def dre_geo(Wl, Vr, R, Ar=0.):
    """doppler rate estimation based on geometry

    doppler rate estimation based on geometry

    Parameters
    ----------
    Wl : float
        Wave length.
    Vr : float, list or array
        Equivalent velocity of radar.
    R : float, list or array
        Minimum slant range from radar to target.
    Ar : float, list or array
        Equivalent squint angle.
    """


