class LRDAnet(th.nn.Module):
    ...

    def __init__(self, Na, Nr, Fsa, Fsr, Ta, Tp, Ka, Kr, fdc, Fc, R0, Vr, La):
        ...

    def forward(self, X):
        ...

    def focus(self, X, pa=None, pr=None):

