def center_dominant_scatters(x, axis=-2, isplot=False):
    """Center the dominant scatters.

    Parameters
    ----------
    x : tensor
        Complex SAR image tensor.
    axis : int, optional
        The axis to be centered in.
    isplot : bool, optional
        Whether to plot.

    Returns
    -------
    tensor
        Centered.

    Raises
    ------
    TypeError
        Intput must be complex image!
    """

def window_data(z, win=None, axis=-2, isplot=False):

