class SarPlat(object):
    r"""SAR platform class.
        sensor
        acquisition
        params
    """

    def name(self):
        ...

    def name(self, value):
        ...

    def sensor(self):
        ...

    def sensor(self, value):
        ...

    def acquisition(self):
        ...

    def acquisition(self, value):
        ...

    def params(self):
        ...

    def params(self, value):
        ...

    def selection(self):
        ...

    def selection(self, value):
        ...

    def printsp(self, verbose=False):

