/* Generated by Cython 0.29.32 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "name": "torchsar.misc.sarplat",
        "sources": [
            "torchsar/misc/sarplat.py"
        ]
    },
    "module_name": "torchsar.misc.sarplat"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_32"
#define CYTHON_HEX_VERSION 0x001D20F0
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC (PYPY_VERSION_HEX >= 0x07030900)
  #endif
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_NOGIL 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
#elif defined(PY_NOGIL)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_NOGIL 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #ifndef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_NOGIL 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0 || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 0
  #elif !defined(CYTHON_FAST_THREAD_STATE)
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL (PY_VERSION_HEX < 0x030A0000)
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #if PY_VERSION_HEX >= 0x030B00A4
    #undef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 0
  #elif !defined(CYTHON_USE_EXC_INFO_STACK)
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_MAJOR_VERSION < 3
    #include "longintrepr.h"
  #endif
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_DefaultClassType PyType_Type
#if PY_VERSION_HEX >= 0x030B00A1
    static CYTHON_INLINE PyCodeObject* __Pyx_PyCode_New(int a, int k, int l, int s, int f,
                                                    PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                                    PyObject *fv, PyObject *cell, PyObject* fn,
                                                    PyObject *name, int fline, PyObject *lnos) {
        PyObject *kwds=NULL, *argcount=NULL, *posonlyargcount=NULL, *kwonlyargcount=NULL;
        PyObject *nlocals=NULL, *stacksize=NULL, *flags=NULL, *replace=NULL, *call_result=NULL, *empty=NULL;
        const char *fn_cstr=NULL;
        const char *name_cstr=NULL;
        PyCodeObject* co=NULL;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        if (!(kwds=PyDict_New())) goto end;
        if (!(argcount=PyLong_FromLong(a))) goto end;
        if (PyDict_SetItemString(kwds, "co_argcount", argcount) != 0) goto end;
        if (!(posonlyargcount=PyLong_FromLong(0))) goto end;
        if (PyDict_SetItemString(kwds, "co_posonlyargcount", posonlyargcount) != 0) goto end;
        if (!(kwonlyargcount=PyLong_FromLong(k))) goto end;
        if (PyDict_SetItemString(kwds, "co_kwonlyargcount", kwonlyargcount) != 0) goto end;
        if (!(nlocals=PyLong_FromLong(l))) goto end;
        if (PyDict_SetItemString(kwds, "co_nlocals", nlocals) != 0) goto end;
        if (!(stacksize=PyLong_FromLong(s))) goto end;
        if (PyDict_SetItemString(kwds, "co_stacksize", stacksize) != 0) goto end;
        if (!(flags=PyLong_FromLong(f))) goto end;
        if (PyDict_SetItemString(kwds, "co_flags", flags) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_code", code) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_consts", c) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_names", n) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_varnames", v) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_freevars", fv) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_cellvars", cell) != 0) goto end;
        if (PyDict_SetItemString(kwds, "co_linetable", lnos) != 0) goto end;
        if (!(fn_cstr=PyUnicode_AsUTF8AndSize(fn, NULL))) goto end;
        if (!(name_cstr=PyUnicode_AsUTF8AndSize(name, NULL))) goto end;
        if (!(co = PyCode_NewEmpty(fn_cstr, name_cstr, fline))) goto end;
        if (!(replace = PyObject_GetAttrString((PyObject*)co, "replace"))) goto cleanup_code_too;
        if (!(empty = PyTuple_New(0))) goto cleanup_code_too; // unfortunately __pyx_empty_tuple isn't available here
        if (!(call_result = PyObject_Call(replace, empty, kwds))) goto cleanup_code_too;
        Py_XDECREF((PyObject*)co);
        co = (PyCodeObject*)call_result;
        call_result = NULL;
        if (0) {
            cleanup_code_too:
            Py_XDECREF((PyObject*)co);
            co = NULL;
        }
        end:
        Py_XDECREF(kwds);
        Py_XDECREF(argcount);
        Py_XDECREF(posonlyargcount);
        Py_XDECREF(kwonlyargcount);
        Py_XDECREF(nlocals);
        Py_XDECREF(stacksize);
        Py_XDECREF(replace);
        Py_XDECREF(call_result);
        Py_XDECREF(empty);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return co;
    }
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsHash_t
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   __Pyx_PyIndex_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #if !defined(_USE_MATH_DEFINES)
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__torchsar__misc__sarplat
#define __PYX_HAVE_API__torchsar__misc__sarplat
/* Early includes */
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "torchsar/misc/sarplat.py",
};

/*--- Type declarations ---*/

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
#if CYTHON_FAST_PYCALL
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif // CYTHON_FAST_PYCALL
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod0.proto */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* UnpackTupleError.proto */
static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

/* UnpackTuple2.proto */
#define __Pyx_unpack_tuple2(tuple, value1, value2, is_tuple, has_known_size, decref_tuple)\
    (likely(is_tuple || PyTuple_Check(tuple)) ?\
        (likely(has_known_size || PyTuple_GET_SIZE(tuple) == 2) ?\
            __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple) :\
            (__Pyx_UnpackTupleError(tuple, 2), -1)) :\
        __Pyx_unpack_tuple2_generic(tuple, value1, value2, has_known_size, decref_tuple))
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
    PyObject* tuple, PyObject** value1, PyObject** value2, int decref_tuple);
static int __Pyx_unpack_tuple2_generic(
    PyObject* tuple, PyObject** value1, PyObject** value2, int has_known_size, int decref_tuple);

/* dict_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* PyFloatBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_TrueDivideObjC(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyFloat_TrueDivideObjC(op1, op2, floatval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceTrueDivide(op1, op2) : PyNumber_TrueDivide(op1, op2))
#endif

/* py_abs.proto */
#if CYTHON_USE_PYLONG_INTERNALS
static PyObject *__Pyx_PyLong_AbsNeg(PyObject *num);
#define __Pyx_PyNumber_Absolute(x)\
    ((likely(PyLong_CheckExact(x))) ?\
         (likely(Py_SIZE(x) >= 0) ? (Py_INCREF(x), (x)) : __Pyx_PyLong_AbsNeg(x)) :\
         PyNumber_Absolute(x))
#else
#define __Pyx_PyNumber_Absolute(x)  PyNumber_Absolute(x)
#endif

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* PyObjectFormat.proto */
#if CYTHON_USE_UNICODE_WRITER
static PyObject* __Pyx_PyObject_Format(PyObject* s, PyObject* f);
#else
#define __Pyx_PyObject_Format(s, f) PyObject_Format(s, f)
#endif

/* JoinPyUnicode.proto */
static PyObject* __Pyx_PyUnicode_Join(PyObject* value_tuple, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char);

/* PyFloatBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_TrueDivideCObj(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyFloat_TrueDivideCObj(op1, op2, floatval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceTrueDivide(op1, op2) : PyNumber_TrueDivide(op1, op2))
#endif

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname);

/* PyObjectFormatAndDecref.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f);
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED 1
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;  // used by FusedFunction for copying defaults
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_Check(obj)  (__Pyx_TypeCheck(obj, __pyx_CyFunctionType))
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* SetNameInClass.proto */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? _PyDict_SetItem_KnownHash(ns, name, value, ((PyASCIIObject *) name)->hash) : PyObject_SetItem(ns, name, value))
#elif CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? PyDict_SetItem(ns, name, value) : PyObject_SetItem(ns, name, value))
#else
#define __Pyx_SetNameInClass(ns, name, value)  PyObject_SetItem(ns, name, value)
#endif

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CStringEquals.proto */
static CYTHON_INLINE int __Pyx_StrEq(const char *, const char *);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);


/* Module declarations from 'torchsar.misc.sarplat' */
#define __Pyx_MODULE_NAME "torchsar.misc.sarplat"
extern int __pyx_module_is_main_torchsar__misc__sarplat;
int __pyx_module_is_main_torchsar__misc__sarplat = 0;

/* Implementation of 'torchsar.misc.sarplat' */
static PyObject *__pyx_builtin_object;
static PyObject *__pyx_builtin_property;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_print;
static const char __pyx_k_C[] = "C";
static const char __pyx_k_H[] = "H";
static const char __pyx_k_V[] = "V";
static const char __pyx_k_f[] = "f";
static const char __pyx_k_k[] = "k";
static const char __pyx_k_v[] = "v";
static const char __pyx_k_Ad[] = "Ad";
static const char __pyx_k_Ai[] = "Ai";
static const char __pyx_k_Ao[] = "Ao";
static const char __pyx_k_Ar[] = "Ar";
static const char __pyx_k_As[] = "As";
static const char __pyx_k_BA[] = "BA";
static const char __pyx_k_BC[] = "BC";
static const char __pyx_k_BG[] = "BG";
static const char __pyx_k_Ba[] = " Ba ";
static const char __pyx_k_Br[] = "Br";
static const char __pyx_k_DA[] = "DA";
static const char __pyx_k_DR[] = "DR";
static const char __pyx_k_DX[] = "DX";
static const char __pyx_k_DY[] = "DY";
static const char __pyx_k_ES[] = "ES";
static const char __pyx_k_Fc[] = "Fc";
static const char __pyx_k_Fs[] = "Fs";
static const char __pyx_k_GM[] = "GM";
static const char __pyx_k_Ge[] = "Ge";
static const char __pyx_k_Ka[] = "Ka";
static const char __pyx_k_Kr[] = "Kr";
static const char __pyx_k_La[] = "La";
static const char __pyx_k_Na[] = "Na";
static const char __pyx_k_Nr[] = "Nr";
static const char __pyx_k_PC[] = "PC";
static const char __pyx_k_PI[] = "PI";
static const char __pyx_k_R0[] = "R0";
static const char __pyx_k_Re[] = "Re";
static const char __pyx_k_Rs[] = "Rs";
static const char __pyx_k_SA[] = "SA";
static const char __pyx_k_SC[] = "SC";
static const char __pyx_k_SG[] = "SG";
static const char __pyx_k_SV[] = "SV";
static const char __pyx_k_Ta[] = "Ta";
static const char __pyx_k_To[] = "To";
static const char __pyx_k_Tp[] = "Tp";
static const char __pyx_k_Tr[] = " > Tr = ";
static const char __pyx_k_Vg[] = "Vg";
static const char __pyx_k_Vr[] = "Vr";
static const char __pyx_k_Vs[] = "Vs";
static const char __pyx_k_Wl[] = "Wl";
static const char __pyx_k_Ws[] = "Ws";
static const char __pyx_k_Xc[] = "Xc";
static const char __pyx_k_Yc[] = "Yc";
static const char __pyx_k__5[] = "!";
static const char __pyx_k__6[] = "\303\227";
static const char __pyx_k__7[] = "= ";
static const char __pyx_k__8[] = "\\";
static const char __pyx_k__9[] = " \342\211\240 ";
static const char __pyx_k_fa[] = "fa";
static const char __pyx_k_fr[] = "fr";
static const char __pyx_k_np[] = "np";
static const char __pyx_k_pi[] = "pi";
static const char __pyx_k_t0[] = "t0";
static const char __pyx_k_ta[] = "ta";
static const char __pyx_k_th[] = "th";
static const char __pyx_k_tr[] = "tr";
static const char __pyx_k_Aba[] = "Aba";
static const char __pyx_k_Abr[] = "Abr";
static const char __pyx_k_Aon[] = "Aon";
static const char __pyx_k_BAV[] = "BAV";
static const char __pyx_k_BWa[] = "BWa";
static const char __pyx_k_EPS[] = "EPS";
static const char __pyx_k_FPa[] = "FPa";
static const char __pyx_k_FPr[] = "FPr";
static const char __pyx_k_Fsa[] = "Fsa";
static const char __pyx_k_Fsr[] = "Fsr";
static const char __pyx_k_PRF[] = "PRF";
static const char __pyx_k_PRT[] = "PRT";
static const char __pyx_k_Rb0[] = "Rb0";
static const char __pyx_k_Rbc[] = "Rbc";
static const char __pyx_k_Rea[] = "Rea";
static const char __pyx_k_Ree[] = "Ree";
static const char __pyx_k_Rs0[] = "Rs0";
static const char __pyx_k_Rsc[] = "Rsc";
static const char __pyx_k_SAV[] = "SAV";
static const char __pyx_k_Tsa[] = "Tsa";
static const char __pyx_k_Tsr[] = "Tsr";
static const char __pyx_k_Xbc[] = "Xbc";
static const char __pyx_k_Xsc[] = "Xsc";
static const char __pyx_k_Ybc[] = "Ybc";
static const char __pyx_k_Ysc[] = "Ysc";
static const char __pyx_k__10[] = " \303\227 ";
static const char __pyx_k__11[] = " = ";
static const char __pyx_k__12[] = " - ";
static const char __pyx_k__13[] = ", ";
static const char __pyx_k__14[] = "].";
static const char __pyx_k__15[] = "================";
static const char __pyx_k__19[] = ": ";
static const char __pyx_k__21[] = "=============================================================";
static const char __pyx_k__23[] = "*";
static const char __pyx_k_abs[] = "abs";
static const char __pyx_k_cos[] = "cos";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_f_2[] = " f";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_sin[] = "sin";
static const char __pyx_k_tac[] = "tac";
static const char __pyx_k_tan[] = "tan";
static const char __pyx_k_Ba_2[] = "Ba";
static const char __pyx_k_Bdop[] = "Bdop";
static const char __pyx_k_Done[] = "---Done!";
static const char __pyx_k_FPr0[] = "FPr0";
static const char __pyx_k_FPr1[] = "FPr1";
static const char __pyx_k_Lsar[] = "Lsar";
static const char __pyx_k_Nsar[] = "Nsar";
static const char __pyx_k_Rfar[] = "Rfar";
static const char __pyx_k_Tr_2[] = "Tr";
static const char __pyx_k_Tsar[] = "Tsar";
static const char __pyx_k_Xfar[] = "Xfar";
static const char __pyx_k_Yfar[] = "Yfar";
static const char __pyx_k_fadc[] = "fadc";
static const char __pyx_k_info[] = "info";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "_name";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_sign[] = "sign";
static const char __pyx_k_sqrt[] = "sqrt";
static const char __pyx_k_tend[] = "tend";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_tfar[] = "tfar";
static const char __pyx_k_used[] = "(used).";
static const char __pyx_k_xmax[] = "xmax";
static const char __pyx_k_xmin[] = "xmin";
static const char __pyx_k_ymax[] = "ymax";
static const char __pyx_k_ymin[] = "ymin";
static const char __pyx_k_Rnear[] = "Rnear";
static const char __pyx_k_SubAd[] = "SubAd";
static const char __pyx_k_SubBA[] = "SubBA";
static const char __pyx_k_SubBC[] = "SubBC";
static const char __pyx_k_SubBa[] = "SubBa";
static const char __pyx_k_SubEA[] = "SubEA";
static const char __pyx_k_SubES[] = "SubES";
static const char __pyx_k_SubKa[] = "SubKa";
static const char __pyx_k_SubNa[] = "SubNa";
static const char __pyx_k_SubNr[] = "SubNr";
static const char __pyx_k_SubR0[] = "SubR0";
static const char __pyx_k_SubSA[] = "SubSA";
static const char __pyx_k_SubSC[] = "SubSC";
static const char __pyx_k_SubTa[] = "SubTa";
static const char __pyx_k_SubTr[] = "SubTr";
static const char __pyx_k_Xnear[] = "Xnear";
static const char __pyx_k_Ynear[] = "Ynear";
static const char __pyx_k_array[] = "array";
static const char __pyx_k_error[] = "error";
static const char __pyx_k_faSub[] = "faSub";
static const char __pyx_k_frSub[] = "frSub";
static const char __pyx_k_items[] = "items";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_param[] = "param";
static const char __pyx_k_print[] = "print";
static const char __pyx_k_shape[] = "shape";
static const char __pyx_k_t0Sub[] = "t0Sub";
static const char __pyx_k_taSub[] = "taSub";
static const char __pyx_k_tnear[] = "tnear";
static const char __pyx_k_torch[] = "torch";
static const char __pyx_k_trSub[] = "trSub";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_PRFmax[] = "PRFmax";
static const char __pyx_k_PRFmin[] = "PRFmin";
static const char __pyx_k_SubAbr[] = "SubAbr";
static const char __pyx_k_SubAon[] = "SubAon";
static const char __pyx_k_SubFPa[] = "SubFPa";
static const char __pyx_k_SubFPr[] = "SubFPr";
static const char __pyx_k_SubRb0[] = "SubRb0";
static const char __pyx_k_SubRbc[] = "SubRbc";
static const char __pyx_k_SubRs0[] = "SubRs0";
static const char __pyx_k_SubRsc[] = "SubRsc";
static const char __pyx_k_SubXbc[] = "SubXbc";
static const char __pyx_k_SubXsc[] = "SubXsc";
static const char __pyx_k_SubYbc[] = "SubYbc";
static const char __pyx_k_SubYsc[] = "SubYsc";
static const char __pyx_k_Tensor[] = "Tensor";
static const char __pyx_k_arccos[] = "arccos";
static const char __pyx_k_degree[] = " degree!";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_name_2[] = "__name__";
static const char __pyx_k_name_3[] = "name";
static const char __pyx_k_object[] = "object";
static const char __pyx_k_params[] = "_params";
static const char __pyx_k_sensor[] = "_sensor";
static const char __pyx_k_setter[] = "setter";
static const char __pyx_k_tstart[] = "tstart";
static const char __pyx_k_Airbone[] = "---Airbone.";
static const char __pyx_k_SarPlat[] = "SarPlat";
static const char __pyx_k_SubFPr0[] = "SubFPr0";
static const char __pyx_k_SubFPr1[] = "SubFPr1";
static const char __pyx_k_SubLsar[] = "SubLsar";
static const char __pyx_k_SubRfar[] = "SubRfar";
static const char __pyx_k_SubTsar[] = "SubTsar";
static const char __pyx_k_SubXfar[] = "SubXfar";
static const char __pyx_k_SubYfar[] = "SubYfar";
static const char __pyx_k_logging[] = "logging";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_printsp[] = "printsp";
static const char __pyx_k_tendSub[] = "tendSub";
static const char __pyx_k_tfarSub[] = "tfarSub";
static const char __pyx_k_verbose[] = "verbose";
static const char __pyx_k_warning[] = "warning";
static const char __pyx_k_xmaxSub[] = "xmaxSub";
static const char __pyx_k_xminSub[] = "xminSub";
static const char __pyx_k_ymaxSub[] = "ymaxSub";
static const char __pyx_k_yminSub[] = "yminSub";
static const char __pyx_k_BeamArea[] = "BeamArea";
static const char __pyx_k_EchoSize[] = "EchoSize";
static const char __pyx_k_SubRnear[] = "SubRnear";
static const char __pyx_k_SubXnear[] = "SubXnear";
static const char __pyx_k_SubYnear[] = "SubYnear";
static const char __pyx_k_linspace[] = "linspace";
static const char __pyx_k_params_2[] = "--------------------------params-----------------------------";
static const char __pyx_k_params_3[] = "params";
static const char __pyx_k_property[] = "property";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_sensor_2[] = "--------------------------sensor-----------------------------";
static const char __pyx_k_sensor_3[] = "sensor";
static const char __pyx_k_shape_of[] = "shape of ";
static const char __pyx_k_tnearSub[] = "tnearSub";
static const char __pyx_k_Nr_Tr_Fsr[] = "Nr \342\211\240 Tr \303\227 Fsr! ";
static const char __pyx_k_Satellite[] = "---Satellite.";
static const char __pyx_k_SceneArea[] = "SceneArea";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_selection[] = "_selection";
static const char __pyx_k_should_in[] = " should in [";
static const char __pyx_k_tstartSub[] = "tstartSub";
static const char __pyx_k_BeamCenter[] = "BeamCenter";
static const char __pyx_k_PlatCenter[] = "PlatCenter";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_SceneCenter[] = "SceneCenter";
static const char __pyx_k_StateVector[] = "StateVector";
static const char __pyx_k_SubBeamArea[] = "SubBeamArea";
static const char __pyx_k_SubEchoSize[] = "SubEchoSize";
static const char __pyx_k_acquisition[] = "_acquisition";
static const char __pyx_k_selection_2[] = "-------------------------selection---------------------------";
static const char __pyx_k_selection_3[] = "selection";
static const char __pyx_k_GeometryMode[] = "GeometryMode";
static const char __pyx_k_SAR_platform[] = "===============SAR platform: ";
static const char __pyx_k_SarPlat_name[] = "SarPlat.name";
static const char __pyx_k_SubSceneArea[] = "SubSceneArea";
static const char __pyx_k_cr_footprint[] = "cr_footprint";
static const char __pyx_k_SubBeamCenter[] = "SubBeamCenter";
static const char __pyx_k_SubEchoAnchor[] = "SubEchoAnchor";
static const char __pyx_k_acquisition_2[] = "-----------------------acquisition---------------------------";
static const char __pyx_k_acquisition_3[] = "acquisition";
static const char __pyx_k_SarPlat_params[] = "SarPlat.params";
static const char __pyx_k_SarPlat_sensor[] = "SarPlat.sensor";
static const char __pyx_k_SubSceneCenter[] = "SubSceneCenter";
static const char __pyx_k_SarPlat_printsp[] = "SarPlat.printsp";
static const char __pyx_k_min_slant_range[] = "min_slant_range";
static const char __pyx_k_SarPlat_selection[] = "SarPlat.selection";
static const char __pyx_k_azimuth_beamwidth[] = "azimuth_beamwidth";
static const char __pyx_k_Nr_should_be_TrFsr[] = "~~~Nr should be Tr\303\227Fsr= ";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_SarPlat_acquisition[] = "SarPlat.acquisition";
static const char __pyx_k_or_Abr_should_be_Abr[] = "\r\n                          or Abr should be Abr= ";
static const char __pyx_k_torchbox_utils_const[] = "torchbox.utils.const";
static const char __pyx_k_name_must_be_a_string[] = "name must be a string!";
static const char __pyx_k_or_Fsr_should_be_NrTr[] = "\r\n                          or Fsr should be Nr\303\267Tr= ";
static const char __pyx_k_params_must_be_a_dict[] = "params must be a dict!";
static const char __pyx_k_sensor_must_be_a_dict[] = "sensor must be a dict!";
static const char __pyx_k_torchsar_misc_sarplat[] = "torchsar.misc.sarplat";
static const char __pyx_k_compute_range_beamwidth2[] = "compute_range_beamwidth2";
static const char __pyx_k_torchsar_misc_sarplat_py[] = "torchsar/misc/sarplat.py";
static const char __pyx_k_acquisition_must_be_a_dict[] = "acquisition must be a dict!";
static const char __pyx_k_EchoSize_should_be_specified[] = "~~~EchoSize should be specified!";
static const char __pyx_k_setted_equivalent_velocity_is[] = ", setted equivalent velocity is ";
static const char __pyx_k_AirboneSatelliteHeightBoundary[] = "AirboneSatelliteHeightBoundary";
static const char __pyx_k_which_maks_a_point_outside_the[] = " which maks a point outside the beam footprint.";
static const char __pyx_k_Check_and_set_SAR_platform_name[] = "---Check and set SAR platform name...";
static const char __pyx_k_Check_and_set_SAR_sensor_parame[] = "---Check and set SAR sensor parameters...";
static const char __pyx_k_Check_and_set_acquisition_param[] = "---Check and set acquisition parameters...";
static const char __pyx_k_Check_and_set_algorithm_paramet[] = "---Check and set algorithm parameters...";
static const char __pyx_k_Check_and_set_sub_area_selectio[] = "---Check and set sub area selection...";
static const char __pyx_k_Computed_equivalent_velocity_is[] = "---Computed equivalent velocity is ";
static const char __pyx_k_Sampling_rate_should_be_in_rang[] = "~~~Sampling rate should be in range [1.1, 2.0]*Ba, \r\n   Fsa ";
static const char __pyx_k_The_antenna_elevation_beamwidth[] = "~~~The antenna elevation beamwidth Abr is too large!";
static const char __pyx_k_The_synthetic_aperture_length_L[] = "~~~The synthetic aperture length Lsar=";
static const char __pyx_k_To_avoid_grating_lobe_issues_PR[] = "~~~To avoid grating lobe issues, PRF=";
static const char __pyx_k_is_large_than_r_n_the_cross_ran[] = " is large than \\r\\n   \\\n                the cross range footprint size Rcr=";
static const char __pyx_k_r_Sampling_rate_should_be_lower[] = "r~~~Sampling rate should be lower to avoid range ambiguity \r\n                   (1.0 \303\267 Fsa - Tp = 1.0 \303\267 ";
static const char __pyx_k_SAR_platform_class_sensor_acquis[] = "SAR platform class.\n        sensor\n        acquisition\n        params\n    ";
static const char __pyx_k_torchsar_sharing_beamwidth_footp[] = "torchsar.sharing.beamwidth_footprint";
static const char __pyx_k_torchsar_sharing_slant_ground_ra[] = "torchsar.sharing.slant_ground_range";
static PyObject *__pyx_n_s_Aba;
static PyObject *__pyx_n_u_Aba;
static PyObject *__pyx_n_s_Abr;
static PyObject *__pyx_n_u_Abr;
static PyObject *__pyx_n_s_Ad;
static PyObject *__pyx_n_u_Ad;
static PyObject *__pyx_n_s_Ai;
static PyObject *__pyx_n_u_Ai;
static PyObject *__pyx_kp_u_Airbone;
static PyObject *__pyx_n_s_AirboneSatelliteHeightBoundary;
static PyObject *__pyx_n_s_Ao;
static PyObject *__pyx_n_s_Aon;
static PyObject *__pyx_n_u_Aon;
static PyObject *__pyx_n_s_Ar;
static PyObject *__pyx_n_u_Ar;
static PyObject *__pyx_n_s_As;
static PyObject *__pyx_n_u_As;
static PyObject *__pyx_n_s_BA;
static PyObject *__pyx_n_s_BAV;
static PyObject *__pyx_n_s_BC;
static PyObject *__pyx_n_u_BG;
static PyObject *__pyx_n_s_BWa;
static PyObject *__pyx_n_u_BWa;
static PyObject *__pyx_kp_u_Ba;
static PyObject *__pyx_n_s_Ba_2;
static PyObject *__pyx_n_s_Bdop;
static PyObject *__pyx_n_u_Bdop;
static PyObject *__pyx_n_u_BeamArea;
static PyObject *__pyx_n_u_BeamCenter;
static PyObject *__pyx_n_u_Br;
static PyObject *__pyx_n_s_C;
static PyObject *__pyx_kp_u_Check_and_set_SAR_platform_name;
static PyObject *__pyx_kp_u_Check_and_set_SAR_sensor_parame;
static PyObject *__pyx_kp_u_Check_and_set_acquisition_param;
static PyObject *__pyx_kp_u_Check_and_set_algorithm_paramet;
static PyObject *__pyx_kp_u_Check_and_set_sub_area_selectio;
static PyObject *__pyx_kp_u_Computed_equivalent_velocity_is;
static PyObject *__pyx_n_u_DA;
static PyObject *__pyx_n_u_DR;
static PyObject *__pyx_n_u_DX;
static PyObject *__pyx_n_u_DY;
static PyObject *__pyx_kp_u_Done;
static PyObject *__pyx_n_s_EPS;
static PyObject *__pyx_n_s_ES;
static PyObject *__pyx_n_u_EchoSize;
static PyObject *__pyx_kp_u_EchoSize_should_be_specified;
static PyObject *__pyx_n_s_FPa;
static PyObject *__pyx_n_u_FPa;
static PyObject *__pyx_n_s_FPr;
static PyObject *__pyx_n_u_FPr;
static PyObject *__pyx_n_s_FPr0;
static PyObject *__pyx_n_s_FPr1;
static PyObject *__pyx_n_s_Fc;
static PyObject *__pyx_n_u_Fc;
static PyObject *__pyx_n_s_Fs;
static PyObject *__pyx_n_u_Fs;
static PyObject *__pyx_n_s_Fsa;
static PyObject *__pyx_n_u_Fsa;
static PyObject *__pyx_n_s_Fsr;
static PyObject *__pyx_n_u_Fsr;
static PyObject *__pyx_n_s_GM;
static PyObject *__pyx_n_s_Ge;
static PyObject *__pyx_n_u_GeometryMode;
static PyObject *__pyx_n_s_H;
static PyObject *__pyx_n_u_H;
static PyObject *__pyx_n_s_Ka;
static PyObject *__pyx_n_u_Ka;
static PyObject *__pyx_n_s_Kr;
static PyObject *__pyx_n_u_Kr;
static PyObject *__pyx_n_s_La;
static PyObject *__pyx_n_u_La;
static PyObject *__pyx_n_s_Lsar;
static PyObject *__pyx_n_u_Lsar;
static PyObject *__pyx_n_s_Na;
static PyObject *__pyx_n_u_Na;
static PyObject *__pyx_n_s_Nr;
static PyObject *__pyx_n_u_Nr;
static PyObject *__pyx_kp_u_Nr_Tr_Fsr;
static PyObject *__pyx_kp_u_Nr_should_be_TrFsr;
static PyObject *__pyx_n_u_Nsar;
static PyObject *__pyx_n_s_PC;
static PyObject *__pyx_n_s_PI;
static PyObject *__pyx_n_s_PRF;
static PyObject *__pyx_n_u_PRF;
static PyObject *__pyx_n_s_PRFmax;
static PyObject *__pyx_n_s_PRFmin;
static PyObject *__pyx_n_s_PRT;
static PyObject *__pyx_n_u_PRT;
static PyObject *__pyx_n_u_PlatCenter;
static PyObject *__pyx_n_s_R0;
static PyObject *__pyx_n_s_Rb0;
static PyObject *__pyx_n_u_Rb0;
static PyObject *__pyx_n_s_Rbc;
static PyObject *__pyx_n_u_Rbc;
static PyObject *__pyx_n_s_Re;
static PyObject *__pyx_n_s_Rea;
static PyObject *__pyx_n_s_Ree;
static PyObject *__pyx_n_s_Rfar;
static PyObject *__pyx_n_u_Rfar;
static PyObject *__pyx_n_s_Rnear;
static PyObject *__pyx_n_u_Rnear;
static PyObject *__pyx_n_s_Rs;
static PyObject *__pyx_n_u_Rs;
static PyObject *__pyx_n_s_Rs0;
static PyObject *__pyx_n_u_Rs0;
static PyObject *__pyx_n_s_Rsc;
static PyObject *__pyx_n_u_Rsc;
static PyObject *__pyx_n_s_SA;
static PyObject *__pyx_kp_u_SAR_platform;
static PyObject *__pyx_kp_s_SAR_platform_class_sensor_acquis;
static PyObject *__pyx_n_s_SAV;
static PyObject *__pyx_n_s_SC;
static PyObject *__pyx_n_u_SG;
static PyObject *__pyx_n_s_SV;
static PyObject *__pyx_kp_u_Sampling_rate_should_be_in_rang;
static PyObject *__pyx_n_s_SarPlat;
static PyObject *__pyx_n_s_SarPlat_acquisition;
static PyObject *__pyx_n_s_SarPlat_name;
static PyObject *__pyx_n_s_SarPlat_params;
static PyObject *__pyx_n_s_SarPlat_printsp;
static PyObject *__pyx_n_s_SarPlat_selection;
static PyObject *__pyx_n_s_SarPlat_sensor;
static PyObject *__pyx_kp_u_Satellite;
static PyObject *__pyx_n_u_SceneArea;
static PyObject *__pyx_n_u_SceneCenter;
static PyObject *__pyx_n_u_StateVector;
static PyObject *__pyx_n_s_SubAbr;
static PyObject *__pyx_n_u_SubAbr;
static PyObject *__pyx_n_s_SubAd;
static PyObject *__pyx_n_u_SubAd;
static PyObject *__pyx_n_s_SubAon;
static PyObject *__pyx_n_u_SubAon;
static PyObject *__pyx_n_s_SubBA;
static PyObject *__pyx_n_s_SubBC;
static PyObject *__pyx_n_s_SubBa;
static PyObject *__pyx_n_u_SubBa;
static PyObject *__pyx_n_u_SubBeamArea;
static PyObject *__pyx_n_u_SubBeamCenter;
static PyObject *__pyx_n_s_SubEA;
static PyObject *__pyx_n_s_SubES;
static PyObject *__pyx_n_u_SubEchoAnchor;
static PyObject *__pyx_n_u_SubEchoSize;
static PyObject *__pyx_n_s_SubFPa;
static PyObject *__pyx_n_u_SubFPa;
static PyObject *__pyx_n_s_SubFPr;
static PyObject *__pyx_n_u_SubFPr;
static PyObject *__pyx_n_s_SubFPr0;
static PyObject *__pyx_n_s_SubFPr1;
static PyObject *__pyx_n_s_SubKa;
static PyObject *__pyx_n_u_SubKa;
static PyObject *__pyx_n_s_SubLsar;
static PyObject *__pyx_n_u_SubLsar;
static PyObject *__pyx_n_s_SubNa;
static PyObject *__pyx_n_u_SubNa;
static PyObject *__pyx_n_s_SubNr;
static PyObject *__pyx_n_u_SubNr;
static PyObject *__pyx_n_s_SubR0;
static PyObject *__pyx_n_s_SubRb0;
static PyObject *__pyx_n_u_SubRb0;
static PyObject *__pyx_n_s_SubRbc;
static PyObject *__pyx_n_u_SubRbc;
static PyObject *__pyx_n_s_SubRfar;
static PyObject *__pyx_n_u_SubRfar;
static PyObject *__pyx_n_s_SubRnear;
static PyObject *__pyx_n_u_SubRnear;
static PyObject *__pyx_n_s_SubRs0;
static PyObject *__pyx_n_u_SubRs0;
static PyObject *__pyx_n_s_SubRsc;
static PyObject *__pyx_n_u_SubRsc;
static PyObject *__pyx_n_s_SubSA;
static PyObject *__pyx_n_s_SubSC;
static PyObject *__pyx_n_u_SubSceneArea;
static PyObject *__pyx_n_u_SubSceneCenter;
static PyObject *__pyx_n_s_SubTa;
static PyObject *__pyx_n_u_SubTa;
static PyObject *__pyx_n_s_SubTr;
static PyObject *__pyx_n_u_SubTr;
static PyObject *__pyx_n_s_SubTsar;
static PyObject *__pyx_n_u_SubTsar;
static PyObject *__pyx_n_s_SubXbc;
static PyObject *__pyx_n_s_SubXfar;
static PyObject *__pyx_n_s_SubXnear;
static PyObject *__pyx_n_s_SubXsc;
static PyObject *__pyx_n_s_SubYbc;
static PyObject *__pyx_n_s_SubYfar;
static PyObject *__pyx_n_s_SubYnear;
static PyObject *__pyx_n_s_SubYsc;
static PyObject *__pyx_n_s_Ta;
static PyObject *__pyx_n_u_Ta;
static PyObject *__pyx_n_s_Tensor;
static PyObject *__pyx_kp_u_The_antenna_elevation_beamwidth;
static PyObject *__pyx_kp_u_The_synthetic_aperture_length_L;
static PyObject *__pyx_n_u_To;
static PyObject *__pyx_kp_u_To_avoid_grating_lobe_issues_PR;
static PyObject *__pyx_n_s_Tp;
static PyObject *__pyx_n_u_Tp;
static PyObject *__pyx_kp_u_Tr;
static PyObject *__pyx_n_s_Tr_2;
static PyObject *__pyx_n_u_Tr_2;
static PyObject *__pyx_n_s_Tsa;
static PyObject *__pyx_n_u_Tsa;
static PyObject *__pyx_n_s_Tsar;
static PyObject *__pyx_n_u_Tsar;
static PyObject *__pyx_n_s_Tsr;
static PyObject *__pyx_n_u_Tsr;
static PyObject *__pyx_n_s_V;
static PyObject *__pyx_n_u_V;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_Vg;
static PyObject *__pyx_n_u_Vg;
static PyObject *__pyx_n_s_Vr;
static PyObject *__pyx_n_u_Vr;
static PyObject *__pyx_n_s_Vs;
static PyObject *__pyx_n_u_Vs;
static PyObject *__pyx_n_s_Wl;
static PyObject *__pyx_n_u_Wl;
static PyObject *__pyx_n_u_Ws;
static PyObject *__pyx_n_s_Xbc;
static PyObject *__pyx_n_s_Xc;
static PyObject *__pyx_n_u_Xc;
static PyObject *__pyx_n_s_Xfar;
static PyObject *__pyx_n_s_Xnear;
static PyObject *__pyx_n_s_Xsc;
static PyObject *__pyx_n_s_Ybc;
static PyObject *__pyx_n_s_Yc;
static PyObject *__pyx_n_u_Yc;
static PyObject *__pyx_n_s_Yfar;
static PyObject *__pyx_n_s_Ynear;
static PyObject *__pyx_n_s_Ysc;
static PyObject *__pyx_kp_u__10;
static PyObject *__pyx_kp_u__11;
static PyObject *__pyx_kp_u__12;
static PyObject *__pyx_kp_u__13;
static PyObject *__pyx_kp_u__14;
static PyObject *__pyx_kp_u__15;
static PyObject *__pyx_kp_u__19;
static PyObject *__pyx_kp_u__21;
static PyObject *__pyx_n_s__23;
static PyObject *__pyx_kp_u__5;
static PyObject *__pyx_kp_u__6;
static PyObject *__pyx_kp_u__7;
static PyObject *__pyx_kp_u__8;
static PyObject *__pyx_kp_u__9;
static PyObject *__pyx_n_s_abs;
static PyObject *__pyx_n_s_acquisition;
static PyObject *__pyx_n_u_acquisition;
static PyObject *__pyx_kp_u_acquisition_2;
static PyObject *__pyx_n_s_acquisition_3;
static PyObject *__pyx_kp_u_acquisition_must_be_a_dict;
static PyObject *__pyx_n_s_arccos;
static PyObject *__pyx_n_s_array;
static PyObject *__pyx_n_s_azimuth_beamwidth;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_compute_range_beamwidth2;
static PyObject *__pyx_n_s_cos;
static PyObject *__pyx_n_s_cr_footprint;
static PyObject *__pyx_kp_u_degree;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_error;
static PyObject *__pyx_n_u_f;
static PyObject *__pyx_kp_u_f_2;
static PyObject *__pyx_n_s_fa;
static PyObject *__pyx_n_u_fa;
static PyObject *__pyx_n_s_faSub;
static PyObject *__pyx_n_u_faSub;
static PyObject *__pyx_n_s_fadc;
static PyObject *__pyx_n_u_fadc;
static PyObject *__pyx_n_s_fr;
static PyObject *__pyx_n_u_fr;
static PyObject *__pyx_n_s_frSub;
static PyObject *__pyx_n_u_frSub;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_info;
static PyObject *__pyx_kp_u_is_large_than_r_n_the_cross_ran;
static PyObject *__pyx_n_s_items;
static PyObject *__pyx_n_s_k;
static PyObject *__pyx_n_s_key;
static PyObject *__pyx_n_s_linspace;
static PyObject *__pyx_n_s_logging;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_min_slant_range;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_n_s_name_3;
static PyObject *__pyx_kp_u_name_must_be_a_string;
static PyObject *__pyx_n_s_np;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_n_s_object;
static PyObject *__pyx_kp_u_or_Abr_should_be_Abr;
static PyObject *__pyx_kp_u_or_Fsr_should_be_NrTr;
static PyObject *__pyx_n_s_param;
static PyObject *__pyx_n_s_params;
static PyObject *__pyx_n_u_params;
static PyObject *__pyx_kp_u_params_2;
static PyObject *__pyx_n_s_params_3;
static PyObject *__pyx_kp_u_params_must_be_a_dict;
static PyObject *__pyx_n_s_pi;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_print;
static PyObject *__pyx_n_s_printsp;
static PyObject *__pyx_n_s_property;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_kp_u_r_Sampling_rate_should_be_lower;
static PyObject *__pyx_n_s_selection;
static PyObject *__pyx_kp_u_selection_2;
static PyObject *__pyx_n_s_selection_3;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_n_s_sensor;
static PyObject *__pyx_n_u_sensor;
static PyObject *__pyx_kp_u_sensor_2;
static PyObject *__pyx_n_s_sensor_3;
static PyObject *__pyx_kp_u_sensor_must_be_a_dict;
static PyObject *__pyx_kp_u_setted_equivalent_velocity_is;
static PyObject *__pyx_n_s_setter;
static PyObject *__pyx_n_s_shape;
static PyObject *__pyx_kp_u_shape_of;
static PyObject *__pyx_kp_u_should_in;
static PyObject *__pyx_n_s_sign;
static PyObject *__pyx_n_s_sin;
static PyObject *__pyx_n_s_sqrt;
static PyObject *__pyx_n_s_t0;
static PyObject *__pyx_n_u_t0;
static PyObject *__pyx_n_s_t0Sub;
static PyObject *__pyx_n_s_ta;
static PyObject *__pyx_n_u_ta;
static PyObject *__pyx_n_s_taSub;
static PyObject *__pyx_n_u_taSub;
static PyObject *__pyx_n_s_tac;
static PyObject *__pyx_n_u_tac;
static PyObject *__pyx_n_s_tan;
static PyObject *__pyx_n_s_tend;
static PyObject *__pyx_n_u_tend;
static PyObject *__pyx_n_s_tendSub;
static PyObject *__pyx_n_u_tendSub;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_tfar;
static PyObject *__pyx_n_u_tfar;
static PyObject *__pyx_n_s_tfarSub;
static PyObject *__pyx_n_u_tfarSub;
static PyObject *__pyx_n_s_th;
static PyObject *__pyx_n_s_tnear;
static PyObject *__pyx_n_u_tnear;
static PyObject *__pyx_n_s_tnearSub;
static PyObject *__pyx_n_u_tnearSub;
static PyObject *__pyx_n_s_torch;
static PyObject *__pyx_n_s_torchbox_utils_const;
static PyObject *__pyx_n_s_torchsar_misc_sarplat;
static PyObject *__pyx_kp_s_torchsar_misc_sarplat_py;
static PyObject *__pyx_n_s_torchsar_sharing_beamwidth_footp;
static PyObject *__pyx_n_s_torchsar_sharing_slant_ground_ra;
static PyObject *__pyx_n_s_tr;
static PyObject *__pyx_n_u_tr;
static PyObject *__pyx_n_s_trSub;
static PyObject *__pyx_n_u_trSub;
static PyObject *__pyx_n_s_tstart;
static PyObject *__pyx_n_u_tstart;
static PyObject *__pyx_n_s_tstartSub;
static PyObject *__pyx_n_u_tstartSub;
static PyObject *__pyx_kp_u_used;
static PyObject *__pyx_n_s_v;
static PyObject *__pyx_n_s_value;
static PyObject *__pyx_n_s_verbose;
static PyObject *__pyx_n_s_warning;
static PyObject *__pyx_kp_u_which_maks_a_point_outside_the;
static PyObject *__pyx_n_s_xmax;
static PyObject *__pyx_n_u_xmax;
static PyObject *__pyx_n_s_xmaxSub;
static PyObject *__pyx_n_u_xmaxSub;
static PyObject *__pyx_n_s_xmin;
static PyObject *__pyx_n_u_xmin;
static PyObject *__pyx_n_s_xminSub;
static PyObject *__pyx_n_u_xminSub;
static PyObject *__pyx_n_s_ymax;
static PyObject *__pyx_n_u_ymax;
static PyObject *__pyx_n_s_ymaxSub;
static PyObject *__pyx_n_u_ymaxSub;
static PyObject *__pyx_n_s_ymin;
static PyObject *__pyx_n_u_ymin;
static PyObject *__pyx_n_s_yminSub;
static PyObject *__pyx_n_u_yminSub;
static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_name(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_2name(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_4sensor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_6sensor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_8acquisition(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_10acquisition(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_12params(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_14params(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_16selection(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_18selection(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_20printsp(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_verbose); /* proto */
static PyObject *__pyx_float_2_;
static PyObject *__pyx_float_0_0;
static PyObject *__pyx_float_0_5;
static PyObject *__pyx_float_1_0;
static PyObject *__pyx_float_1_1;
static PyObject *__pyx_float_2_0;
static PyObject *__pyx_float_1_25;
static PyObject *__pyx_float_180_;
static PyObject *__pyx_float_60e3;
static PyObject *__pyx_float_0_886;
static PyObject *__pyx_float_1_905;
static PyObject *__pyx_float_neg_2_0;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_3;
static PyObject *__pyx_int_neg_1;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__20;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_tuple__35;
static PyObject *__pyx_tuple__37;
static PyObject *__pyx_tuple__39;
static PyObject *__pyx_tuple__41;
static PyObject *__pyx_tuple__43;
static PyObject *__pyx_tuple__45;
static PyObject *__pyx_tuple__47;
static PyObject *__pyx_codeobj__26;
static PyObject *__pyx_codeobj__28;
static PyObject *__pyx_codeobj__30;
static PyObject *__pyx_codeobj__32;
static PyObject *__pyx_codeobj__34;
static PyObject *__pyx_codeobj__36;
static PyObject *__pyx_codeobj__38;
static PyObject *__pyx_codeobj__40;
static PyObject *__pyx_codeobj__42;
static PyObject *__pyx_codeobj__44;
static PyObject *__pyx_codeobj__46;
/* Late includes */

/* "torchsar/misc/sarplat.py":46
 * 
 *     @property
 *     def name(self):             # <<<<<<<<<<<<<<
 *         return self._name
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_1name(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_1name = {"name", (PyCFunction)__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_1name, METH_O, 0};
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_1name(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("name (wrapper)", 0);
  __pyx_r = __pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_name(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_name(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("name", 0);

  /* "torchsar/misc/sarplat.py":47
 *     @property
 *     def name(self):
 *         return self._name             # <<<<<<<<<<<<<<
 * 
 *     @name.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "torchsar/misc/sarplat.py":46
 * 
 *     @property
 *     def name(self):             # <<<<<<<<<<<<<<
 *         return self._name
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("torchsar.misc.sarplat.SarPlat.name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "torchsar/misc/sarplat.py":50
 * 
 *     @name.setter
 *     def name(self, value):             # <<<<<<<<<<<<<<
 *         logging.info("---Check and set SAR platform name...")
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_3name(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_3name = {"name", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_3name, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_3name(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_value = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("name (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_value,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("name", 1, 2, 2, 1); __PYX_ERR(0, 50, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "name") < 0)) __PYX_ERR(0, 50, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_value = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("name", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 50, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("torchsar.misc.sarplat.SarPlat.name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_2name(__pyx_self, __pyx_v_self, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_2name(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("name", 0);

  /* "torchsar/misc/sarplat.py":51
 *     @name.setter
 *     def name(self, value):
 *         logging.info("---Check and set SAR platform name...")             # <<<<<<<<<<<<<<
 * 
 *         if not isinstance(value, str):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_logging); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_info); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_kp_u_Check_and_set_SAR_platform_name) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_Check_and_set_SAR_platform_name);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":53
 *         logging.info("---Check and set SAR platform name...")
 * 
 *         if not isinstance(value, str):             # <<<<<<<<<<<<<<
 *             raise ValueError('name must be a string!')
 *         self._name = value
 */
  __pyx_t_4 = PyUnicode_Check(__pyx_v_value); 
  __pyx_t_5 = ((!(__pyx_t_4 != 0)) != 0);
  if (unlikely(__pyx_t_5)) {

    /* "torchsar/misc/sarplat.py":54
 * 
 *         if not isinstance(value, str):
 *             raise ValueError('name must be a string!')             # <<<<<<<<<<<<<<
 *         self._name = value
 *         logging.info("---Done!")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 54, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 54, __pyx_L1_error)

    /* "torchsar/misc/sarplat.py":53
 *         logging.info("---Check and set SAR platform name...")
 * 
 *         if not isinstance(value, str):             # <<<<<<<<<<<<<<
 *             raise ValueError('name must be a string!')
 *         self._name = value
 */
  }

  /* "torchsar/misc/sarplat.py":55
 *         if not isinstance(value, str):
 *             raise ValueError('name must be a string!')
 *         self._name = value             # <<<<<<<<<<<<<<
 *         logging.info("---Done!")
 * 
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_name, __pyx_v_value) < 0) __PYX_ERR(0, 55, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":56
 *             raise ValueError('name must be a string!')
 *         self._name = value
 *         logging.info("---Done!")             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_logging); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_info); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_kp_u_Done) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_Done);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":50
 * 
 *     @name.setter
 *     def name(self, value):             # <<<<<<<<<<<<<<
 *         logging.info("---Check and set SAR platform name...")
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("torchsar.misc.sarplat.SarPlat.name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "torchsar/misc/sarplat.py":59
 * 
 *     @property
 *     def sensor(self):             # <<<<<<<<<<<<<<
 *         return self._sensor
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_5sensor(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_5sensor = {"sensor", (PyCFunction)__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_5sensor, METH_O, 0};
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_5sensor(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("sensor (wrapper)", 0);
  __pyx_r = __pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_4sensor(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_4sensor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("sensor", 0);

  /* "torchsar/misc/sarplat.py":60
 *     @property
 *     def sensor(self):
 *         return self._sensor             # <<<<<<<<<<<<<<
 * 
 *     @sensor.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "torchsar/misc/sarplat.py":59
 * 
 *     @property
 *     def sensor(self):             # <<<<<<<<<<<<<<
 *         return self._sensor
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("torchsar.misc.sarplat.SarPlat.sensor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "torchsar/misc/sarplat.py":63
 * 
 *     @sensor.setter
 *     def sensor(self, value):             # <<<<<<<<<<<<<<
 *         logging.info("---Check and set SAR sensor parameters...")
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_7sensor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_7sensor = {"sensor", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_7sensor, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_7sensor(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_value = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("sensor (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_value,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("sensor", 1, 2, 2, 1); __PYX_ERR(0, 63, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "sensor") < 0)) __PYX_ERR(0, 63, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_value = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("sensor", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 63, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("torchsar.misc.sarplat.SarPlat.sensor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_6sensor(__pyx_self, __pyx_v_self, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_6sensor(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_k = NULL;
  PyObject *__pyx_v_v = NULL;
  PyObject *__pyx_v_Re = NULL;
  PyObject *__pyx_v_Rs = NULL;
  PyObject *__pyx_v_Vr = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("sensor", 0);

  /* "torchsar/misc/sarplat.py":64
 *     @sensor.setter
 *     def sensor(self, value):
 *         logging.info("---Check and set SAR sensor parameters...")             # <<<<<<<<<<<<<<
 * 
 *         if not isinstance(value, dict):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_logging); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_info); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_kp_u_Check_and_set_SAR_sensor_parame) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_Check_and_set_SAR_sensor_parame);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":66
 *         logging.info("---Check and set SAR sensor parameters...")
 * 
 *         if not isinstance(value, dict):             # <<<<<<<<<<<<<<
 *             raise ValueError('sensor must be a dict!')
 *         if hasattr(self, '_sensor'):
 */
  __pyx_t_4 = PyDict_Check(__pyx_v_value); 
  __pyx_t_5 = ((!(__pyx_t_4 != 0)) != 0);
  if (unlikely(__pyx_t_5)) {

    /* "torchsar/misc/sarplat.py":67
 * 
 *         if not isinstance(value, dict):
 *             raise ValueError('sensor must be a dict!')             # <<<<<<<<<<<<<<
 *         if hasattr(self, '_sensor'):
 *             for k, v in value.items():
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 67, __pyx_L1_error)

    /* "torchsar/misc/sarplat.py":66
 *         logging.info("---Check and set SAR sensor parameters...")
 * 
 *         if not isinstance(value, dict):             # <<<<<<<<<<<<<<
 *             raise ValueError('sensor must be a dict!')
 *         if hasattr(self, '_sensor'):
 */
  }

  /* "torchsar/misc/sarplat.py":68
 *         if not isinstance(value, dict):
 *             raise ValueError('sensor must be a dict!')
 *         if hasattr(self, '_sensor'):             # <<<<<<<<<<<<<<
 *             for k, v in value.items():
 *                 self._sensor[k] = v
 */
  __pyx_t_5 = __Pyx_HasAttr(__pyx_v_self, __pyx_n_u_sensor); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 68, __pyx_L1_error)
  __pyx_t_4 = (__pyx_t_5 != 0);
  if (__pyx_t_4) {

    /* "torchsar/misc/sarplat.py":69
 *             raise ValueError('sensor must be a dict!')
 *         if hasattr(self, '_sensor'):
 *             for k, v in value.items():             # <<<<<<<<<<<<<<
 *                 self._sensor[k] = v
 *         else:
 */
    __pyx_t_6 = 0;
    if (unlikely(__pyx_v_value == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
      __PYX_ERR(0, 69, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_dict_iterator(__pyx_v_value, 0, __pyx_n_s_items, (&__pyx_t_7), (&__pyx_t_8)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 69, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_1);
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
    while (1) {
      __pyx_t_9 = __Pyx_dict_iter_next(__pyx_t_1, __pyx_t_7, &__pyx_t_6, &__pyx_t_3, &__pyx_t_2, NULL, __pyx_t_8);
      if (unlikely(__pyx_t_9 == 0)) break;
      if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 69, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_XDECREF_SET(__pyx_v_k, __pyx_t_3);
      __pyx_t_3 = 0;
      __Pyx_XDECREF_SET(__pyx_v_v, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "torchsar/misc/sarplat.py":70
 *         if hasattr(self, '_sensor'):
 *             for k, v in value.items():
 *                 self._sensor[k] = v             # <<<<<<<<<<<<<<
 *         else:
 *             self._sensor = value
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 70, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      if (unlikely(PyObject_SetItem(__pyx_t_2, __pyx_v_k, __pyx_v_v) < 0)) __PYX_ERR(0, 70, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "torchsar/misc/sarplat.py":68
 *         if not isinstance(value, dict):
 *             raise ValueError('sensor must be a dict!')
 *         if hasattr(self, '_sensor'):             # <<<<<<<<<<<<<<
 *             for k, v in value.items():
 *                 self._sensor[k] = v
 */
    goto __pyx_L4;
  }

  /* "torchsar/misc/sarplat.py":72
 *                 self._sensor[k] = v
 *         else:
 *             self._sensor = value             # <<<<<<<<<<<<<<
 * 
 *         self._sensor['Br'] = self._sensor['Tp'] * np.abs(self._sensor['Kr'])  # chirp rate in Hz/s
 */
  /*else*/ {
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_sensor, __pyx_v_value) < 0) __PYX_ERR(0, 72, __pyx_L1_error)
  }
  __pyx_L4:;

  /* "torchsar/misc/sarplat.py":74
 *             self._sensor = value
 * 
 *         self._sensor['Br'] = self._sensor['Tp'] * np.abs(self._sensor['Kr'])  # chirp rate in Hz/s             # <<<<<<<<<<<<<<
 *         self._sensor['Wl'] = C / self._sensor['Fc']  # wave length in m
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Tp); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_abs); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_11 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_Kr); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_10);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_10, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_3, __pyx_t_11) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_11);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyNumber_Multiply(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(PyObject_SetItem(__pyx_t_1, __pyx_n_u_Br, __pyx_t_10) < 0)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "torchsar/misc/sarplat.py":75
 * 
 *         self._sensor['Br'] = self._sensor['Tp'] * np.abs(self._sensor['Kr'])  # chirp rate in Hz/s
 *         self._sensor['Wl'] = C / self._sensor['Fc']  # wave length in m             # <<<<<<<<<<<<<<
 * 
 *         Re = Ree
 */
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_C); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Fc); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_t_10, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (unlikely(PyObject_SetItem(__pyx_t_2, __pyx_n_u_Wl, __pyx_t_1) < 0)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":77
 *         self._sensor['Wl'] = C / self._sensor['Fc']  # wave length in m
 * 
 *         Re = Ree             # <<<<<<<<<<<<<<
 *         Rs = self._sensor['H'] + Re
 *         # Rs = self._sensor['H'] + Ree
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_Ree); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_Re = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":78
 * 
 *         Re = Ree
 *         Rs = self._sensor['H'] + Re             # <<<<<<<<<<<<<<
 *         # Rs = self._sensor['H'] + Ree
 *         self._sensor['Rs'] = Rs
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_H); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Add(__pyx_t_2, __pyx_v_Re); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_Rs = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":80
 *         Rs = self._sensor['H'] + Re
 *         # Rs = self._sensor['H'] + Ree
 *         self._sensor['Rs'] = Rs             # <<<<<<<<<<<<<<
 *         if value['H'] < AirboneSatelliteHeightBoundary:
 *             logging.info("---Airbone.")
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(PyObject_SetItem(__pyx_t_1, __pyx_n_u_Rs, __pyx_v_Rs) < 0)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":81
 *         # Rs = self._sensor['H'] + Ree
 *         self._sensor['Rs'] = Rs
 *         if value['H'] < AirboneSatelliteHeightBoundary:             # <<<<<<<<<<<<<<
 *             logging.info("---Airbone.")
 *             self._sensor['Vs'] = self._sensor['V']
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_value, __pyx_n_u_H); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_AirboneSatelliteHeightBoundary); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_LT); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (__pyx_t_4) {

    /* "torchsar/misc/sarplat.py":82
 *         self._sensor['Rs'] = Rs
 *         if value['H'] < AirboneSatelliteHeightBoundary:
 *             logging.info("---Airbone.")             # <<<<<<<<<<<<<<
 *             self._sensor['Vs'] = self._sensor['V']
 *             self._sensor['Vr'] = self._sensor['V']
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_logging); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 82, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_info); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 82, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_10 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_2, __pyx_kp_u_Airbone) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_kp_u_Airbone);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 82, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

    /* "torchsar/misc/sarplat.py":83
 *         if value['H'] < AirboneSatelliteHeightBoundary:
 *             logging.info("---Airbone.")
 *             self._sensor['Vs'] = self._sensor['V']             # <<<<<<<<<<<<<<
 *             self._sensor['Vr'] = self._sensor['V']
 *             self._sensor['Vg'] = self._sensor['V']
 */
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 83, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_10, __pyx_n_u_V); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 83, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 83, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (unlikely(PyObject_SetItem(__pyx_t_10, __pyx_n_u_Vs, __pyx_t_1) < 0)) __PYX_ERR(0, 83, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "torchsar/misc/sarplat.py":84
 *             logging.info("---Airbone.")
 *             self._sensor['Vs'] = self._sensor['V']
 *             self._sensor['Vr'] = self._sensor['V']             # <<<<<<<<<<<<<<
 *             self._sensor['Vg'] = self._sensor['V']
 *             self._sensor['To'] = -1
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 84, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_V); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 84, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 84, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(PyObject_SetItem(__pyx_t_1, __pyx_n_u_Vr, __pyx_t_10) < 0)) __PYX_ERR(0, 84, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

    /* "torchsar/misc/sarplat.py":85
 *             self._sensor['Vs'] = self._sensor['V']
 *             self._sensor['Vr'] = self._sensor['V']
 *             self._sensor['Vg'] = self._sensor['V']             # <<<<<<<<<<<<<<
 *             self._sensor['To'] = -1
 *             self._sensor['Ws'] = -1
 */
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 85, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_10, __pyx_n_u_V); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 85, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 85, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (unlikely(PyObject_SetItem(__pyx_t_10, __pyx_n_u_Vg, __pyx_t_1) < 0)) __PYX_ERR(0, 85, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "torchsar/misc/sarplat.py":86
 *             self._sensor['Vr'] = self._sensor['V']
 *             self._sensor['Vg'] = self._sensor['V']
 *             self._sensor['To'] = -1             # <<<<<<<<<<<<<<
 *             self._sensor['Ws'] = -1
 *         else:
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 86, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(PyObject_SetItem(__pyx_t_1, __pyx_n_u_To, __pyx_int_neg_1) < 0)) __PYX_ERR(0, 86, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "torchsar/misc/sarplat.py":87
 *             self._sensor['Vg'] = self._sensor['V']
 *             self._sensor['To'] = -1
 *             self._sensor['Ws'] = -1             # <<<<<<<<<<<<<<
 *         else:
 *             logging.info("---Satellite.")
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 87, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(PyObject_SetItem(__pyx_t_1, __pyx_n_u_Ws, __pyx_int_neg_1) < 0)) __PYX_ERR(0, 87, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "torchsar/misc/sarplat.py":81
 *         # Rs = self._sensor['H'] + Ree
 *         self._sensor['Rs'] = Rs
 *         if value['H'] < AirboneSatelliteHeightBoundary:             # <<<<<<<<<<<<<<
 *             logging.info("---Airbone.")
 *             self._sensor['Vs'] = self._sensor['V']
 */
    goto __pyx_L7;
  }

  /* "torchsar/misc/sarplat.py":89
 *             self._sensor['Ws'] = -1
 *         else:
 *             logging.info("---Satellite.")             # <<<<<<<<<<<<<<
 *             # Earth Bending Geometry (Vg < Vr < Vs, Vr = \sqrt{Vg*Vs})
 *             self._sensor['To'] = 2 * PI * np.sqrt((Rs)**3 / Ge)
 */
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_logging); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 89, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_info); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 89, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_10, __pyx_kp_u_Satellite) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_kp_u_Satellite);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 89, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "torchsar/misc/sarplat.py":91
 *             logging.info("---Satellite.")
 *             # Earth Bending Geometry (Vg < Vr < Vs, Vr = \sqrt{Vg*Vs})
 *             self._sensor['To'] = 2 * PI * np.sqrt((Rs)**3 / Ge)             # <<<<<<<<<<<<<<
 *             self._sensor['Ws'] = 2 * PI / self._sensor['To']
 *             self._sensor['Vs'] = Rs * self._sensor['Ws']
 */
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_PI); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyNumber_Multiply(__pyx_int_2, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = PyNumber_Power(__pyx_v_Rs, __pyx_int_3, Py_None); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_Ge); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_12 = __Pyx_PyNumber_Divide(__pyx_t_10, __pyx_t_3); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_11))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_11);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_11, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_11, __pyx_t_3, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_t_12);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_11 = PyNumber_Multiply(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(PyObject_SetItem(__pyx_t_1, __pyx_n_u_To, __pyx_t_11) < 0)) __PYX_ERR(0, 91, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

    /* "torchsar/misc/sarplat.py":92
 *             # Earth Bending Geometry (Vg < Vr < Vs, Vr = \sqrt{Vg*Vs})
 *             self._sensor['To'] = 2 * PI * np.sqrt((Rs)**3 / Ge)
 *             self._sensor['Ws'] = 2 * PI / self._sensor['To']             # <<<<<<<<<<<<<<
 *             self._sensor['Vs'] = Rs * self._sensor['Ws']
 *             self._sensor['Vr'] = self._sensor['V']
 */
    __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_n_s_PI); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 92, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_1 = PyNumber_Multiply(__pyx_int_2, __pyx_t_11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 92, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 92, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_11, __pyx_n_u_To); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 92, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_11 = __Pyx_PyNumber_Divide(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 92, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 92, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (unlikely(PyObject_SetItem(__pyx_t_2, __pyx_n_u_Ws, __pyx_t_11) < 0)) __PYX_ERR(0, 92, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

    /* "torchsar/misc/sarplat.py":93
 *             self._sensor['To'] = 2 * PI * np.sqrt((Rs)**3 / Ge)
 *             self._sensor['Ws'] = 2 * PI / self._sensor['To']
 *             self._sensor['Vs'] = Rs * self._sensor['Ws']             # <<<<<<<<<<<<<<
 *             self._sensor['Vr'] = self._sensor['V']
 *             # Equivalent velocity (Linear geometry)
 */
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 93, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_11, __pyx_n_u_Ws); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 93, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_11 = PyNumber_Multiply(__pyx_v_Rs, __pyx_t_2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 93, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 93, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (unlikely(PyObject_SetItem(__pyx_t_2, __pyx_n_u_Vs, __pyx_t_11) < 0)) __PYX_ERR(0, 93, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

    /* "torchsar/misc/sarplat.py":94
 *             self._sensor['Ws'] = 2 * PI / self._sensor['To']
 *             self._sensor['Vs'] = Rs * self._sensor['Ws']
 *             self._sensor['Vr'] = self._sensor['V']             # <<<<<<<<<<<<<<
 *             # Equivalent velocity (Linear geometry)
 *             self._sensor['Vg'] = self._sensor['Vr']**2 / self._sensor['Vs']  # ground velocity
 */
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 94, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_11, __pyx_n_u_V); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 94, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 94, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    if (unlikely(PyObject_SetItem(__pyx_t_11, __pyx_n_u_Vr, __pyx_t_2) < 0)) __PYX_ERR(0, 94, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "torchsar/misc/sarplat.py":96
 *             self._sensor['Vr'] = self._sensor['V']
 *             # Equivalent velocity (Linear geometry)
 *             self._sensor['Vg'] = self._sensor['Vr']**2 / self._sensor['Vs']  # ground velocity             # <<<<<<<<<<<<<<
 *         Vr = self._sensor['Vs'] * np.sqrt(Re / Rs)
 *         logging.info("---Computed equivalent velocity is " + str(Vr) +
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 96, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_11 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_Vr); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 96, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyNumber_Power(__pyx_t_11, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 96, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 96, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_11, __pyx_n_u_Vs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 96, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_11 = __Pyx_PyNumber_Divide(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 96, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 96, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(PyObject_SetItem(__pyx_t_1, __pyx_n_u_Vg, __pyx_t_11) < 0)) __PYX_ERR(0, 96, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __pyx_L7:;

  /* "torchsar/misc/sarplat.py":97
 *             # Equivalent velocity (Linear geometry)
 *             self._sensor['Vg'] = self._sensor['Vr']**2 / self._sensor['Vs']  # ground velocity
 *         Vr = self._sensor['Vs'] * np.sqrt(Re / Rs)             # <<<<<<<<<<<<<<
 *         logging.info("---Computed equivalent velocity is " + str(Vr) +
 *                      ", setted equivalent velocity is " + str(self._sensor['Vr']) + "(used).")
 */
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_11, __pyx_n_u_Vs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyNumber_Divide(__pyx_v_Re, __pyx_v_Rs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  __pyx_t_11 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_12, __pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = PyNumber_Multiply(__pyx_t_1, __pyx_t_11); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_v_Vr = __pyx_t_12;
  __pyx_t_12 = 0;

  /* "torchsar/misc/sarplat.py":98
 *             self._sensor['Vg'] = self._sensor['Vr']**2 / self._sensor['Vs']  # ground velocity
 *         Vr = self._sensor['Vs'] * np.sqrt(Re / Rs)
 *         logging.info("---Computed equivalent velocity is " + str(Vr) +             # <<<<<<<<<<<<<<
 *                      ", setted equivalent velocity is " + str(self._sensor['Vr']) + "(used).")
 *         self._sensor['Vs'] = Vr / np.sqrt(Re / Rs)  # reset
 */
  __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_n_s_logging); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_n_s_info); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyUnicode_Type)), __pyx_v_Vr); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_2 = __Pyx_PyUnicode_Concat(__pyx_kp_u_Computed_equivalent_velocity_is, __pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = __Pyx_PyUnicode_Concat(__pyx_t_2, __pyx_kp_u_setted_equivalent_velocity_is); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":99
 *         Vr = self._sensor['Vs'] * np.sqrt(Re / Rs)
 *         logging.info("---Computed equivalent velocity is " + str(Vr) +
 *                      ", setted equivalent velocity is " + str(self._sensor['Vr']) + "(used).")             # <<<<<<<<<<<<<<
 *         self._sensor['Vs'] = Vr / np.sqrt(Re / Rs)  # reset
 *         logging.info("---Done!")
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_Vr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyUnicode_Type)), __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyUnicode_Concat(__pyx_t_11, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyUnicode_Concat(__pyx_t_3, __pyx_kp_u_used); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_12 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

  /* "torchsar/misc/sarplat.py":100
 *         logging.info("---Computed equivalent velocity is " + str(Vr) +
 *                      ", setted equivalent velocity is " + str(self._sensor['Vr']) + "(used).")
 *         self._sensor['Vs'] = Vr / np.sqrt(Re / Rs)  # reset             # <<<<<<<<<<<<<<
 *         logging.info("---Done!")
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_v_Re, __pyx_v_Rs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_12 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyNumber_Divide(__pyx_v_Vr, __pyx_t_12); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  if (unlikely(PyObject_SetItem(__pyx_t_12, __pyx_n_u_Vs, __pyx_t_2) < 0)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":101
 *                      ", setted equivalent velocity is " + str(self._sensor['Vr']) + "(used).")
 *         self._sensor['Vs'] = Vr / np.sqrt(Re / Rs)  # reset
 *         logging.info("---Done!")             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_logging); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_info); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_12)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_12);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_12) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_12, __pyx_kp_u_Done) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_kp_u_Done);
  __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":63
 * 
 *     @sensor.setter
 *     def sensor(self, value):             # <<<<<<<<<<<<<<
 *         logging.info("---Check and set SAR sensor parameters...")
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("torchsar.misc.sarplat.SarPlat.sensor", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_k);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_XDECREF(__pyx_v_Re);
  __Pyx_XDECREF(__pyx_v_Rs);
  __Pyx_XDECREF(__pyx_v_Vr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "torchsar/misc/sarplat.py":104
 * 
 *     @property
 *     def acquisition(self):             # <<<<<<<<<<<<<<
 *         return self._acquisition
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_9acquisition(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_9acquisition = {"acquisition", (PyCFunction)__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_9acquisition, METH_O, 0};
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_9acquisition(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("acquisition (wrapper)", 0);
  __pyx_r = __pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_8acquisition(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_8acquisition(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("acquisition", 0);

  /* "torchsar/misc/sarplat.py":105
 *     @property
 *     def acquisition(self):
 *         return self._acquisition             # <<<<<<<<<<<<<<
 * 
 *     @acquisition.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "torchsar/misc/sarplat.py":104
 * 
 *     @property
 *     def acquisition(self):             # <<<<<<<<<<<<<<
 *         return self._acquisition
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("torchsar.misc.sarplat.SarPlat.acquisition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "torchsar/misc/sarplat.py":108
 * 
 *     @acquisition.setter
 *     def acquisition(self, value):             # <<<<<<<<<<<<<<
 *         logging.info("---Check and set acquisition parameters...")
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_11acquisition(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_11acquisition = {"acquisition", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_11acquisition, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_11acquisition(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_value = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("acquisition (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_value,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("acquisition", 1, 2, 2, 1); __PYX_ERR(0, 108, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "acquisition") < 0)) __PYX_ERR(0, 108, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_value = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("acquisition", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 108, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("torchsar.misc.sarplat.SarPlat.acquisition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_10acquisition(__pyx_self, __pyx_v_self, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_10acquisition(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_k = NULL;
  PyObject *__pyx_v_v = NULL;
  PyObject *__pyx_v_H = NULL;
  PyObject *__pyx_v_V = NULL;
  PyObject *__pyx_v_Vr = NULL;
  PyObject *__pyx_v_Vs = NULL;
  PyObject *__pyx_v_PRF = NULL;
  PyObject *__pyx_v_Fsr = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_Tp = NULL;
  PyObject *__pyx_v_Rs = NULL;
  PyObject *__pyx_v_As = NULL;
  PyObject *__pyx_v_Ad = NULL;
  PyObject *__pyx_v_Aon = NULL;
  PyObject *__pyx_v_Aba = NULL;
  PyObject *__pyx_v_Abr = NULL;
  PyObject *__pyx_v_SA = NULL;
  PyObject *__pyx_v_PC = NULL;
  PyObject *__pyx_v_ES = NULL;
  PyObject *__pyx_v_SV = NULL;
  PyObject *__pyx_v_Ar = NULL;
  PyObject *__pyx_v_SAV = NULL;
  PyObject *__pyx_v_BAV = NULL;
  PyObject *__pyx_v_Nr = NULL;
  PyObject *__pyx_v_Rnear = NULL;
  PyObject *__pyx_v_Rfar = NULL;
  PyObject *__pyx_v_FPr = NULL;
  PyObject *__pyx_v_Rsc = NULL;
  PyObject *__pyx_v_Rs0 = NULL;
  PyObject *__pyx_v_Ysc = NULL;
  PyObject *__pyx_v_Xsc = NULL;
  PyObject *__pyx_v_SC = NULL;
  PyObject *__pyx_v_Rbc = NULL;
  PyObject *__pyx_v_Rb0 = NULL;
  PyObject *__pyx_v_Ybc = NULL;
  PyObject *__pyx_v_Xbc = NULL;
  PyObject *__pyx_v_BC = NULL;
  PyObject *__pyx_v_Ynear = NULL;
  PyObject *__pyx_v_Yfar = NULL;
  PyObject *__pyx_v_Xnear = NULL;
  PyObject *__pyx_v_Xfar = NULL;
  PyObject *__pyx_v_Na = NULL;
  PyObject *__pyx_v_Ta = NULL;
  PyObject *__pyx_v_FPr0 = NULL;
  PyObject *__pyx_v_FPr1 = NULL;
  PyObject *__pyx_v_Ai = NULL;
  PyObject *__pyx_v_Ao = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  PyObject *(*__pyx_t_15)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("acquisition", 0);

  /* "torchsar/misc/sarplat.py":109
 *     @acquisition.setter
 *     def acquisition(self, value):
 *         logging.info("---Check and set acquisition parameters...")             # <<<<<<<<<<<<<<
 * 
 *         if value is not None:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_logging); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_info); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_kp_u_Check_and_set_acquisition_param) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_Check_and_set_acquisition_param);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":111
 *         logging.info("---Check and set acquisition parameters...")
 * 
 *         if value is not None:             # <<<<<<<<<<<<<<
 *             if not isinstance(value, dict):
 *                 raise ValueError('acquisition must be a dict!')
 */
  __pyx_t_4 = (__pyx_v_value != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "torchsar/misc/sarplat.py":112
 * 
 *         if value is not None:
 *             if not isinstance(value, dict):             # <<<<<<<<<<<<<<
 *                 raise ValueError('acquisition must be a dict!')
 *             if hasattr(self, '_acquisition'):
 */
    __pyx_t_5 = PyDict_Check(__pyx_v_value); 
    __pyx_t_4 = ((!(__pyx_t_5 != 0)) != 0);
    if (unlikely(__pyx_t_4)) {

      /* "torchsar/misc/sarplat.py":113
 *         if value is not None:
 *             if not isinstance(value, dict):
 *                 raise ValueError('acquisition must be a dict!')             # <<<<<<<<<<<<<<
 *             if hasattr(self, '_acquisition'):
 *                 for k, v in value.items():
 */
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 113, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 113, __pyx_L1_error)

      /* "torchsar/misc/sarplat.py":112
 * 
 *         if value is not None:
 *             if not isinstance(value, dict):             # <<<<<<<<<<<<<<
 *                 raise ValueError('acquisition must be a dict!')
 *             if hasattr(self, '_acquisition'):
 */
    }

    /* "torchsar/misc/sarplat.py":114
 *             if not isinstance(value, dict):
 *                 raise ValueError('acquisition must be a dict!')
 *             if hasattr(self, '_acquisition'):             # <<<<<<<<<<<<<<
 *                 for k, v in value.items():
 *                     self._acquisition[k] = v
 */
    __pyx_t_4 = __Pyx_HasAttr(__pyx_v_self, __pyx_n_u_acquisition); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 114, __pyx_L1_error)
    __pyx_t_5 = (__pyx_t_4 != 0);
    if (__pyx_t_5) {

      /* "torchsar/misc/sarplat.py":115
 *                 raise ValueError('acquisition must be a dict!')
 *             if hasattr(self, '_acquisition'):
 *                 for k, v in value.items():             # <<<<<<<<<<<<<<
 *                     self._acquisition[k] = v
 *             else:
 */
      __pyx_t_6 = 0;
      if (unlikely(__pyx_v_value == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
        __PYX_ERR(0, 115, __pyx_L1_error)
      }
      __pyx_t_3 = __Pyx_dict_iterator(__pyx_v_value, 0, __pyx_n_s_items, (&__pyx_t_7), (&__pyx_t_8)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 115, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_XDECREF(__pyx_t_1);
      __pyx_t_1 = __pyx_t_3;
      __pyx_t_3 = 0;
      while (1) {
        __pyx_t_9 = __Pyx_dict_iter_next(__pyx_t_1, __pyx_t_7, &__pyx_t_6, &__pyx_t_3, &__pyx_t_2, NULL, __pyx_t_8);
        if (unlikely(__pyx_t_9 == 0)) break;
        if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 115, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_k, __pyx_t_3);
        __pyx_t_3 = 0;
        __Pyx_XDECREF_SET(__pyx_v_v, __pyx_t_2);
        __pyx_t_2 = 0;

        /* "torchsar/misc/sarplat.py":116
 *             if hasattr(self, '_acquisition'):
 *                 for k, v in value.items():
 *                     self._acquisition[k] = v             # <<<<<<<<<<<<<<
 *             else:
 *                 self._acquisition = value
 */
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 116, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyObject_SetItem(__pyx_t_2, __pyx_v_k, __pyx_v_v) < 0)) __PYX_ERR(0, 116, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "torchsar/misc/sarplat.py":114
 *             if not isinstance(value, dict):
 *                 raise ValueError('acquisition must be a dict!')
 *             if hasattr(self, '_acquisition'):             # <<<<<<<<<<<<<<
 *                 for k, v in value.items():
 *                     self._acquisition[k] = v
 */
      goto __pyx_L5;
    }

    /* "torchsar/misc/sarplat.py":118
 *                     self._acquisition[k] = v
 *             else:
 *                 self._acquisition = value             # <<<<<<<<<<<<<<
 * 
 *         H = self._sensor['H']
 */
    /*else*/ {
      if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_acquisition, __pyx_v_value) < 0) __PYX_ERR(0, 118, __pyx_L1_error)
    }
    __pyx_L5:;

    /* "torchsar/misc/sarplat.py":111
 *         logging.info("---Check and set acquisition parameters...")
 * 
 *         if value is not None:             # <<<<<<<<<<<<<<
 *             if not isinstance(value, dict):
 *                 raise ValueError('acquisition must be a dict!')
 */
  }

  /* "torchsar/misc/sarplat.py":120
 *                 self._acquisition = value
 * 
 *         H = self._sensor['H']             # <<<<<<<<<<<<<<
 *         V = self._sensor['V']
 *         Vr = self._sensor['Vr']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_H); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_H = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":121
 * 
 *         H = self._sensor['H']
 *         V = self._sensor['V']             # <<<<<<<<<<<<<<
 *         Vr = self._sensor['Vr']
 *         Vs = self._sensor['Vs']
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_V); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 121, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_V = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":122
 *         H = self._sensor['H']
 *         V = self._sensor['V']
 *         Vr = self._sensor['Vr']             # <<<<<<<<<<<<<<
 *         Vs = self._sensor['Vs']
 *         PRF = self._sensor['PRF']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Vr); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Vr = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":123
 *         V = self._sensor['V']
 *         Vr = self._sensor['Vr']
 *         Vs = self._sensor['Vs']             # <<<<<<<<<<<<<<
 *         PRF = self._sensor['PRF']
 *         Fsr = self._sensor['Fs']
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_Vs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_Vs = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":124
 *         Vr = self._sensor['Vr']
 *         Vs = self._sensor['Vs']
 *         PRF = self._sensor['PRF']             # <<<<<<<<<<<<<<
 *         Fsr = self._sensor['Fs']
 *         Tp = self._sensor['Tp']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_PRF); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_PRF = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":125
 *         Vs = self._sensor['Vs']
 *         PRF = self._sensor['PRF']
 *         Fsr = self._sensor['Fs']             # <<<<<<<<<<<<<<
 *         Tp = self._sensor['Tp']
 *         Rs = self._sensor['Rs']
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_Fs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_Fsr = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":126
 *         PRF = self._sensor['PRF']
 *         Fsr = self._sensor['Fs']
 *         Tp = self._sensor['Tp']             # <<<<<<<<<<<<<<
 *         Rs = self._sensor['Rs']
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Tp); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Tp = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":127
 *         Fsr = self._sensor['Fs']
 *         Tp = self._sensor['Tp']
 *         Rs = self._sensor['Rs']             # <<<<<<<<<<<<<<
 * 
 *         As = value['As']
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_Rs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_Rs = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":129
 *         Rs = self._sensor['Rs']
 * 
 *         As = value['As']             # <<<<<<<<<<<<<<
 *         Ad = value['Ad']
 *         Aon = value['Aon']
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_value, __pyx_n_u_As); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_As = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":130
 * 
 *         As = value['As']
 *         Ad = value['Ad']             # <<<<<<<<<<<<<<
 *         Aon = value['Aon']
 *         Aba = value['Aba']
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_value, __pyx_n_u_Ad); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_Ad = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":131
 *         As = value['As']
 *         Ad = value['Ad']
 *         Aon = value['Aon']             # <<<<<<<<<<<<<<
 *         Aba = value['Aba']
 *         Abr = value['Abr']
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_value, __pyx_n_u_Aon); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 131, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_Aon = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":132
 *         Ad = value['Ad']
 *         Aon = value['Aon']
 *         Aba = value['Aba']             # <<<<<<<<<<<<<<
 *         Abr = value['Abr']
 *         SA = value['SceneArea']
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_value, __pyx_n_u_Aba); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_Aba = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":133
 *         Aon = value['Aon']
 *         Aba = value['Aba']
 *         Abr = value['Abr']             # <<<<<<<<<<<<<<
 *         SA = value['SceneArea']
 *         PC = value['PlatCenter']
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_value, __pyx_n_u_Abr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_Abr = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":134
 *         Aba = value['Aba']
 *         Abr = value['Abr']
 *         SA = value['SceneArea']             # <<<<<<<<<<<<<<
 *         PC = value['PlatCenter']
 *         ES = value['EchoSize']
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_value, __pyx_n_u_SceneArea); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_SA = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":135
 *         Abr = value['Abr']
 *         SA = value['SceneArea']
 *         PC = value['PlatCenter']             # <<<<<<<<<<<<<<
 *         ES = value['EchoSize']
 *         SV = value['StateVector']
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_value, __pyx_n_u_PlatCenter); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_PC = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":136
 *         SA = value['SceneArea']
 *         PC = value['PlatCenter']
 *         ES = value['EchoSize']             # <<<<<<<<<<<<<<
 *         SV = value['StateVector']
 *         if SV is None:
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_value, __pyx_n_u_EchoSize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_ES = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":137
 *         PC = value['PlatCenter']
 *         ES = value['EchoSize']
 *         SV = value['StateVector']             # <<<<<<<<<<<<<<
 *         if SV is None:
 *             SV = []
 */
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_value, __pyx_n_u_StateVector); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 137, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_SV = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":138
 *         ES = value['EchoSize']
 *         SV = value['StateVector']
 *         if SV is None:             # <<<<<<<<<<<<<<
 *             SV = []
 * 
 */
  __pyx_t_5 = (__pyx_v_SV == Py_None);
  __pyx_t_4 = (__pyx_t_5 != 0);
  if (__pyx_t_4) {

    /* "torchsar/misc/sarplat.py":139
 *         SV = value['StateVector']
 *         if SV is None:
 *             SV = []             # <<<<<<<<<<<<<<
 * 
 *         if Aba is None:
 */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 139, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_SV, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "torchsar/misc/sarplat.py":138
 *         ES = value['EchoSize']
 *         SV = value['StateVector']
 *         if SV is None:             # <<<<<<<<<<<<<<
 *             SV = []
 * 
 */
  }

  /* "torchsar/misc/sarplat.py":141
 *             SV = []
 * 
 *         if Aba is None:             # <<<<<<<<<<<<<<
 *             Aba = azimuth_beamwidth(self._sensor['Wl'], self._sensor['La'])
 * 
 */
  __pyx_t_4 = (__pyx_v_Aba == Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "torchsar/misc/sarplat.py":142
 * 
 *         if Aba is None:
 *             Aba = azimuth_beamwidth(self._sensor['Wl'], self._sensor['La'])             # <<<<<<<<<<<<<<
 * 
 *         # As: squint angle in earth curve geometry, Ar: squint angle in line geometry
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_azimuth_beamwidth); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 142, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 142, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_10 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_Wl); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 142, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 142, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_11 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_La); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 142, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_8 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_t_10, __pyx_t_11};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 142, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_3, __pyx_t_10, __pyx_t_11};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 142, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    } else
    #endif
    {
      __pyx_t_12 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 142, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      if (__pyx_t_3) {
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_3); __pyx_t_3 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_10);
      PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_8, __pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_11);
      PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_8, __pyx_t_11);
      __pyx_t_10 = 0;
      __pyx_t_11 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 142, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_Aba, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "torchsar/misc/sarplat.py":141
 *             SV = []
 * 
 *         if Aba is None:             # <<<<<<<<<<<<<<
 *             Aba = azimuth_beamwidth(self._sensor['Wl'], self._sensor['La'])
 * 
 */
  }

  /* "torchsar/misc/sarplat.py":145
 * 
 *         # As: squint angle in earth curve geometry, Ar: squint angle in line geometry
 *         Ar = (Vs / Vr) * As             # <<<<<<<<<<<<<<
 * 
 *         if PC is None:
 */
  __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_v_Vs, __pyx_v_Vr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Multiply(__pyx_t_1, __pyx_v_As); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Ar = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":147
 *         Ar = (Vs / Vr) * As
 * 
 *         if PC is None:             # <<<<<<<<<<<<<<
 *             PC = [0.0, 0.0, self._sensor['H']]
 * 
 */
  __pyx_t_5 = (__pyx_v_PC == Py_None);
  __pyx_t_4 = (__pyx_t_5 != 0);
  if (__pyx_t_4) {

    /* "torchsar/misc/sarplat.py":148
 * 
 *         if PC is None:
 *             PC = [0.0, 0.0, self._sensor['H']]             # <<<<<<<<<<<<<<
 * 
 *         if value['EchoSize'] is None:
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_H); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyList_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_float_0_0);
    __Pyx_GIVEREF(__pyx_float_0_0);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_float_0_0);
    __Pyx_INCREF(__pyx_float_0_0);
    __Pyx_GIVEREF(__pyx_float_0_0);
    PyList_SET_ITEM(__pyx_t_2, 1, __pyx_float_0_0);
    __Pyx_GIVEREF(__pyx_t_1);
    PyList_SET_ITEM(__pyx_t_2, 2, __pyx_t_1);
    __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_PC, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "torchsar/misc/sarplat.py":147
 *         Ar = (Vs / Vr) * As
 * 
 *         if PC is None:             # <<<<<<<<<<<<<<
 *             PC = [0.0, 0.0, self._sensor['H']]
 * 
 */
  }

  /* "torchsar/misc/sarplat.py":150
 *             PC = [0.0, 0.0, self._sensor['H']]
 * 
 *         if value['EchoSize'] is None:             # <<<<<<<<<<<<<<
 *             logging.error("~~~EchoSize should be specified!")
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_v_value, __pyx_n_u_EchoSize); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = (__pyx_t_2 == Py_None);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "torchsar/misc/sarplat.py":151
 * 
 *         if value['EchoSize'] is None:
 *             logging.error("~~~EchoSize should be specified!")             # <<<<<<<<<<<<<<
 * 
 *         SAV = [0, 0, 0, 0]
 */
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_logging); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 151, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_error); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 151, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_12))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_12);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_12, function);
      }
    }
    __pyx_t_2 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_12, __pyx_t_1, __pyx_kp_u_EchoSize_should_be_specified) : __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_kp_u_EchoSize_should_be_specified);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 151, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "torchsar/misc/sarplat.py":150
 *             PC = [0.0, 0.0, self._sensor['H']]
 * 
 *         if value['EchoSize'] is None:             # <<<<<<<<<<<<<<
 *             logging.error("~~~EchoSize should be specified!")
 * 
 */
  }

  /* "torchsar/misc/sarplat.py":153
 *             logging.error("~~~EchoSize should be specified!")
 * 
 *         SAV = [0, 0, 0, 0]             # <<<<<<<<<<<<<<
 *         BAV = [0, 0, 0, 0]
 *         if SA is None:
 */
  __pyx_t_2 = PyList_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_int_0);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_int_0);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_int_0);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_int_0);
  __pyx_v_SAV = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":154
 * 
 *         SAV = [0, 0, 0, 0]
 *         BAV = [0, 0, 0, 0]             # <<<<<<<<<<<<<<
 *         if SA is None:
 *             SA = [None, None, None, None]
 */
  __pyx_t_2 = PyList_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 154, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_int_0);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_int_0);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_int_0);
  __Pyx_INCREF(__pyx_int_0);
  __Pyx_GIVEREF(__pyx_int_0);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_int_0);
  __pyx_v_BAV = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":155
 *         SAV = [0, 0, 0, 0]
 *         BAV = [0, 0, 0, 0]
 *         if SA is None:             # <<<<<<<<<<<<<<
 *             SA = [None, None, None, None]
 * 
 */
  __pyx_t_5 = (__pyx_v_SA == Py_None);
  __pyx_t_4 = (__pyx_t_5 != 0);
  if (__pyx_t_4) {

    /* "torchsar/misc/sarplat.py":156
 *         BAV = [0, 0, 0, 0]
 *         if SA is None:
 *             SA = [None, None, None, None]             # <<<<<<<<<<<<<<
 * 
 *         if Abr is None:
 */
    __pyx_t_2 = PyList_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 156, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyList_SET_ITEM(__pyx_t_2, 0, Py_None);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyList_SET_ITEM(__pyx_t_2, 1, Py_None);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyList_SET_ITEM(__pyx_t_2, 2, Py_None);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyList_SET_ITEM(__pyx_t_2, 3, Py_None);
    __Pyx_DECREF_SET(__pyx_v_SA, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "torchsar/misc/sarplat.py":155
 *         SAV = [0, 0, 0, 0]
 *         BAV = [0, 0, 0, 0]
 *         if SA is None:             # <<<<<<<<<<<<<<
 *             SA = [None, None, None, None]
 * 
 */
  }

  /* "torchsar/misc/sarplat.py":158
 *             SA = [None, None, None, None]
 * 
 *         if Abr is None:             # <<<<<<<<<<<<<<
 *             Nr = ES[1]
 *             Abr = compute_range_beamwidth2(Nr, Fsr, H, Aon)
 */
  __pyx_t_4 = (__pyx_v_Abr == Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "torchsar/misc/sarplat.py":159
 * 
 *         if Abr is None:
 *             Nr = ES[1]             # <<<<<<<<<<<<<<
 *             Abr = compute_range_beamwidth2(Nr, Fsr, H, Aon)
 *         if Ad is None:
 */
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_ES, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 159, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_Nr = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "torchsar/misc/sarplat.py":160
 *         if Abr is None:
 *             Nr = ES[1]
 *             Abr = compute_range_beamwidth2(Nr, Fsr, H, Aon)             # <<<<<<<<<<<<<<
 *         if Ad is None:
 *             Ad = Aon + Abr / 2.
 */
    __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_compute_range_beamwidth2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 160, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_1 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_12))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_12);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_12, function);
        __pyx_t_8 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_12)) {
      PyObject *__pyx_temp[5] = {__pyx_t_1, __pyx_v_Nr, __pyx_v_Fsr, __pyx_v_H, __pyx_v_Aon};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_12, __pyx_temp+1-__pyx_t_8, 4+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_12)) {
      PyObject *__pyx_temp[5] = {__pyx_t_1, __pyx_v_Nr, __pyx_v_Fsr, __pyx_v_H, __pyx_v_Aon};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_12, __pyx_temp+1-__pyx_t_8, 4+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    {
      __pyx_t_11 = PyTuple_New(4+__pyx_t_8); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 160, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      if (__pyx_t_1) {
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_1); __pyx_t_1 = NULL;
      }
      __Pyx_INCREF(__pyx_v_Nr);
      __Pyx_GIVEREF(__pyx_v_Nr);
      PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_8, __pyx_v_Nr);
      __Pyx_INCREF(__pyx_v_Fsr);
      __Pyx_GIVEREF(__pyx_v_Fsr);
      PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_8, __pyx_v_Fsr);
      __Pyx_INCREF(__pyx_v_H);
      __Pyx_GIVEREF(__pyx_v_H);
      PyTuple_SET_ITEM(__pyx_t_11, 2+__pyx_t_8, __pyx_v_H);
      __Pyx_INCREF(__pyx_v_Aon);
      __Pyx_GIVEREF(__pyx_v_Aon);
      PyTuple_SET_ITEM(__pyx_t_11, 3+__pyx_t_8, __pyx_v_Aon);
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_12, __pyx_t_11, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    }
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF_SET(__pyx_v_Abr, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "torchsar/misc/sarplat.py":158
 *             SA = [None, None, None, None]
 * 
 *         if Abr is None:             # <<<<<<<<<<<<<<
 *             Nr = ES[1]
 *             Abr = compute_range_beamwidth2(Nr, Fsr, H, Aon)
 */
  }

  /* "torchsar/misc/sarplat.py":161
 *             Nr = ES[1]
 *             Abr = compute_range_beamwidth2(Nr, Fsr, H, Aon)
 *         if Ad is None:             # <<<<<<<<<<<<<<
 *             Ad = Aon + Abr / 2.
 *             # pdict['Abr'] = math.acos(pdict['H'] / pdict['Rfar']) - abs(pdict['Aon'])
 */
  __pyx_t_5 = (__pyx_v_Ad == Py_None);
  __pyx_t_4 = (__pyx_t_5 != 0);
  if (__pyx_t_4) {

    /* "torchsar/misc/sarplat.py":162
 *             Abr = compute_range_beamwidth2(Nr, Fsr, H, Aon)
 *         if Ad is None:
 *             Ad = Aon + Abr / 2.             # <<<<<<<<<<<<<<
 *             # pdict['Abr'] = math.acos(pdict['H'] / pdict['Rfar']) - abs(pdict['Aon'])
 *             Ad = np.sign(Aon) * (PI / 2. - (abs(Aon) + Abr / 2.))
 */
    __pyx_t_2 = __Pyx_PyFloat_TrueDivideObjC(__pyx_v_Abr, __pyx_float_2_, 2., 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 162, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_12 = PyNumber_Add(__pyx_v_Aon, __pyx_t_2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 162, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_Ad, __pyx_t_12);
    __pyx_t_12 = 0;

    /* "torchsar/misc/sarplat.py":164
 *             Ad = Aon + Abr / 2.
 *             # pdict['Abr'] = math.acos(pdict['H'] / pdict['Rfar']) - abs(pdict['Aon'])
 *             Ad = np.sign(Aon) * (PI / 2. - (abs(Aon) + Abr / 2.))             # <<<<<<<<<<<<<<
 * 
 *         if (Ad - Abr / 2.0) <= 0 or (Ad + Abr / 2.0) >= np.pi / 2.0:
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_sign); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_11))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_11);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_11, function);
      }
    }
    __pyx_t_12 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_11, __pyx_t_2, __pyx_v_Aon) : __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_v_Aon);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_n_s_PI); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_2 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_11, __pyx_float_2_, 2., 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_11 = __Pyx_PyNumber_Absolute(__pyx_v_Aon); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_1 = __Pyx_PyFloat_TrueDivideObjC(__pyx_v_Abr, __pyx_float_2_, 2., 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = PyNumber_Add(__pyx_t_11, __pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyNumber_Subtract(__pyx_t_2, __pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = PyNumber_Multiply(__pyx_t_12, __pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_Ad, __pyx_t_10);
    __pyx_t_10 = 0;

    /* "torchsar/misc/sarplat.py":161
 *             Nr = ES[1]
 *             Abr = compute_range_beamwidth2(Nr, Fsr, H, Aon)
 *         if Ad is None:             # <<<<<<<<<<<<<<
 *             Ad = Aon + Abr / 2.
 *             # pdict['Abr'] = math.acos(pdict['H'] / pdict['Rfar']) - abs(pdict['Aon'])
 */
  }

  /* "torchsar/misc/sarplat.py":166
 *             Ad = np.sign(Aon) * (PI / 2. - (abs(Aon) + Abr / 2.))
 * 
 *         if (Ad - Abr / 2.0) <= 0 or (Ad + Abr / 2.0) >= np.pi / 2.0:             # <<<<<<<<<<<<<<
 *             logging.error(r"~~~The antenna elevation beamwidth Abr is too large!")
 * 
 */
  __pyx_t_10 = __Pyx_PyFloat_TrueDivideObjC(__pyx_v_Abr, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_1 = PyNumber_Subtract(__pyx_v_Ad, __pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyObject_RichCompare(__pyx_t_1, __pyx_int_0, Py_LE); __Pyx_XGOTREF(__pyx_t_10); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (!__pyx_t_5) {
  } else {
    __pyx_t_4 = __pyx_t_5;
    goto __pyx_L16_bool_binop_done;
  }
  __pyx_t_10 = __Pyx_PyFloat_TrueDivideObjC(__pyx_v_Abr, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_1 = PyNumber_Add(__pyx_v_Ad, __pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_pi); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_12, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = PyObject_RichCompare(__pyx_t_1, __pyx_t_10, Py_GE); __Pyx_XGOTREF(__pyx_t_12); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_12); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_4 = __pyx_t_5;
  __pyx_L16_bool_binop_done:;
  if (__pyx_t_4) {

    /* "torchsar/misc/sarplat.py":167
 * 
 *         if (Ad - Abr / 2.0) <= 0 or (Ad + Abr / 2.0) >= np.pi / 2.0:
 *             logging.error(r"~~~The antenna elevation beamwidth Abr is too large!")             # <<<<<<<<<<<<<<
 * 
 *         Rnear = H / (np.cos(Aon) + EPS)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_logging); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_error); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_12 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_10, __pyx_kp_u_The_antenna_elevation_beamwidth) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_kp_u_The_antenna_elevation_beamwidth);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

    /* "torchsar/misc/sarplat.py":166
 *             Ad = np.sign(Aon) * (PI / 2. - (abs(Aon) + Abr / 2.))
 * 
 *         if (Ad - Abr / 2.0) <= 0 or (Ad + Abr / 2.0) >= np.pi / 2.0:             # <<<<<<<<<<<<<<
 *             logging.error(r"~~~The antenna elevation beamwidth Abr is too large!")
 * 
 */
  }

  /* "torchsar/misc/sarplat.py":169
 *             logging.error(r"~~~The antenna elevation beamwidth Abr is too large!")
 * 
 *         Rnear = H / (np.cos(Aon) + EPS)             # <<<<<<<<<<<<<<
 *         Rfar = H / (np.cos(Aon + np.sign(Aon) * Abr) + EPS)
 *         FPr = np.sqrt(Rfar**2 + Rnear**2 - 2. * Rnear * Rfar * np.cos(Abr) + EPS)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_cos); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_10);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_10, function);
    }
  }
  __pyx_t_12 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_1, __pyx_v_Aon) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_v_Aon);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_EPS); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_1 = PyNumber_Add(__pyx_t_12, __pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyNumber_Divide(__pyx_v_H, __pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 169, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Rnear = __pyx_t_10;
  __pyx_t_10 = 0;

  /* "torchsar/misc/sarplat.py":170
 * 
 *         Rnear = H / (np.cos(Aon) + EPS)
 *         Rfar = H / (np.cos(Aon + np.sign(Aon) * Abr) + EPS)             # <<<<<<<<<<<<<<
 *         FPr = np.sqrt(Rfar**2 + Rnear**2 - 2. * Rnear * Rfar * np.cos(Abr) + EPS)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_cos); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_sign); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_11))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_11);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_11, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_11, __pyx_t_2, __pyx_v_Aon) : __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_v_Aon);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = PyNumber_Multiply(__pyx_t_1, __pyx_v_Abr); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Add(__pyx_v_Aon, __pyx_t_11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  __pyx_t_10 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_12, __pyx_t_11, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_EPS); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_1 = PyNumber_Add(__pyx_t_10, __pyx_t_12); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyNumber_Divide(__pyx_v_H, __pyx_t_1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Rfar = __pyx_t_12;
  __pyx_t_12 = 0;

  /* "torchsar/misc/sarplat.py":171
 *         Rnear = H / (np.cos(Aon) + EPS)
 *         Rfar = H / (np.cos(Aon + np.sign(Aon) * Abr) + EPS)
 *         FPr = np.sqrt(Rfar**2 + Rnear**2 - 2. * Rnear * Rfar * np.cos(Abr) + EPS)             # <<<<<<<<<<<<<<
 * 
 *         Rsc = 0.5 * np.sqrt(2. * Rfar**2 + 2. * Rnear**2 - FPr**2 + EPS)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Power(__pyx_v_Rfar, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_11 = PyNumber_Power(__pyx_v_Rnear, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_2 = PyNumber_Add(__pyx_t_1, __pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = PyNumber_Multiply(__pyx_float_2_, __pyx_v_Rnear); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_1 = PyNumber_Multiply(__pyx_t_11, __pyx_v_Rfar); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_cos); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_13);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_13, function);
    }
  }
  __pyx_t_11 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_13, __pyx_t_3, __pyx_v_Abr) : __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_v_Abr);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = PyNumber_Multiply(__pyx_t_1, __pyx_t_11); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = PyNumber_Subtract(__pyx_t_2, __pyx_t_13); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_n_s_EPS); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_2 = PyNumber_Add(__pyx_t_11, __pyx_t_13); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_10);
    if (likely(__pyx_t_13)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_13);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_10, function);
    }
  }
  __pyx_t_12 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_13, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_2);
  __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_v_FPr = __pyx_t_12;
  __pyx_t_12 = 0;

  /* "torchsar/misc/sarplat.py":173
 *         FPr = np.sqrt(Rfar**2 + Rnear**2 - 2. * Rnear * Rfar * np.cos(Abr) + EPS)
 * 
 *         Rsc = 0.5 * np.sqrt(2. * Rfar**2 + 2. * Rnear**2 - FPr**2 + EPS)             # <<<<<<<<<<<<<<
 *         Rs0 = Rsc * np.cos(Ar)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyNumber_Power(__pyx_v_Rfar, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_13 = PyNumber_Multiply(__pyx_float_2_, __pyx_t_10); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyNumber_Power(__pyx_v_Rnear, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = PyNumber_Multiply(__pyx_float_2_, __pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyNumber_Add(__pyx_t_13, __pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = PyNumber_Power(__pyx_v_FPr, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_13 = PyNumber_Subtract(__pyx_t_10, __pyx_t_11); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_n_s_EPS); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_10 = PyNumber_Add(__pyx_t_13, __pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_12 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_11, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyNumber_Multiply(__pyx_float_0_5, __pyx_t_12); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_Rsc = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":174
 * 
 *         Rsc = 0.5 * np.sqrt(2. * Rfar**2 + 2. * Rnear**2 - FPr**2 + EPS)
 *         Rs0 = Rsc * np.cos(Ar)             # <<<<<<<<<<<<<<
 * 
 *         Ysc = Rsc * np.sin(Ar)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_np); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_cos); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_10);
    if (likely(__pyx_t_12)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_12);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_10, function);
    }
  }
  __pyx_t_2 = (__pyx_t_12) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_12, __pyx_v_Ar) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_v_Ar);
  __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyNumber_Multiply(__pyx_v_Rsc, __pyx_t_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_Rs0 = __pyx_t_10;
  __pyx_t_10 = 0;

  /* "torchsar/misc/sarplat.py":176
 *         Rs0 = Rsc * np.cos(Ar)
 * 
 *         Ysc = Rsc * np.sin(Ar)             # <<<<<<<<<<<<<<
 *         Xsc = np.sqrt(np.abs(Rs0**2 - H**2))
 *         SC = [Xsc, Ysc, 0.0]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_sin); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  __pyx_t_10 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_12, __pyx_t_2, __pyx_v_Ar) : __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_v_Ar);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = PyNumber_Multiply(__pyx_v_Rsc, __pyx_t_10); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_v_Ysc = __pyx_t_12;
  __pyx_t_12 = 0;

  /* "torchsar/misc/sarplat.py":177
 * 
 *         Ysc = Rsc * np.sin(Ar)
 *         Xsc = np.sqrt(np.abs(Rs0**2 - H**2))             # <<<<<<<<<<<<<<
 *         SC = [Xsc, Ysc, 0.0]
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_n_s_np); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_n_s_abs); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = PyNumber_Power(__pyx_v_Rs0, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_1 = PyNumber_Power(__pyx_v_H, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyNumber_Subtract(__pyx_t_11, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_13);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_13, function);
    }
  }
  __pyx_t_10 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_13, __pyx_t_1, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_13)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_13);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_12 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_13, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_10);
  __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_Xsc = __pyx_t_12;
  __pyx_t_12 = 0;

  /* "torchsar/misc/sarplat.py":178
 *         Ysc = Rsc * np.sin(Ar)
 *         Xsc = np.sqrt(np.abs(Rs0**2 - H**2))
 *         SC = [Xsc, Ysc, 0.0]             # <<<<<<<<<<<<<<
 * 
 *         Rbc = H / (np.sin(Ad) + EPS)
 */
  __pyx_t_12 = PyList_New(3); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 178, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_INCREF(__pyx_v_Xsc);
  __Pyx_GIVEREF(__pyx_v_Xsc);
  PyList_SET_ITEM(__pyx_t_12, 0, __pyx_v_Xsc);
  __Pyx_INCREF(__pyx_v_Ysc);
  __Pyx_GIVEREF(__pyx_v_Ysc);
  PyList_SET_ITEM(__pyx_t_12, 1, __pyx_v_Ysc);
  __Pyx_INCREF(__pyx_float_0_0);
  __Pyx_GIVEREF(__pyx_float_0_0);
  PyList_SET_ITEM(__pyx_t_12, 2, __pyx_float_0_0);
  __pyx_v_SC = ((PyObject*)__pyx_t_12);
  __pyx_t_12 = 0;

  /* "torchsar/misc/sarplat.py":180
 *         SC = [Xsc, Ysc, 0.0]
 * 
 *         Rbc = H / (np.sin(Ad) + EPS)             # <<<<<<<<<<<<<<
 *         Rb0 = Rbc * np.cos(Ar)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 180, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_sin); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 180, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_10);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_10, function);
    }
  }
  __pyx_t_12 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_2, __pyx_v_Ad) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_v_Ad);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 180, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_EPS); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 180, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_2 = PyNumber_Add(__pyx_t_12, __pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 180, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyNumber_Divide(__pyx_v_H, __pyx_t_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 180, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_Rbc = __pyx_t_10;
  __pyx_t_10 = 0;

  /* "torchsar/misc/sarplat.py":181
 * 
 *         Rbc = H / (np.sin(Ad) + EPS)
 *         Rb0 = Rbc * np.cos(Ar)             # <<<<<<<<<<<<<<
 * 
 *         Ybc = Rbc * np.sin(Ar)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_cos); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  __pyx_t_10 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_12, __pyx_t_2, __pyx_v_Ar) : __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_v_Ar);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = PyNumber_Multiply(__pyx_v_Rbc, __pyx_t_10); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_v_Rb0 = __pyx_t_12;
  __pyx_t_12 = 0;

  /* "torchsar/misc/sarplat.py":183
 *         Rb0 = Rbc * np.cos(Ar)
 * 
 *         Ybc = Rbc * np.sin(Ar)             # <<<<<<<<<<<<<<
 *         Xbc = np.sqrt(np.abs(Rb0**2 - H**2))
 *         BC = [Xbc, Ybc, 0.0]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_sin); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_12 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_10, __pyx_v_Ar) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_Ar);
  __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyNumber_Multiply(__pyx_v_Rbc, __pyx_t_12); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_v_Ybc = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":184
 * 
 *         Ybc = Rbc * np.sin(Ar)
 *         Xbc = np.sqrt(np.abs(Rb0**2 - H**2))             # <<<<<<<<<<<<<<
 *         BC = [Xbc, Ybc, 0.0]
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_np); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_n_s_np); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_13, __pyx_n_s_abs); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = PyNumber_Power(__pyx_v_Rb0, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_1 = PyNumber_Power(__pyx_v_H, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_11 = PyNumber_Subtract(__pyx_t_13, __pyx_t_1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_12 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_1, __pyx_t_11) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_11);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_10);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_10, function);
    }
  }
  __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_3, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_12);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_v_Xbc = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":185
 *         Ybc = Rbc * np.sin(Ar)
 *         Xbc = np.sqrt(np.abs(Rb0**2 - H**2))
 *         BC = [Xbc, Ybc, 0.0]             # <<<<<<<<<<<<<<
 * 
 *         if (SA[0] is None) or (SA[1] is None):
 */
  __pyx_t_2 = PyList_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_Xbc);
  __Pyx_GIVEREF(__pyx_v_Xbc);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_v_Xbc);
  __Pyx_INCREF(__pyx_v_Ybc);
  __Pyx_GIVEREF(__pyx_v_Ybc);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_v_Ybc);
  __Pyx_INCREF(__pyx_float_0_0);
  __Pyx_GIVEREF(__pyx_float_0_0);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_float_0_0);
  __pyx_v_BC = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":187
 *         BC = [Xbc, Ybc, 0.0]
 * 
 *         if (SA[0] is None) or (SA[1] is None):             # <<<<<<<<<<<<<<
 *             Ynear = Rnear * np.sin(Ar)
 *             Yfar = Rfar * np.sin(Ar)
 */
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_SA, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = (__pyx_t_2 == Py_None);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_14 = (__pyx_t_5 != 0);
  if (!__pyx_t_14) {
  } else {
    __pyx_t_4 = __pyx_t_14;
    goto __pyx_L19_bool_binop_done;
  }
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_SA, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_14 = (__pyx_t_2 == Py_None);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = (__pyx_t_14 != 0);
  __pyx_t_4 = __pyx_t_5;
  __pyx_L19_bool_binop_done:;
  if (__pyx_t_4) {

    /* "torchsar/misc/sarplat.py":188
 * 
 *         if (SA[0] is None) or (SA[1] is None):
 *             Ynear = Rnear * np.sin(Ar)             # <<<<<<<<<<<<<<
 *             Yfar = Rfar * np.sin(Ar)
 *             Xnear = np.sqrt((Rnear ** 2 - H**2) - Ynear ** 2 + EPS)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_sin); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_12))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_12);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_12, function);
      }
    }
    __pyx_t_2 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_12, __pyx_t_10, __pyx_v_Ar) : __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_v_Ar);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_t_12 = PyNumber_Multiply(__pyx_v_Rnear, __pyx_t_2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 188, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_Ynear = __pyx_t_12;
    __pyx_t_12 = 0;

    /* "torchsar/misc/sarplat.py":189
 *         if (SA[0] is None) or (SA[1] is None):
 *             Ynear = Rnear * np.sin(Ar)
 *             Yfar = Rfar * np.sin(Ar)             # <<<<<<<<<<<<<<
 *             Xnear = np.sqrt((Rnear ** 2 - H**2) - Ynear ** 2 + EPS)
 *             Xfar = np.sqrt((Rfar ** 2 - H**2) - Yfar ** 2 + EPS)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_sin); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_10);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_10, function);
      }
    }
    __pyx_t_12 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_2, __pyx_v_Ar) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_v_Ar);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = PyNumber_Multiply(__pyx_v_Rfar, __pyx_t_12); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_v_Yfar = __pyx_t_10;
    __pyx_t_10 = 0;

    /* "torchsar/misc/sarplat.py":190
 *             Ynear = Rnear * np.sin(Ar)
 *             Yfar = Rfar * np.sin(Ar)
 *             Xnear = np.sqrt((Rnear ** 2 - H**2) - Ynear ** 2 + EPS)             # <<<<<<<<<<<<<<
 *             Xfar = np.sqrt((Rfar ** 2 - H**2) - Yfar ** 2 + EPS)
 *             SAV[0] = Xnear - Xsc
 */
    __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_np); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_t_12 = PyNumber_Power(__pyx_v_Rnear, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_3 = PyNumber_Power(__pyx_v_H, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_11 = PyNumber_Subtract(__pyx_t_12, __pyx_t_3); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyNumber_Power(__pyx_v_Ynear, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_12 = PyNumber_Subtract(__pyx_t_11, __pyx_t_3); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_EPS); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_11 = PyNumber_Add(__pyx_t_12, __pyx_t_3); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_10 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_t_11) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_11);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_Xnear = __pyx_t_10;
    __pyx_t_10 = 0;

    /* "torchsar/misc/sarplat.py":191
 *             Yfar = Rfar * np.sin(Ar)
 *             Xnear = np.sqrt((Rnear ** 2 - H**2) - Ynear ** 2 + EPS)
 *             Xfar = np.sqrt((Rfar ** 2 - H**2) - Yfar ** 2 + EPS)             # <<<<<<<<<<<<<<
 *             SAV[0] = Xnear - Xsc
 *             SAV[1] = Xfar - Xsc
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyNumber_Power(__pyx_v_Rfar, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyNumber_Power(__pyx_v_H, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_12 = PyNumber_Subtract(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyNumber_Power(__pyx_v_Yfar, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = PyNumber_Subtract(__pyx_t_12, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_EPS); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_12 = PyNumber_Add(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_11))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_11);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_11, function);
      }
    }
    __pyx_t_10 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_11, __pyx_t_3, __pyx_t_12) : __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_t_12);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_v_Xfar = __pyx_t_10;
    __pyx_t_10 = 0;

    /* "torchsar/misc/sarplat.py":192
 *             Xnear = np.sqrt((Rnear ** 2 - H**2) - Ynear ** 2 + EPS)
 *             Xfar = np.sqrt((Rfar ** 2 - H**2) - Yfar ** 2 + EPS)
 *             SAV[0] = Xnear - Xsc             # <<<<<<<<<<<<<<
 *             SAV[1] = Xfar - Xsc
 *         else:
 */
    __pyx_t_10 = PyNumber_Subtract(__pyx_v_Xnear, __pyx_v_Xsc); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 192, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (unlikely(__Pyx_SetItemInt(__pyx_v_SAV, 0, __pyx_t_10, long, 1, __Pyx_PyInt_From_long, 1, 0, 1) < 0)) __PYX_ERR(0, 192, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

    /* "torchsar/misc/sarplat.py":193
 *             Xfar = np.sqrt((Rfar ** 2 - H**2) - Yfar ** 2 + EPS)
 *             SAV[0] = Xnear - Xsc
 *             SAV[1] = Xfar - Xsc             # <<<<<<<<<<<<<<
 *         else:
 *             SAV[0] = SA[0]
 */
    __pyx_t_10 = PyNumber_Subtract(__pyx_v_Xfar, __pyx_v_Xsc); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (unlikely(__Pyx_SetItemInt(__pyx_v_SAV, 1, __pyx_t_10, long, 1, __Pyx_PyInt_From_long, 1, 0, 1) < 0)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

    /* "torchsar/misc/sarplat.py":187
 *         BC = [Xbc, Ybc, 0.0]
 * 
 *         if (SA[0] is None) or (SA[1] is None):             # <<<<<<<<<<<<<<
 *             Ynear = Rnear * np.sin(Ar)
 *             Yfar = Rfar * np.sin(Ar)
 */
    goto __pyx_L18;
  }

  /* "torchsar/misc/sarplat.py":195
 *             SAV[1] = Xfar - Xsc
 *         else:
 *             SAV[0] = SA[0]             # <<<<<<<<<<<<<<
 *             SAV[1] = SA[1]
 * 
 */
  /*else*/ {
    __pyx_t_10 = __Pyx_GetItemInt(__pyx_v_SA, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 195, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (unlikely(__Pyx_SetItemInt(__pyx_v_SAV, 0, __pyx_t_10, long, 1, __Pyx_PyInt_From_long, 1, 0, 1) < 0)) __PYX_ERR(0, 195, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

    /* "torchsar/misc/sarplat.py":196
 *         else:
 *             SAV[0] = SA[0]
 *             SAV[1] = SA[1]             # <<<<<<<<<<<<<<
 * 
 *         if (SA[2] is None) or (SA[3] is None):
 */
    __pyx_t_10 = __Pyx_GetItemInt(__pyx_v_SA, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 196, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (unlikely(__Pyx_SetItemInt(__pyx_v_SAV, 1, __pyx_t_10, long, 1, __Pyx_PyInt_From_long, 1, 0, 1) < 0)) __PYX_ERR(0, 196, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __pyx_L18:;

  /* "torchsar/misc/sarplat.py":198
 *             SAV[1] = SA[1]
 * 
 *         if (SA[2] is None) or (SA[3] is None):             # <<<<<<<<<<<<<<
 *             Na, Nr = ES
 *             Ta = Na / PRF
 */
  __pyx_t_10 = __Pyx_GetItemInt(__pyx_v_SA, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_5 = (__pyx_t_10 == Py_None);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_14 = (__pyx_t_5 != 0);
  if (!__pyx_t_14) {
  } else {
    __pyx_t_4 = __pyx_t_14;
    goto __pyx_L22_bool_binop_done;
  }
  __pyx_t_10 = __Pyx_GetItemInt(__pyx_v_SA, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_14 = (__pyx_t_10 == Py_None);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_5 = (__pyx_t_14 != 0);
  __pyx_t_4 = __pyx_t_5;
  __pyx_L22_bool_binop_done:;
  if (__pyx_t_4) {

    /* "torchsar/misc/sarplat.py":199
 * 
 *         if (SA[2] is None) or (SA[3] is None):
 *             Na, Nr = ES             # <<<<<<<<<<<<<<
 *             Ta = Na / PRF
 *             SAV[2] = -Ta * V / 2.0
 */
    if ((likely(PyTuple_CheckExact(__pyx_v_ES))) || (PyList_CheckExact(__pyx_v_ES))) {
      PyObject* sequence = __pyx_v_ES;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 199, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_10 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_11 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_10 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_11 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_11);
      #else
      __pyx_t_10 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 199, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_11 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 199, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      #endif
    } else {
      Py_ssize_t index = -1;
      __pyx_t_12 = PyObject_GetIter(__pyx_v_ES); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 199, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_15 = Py_TYPE(__pyx_t_12)->tp_iternext;
      index = 0; __pyx_t_10 = __pyx_t_15(__pyx_t_12); if (unlikely(!__pyx_t_10)) goto __pyx_L24_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_10);
      index = 1; __pyx_t_11 = __pyx_t_15(__pyx_t_12); if (unlikely(!__pyx_t_11)) goto __pyx_L24_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_11);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_15(__pyx_t_12), 2) < 0) __PYX_ERR(0, 199, __pyx_L1_error)
      __pyx_t_15 = NULL;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      goto __pyx_L25_unpacking_done;
      __pyx_L24_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_15 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 199, __pyx_L1_error)
      __pyx_L25_unpacking_done:;
    }
    __pyx_v_Na = __pyx_t_10;
    __pyx_t_10 = 0;
    __Pyx_XDECREF_SET(__pyx_v_Nr, __pyx_t_11);
    __pyx_t_11 = 0;

    /* "torchsar/misc/sarplat.py":200
 *         if (SA[2] is None) or (SA[3] is None):
 *             Na, Nr = ES
 *             Ta = Na / PRF             # <<<<<<<<<<<<<<
 *             SAV[2] = -Ta * V / 2.0
 *             SAV[3] = Ta * V / 2.0
 */
    __pyx_t_11 = __Pyx_PyNumber_Divide(__pyx_v_Na, __pyx_v_PRF); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 200, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_v_Ta = __pyx_t_11;
    __pyx_t_11 = 0;

    /* "torchsar/misc/sarplat.py":201
 *             Na, Nr = ES
 *             Ta = Na / PRF
 *             SAV[2] = -Ta * V / 2.0             # <<<<<<<<<<<<<<
 *             SAV[3] = Ta * V / 2.0
 *         else:
 */
    __pyx_t_11 = PyNumber_Negative(__pyx_v_Ta); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 201, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_10 = PyNumber_Multiply(__pyx_t_11, __pyx_v_V); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 201, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_11 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_10, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 201, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(__Pyx_SetItemInt(__pyx_v_SAV, 2, __pyx_t_11, long, 1, __Pyx_PyInt_From_long, 1, 0, 1) < 0)) __PYX_ERR(0, 201, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

    /* "torchsar/misc/sarplat.py":202
 *             Ta = Na / PRF
 *             SAV[2] = -Ta * V / 2.0
 *             SAV[3] = Ta * V / 2.0             # <<<<<<<<<<<<<<
 *         else:
 *             SAV[2] = SA[2]
 */
    __pyx_t_11 = PyNumber_Multiply(__pyx_v_Ta, __pyx_v_V); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 202, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_10 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_11, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 202, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(__Pyx_SetItemInt(__pyx_v_SAV, 3, __pyx_t_10, long, 1, __Pyx_PyInt_From_long, 1, 0, 1) < 0)) __PYX_ERR(0, 202, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

    /* "torchsar/misc/sarplat.py":198
 *             SAV[1] = SA[1]
 * 
 *         if (SA[2] is None) or (SA[3] is None):             # <<<<<<<<<<<<<<
 *             Na, Nr = ES
 *             Ta = Na / PRF
 */
    goto __pyx_L21;
  }

  /* "torchsar/misc/sarplat.py":204
 *             SAV[3] = Ta * V / 2.0
 *         else:
 *             SAV[2] = SA[2]             # <<<<<<<<<<<<<<
 *             SAV[3] = SA[3]
 * 
 */
  /*else*/ {
    __pyx_t_10 = __Pyx_GetItemInt(__pyx_v_SA, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 204, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (unlikely(__Pyx_SetItemInt(__pyx_v_SAV, 2, __pyx_t_10, long, 1, __Pyx_PyInt_From_long, 1, 0, 1) < 0)) __PYX_ERR(0, 204, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

    /* "torchsar/misc/sarplat.py":205
 *         else:
 *             SAV[2] = SA[2]
 *             SAV[3] = SA[3]             # <<<<<<<<<<<<<<
 * 
 *         Na, Nr = ES
 */
    __pyx_t_10 = __Pyx_GetItemInt(__pyx_v_SA, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (unlikely(__Pyx_SetItemInt(__pyx_v_SAV, 3, __pyx_t_10, long, 1, __Pyx_PyInt_From_long, 1, 0, 1) < 0)) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __pyx_L21:;

  /* "torchsar/misc/sarplat.py":207
 *             SAV[3] = SA[3]
 * 
 *         Na, Nr = ES             # <<<<<<<<<<<<<<
 *         Ta = Na / PRF
 *         FPr0 = H / np.tan(Ad + Abr / 2.) - H / np.tan(Ad)
 */
  if ((likely(PyTuple_CheckExact(__pyx_v_ES))) || (PyList_CheckExact(__pyx_v_ES))) {
    PyObject* sequence = __pyx_v_ES;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 207, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_10 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_11 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_10 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_11 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_10);
    __Pyx_INCREF(__pyx_t_11);
    #else
    __pyx_t_10 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 207, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 207, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    #endif
  } else {
    Py_ssize_t index = -1;
    __pyx_t_12 = PyObject_GetIter(__pyx_v_ES); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 207, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_15 = Py_TYPE(__pyx_t_12)->tp_iternext;
    index = 0; __pyx_t_10 = __pyx_t_15(__pyx_t_12); if (unlikely(!__pyx_t_10)) goto __pyx_L26_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_10);
    index = 1; __pyx_t_11 = __pyx_t_15(__pyx_t_12); if (unlikely(!__pyx_t_11)) goto __pyx_L26_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_11);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_15(__pyx_t_12), 2) < 0) __PYX_ERR(0, 207, __pyx_L1_error)
    __pyx_t_15 = NULL;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    goto __pyx_L27_unpacking_done;
    __pyx_L26_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_t_15 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 207, __pyx_L1_error)
    __pyx_L27_unpacking_done:;
  }
  __Pyx_XDECREF_SET(__pyx_v_Na, __pyx_t_10);
  __pyx_t_10 = 0;
  __Pyx_XDECREF_SET(__pyx_v_Nr, __pyx_t_11);
  __pyx_t_11 = 0;

  /* "torchsar/misc/sarplat.py":208
 * 
 *         Na, Nr = ES
 *         Ta = Na / PRF             # <<<<<<<<<<<<<<
 *         FPr0 = H / np.tan(Ad + Abr / 2.) - H / np.tan(Ad)
 *         FPr1 = H / np.tan(Ad - Abr / 2.) - H / np.tan(Ad)
 */
  __pyx_t_11 = __Pyx_PyNumber_Divide(__pyx_v_Na, __pyx_v_PRF); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_XDECREF_SET(__pyx_v_Ta, __pyx_t_11);
  __pyx_t_11 = 0;

  /* "torchsar/misc/sarplat.py":209
 *         Na, Nr = ES
 *         Ta = Na / PRF
 *         FPr0 = H / np.tan(Ad + Abr / 2.) - H / np.tan(Ad)             # <<<<<<<<<<<<<<
 *         FPr1 = H / np.tan(Ad - Abr / 2.) - H / np.tan(Ad)
 *         BAV = (FPr0, FPr1, -Ta * V / 2.0, Ta * V / 2.0)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_tan); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyFloat_TrueDivideObjC(__pyx_v_Abr, __pyx_float_2_, 2., 0, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_3 = PyNumber_Add(__pyx_v_Ad, __pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  __pyx_t_11 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_12, __pyx_t_10, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyNumber_Divide(__pyx_v_H, __pyx_t_11); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_tan); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_10);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_10, function);
    }
  }
  __pyx_t_11 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_3, __pyx_v_Ad) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_v_Ad);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyNumber_Divide(__pyx_v_H, __pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = PyNumber_Subtract(__pyx_t_12, __pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_v_FPr0 = __pyx_t_11;
  __pyx_t_11 = 0;

  /* "torchsar/misc/sarplat.py":210
 *         Ta = Na / PRF
 *         FPr0 = H / np.tan(Ad + Abr / 2.) - H / np.tan(Ad)
 *         FPr1 = H / np.tan(Ad - Abr / 2.) - H / np.tan(Ad)             # <<<<<<<<<<<<<<
 *         BAV = (FPr0, FPr1, -Ta * V / 2.0, Ta * V / 2.0)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_tan); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyFloat_TrueDivideObjC(__pyx_v_Abr, __pyx_float_2_, 2., 0, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_3 = PyNumber_Subtract(__pyx_v_Ad, __pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  __pyx_t_11 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_12, __pyx_t_10, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = __Pyx_PyNumber_Divide(__pyx_v_H, __pyx_t_11); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_tan); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_10);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_10, function);
    }
  }
  __pyx_t_11 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_3, __pyx_v_Ad) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_v_Ad);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyNumber_Divide(__pyx_v_H, __pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = PyNumber_Subtract(__pyx_t_12, __pyx_t_10); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_v_FPr1 = __pyx_t_11;
  __pyx_t_11 = 0;

  /* "torchsar/misc/sarplat.py":211
 *         FPr0 = H / np.tan(Ad + Abr / 2.) - H / np.tan(Ad)
 *         FPr1 = H / np.tan(Ad - Abr / 2.) - H / np.tan(Ad)
 *         BAV = (FPr0, FPr1, -Ta * V / 2.0, Ta * V / 2.0)             # <<<<<<<<<<<<<<
 * 
 *         if H < AirboneSatelliteHeightBoundary:
 */
  __pyx_t_11 = PyNumber_Negative(__pyx_v_Ta); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_10 = PyNumber_Multiply(__pyx_t_11, __pyx_v_V); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_10, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyNumber_Multiply(__pyx_v_Ta, __pyx_v_V); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_12 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_10, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyTuple_New(4); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_INCREF(__pyx_v_FPr0);
  __Pyx_GIVEREF(__pyx_v_FPr0);
  PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_v_FPr0);
  __Pyx_INCREF(__pyx_v_FPr1);
  __Pyx_GIVEREF(__pyx_v_FPr1);
  PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_v_FPr1);
  __Pyx_GIVEREF(__pyx_t_11);
  PyTuple_SET_ITEM(__pyx_t_10, 2, __pyx_t_11);
  __Pyx_GIVEREF(__pyx_t_12);
  PyTuple_SET_ITEM(__pyx_t_10, 3, __pyx_t_12);
  __pyx_t_11 = 0;
  __pyx_t_12 = 0;
  __Pyx_DECREF_SET(__pyx_v_BAV, __pyx_t_10);
  __pyx_t_10 = 0;

  /* "torchsar/misc/sarplat.py":213
 *         BAV = (FPr0, FPr1, -Ta * V / 2.0, Ta * V / 2.0)
 * 
 *         if H < AirboneSatelliteHeightBoundary:             # <<<<<<<<<<<<<<
 *             logging.info("---Airbone.")
 *             Ai = PI / 2. - Ad
 */
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_AirboneSatelliteHeightBoundary); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_12 = PyObject_RichCompare(__pyx_v_H, __pyx_t_10, Py_LT); __Pyx_XGOTREF(__pyx_t_12); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_12); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (__pyx_t_4) {

    /* "torchsar/misc/sarplat.py":214
 * 
 *         if H < AirboneSatelliteHeightBoundary:
 *             logging.info("---Airbone.")             # <<<<<<<<<<<<<<
 *             Ai = PI / 2. - Ad
 *         else:
 */
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_logging); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 214, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_info); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 214, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_11))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_11);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_11, function);
      }
    }
    __pyx_t_12 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_11, __pyx_t_10, __pyx_kp_u_Airbone) : __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_kp_u_Airbone);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 214, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

    /* "torchsar/misc/sarplat.py":215
 *         if H < AirboneSatelliteHeightBoundary:
 *             logging.info("---Airbone.")
 *             Ai = PI / 2. - Ad             # <<<<<<<<<<<<<<
 *         else:
 *             Ao = np.arccos((Rs**2 + Rea**2 - Rbc**2) / (2. * Rs * Rea))
 */
    __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_PI); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 215, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_11 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_12, __pyx_float_2_, 2., 0, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 215, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_t_12 = PyNumber_Subtract(__pyx_t_11, __pyx_v_Ad); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 215, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_v_Ai = __pyx_t_12;
    __pyx_t_12 = 0;

    /* "torchsar/misc/sarplat.py":213
 *         BAV = (FPr0, FPr1, -Ta * V / 2.0, Ta * V / 2.0)
 * 
 *         if H < AirboneSatelliteHeightBoundary:             # <<<<<<<<<<<<<<
 *             logging.info("---Airbone.")
 *             Ai = PI / 2. - Ad
 */
    goto __pyx_L28;
  }

  /* "torchsar/misc/sarplat.py":217
 *             Ai = PI / 2. - Ad
 *         else:
 *             Ao = np.arccos((Rs**2 + Rea**2 - Rbc**2) / (2. * Rs * Rea))             # <<<<<<<<<<<<<<
 *             Ai = (PI / 2. - Ad) + Ao
 *             logging.info("---Satellite.")
 */
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_n_s_np); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_n_s_arccos); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_11 = PyNumber_Power(__pyx_v_Rs, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_Rea); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = PyNumber_Power(__pyx_t_3, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyNumber_Add(__pyx_t_11, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyNumber_Power(__pyx_v_Rbc, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_11 = PyNumber_Subtract(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyNumber_Multiply(__pyx_float_2_, __pyx_v_Rs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_Rea); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = PyNumber_Multiply(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyNumber_Divide(__pyx_t_11, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_10);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_10, function);
      }
    }
    __pyx_t_12 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_1, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_3);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_v_Ao = __pyx_t_12;
    __pyx_t_12 = 0;

    /* "torchsar/misc/sarplat.py":218
 *         else:
 *             Ao = np.arccos((Rs**2 + Rea**2 - Rbc**2) / (2. * Rs * Rea))
 *             Ai = (PI / 2. - Ad) + Ao             # <<<<<<<<<<<<<<
 *             logging.info("---Satellite.")
 *         logging.info('---Done!')
 */
    __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_PI); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 218, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_10 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_12, __pyx_float_2_, 2., 0, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 218, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_t_12 = PyNumber_Subtract(__pyx_t_10, __pyx_v_Ad); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 218, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = PyNumber_Add(__pyx_t_12, __pyx_v_Ao); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 218, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_v_Ai = __pyx_t_10;
    __pyx_t_10 = 0;

    /* "torchsar/misc/sarplat.py":219
 *             Ao = np.arccos((Rs**2 + Rea**2 - Rbc**2) / (2. * Rs * Rea))
 *             Ai = (PI / 2. - Ad) + Ao
 *             logging.info("---Satellite.")             # <<<<<<<<<<<<<<
 *         logging.info('---Done!')
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_logging); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 219, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_info); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 219, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_t_12 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_12)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_12);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_10 = (__pyx_t_12) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_12, __pyx_kp_u_Satellite) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_Satellite);
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
    if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 219, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __pyx_L28:;

  /* "torchsar/misc/sarplat.py":220
 *             Ai = (PI / 2. - Ad) + Ao
 *             logging.info("---Satellite.")
 *         logging.info('---Done!')             # <<<<<<<<<<<<<<
 * 
 *         value['StateVector'] = SV
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_logging); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_info); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  __pyx_t_10 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_12, __pyx_t_3, __pyx_kp_u_Done) : __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_kp_u_Done);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 220, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "torchsar/misc/sarplat.py":222
 *         logging.info('---Done!')
 * 
 *         value['StateVector'] = SV             # <<<<<<<<<<<<<<
 *         value['PlatCenter'] = np.array(PC)
 *         value['SceneCenter'] = np.array(SC)
 */
  if (unlikely(PyObject_SetItem(__pyx_v_value, __pyx_n_u_StateVector, __pyx_v_SV) < 0)) __PYX_ERR(0, 222, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":223
 * 
 *         value['StateVector'] = SV
 *         value['PlatCenter'] = np.array(PC)             # <<<<<<<<<<<<<<
 *         value['SceneCenter'] = np.array(SC)
 *         value['BeamCenter'] = np.array(BC)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_np); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 223, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_array); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 223, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_12)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_12);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_10 = (__pyx_t_12) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_12, __pyx_v_PC) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_PC);
  __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 223, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(PyObject_SetItem(__pyx_v_value, __pyx_n_u_PlatCenter, __pyx_t_10) < 0)) __PYX_ERR(0, 223, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "torchsar/misc/sarplat.py":224
 *         value['StateVector'] = SV
 *         value['PlatCenter'] = np.array(PC)
 *         value['SceneCenter'] = np.array(SC)             # <<<<<<<<<<<<<<
 *         value['BeamCenter'] = np.array(BC)
 *         value['SceneArea'] = np.array(SAV)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_array); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  __pyx_t_10 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_12, __pyx_t_3, __pyx_v_SC) : __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_v_SC);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (unlikely(PyObject_SetItem(__pyx_v_value, __pyx_n_u_SceneCenter, __pyx_t_10) < 0)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "torchsar/misc/sarplat.py":225
 *         value['PlatCenter'] = np.array(PC)
 *         value['SceneCenter'] = np.array(SC)
 *         value['BeamCenter'] = np.array(BC)             # <<<<<<<<<<<<<<
 *         value['SceneArea'] = np.array(SAV)
 *         value['BeamArea'] = np.array(BAV)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_np); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 225, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_array); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 225, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_12)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_12);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_10 = (__pyx_t_12) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_12, __pyx_v_BC) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_BC);
  __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 225, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(PyObject_SetItem(__pyx_v_value, __pyx_n_u_BeamCenter, __pyx_t_10) < 0)) __PYX_ERR(0, 225, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "torchsar/misc/sarplat.py":226
 *         value['SceneCenter'] = np.array(SC)
 *         value['BeamCenter'] = np.array(BC)
 *         value['SceneArea'] = np.array(SAV)             # <<<<<<<<<<<<<<
 *         value['BeamArea'] = np.array(BAV)
 *         value['Rbc'] = Rbc
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 226, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_array); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 226, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_12))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_12);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_12, function);
    }
  }
  __pyx_t_10 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_12, __pyx_t_3, __pyx_v_SAV) : __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_v_SAV);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 226, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (unlikely(PyObject_SetItem(__pyx_v_value, __pyx_n_u_SceneArea, __pyx_t_10) < 0)) __PYX_ERR(0, 226, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "torchsar/misc/sarplat.py":227
 *         value['BeamCenter'] = np.array(BC)
 *         value['SceneArea'] = np.array(SAV)
 *         value['BeamArea'] = np.array(BAV)             # <<<<<<<<<<<<<<
 *         value['Rbc'] = Rbc
 *         value['Rb0'] = Rb0
 */
  __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_n_s_np); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_n_s_array); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
  __pyx_t_12 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_12)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_12);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_10 = (__pyx_t_12) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_12, __pyx_v_BAV) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_BAV);
  __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(PyObject_SetItem(__pyx_v_value, __pyx_n_u_BeamArea, __pyx_t_10) < 0)) __PYX_ERR(0, 227, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

  /* "torchsar/misc/sarplat.py":228
 *         value['SceneArea'] = np.array(SAV)
 *         value['BeamArea'] = np.array(BAV)
 *         value['Rbc'] = Rbc             # <<<<<<<<<<<<<<
 *         value['Rb0'] = Rb0
 *         value['Rsc'] = Rsc
 */
  if (unlikely(PyObject_SetItem(__pyx_v_value, __pyx_n_u_Rbc, __pyx_v_Rbc) < 0)) __PYX_ERR(0, 228, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":229
 *         value['BeamArea'] = np.array(BAV)
 *         value['Rbc'] = Rbc
 *         value['Rb0'] = Rb0             # <<<<<<<<<<<<<<
 *         value['Rsc'] = Rsc
 *         value['Rs0'] = Rs0
 */
  if (unlikely(PyObject_SetItem(__pyx_v_value, __pyx_n_u_Rb0, __pyx_v_Rb0) < 0)) __PYX_ERR(0, 229, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":230
 *         value['Rbc'] = Rbc
 *         value['Rb0'] = Rb0
 *         value['Rsc'] = Rsc             # <<<<<<<<<<<<<<
 *         value['Rs0'] = Rs0
 *         value['Rnear'] = Rnear
 */
  if (unlikely(PyObject_SetItem(__pyx_v_value, __pyx_n_u_Rsc, __pyx_v_Rsc) < 0)) __PYX_ERR(0, 230, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":231
 *         value['Rb0'] = Rb0
 *         value['Rsc'] = Rsc
 *         value['Rs0'] = Rs0             # <<<<<<<<<<<<<<
 *         value['Rnear'] = Rnear
 *         value['Rfar'] = Rfar
 */
  if (unlikely(PyObject_SetItem(__pyx_v_value, __pyx_n_u_Rs0, __pyx_v_Rs0) < 0)) __PYX_ERR(0, 231, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":232
 *         value['Rsc'] = Rsc
 *         value['Rs0'] = Rs0
 *         value['Rnear'] = Rnear             # <<<<<<<<<<<<<<
 *         value['Rfar'] = Rfar
 *         value['Aba'] = Aba
 */
  if (unlikely(PyObject_SetItem(__pyx_v_value, __pyx_n_u_Rnear, __pyx_v_Rnear) < 0)) __PYX_ERR(0, 232, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":233
 *         value['Rs0'] = Rs0
 *         value['Rnear'] = Rnear
 *         value['Rfar'] = Rfar             # <<<<<<<<<<<<<<
 *         value['Aba'] = Aba
 *         value['Abr'] = Abr
 */
  if (unlikely(PyObject_SetItem(__pyx_v_value, __pyx_n_u_Rfar, __pyx_v_Rfar) < 0)) __PYX_ERR(0, 233, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":234
 *         value['Rnear'] = Rnear
 *         value['Rfar'] = Rfar
 *         value['Aba'] = Aba             # <<<<<<<<<<<<<<
 *         value['Abr'] = Abr
 *         value['Ar'] = Ar
 */
  if (unlikely(PyObject_SetItem(__pyx_v_value, __pyx_n_u_Aba, __pyx_v_Aba) < 0)) __PYX_ERR(0, 234, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":235
 *         value['Rfar'] = Rfar
 *         value['Aba'] = Aba
 *         value['Abr'] = Abr             # <<<<<<<<<<<<<<
 *         value['Ar'] = Ar
 *         value['Ai'] = Ai
 */
  if (unlikely(PyObject_SetItem(__pyx_v_value, __pyx_n_u_Abr, __pyx_v_Abr) < 0)) __PYX_ERR(0, 235, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":236
 *         value['Aba'] = Aba
 *         value['Abr'] = Abr
 *         value['Ar'] = Ar             # <<<<<<<<<<<<<<
 *         value['Ai'] = Ai
 *         value['Ad'] = Ad
 */
  if (unlikely(PyObject_SetItem(__pyx_v_value, __pyx_n_u_Ar, __pyx_v_Ar) < 0)) __PYX_ERR(0, 236, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":237
 *         value['Abr'] = Abr
 *         value['Ar'] = Ar
 *         value['Ai'] = Ai             # <<<<<<<<<<<<<<
 *         value['Ad'] = Ad
 *         value['Aon'] = Aon
 */
  if (unlikely(PyObject_SetItem(__pyx_v_value, __pyx_n_u_Ai, __pyx_v_Ai) < 0)) __PYX_ERR(0, 237, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":238
 *         value['Ar'] = Ar
 *         value['Ai'] = Ai
 *         value['Ad'] = Ad             # <<<<<<<<<<<<<<
 *         value['Aon'] = Aon
 * 
 */
  if (unlikely(PyObject_SetItem(__pyx_v_value, __pyx_n_u_Ad, __pyx_v_Ad) < 0)) __PYX_ERR(0, 238, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":239
 *         value['Ai'] = Ai
 *         value['Ad'] = Ad
 *         value['Aon'] = Aon             # <<<<<<<<<<<<<<
 * 
 *         self._acquisition = value
 */
  if (unlikely(PyObject_SetItem(__pyx_v_value, __pyx_n_u_Aon, __pyx_v_Aon) < 0)) __PYX_ERR(0, 239, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":241
 *         value['Aon'] = Aon
 * 
 *         self._acquisition = value             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_acquisition, __pyx_v_value) < 0) __PYX_ERR(0, 241, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":108
 * 
 *     @acquisition.setter
 *     def acquisition(self, value):             # <<<<<<<<<<<<<<
 *         logging.info("---Check and set acquisition parameters...")
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("torchsar.misc.sarplat.SarPlat.acquisition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_k);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_XDECREF(__pyx_v_H);
  __Pyx_XDECREF(__pyx_v_V);
  __Pyx_XDECREF(__pyx_v_Vr);
  __Pyx_XDECREF(__pyx_v_Vs);
  __Pyx_XDECREF(__pyx_v_PRF);
  __Pyx_XDECREF(__pyx_v_Fsr);
  __Pyx_XDECREF(__pyx_v_Tp);
  __Pyx_XDECREF(__pyx_v_Rs);
  __Pyx_XDECREF(__pyx_v_As);
  __Pyx_XDECREF(__pyx_v_Ad);
  __Pyx_XDECREF(__pyx_v_Aon);
  __Pyx_XDECREF(__pyx_v_Aba);
  __Pyx_XDECREF(__pyx_v_Abr);
  __Pyx_XDECREF(__pyx_v_SA);
  __Pyx_XDECREF(__pyx_v_PC);
  __Pyx_XDECREF(__pyx_v_ES);
  __Pyx_XDECREF(__pyx_v_SV);
  __Pyx_XDECREF(__pyx_v_Ar);
  __Pyx_XDECREF(__pyx_v_SAV);
  __Pyx_XDECREF(__pyx_v_BAV);
  __Pyx_XDECREF(__pyx_v_Nr);
  __Pyx_XDECREF(__pyx_v_Rnear);
  __Pyx_XDECREF(__pyx_v_Rfar);
  __Pyx_XDECREF(__pyx_v_FPr);
  __Pyx_XDECREF(__pyx_v_Rsc);
  __Pyx_XDECREF(__pyx_v_Rs0);
  __Pyx_XDECREF(__pyx_v_Ysc);
  __Pyx_XDECREF(__pyx_v_Xsc);
  __Pyx_XDECREF(__pyx_v_SC);
  __Pyx_XDECREF(__pyx_v_Rbc);
  __Pyx_XDECREF(__pyx_v_Rb0);
  __Pyx_XDECREF(__pyx_v_Ybc);
  __Pyx_XDECREF(__pyx_v_Xbc);
  __Pyx_XDECREF(__pyx_v_BC);
  __Pyx_XDECREF(__pyx_v_Ynear);
  __Pyx_XDECREF(__pyx_v_Yfar);
  __Pyx_XDECREF(__pyx_v_Xnear);
  __Pyx_XDECREF(__pyx_v_Xfar);
  __Pyx_XDECREF(__pyx_v_Na);
  __Pyx_XDECREF(__pyx_v_Ta);
  __Pyx_XDECREF(__pyx_v_FPr0);
  __Pyx_XDECREF(__pyx_v_FPr1);
  __Pyx_XDECREF(__pyx_v_Ai);
  __Pyx_XDECREF(__pyx_v_Ao);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "torchsar/misc/sarplat.py":244
 * 
 *     @property
 *     def params(self):             # <<<<<<<<<<<<<<
 *         return self._params
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_13params(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_13params = {"params", (PyCFunction)__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_13params, METH_O, 0};
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_13params(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("params (wrapper)", 0);
  __pyx_r = __pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_12params(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_12params(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("params", 0);

  /* "torchsar/misc/sarplat.py":245
 *     @property
 *     def params(self):
 *         return self._params             # <<<<<<<<<<<<<<
 * 
 *     @params.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "torchsar/misc/sarplat.py":244
 * 
 *     @property
 *     def params(self):             # <<<<<<<<<<<<<<
 *         return self._params
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("torchsar.misc.sarplat.SarPlat.params", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "torchsar/misc/sarplat.py":248
 * 
 *     @params.setter
 *     def params(self, value):             # <<<<<<<<<<<<<<
 * 
 *         logging.info("---Check and set algorithm parameters...")
 */

/* Python wrapper */
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_15params(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_15params = {"params", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_15params, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_15params(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_value = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("params (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_value,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("params", 1, 2, 2, 1); __PYX_ERR(0, 248, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "params") < 0)) __PYX_ERR(0, 248, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_value = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("params", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 248, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("torchsar.misc.sarplat.SarPlat.params", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_14params(__pyx_self, __pyx_v_self, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_14params(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_k = NULL;
  PyObject *__pyx_v_v = NULL;
  PyObject *__pyx_v_GM = NULL;
  PyObject *__pyx_v_H = NULL;
  PyObject *__pyx_v_V = NULL;
  PyObject *__pyx_v_Vr = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_Vg = NULL;
  PyObject *__pyx_v_Vs = NULL;
  PyObject *__pyx_v_La = NULL;
  PyObject *__pyx_v_Fsa = NULL;
  PyObject *__pyx_v_Fsr = NULL;
  PyObject *__pyx_v_Tp = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_Fc = NULL;
  PyObject *__pyx_v_Wl = NULL;
  PyObject *__pyx_v_Kr = NULL;
  PyObject *__pyx_v_Rbc = NULL;
  PyObject *__pyx_v_Rb0 = NULL;
  PyObject *__pyx_v_Rsc = NULL;
  PyObject *__pyx_v_Rs0 = NULL;
  PyObject *__pyx_v_Rnear = NULL;
  PyObject *__pyx_v_Rfar = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_PC = NULL;
  PyObject *__pyx_v_SC = NULL;
  PyObject *__pyx_v_BC = NULL;
  PyObject *__pyx_v_SA = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_BA = NULL;
  PyObject *__pyx_v_ES = NULL;
  PyObject *__pyx_v_As = NULL;
  PyObject *__pyx_v_Ar = NULL;
  PyObject *__pyx_v_Ad = NULL;
  PyObject *__pyx_v_Aon = NULL;
  PyObject *__pyx_v_Aba = NULL;
  PyObject *__pyx_v_Abr = NULL;
  PyObject *__pyx_v_Ai = NULL;
  PyObject *__pyx_v_Xc = NULL;
  PyObject *__pyx_v_Yc = NULL;
  PyObject *__pyx_v_t0 = NULL;
  PyObject *__pyx_v_R0 = NULL;
  PyObject *__pyx_v_Lsar = NULL;
  PyObject *__pyx_v_Ka = NULL;
  PyObject *__pyx_v_Tsar = NULL;
  PyObject *__pyx_v_Ba = NULL;
  PyObject *__pyx_v_PRF = NULL;
  PyObject *__pyx_v_PRT = NULL;
  PyObject *__pyx_v_Tsa = NULL;
  PyObject *__pyx_v_Fs = NULL;
  PyObject *__pyx_v_Tsr = NULL;
  PyObject *__pyx_v_Na = NULL;
  PyObject *__pyx_v_Nr = NULL;
  PyObject *__pyx_v_Ta = NULL;
  PyObject *__pyx_v_ta = NULL;
  PyObject *__pyx_v_fa = NULL;
  PyObject *__pyx_v_tnear = NULL;
  PyObject *__pyx_v_tfar = NULL;
  PyObject *__pyx_v_tstart = NULL;
  PyObject *__pyx_v_tend = NULL;
  PyObject *__pyx_v_Tr = NULL;
  PyObject *__pyx_v_tr = NULL;
  PyObject *__pyx_v_fr = NULL;
  PyObject *__pyx_v_tac = NULL;
  PyObject *__pyx_v_fadc = NULL;
  PyObject *__pyx_v_Bdop = NULL;
  PyObject *__pyx_v_xmin = NULL;
  PyObject *__pyx_v_xmax = NULL;
  PyObject *__pyx_v_ymin = NULL;
  PyObject *__pyx_v_ymax = NULL;
  PyObject *__pyx_v_FPa = NULL;
  PyObject *__pyx_v_FPr = NULL;
  PyObject *__pyx_v_BWa = NULL;
  PyObject *__pyx_v_PRFmin = NULL;
  PyObject *__pyx_v_PRFmax = NULL;
  PyObject *__pyx_v_param = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  Py_UCS4 __pyx_t_11;
  PyObject *(*__pyx_t_12)(PyObject *);
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("params", 0);

  /* "torchsar/misc/sarplat.py":250
 *     def params(self, value):
 * 
 *         logging.info("---Check and set algorithm parameters...")             # <<<<<<<<<<<<<<
 * 
 *         if value is not None:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_logging); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_info); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_kp_u_Check_and_set_algorithm_paramet) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_Check_and_set_algorithm_paramet);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":252
 *         logging.info("---Check and set algorithm parameters...")
 * 
 *         if value is not None:             # <<<<<<<<<<<<<<
 *             if not isinstance(value, dict):
 *                 raise ValueError('params must be a dict!')
 */
  __pyx_t_4 = (__pyx_v_value != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "torchsar/misc/sarplat.py":253
 * 
 *         if value is not None:
 *             if not isinstance(value, dict):             # <<<<<<<<<<<<<<
 *                 raise ValueError('params must be a dict!')
 *             if hasattr(self, '_params'):
 */
    __pyx_t_5 = PyDict_Check(__pyx_v_value); 
    __pyx_t_4 = ((!(__pyx_t_5 != 0)) != 0);
    if (unlikely(__pyx_t_4)) {

      /* "torchsar/misc/sarplat.py":254
 *         if value is not None:
 *             if not isinstance(value, dict):
 *                 raise ValueError('params must be a dict!')             # <<<<<<<<<<<<<<
 *             if hasattr(self, '_params'):
 *                 for k, v in value.items():
 */
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 254, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 254, __pyx_L1_error)

      /* "torchsar/misc/sarplat.py":253
 * 
 *         if value is not None:
 *             if not isinstance(value, dict):             # <<<<<<<<<<<<<<
 *                 raise ValueError('params must be a dict!')
 *             if hasattr(self, '_params'):
 */
    }

    /* "torchsar/misc/sarplat.py":255
 *             if not isinstance(value, dict):
 *                 raise ValueError('params must be a dict!')
 *             if hasattr(self, '_params'):             # <<<<<<<<<<<<<<
 *                 for k, v in value.items():
 *                     self._params[k] = v
 */
    __pyx_t_4 = __Pyx_HasAttr(__pyx_v_self, __pyx_n_u_params); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 255, __pyx_L1_error)
    __pyx_t_5 = (__pyx_t_4 != 0);
    if (__pyx_t_5) {

      /* "torchsar/misc/sarplat.py":256
 *                 raise ValueError('params must be a dict!')
 *             if hasattr(self, '_params'):
 *                 for k, v in value.items():             # <<<<<<<<<<<<<<
 *                     self._params[k] = v
 *             else:
 */
      __pyx_t_6 = 0;
      if (unlikely(__pyx_v_value == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
        __PYX_ERR(0, 256, __pyx_L1_error)
      }
      __pyx_t_3 = __Pyx_dict_iterator(__pyx_v_value, 0, __pyx_n_s_items, (&__pyx_t_7), (&__pyx_t_8)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 256, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_XDECREF(__pyx_t_1);
      __pyx_t_1 = __pyx_t_3;
      __pyx_t_3 = 0;
      while (1) {
        __pyx_t_9 = __Pyx_dict_iter_next(__pyx_t_1, __pyx_t_7, &__pyx_t_6, &__pyx_t_3, &__pyx_t_2, NULL, __pyx_t_8);
        if (unlikely(__pyx_t_9 == 0)) break;
        if (unlikely(__pyx_t_9 == -1)) __PYX_ERR(0, 256, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_k, __pyx_t_3);
        __pyx_t_3 = 0;
        __Pyx_XDECREF_SET(__pyx_v_v, __pyx_t_2);
        __pyx_t_2 = 0;

        /* "torchsar/misc/sarplat.py":257
 *             if hasattr(self, '_params'):
 *                 for k, v in value.items():
 *                     self._params[k] = v             # <<<<<<<<<<<<<<
 *             else:
 *                 self._params = value
 */
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 257, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        if (unlikely(PyObject_SetItem(__pyx_t_2, __pyx_v_k, __pyx_v_v) < 0)) __PYX_ERR(0, 257, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "torchsar/misc/sarplat.py":255
 *             if not isinstance(value, dict):
 *                 raise ValueError('params must be a dict!')
 *             if hasattr(self, '_params'):             # <<<<<<<<<<<<<<
 *                 for k, v in value.items():
 *                     self._params[k] = v
 */
      goto __pyx_L5;
    }

    /* "torchsar/misc/sarplat.py":259
 *                     self._params[k] = v
 *             else:
 *                 self._params = value             # <<<<<<<<<<<<<<
 * 
 *             return self._params
 */
    /*else*/ {
      if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_params, __pyx_v_value) < 0) __PYX_ERR(0, 259, __pyx_L1_error)
    }
    __pyx_L5:;

    /* "torchsar/misc/sarplat.py":261
 *                 self._params = value
 * 
 *             return self._params             # <<<<<<<<<<<<<<
 * 
 *         GM = self._params['GeometryMode']
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 261, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "torchsar/misc/sarplat.py":252
 *         logging.info("---Check and set algorithm parameters...")
 * 
 *         if value is not None:             # <<<<<<<<<<<<<<
 *             if not isinstance(value, dict):
 *                 raise ValueError('params must be a dict!')
 */
  }

  /* "torchsar/misc/sarplat.py":263
 *             return self._params
 * 
 *         GM = self._params['GeometryMode']             # <<<<<<<<<<<<<<
 *         if not isinstance(self._sensor, dict):
 *             raise ValueError('sensor must be a dict!')
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_GeometryMode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_GM = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":264
 * 
 *         GM = self._params['GeometryMode']
 *         if not isinstance(self._sensor, dict):             # <<<<<<<<<<<<<<
 *             raise ValueError('sensor must be a dict!')
 *         if not isinstance(self._acquisition, dict):
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyDict_Check(__pyx_t_2); 
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = ((!(__pyx_t_5 != 0)) != 0);
  if (unlikely(__pyx_t_4)) {

    /* "torchsar/misc/sarplat.py":265
 *         GM = self._params['GeometryMode']
 *         if not isinstance(self._sensor, dict):
 *             raise ValueError('sensor must be a dict!')             # <<<<<<<<<<<<<<
 *         if not isinstance(self._acquisition, dict):
 *             raise ValueError('acquisition must be a dict!')
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 265, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 265, __pyx_L1_error)

    /* "torchsar/misc/sarplat.py":264
 * 
 *         GM = self._params['GeometryMode']
 *         if not isinstance(self._sensor, dict):             # <<<<<<<<<<<<<<
 *             raise ValueError('sensor must be a dict!')
 *         if not isinstance(self._acquisition, dict):
 */
  }

  /* "torchsar/misc/sarplat.py":266
 *         if not isinstance(self._sensor, dict):
 *             raise ValueError('sensor must be a dict!')
 *         if not isinstance(self._acquisition, dict):             # <<<<<<<<<<<<<<
 *             raise ValueError('acquisition must be a dict!')
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyDict_Check(__pyx_t_2); 
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = ((!(__pyx_t_4 != 0)) != 0);
  if (unlikely(__pyx_t_5)) {

    /* "torchsar/misc/sarplat.py":267
 *             raise ValueError('sensor must be a dict!')
 *         if not isinstance(self._acquisition, dict):
 *             raise ValueError('acquisition must be a dict!')             # <<<<<<<<<<<<<<
 * 
 *         H = self._sensor['H']
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 267, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 267, __pyx_L1_error)

    /* "torchsar/misc/sarplat.py":266
 *         if not isinstance(self._sensor, dict):
 *             raise ValueError('sensor must be a dict!')
 *         if not isinstance(self._acquisition, dict):             # <<<<<<<<<<<<<<
 *             raise ValueError('acquisition must be a dict!')
 * 
 */
  }

  /* "torchsar/misc/sarplat.py":269
 *             raise ValueError('acquisition must be a dict!')
 * 
 *         H = self._sensor['H']             # <<<<<<<<<<<<<<
 *         V = self._sensor['V']
 *         Vr = self._sensor['Vr']
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_H); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_H = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":270
 * 
 *         H = self._sensor['H']
 *         V = self._sensor['V']             # <<<<<<<<<<<<<<
 *         Vr = self._sensor['Vr']
 *         Vg = self._sensor['Vg']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_V); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_V = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":271
 *         H = self._sensor['H']
 *         V = self._sensor['V']
 *         Vr = self._sensor['Vr']             # <<<<<<<<<<<<<<
 *         Vg = self._sensor['Vg']
 *         Vs = self._sensor['Vs']
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 271, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_Vr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 271, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_Vr = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":272
 *         V = self._sensor['V']
 *         Vr = self._sensor['Vr']
 *         Vg = self._sensor['Vg']             # <<<<<<<<<<<<<<
 *         Vs = self._sensor['Vs']
 *         La = self._sensor['La']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Vg); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 272, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Vg = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":273
 *         Vr = self._sensor['Vr']
 *         Vg = self._sensor['Vg']
 *         Vs = self._sensor['Vs']             # <<<<<<<<<<<<<<
 *         La = self._sensor['La']
 *         Fsa = self._sensor['PRF']  # sampling rate in Azimuth dimension
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_Vs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 273, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_Vs = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":274
 *         Vg = self._sensor['Vg']
 *         Vs = self._sensor['Vs']
 *         La = self._sensor['La']             # <<<<<<<<<<<<<<
 *         Fsa = self._sensor['PRF']  # sampling rate in Azimuth dimension
 *         Fsr = self._sensor['Fs']  # sampling rate in range dimension
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 274, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_La); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 274, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_La = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":275
 *         Vs = self._sensor['Vs']
 *         La = self._sensor['La']
 *         Fsa = self._sensor['PRF']  # sampling rate in Azimuth dimension             # <<<<<<<<<<<<<<
 *         Fsr = self._sensor['Fs']  # sampling rate in range dimension
 *         Tp = self._sensor['Tp']  # Pulse repetition period
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_PRF); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_Fsa = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":276
 *         La = self._sensor['La']
 *         Fsa = self._sensor['PRF']  # sampling rate in Azimuth dimension
 *         Fsr = self._sensor['Fs']  # sampling rate in range dimension             # <<<<<<<<<<<<<<
 *         Tp = self._sensor['Tp']  # Pulse repetition period
 *         Fc = self._sensor['Fc']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Fs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Fsr = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":277
 *         Fsa = self._sensor['PRF']  # sampling rate in Azimuth dimension
 *         Fsr = self._sensor['Fs']  # sampling rate in range dimension
 *         Tp = self._sensor['Tp']  # Pulse repetition period             # <<<<<<<<<<<<<<
 *         Fc = self._sensor['Fc']
 *         Wl = self._sensor['Wl']
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 277, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_Tp); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 277, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_Tp = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":278
 *         Fsr = self._sensor['Fs']  # sampling rate in range dimension
 *         Tp = self._sensor['Tp']  # Pulse repetition period
 *         Fc = self._sensor['Fc']             # <<<<<<<<<<<<<<
 *         Wl = self._sensor['Wl']
 *         Kr = self._sensor['Kr']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 278, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Fc); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 278, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Fc = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":279
 *         Tp = self._sensor['Tp']  # Pulse repetition period
 *         Fc = self._sensor['Fc']
 *         Wl = self._sensor['Wl']             # <<<<<<<<<<<<<<
 *         Kr = self._sensor['Kr']
 *         La = self._sensor['La']
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 279, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_Wl); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 279, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_Wl = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":280
 *         Fc = self._sensor['Fc']
 *         Wl = self._sensor['Wl']
 *         Kr = self._sensor['Kr']             # <<<<<<<<<<<<<<
 *         La = self._sensor['La']
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 280, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Kr); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 280, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Kr = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":281
 *         Wl = self._sensor['Wl']
 *         Kr = self._sensor['Kr']
 *         La = self._sensor['La']             # <<<<<<<<<<<<<<
 * 
 *         Rbc = self._acquisition['Rbc']
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_La); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_La, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":283
 *         La = self._sensor['La']
 * 
 *         Rbc = self._acquisition['Rbc']             # <<<<<<<<<<<<<<
 *         Rb0 = self._acquisition['Rb0']
 *         Rsc = self._acquisition['Rsc']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Rbc); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Rbc = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":284
 * 
 *         Rbc = self._acquisition['Rbc']
 *         Rb0 = self._acquisition['Rb0']             # <<<<<<<<<<<<<<
 *         Rsc = self._acquisition['Rsc']
 *         Rs0 = self._acquisition['Rs0']
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 284, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_Rb0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 284, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_Rb0 = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":285
 *         Rbc = self._acquisition['Rbc']
 *         Rb0 = self._acquisition['Rb0']
 *         Rsc = self._acquisition['Rsc']             # <<<<<<<<<<<<<<
 *         Rs0 = self._acquisition['Rs0']
 *         Rbc = self._acquisition['Rbc']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Rsc); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Rsc = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":286
 *         Rb0 = self._acquisition['Rb0']
 *         Rsc = self._acquisition['Rsc']
 *         Rs0 = self._acquisition['Rs0']             # <<<<<<<<<<<<<<
 *         Rbc = self._acquisition['Rbc']
 *         Rsc = self._acquisition['Rsc']
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 286, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_Rs0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 286, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_Rs0 = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":287
 *         Rsc = self._acquisition['Rsc']
 *         Rs0 = self._acquisition['Rs0']
 *         Rbc = self._acquisition['Rbc']             # <<<<<<<<<<<<<<
 *         Rsc = self._acquisition['Rsc']
 *         Rnear = self._acquisition['Rnear']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 287, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Rbc); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 287, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF_SET(__pyx_v_Rbc, __pyx_t_2);
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":288
 *         Rs0 = self._acquisition['Rs0']
 *         Rbc = self._acquisition['Rbc']
 *         Rsc = self._acquisition['Rsc']             # <<<<<<<<<<<<<<
 *         Rnear = self._acquisition['Rnear']
 *         Rfar = self._acquisition['Rfar']
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 288, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_Rsc); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 288, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_Rsc, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":289
 *         Rbc = self._acquisition['Rbc']
 *         Rsc = self._acquisition['Rsc']
 *         Rnear = self._acquisition['Rnear']             # <<<<<<<<<<<<<<
 *         Rfar = self._acquisition['Rfar']
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 289, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Rnear); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 289, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Rnear = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":290
 *         Rsc = self._acquisition['Rsc']
 *         Rnear = self._acquisition['Rnear']
 *         Rfar = self._acquisition['Rfar']             # <<<<<<<<<<<<<<
 * 
 *         PC = self._acquisition['PlatCenter']
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 290, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_Rfar); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 290, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_Rfar = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":292
 *         Rfar = self._acquisition['Rfar']
 * 
 *         PC = self._acquisition['PlatCenter']             # <<<<<<<<<<<<<<
 *         SC = self._acquisition['SceneCenter']
 *         BC = self._acquisition['BeamCenter']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 292, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_PlatCenter); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 292, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_PC = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":293
 * 
 *         PC = self._acquisition['PlatCenter']
 *         SC = self._acquisition['SceneCenter']             # <<<<<<<<<<<<<<
 *         BC = self._acquisition['BeamCenter']
 *         SA = self._acquisition['SceneArea']
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 293, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_SceneCenter); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 293, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_SC = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":294
 *         PC = self._acquisition['PlatCenter']
 *         SC = self._acquisition['SceneCenter']
 *         BC = self._acquisition['BeamCenter']             # <<<<<<<<<<<<<<
 *         SA = self._acquisition['SceneArea']
 *         BA = self._acquisition['BeamArea']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_BeamCenter); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 294, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_BC = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":295
 *         SC = self._acquisition['SceneCenter']
 *         BC = self._acquisition['BeamCenter']
 *         SA = self._acquisition['SceneArea']             # <<<<<<<<<<<<<<
 *         BA = self._acquisition['BeamArea']
 *         ES = self._acquisition['EchoSize']
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 295, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_SceneArea); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 295, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_SA = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":296
 *         BC = self._acquisition['BeamCenter']
 *         SA = self._acquisition['SceneArea']
 *         BA = self._acquisition['BeamArea']             # <<<<<<<<<<<<<<
 *         ES = self._acquisition['EchoSize']
 *         As = self._acquisition['As']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 296, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_BeamArea); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 296, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_BA = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":297
 *         SA = self._acquisition['SceneArea']
 *         BA = self._acquisition['BeamArea']
 *         ES = self._acquisition['EchoSize']             # <<<<<<<<<<<<<<
 *         As = self._acquisition['As']
 *         Ar = self._acquisition['Ar']
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_EchoSize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_ES = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":298
 *         BA = self._acquisition['BeamArea']
 *         ES = self._acquisition['EchoSize']
 *         As = self._acquisition['As']             # <<<<<<<<<<<<<<
 *         Ar = self._acquisition['Ar']
 *         Ad = self._acquisition['Ad']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_As); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_As = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":299
 *         ES = self._acquisition['EchoSize']
 *         As = self._acquisition['As']
 *         Ar = self._acquisition['Ar']             # <<<<<<<<<<<<<<
 *         Ad = self._acquisition['Ad']
 *         Aon = self._acquisition['Aon']
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 299, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_Ar); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 299, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_Ar = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":300
 *         As = self._acquisition['As']
 *         Ar = self._acquisition['Ar']
 *         Ad = self._acquisition['Ad']             # <<<<<<<<<<<<<<
 *         Aon = self._acquisition['Aon']
 *         Aba = self._acquisition['Aba']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 300, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Ad); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 300, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Ad = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":301
 *         Ar = self._acquisition['Ar']
 *         Ad = self._acquisition['Ad']
 *         Aon = self._acquisition['Aon']             # <<<<<<<<<<<<<<
 *         Aba = self._acquisition['Aba']
 *         Abr = self._acquisition['Abr']
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 301, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_Aon); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 301, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_Aon = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":302
 *         Ad = self._acquisition['Ad']
 *         Aon = self._acquisition['Aon']
 *         Aba = self._acquisition['Aba']             # <<<<<<<<<<<<<<
 *         Abr = self._acquisition['Abr']
 *         Ai = self._acquisition['Ai']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 302, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Aba); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 302, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Aba = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":303
 *         Aon = self._acquisition['Aon']
 *         Aba = self._acquisition['Aba']
 *         Abr = self._acquisition['Abr']             # <<<<<<<<<<<<<<
 *         Ai = self._acquisition['Ai']
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 303, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_Abr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 303, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_Abr = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":304
 *         Aba = self._acquisition['Aba']
 *         Abr = self._acquisition['Abr']
 *         Ai = self._acquisition['Ai']             # <<<<<<<<<<<<<<
 * 
 *         if GM == 'BG':
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Ai); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Ai = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":306
 *         Ai = self._acquisition['Ai']
 * 
 *         if GM == 'BG':             # <<<<<<<<<<<<<<
 *             Xc = BC[0]
 *             Yc = BC[1]
 */
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_GM, __pyx_n_u_BG, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 306, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "torchsar/misc/sarplat.py":307
 * 
 *         if GM == 'BG':
 *             Xc = BC[0]             # <<<<<<<<<<<<<<
 *             Yc = BC[1]
 *             t0 = 2 * Rbc / C
 */
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_BC, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_Xc = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "torchsar/misc/sarplat.py":308
 *         if GM == 'BG':
 *             Xc = BC[0]
 *             Yc = BC[1]             # <<<<<<<<<<<<<<
 *             t0 = 2 * Rbc / C
 *             R0 = Rb0
 */
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_BC, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 308, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_Yc = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "torchsar/misc/sarplat.py":309
 *             Xc = BC[0]
 *             Yc = BC[1]
 *             t0 = 2 * Rbc / C             # <<<<<<<<<<<<<<
 *             R0 = Rb0
 *         if GM == 'SG':
 */
    __pyx_t_2 = PyNumber_Multiply(__pyx_int_2, __pyx_v_Rbc); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 309, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_C); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 309, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyNumber_Divide(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 309, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_t0 = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "torchsar/misc/sarplat.py":310
 *             Yc = BC[1]
 *             t0 = 2 * Rbc / C
 *             R0 = Rb0             # <<<<<<<<<<<<<<
 *         if GM == 'SG':
 *             t0 = 2 * Rsc / C
 */
    __Pyx_INCREF(__pyx_v_Rb0);
    __pyx_v_R0 = __pyx_v_Rb0;

    /* "torchsar/misc/sarplat.py":306
 *         Ai = self._acquisition['Ai']
 * 
 *         if GM == 'BG':             # <<<<<<<<<<<<<<
 *             Xc = BC[0]
 *             Yc = BC[1]
 */
  }

  /* "torchsar/misc/sarplat.py":311
 *             t0 = 2 * Rbc / C
 *             R0 = Rb0
 *         if GM == 'SG':             # <<<<<<<<<<<<<<
 *             t0 = 2 * Rsc / C
 *             Xc = SC[0]
 */
  __pyx_t_5 = (__Pyx_PyUnicode_Equals(__pyx_v_GM, __pyx_n_u_SG, Py_EQ)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 311, __pyx_L1_error)
  if (__pyx_t_5) {

    /* "torchsar/misc/sarplat.py":312
 *             R0 = Rb0
 *         if GM == 'SG':
 *             t0 = 2 * Rsc / C             # <<<<<<<<<<<<<<
 *             Xc = SC[0]
 *             Yc = SC[1]
 */
    __pyx_t_3 = PyNumber_Multiply(__pyx_int_2, __pyx_v_Rsc); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 312, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_C); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 312, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyNumber_Divide(__pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 312, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF_SET(__pyx_v_t0, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "torchsar/misc/sarplat.py":313
 *         if GM == 'SG':
 *             t0 = 2 * Rsc / C
 *             Xc = SC[0]             # <<<<<<<<<<<<<<
 *             Yc = SC[1]
 *             R0 = Rs0
 */
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_SC, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 313, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_Xc, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "torchsar/misc/sarplat.py":314
 *             t0 = 2 * Rsc / C
 *             Xc = SC[0]
 *             Yc = SC[1]             # <<<<<<<<<<<<<<
 *             R0 = Rs0
 * 
 */
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_SC, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 314, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_Yc, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "torchsar/misc/sarplat.py":315
 *             Xc = SC[0]
 *             Yc = SC[1]
 *             R0 = Rs0             # <<<<<<<<<<<<<<
 * 
 *         Lsar = self._sensor['Wl'] * Rnear / La  # Rb0, Rbc, R?
 */
    __Pyx_INCREF(__pyx_v_Rs0);
    __Pyx_XDECREF_SET(__pyx_v_R0, __pyx_v_Rs0);

    /* "torchsar/misc/sarplat.py":311
 *             t0 = 2 * Rbc / C
 *             R0 = Rb0
 *         if GM == 'SG':             # <<<<<<<<<<<<<<
 *             t0 = 2 * Rsc / C
 *             Xc = SC[0]
 */
  }

  /* "torchsar/misc/sarplat.py":317
 *             R0 = Rs0
 * 
 *         Lsar = self._sensor['Wl'] * Rnear / La  # Rb0, Rbc, R?             # <<<<<<<<<<<<<<
 *         # Ka = (2.0 * Vr**2) / (Wl * Rs0)
 *         Ka = (-2.0 * Vr**2 * np.cos(Ar)**3) / (Wl * Rnear)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_Wl); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyNumber_Multiply(__pyx_t_1, __pyx_v_Rnear); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_t_2, __pyx_v_La); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_Lsar = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":319
 *         Lsar = self._sensor['Wl'] * Rnear / La  # Rb0, Rbc, R?
 *         # Ka = (2.0 * Vr**2) / (Wl * Rs0)
 *         Ka = (-2.0 * Vr**2 * np.cos(Ar)**3) / (Wl * Rnear)             # <<<<<<<<<<<<<<
 *         Tsar = Lsar / Vr
 *         Ba = np.abs(Ka * Tsar)
 */
  __pyx_t_1 = PyNumber_Power(__pyx_v_Vr, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyNumber_Multiply(__pyx_float_neg_2_0, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_cos); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_10);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_10, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_3, __pyx_v_Ar) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_v_Ar);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyNumber_Power(__pyx_t_1, __pyx_int_3, Py_None); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Multiply(__pyx_t_2, __pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyNumber_Multiply(__pyx_v_Wl, __pyx_v_Rnear); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_2 = __Pyx_PyNumber_Divide(__pyx_t_1, __pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_v_Ka = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":320
 *         # Ka = (2.0 * Vr**2) / (Wl * Rs0)
 *         Ka = (-2.0 * Vr**2 * np.cos(Ar)**3) / (Wl * Rnear)
 *         Tsar = Lsar / Vr             # <<<<<<<<<<<<<<
 *         Ba = np.abs(Ka * Tsar)
 * 
 */
  __pyx_t_2 = __Pyx_PyNumber_Divide(__pyx_v_Lsar, __pyx_v_Vr); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 320, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_Tsar = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":321
 *         Ka = (-2.0 * Vr**2 * np.cos(Ar)**3) / (Wl * Rnear)
 *         Tsar = Lsar / Vr
 *         Ba = np.abs(Ka * Tsar)             # <<<<<<<<<<<<<<
 * 
 *         if self._sensor['PRF'] is None:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_abs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyNumber_Multiply(__pyx_v_Ka, __pyx_v_Tsar); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_3, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_10);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 321, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Ba = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":323
 *         Ba = np.abs(Ka * Tsar)
 * 
 *         if self._sensor['PRF'] is None:             # <<<<<<<<<<<<<<
 *             PRF = (1.25 * Ba)
 *         else:
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_n_u_PRF); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = (__pyx_t_1 == Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = (__pyx_t_5 != 0);
  if (__pyx_t_4) {

    /* "torchsar/misc/sarplat.py":324
 * 
 *         if self._sensor['PRF'] is None:
 *             PRF = (1.25 * Ba)             # <<<<<<<<<<<<<<
 *         else:
 *             PRF = self._sensor['PRF']
 */
    __pyx_t_1 = PyNumber_Multiply(__pyx_float_1_25, __pyx_v_Ba); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 324, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_PRF = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "torchsar/misc/sarplat.py":323
 *         Ba = np.abs(Ka * Tsar)
 * 
 *         if self._sensor['PRF'] is None:             # <<<<<<<<<<<<<<
 *             PRF = (1.25 * Ba)
 *         else:
 */
    goto __pyx_L12;
  }

  /* "torchsar/misc/sarplat.py":326
 *             PRF = (1.25 * Ba)
 *         else:
 *             PRF = self._sensor['PRF']             # <<<<<<<<<<<<<<
 * 
 *         Fsa = PRF
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 326, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_PRF); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 326, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_PRF = __pyx_t_2;
    __pyx_t_2 = 0;
  }
  __pyx_L12:;

  /* "torchsar/misc/sarplat.py":328
 *             PRF = self._sensor['PRF']
 * 
 *         Fsa = PRF             # <<<<<<<<<<<<<<
 *         if Fsa / Ba < 1.1 or Fsa / Ba > 2.0:
 *             print(Fsa, Ba)
 */
  __Pyx_INCREF(__pyx_v_PRF);
  __Pyx_DECREF_SET(__pyx_v_Fsa, __pyx_v_PRF);

  /* "torchsar/misc/sarplat.py":329
 * 
 *         Fsa = PRF
 *         if Fsa / Ba < 1.1 or Fsa / Ba > 2.0:             # <<<<<<<<<<<<<<
 *             print(Fsa, Ba)
 *             logging.warning(
 */
  __pyx_t_2 = __Pyx_PyNumber_Divide(__pyx_v_Fsa, __pyx_v_Ba); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 329, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_2, __pyx_float_1_1, Py_LT); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 329, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 329, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!__pyx_t_5) {
  } else {
    __pyx_t_4 = __pyx_t_5;
    goto __pyx_L14_bool_binop_done;
  }
  __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_v_Fsa, __pyx_v_Ba); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 329, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_float_2_0, Py_GT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 329, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 329, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __pyx_t_5;
  __pyx_L14_bool_binop_done:;
  if (__pyx_t_4) {

    /* "torchsar/misc/sarplat.py":330
 *         Fsa = PRF
 *         if Fsa / Ba < 1.1 or Fsa / Ba > 2.0:
 *             print(Fsa, Ba)             # <<<<<<<<<<<<<<
 *             logging.warning(
 *                 "~~~Sampling rate should be in range [1.1, 2.0]*Ba, \r\n   Fsa %f Ba %f!" % (Fsa, Ba))
 */
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 330, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_Fsa);
    __Pyx_GIVEREF(__pyx_v_Fsa);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_Fsa);
    __Pyx_INCREF(__pyx_v_Ba);
    __Pyx_GIVEREF(__pyx_v_Ba);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_Ba);
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 330, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "torchsar/misc/sarplat.py":331
 *         if Fsa / Ba < 1.1 or Fsa / Ba > 2.0:
 *             print(Fsa, Ba)
 *             logging.warning(             # <<<<<<<<<<<<<<
 *                 "~~~Sampling rate should be in range [1.1, 2.0]*Ba, \r\n   Fsa %f Ba %f!" % (Fsa, Ba))
 * 
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_logging); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_warning); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "torchsar/misc/sarplat.py":332
 *             print(Fsa, Ba)
 *             logging.warning(
 *                 "~~~Sampling rate should be in range [1.1, 2.0]*Ba, \r\n   Fsa %f Ba %f!" % (Fsa, Ba))             # <<<<<<<<<<<<<<
 * 
 *         PRT = 1.0 / PRF  # azimuth sampling
 */
    __pyx_t_2 = PyTuple_New(5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 332, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = 0;
    __pyx_t_11 = 127;
    __Pyx_INCREF(__pyx_kp_u_Sampling_rate_should_be_in_rang);
    __pyx_t_7 += 60;
    __Pyx_GIVEREF(__pyx_kp_u_Sampling_rate_should_be_in_rang);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_kp_u_Sampling_rate_should_be_in_rang);
    __pyx_t_3 = __Pyx_PyObject_Format(__pyx_v_Fsa, __pyx_n_u_f); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 332, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_3 = 0;
    __Pyx_INCREF(__pyx_kp_u_Ba);
    __pyx_t_7 += 4;
    __Pyx_GIVEREF(__pyx_kp_u_Ba);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_kp_u_Ba);
    __pyx_t_3 = __Pyx_PyObject_Format(__pyx_v_Ba, __pyx_n_u_f); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 332, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_3);
    __pyx_t_3 = 0;
    __Pyx_INCREF(__pyx_kp_u__5);
    __pyx_t_7 += 1;
    __Pyx_GIVEREF(__pyx_kp_u__5);
    PyTuple_SET_ITEM(__pyx_t_2, 4, __pyx_kp_u__5);
    __pyx_t_3 = __Pyx_PyUnicode_Join(__pyx_t_2, 5, __pyx_t_7, __pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 332, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_10);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_10, function);
      }
    }
    __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_3);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "torchsar/misc/sarplat.py":329
 * 
 *         Fsa = PRF
 *         if Fsa / Ba < 1.1 or Fsa / Ba > 2.0:             # <<<<<<<<<<<<<<
 *             print(Fsa, Ba)
 *             logging.warning(
 */
  }

  /* "torchsar/misc/sarplat.py":334
 *                 "~~~Sampling rate should be in range [1.1, 2.0]*Ba, \r\n   Fsa %f Ba %f!" % (Fsa, Ba))
 * 
 *         PRT = 1.0 / PRF  # azimuth sampling             # <<<<<<<<<<<<<<
 *         Tsa = 1.0 / PRF
 * 
 */
  __pyx_t_1 = __Pyx_PyFloat_TrueDivideCObj(__pyx_float_1_0, __pyx_v_PRF, 1.0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 334, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_PRT = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":335
 * 
 *         PRT = 1.0 / PRF  # azimuth sampling
 *         Tsa = 1.0 / PRF             # <<<<<<<<<<<<<<
 * 
 *         if self._sensor['Fs'] is None:
 */
  __pyx_t_1 = __Pyx_PyFloat_TrueDivideCObj(__pyx_float_1_0, __pyx_v_PRF, 1.0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 335, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_Tsa = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":337
 *         Tsa = 1.0 / PRF
 * 
 *         if self._sensor['Fs'] is None:             # <<<<<<<<<<<<<<
 *             Fs = (1.905 * self._sensor['Br'])
 *         else:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_10 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Fs); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 337, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = (__pyx_t_10 == Py_None);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "torchsar/misc/sarplat.py":338
 * 
 *         if self._sensor['Fs'] is None:
 *             Fs = (1.905 * self._sensor['Br'])             # <<<<<<<<<<<<<<
 *         else:
 *             Fs = self._sensor['Fs']
 */
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 338, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_10, __pyx_n_u_Br); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 338, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = PyNumber_Multiply(__pyx_float_1_905, __pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 338, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_Fs = __pyx_t_10;
    __pyx_t_10 = 0;

    /* "torchsar/misc/sarplat.py":337
 *         Tsa = 1.0 / PRF
 * 
 *         if self._sensor['Fs'] is None:             # <<<<<<<<<<<<<<
 *             Fs = (1.905 * self._sensor['Br'])
 *         else:
 */
    goto __pyx_L16;
  }

  /* "torchsar/misc/sarplat.py":340
 *             Fs = (1.905 * self._sensor['Br'])
 *         else:
 *             Fs = self._sensor['Fs']             # <<<<<<<<<<<<<<
 * 
 *         Fsr = Fs
 */
  /*else*/ {
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 340, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_10, __pyx_n_u_Fs); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 340, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_v_Fs = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __pyx_L16:;

  /* "torchsar/misc/sarplat.py":342
 *             Fs = self._sensor['Fs']
 * 
 *         Fsr = Fs             # <<<<<<<<<<<<<<
 *         Tsr = 1.0 / Fsr
 * 
 */
  __Pyx_INCREF(__pyx_v_Fs);
  __Pyx_DECREF_SET(__pyx_v_Fsr, __pyx_v_Fs);

  /* "torchsar/misc/sarplat.py":343
 * 
 *         Fsr = Fs
 *         Tsr = 1.0 / Fsr             # <<<<<<<<<<<<<<
 * 
 *         Na, Nr = ES
 */
  __pyx_t_1 = __Pyx_PyFloat_TrueDivideCObj(__pyx_float_1_0, __pyx_v_Fsr, 1.0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 343, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_Tsr = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":345
 *         Tsr = 1.0 / Fsr
 * 
 *         Na, Nr = ES             # <<<<<<<<<<<<<<
 * 
 *         Ta = Na * 1.0 / Fsa
 */
  if ((likely(PyTuple_CheckExact(__pyx_v_ES))) || (PyList_CheckExact(__pyx_v_ES))) {
    PyObject* sequence = __pyx_v_ES;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 345, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_10 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_10 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_10);
    #else
    __pyx_t_1 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 345, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 345, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    #endif
  } else {
    Py_ssize_t index = -1;
    __pyx_t_3 = PyObject_GetIter(__pyx_v_ES); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 345, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_12 = Py_TYPE(__pyx_t_3)->tp_iternext;
    index = 0; __pyx_t_1 = __pyx_t_12(__pyx_t_3); if (unlikely(!__pyx_t_1)) goto __pyx_L17_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    index = 1; __pyx_t_10 = __pyx_t_12(__pyx_t_3); if (unlikely(!__pyx_t_10)) goto __pyx_L17_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_10);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_12(__pyx_t_3), 2) < 0) __PYX_ERR(0, 345, __pyx_L1_error)
    __pyx_t_12 = NULL;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L18_unpacking_done;
    __pyx_L17_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_12 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 345, __pyx_L1_error)
    __pyx_L18_unpacking_done:;
  }
  __pyx_v_Na = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_v_Nr = __pyx_t_10;
  __pyx_t_10 = 0;

  /* "torchsar/misc/sarplat.py":347
 *         Na, Nr = ES
 * 
 *         Ta = Na * 1.0 / Fsa             # <<<<<<<<<<<<<<
 *         ta = np.linspace(-Ta / 2.0, Ta / 2.0, Na)
 *         fa = np.linspace(-Fsa / 2.0, Fsa / 2.0, Na)
 */
  __pyx_t_10 = PyNumber_Multiply(__pyx_v_Na, __pyx_float_1_0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 347, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_t_10, __pyx_v_Fsa); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 347, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_v_Ta = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":348
 * 
 *         Ta = Na * 1.0 / Fsa
 *         ta = np.linspace(-Ta / 2.0, Ta / 2.0, Na)             # <<<<<<<<<<<<<<
 *         fa = np.linspace(-Fsa / 2.0, Fsa / 2.0, Na)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_linspace); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyNumber_Negative(__pyx_v_Ta); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_2 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_10, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyFloat_TrueDivideObjC(__pyx_v_Ta, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_13 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_13)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_13);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_t_2, __pyx_t_10, __pyx_v_Na};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 348, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[4] = {__pyx_t_13, __pyx_t_2, __pyx_t_10, __pyx_v_Na};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 348, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  } else
  #endif
  {
    __pyx_t_14 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 348, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    if (__pyx_t_13) {
      __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_13); __pyx_t_13 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_14, 0+__pyx_t_8, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_10);
    PyTuple_SET_ITEM(__pyx_t_14, 1+__pyx_t_8, __pyx_t_10);
    __Pyx_INCREF(__pyx_v_Na);
    __Pyx_GIVEREF(__pyx_v_Na);
    PyTuple_SET_ITEM(__pyx_t_14, 2+__pyx_t_8, __pyx_v_Na);
    __pyx_t_2 = 0;
    __pyx_t_10 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_14, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 348, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_ta = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":349
 *         Ta = Na * 1.0 / Fsa
 *         ta = np.linspace(-Ta / 2.0, Ta / 2.0, Na)
 *         fa = np.linspace(-Fsa / 2.0, Fsa / 2.0, Na)             # <<<<<<<<<<<<<<
 * 
 *         tnear = Rnear / C
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_linspace); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Negative(__pyx_v_Fsa); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_10 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_3, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyFloat_TrueDivideObjC(__pyx_v_Fsa, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 349, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_14))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_14);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_14, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_14)) {
    PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_t_10, __pyx_t_3, __pyx_v_Na};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_14, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 349, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_14)) {
    PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_t_10, __pyx_t_3, __pyx_v_Na};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_14, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 349, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_13 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 349, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    if (__pyx_t_2) {
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_2); __pyx_t_2 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_10);
    PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_8, __pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_8, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_Na);
    __Pyx_GIVEREF(__pyx_v_Na);
    PyTuple_SET_ITEM(__pyx_t_13, 2+__pyx_t_8, __pyx_v_Na);
    __pyx_t_10 = 0;
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_14, __pyx_t_13, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 349, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_v_fa = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":351
 *         fa = np.linspace(-Fsa / 2.0, Fsa / 2.0, Na)
 * 
 *         tnear = Rnear / C             # <<<<<<<<<<<<<<
 *         tfar = Rfar / C
 *         tstart = 2.0 * tnear
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_C); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 351, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_14 = __Pyx_PyNumber_Divide(__pyx_v_Rnear, __pyx_t_1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 351, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_tnear = __pyx_t_14;
  __pyx_t_14 = 0;

  /* "torchsar/misc/sarplat.py":352
 * 
 *         tnear = Rnear / C
 *         tfar = Rfar / C             # <<<<<<<<<<<<<<
 *         tstart = 2.0 * tnear
 *         # tend = 2.0 * tfar + Tp
 */
  __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_n_s_C); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 352, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_v_Rfar, __pyx_t_14); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 352, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_v_tfar = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":353
 *         tnear = Rnear / C
 *         tfar = Rfar / C
 *         tstart = 2.0 * tnear             # <<<<<<<<<<<<<<
 *         # tend = 2.0 * tfar + Tp
 *         tend = 2.0 * tfar
 */
  __pyx_t_1 = PyNumber_Multiply(__pyx_float_2_0, __pyx_v_tnear); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 353, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_tstart = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":355
 *         tstart = 2.0 * tnear
 *         # tend = 2.0 * tfar + Tp
 *         tend = 2.0 * tfar             # <<<<<<<<<<<<<<
 * 
 *         Tr = tend - tstart
 */
  __pyx_t_1 = PyNumber_Multiply(__pyx_float_2_0, __pyx_v_tfar); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 355, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_tend = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":357
 *         tend = 2.0 * tfar
 * 
 *         Tr = tend - tstart             # <<<<<<<<<<<<<<
 *         if abs(Nr - Tr * Fsr) > 1:
 *             Abr = compute_range_beamwidth2(Nr, Fsr, H, Aon)
 */
  __pyx_t_1 = PyNumber_Subtract(__pyx_v_tend, __pyx_v_tstart); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 357, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_Tr = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":358
 * 
 *         Tr = tend - tstart
 *         if abs(Nr - Tr * Fsr) > 1:             # <<<<<<<<<<<<<<
 *             Abr = compute_range_beamwidth2(Nr, Fsr, H, Aon)
 *             logging.error("~~~Nr should be TrFsr= % f% f= % f\r\n\
 */
  __pyx_t_1 = PyNumber_Multiply(__pyx_v_Tr, __pyx_v_Fsr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_14 = PyNumber_Subtract(__pyx_v_Nr, __pyx_t_1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyNumber_Absolute(__pyx_t_14); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_14 = PyObject_RichCompare(__pyx_t_1, __pyx_int_1, Py_GT); __Pyx_XGOTREF(__pyx_t_14); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_14); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  if (unlikely(__pyx_t_5)) {

    /* "torchsar/misc/sarplat.py":359
 *         Tr = tend - tstart
 *         if abs(Nr - Tr * Fsr) > 1:
 *             Abr = compute_range_beamwidth2(Nr, Fsr, H, Aon)             # <<<<<<<<<<<<<<
 *             logging.error("~~~Nr should be TrFsr= % f% f= % f\r\n\
 *                           or Fsr should be NrTr= % f\\% f= % f\r\n\
 */
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_compute_range_beamwidth2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 359, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_13 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_13)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_13);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
        __pyx_t_8 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[5] = {__pyx_t_13, __pyx_v_Nr, __pyx_v_Fsr, __pyx_v_H, __pyx_v_Aon};
      __pyx_t_14 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_8, 4+__pyx_t_8); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 359, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_GOTREF(__pyx_t_14);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
      PyObject *__pyx_temp[5] = {__pyx_t_13, __pyx_v_Nr, __pyx_v_Fsr, __pyx_v_H, __pyx_v_Aon};
      __pyx_t_14 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_8, 4+__pyx_t_8); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 359, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_GOTREF(__pyx_t_14);
    } else
    #endif
    {
      __pyx_t_3 = PyTuple_New(4+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 359, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (__pyx_t_13) {
        __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_13); __pyx_t_13 = NULL;
      }
      __Pyx_INCREF(__pyx_v_Nr);
      __Pyx_GIVEREF(__pyx_v_Nr);
      PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_8, __pyx_v_Nr);
      __Pyx_INCREF(__pyx_v_Fsr);
      __Pyx_GIVEREF(__pyx_v_Fsr);
      PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_8, __pyx_v_Fsr);
      __Pyx_INCREF(__pyx_v_H);
      __Pyx_GIVEREF(__pyx_v_H);
      PyTuple_SET_ITEM(__pyx_t_3, 2+__pyx_t_8, __pyx_v_H);
      __Pyx_INCREF(__pyx_v_Aon);
      __Pyx_GIVEREF(__pyx_v_Aon);
      PyTuple_SET_ITEM(__pyx_t_3, 3+__pyx_t_8, __pyx_v_Aon);
      __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, NULL); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 359, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_Abr, __pyx_t_14);
    __pyx_t_14 = 0;

    /* "torchsar/misc/sarplat.py":360
 *         if abs(Nr - Tr * Fsr) > 1:
 *             Abr = compute_range_beamwidth2(Nr, Fsr, H, Aon)
 *             logging.error("~~~Nr should be TrFsr= % f% f= % f\r\n\             # <<<<<<<<<<<<<<
 *                           or Fsr should be NrTr= % f\\% f= % f\r\n\
 *                           or Abr should be Abr= % f degree!" %
 */
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_logging); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_error); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyTuple_New(15); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = 0;
    __pyx_t_11 = 127;
    __Pyx_INCREF(__pyx_kp_u_Nr_should_be_TrFsr);
    __pyx_t_11 = (255 > __pyx_t_11) ? 255 : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_kp_u_Nr_should_be_TrFsr);
    __Pyx_GIVEREF(__pyx_kp_u_Nr_should_be_TrFsr);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_kp_u_Nr_should_be_TrFsr);

    /* "torchsar/misc/sarplat.py":363
 *                           or Fsr should be NrTr= % f\\% f= % f\r\n\
 *                           or Abr should be Abr= % f degree!" %
 *                           (Tr, Fsr, Tr * Fsr, Nr, Tr, Nr / Tr, Abr * 180. / PI))             # <<<<<<<<<<<<<<
 *             raise ValueError("Nr  Tr  Fsr! %f  %f  %f = %f" %
 *                              (Nr, Tr, Fsr, Tr * Fsr))
 */
    __pyx_t_13 = __Pyx_PyObject_Format(__pyx_v_Tr, __pyx_kp_u_f_2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_13) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_13) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_13);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_13);
    __pyx_t_13 = 0;
    __Pyx_INCREF(__pyx_kp_u__6);
    __pyx_t_11 = (255 > __pyx_t_11) ? 255 : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_kp_u__6);
    __Pyx_GIVEREF(__pyx_kp_u__6);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_kp_u__6);
    __pyx_t_13 = __Pyx_PyObject_Format(__pyx_v_Fsr, __pyx_kp_u_f_2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_13) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_13) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_13);
    PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_13);
    __pyx_t_13 = 0;
    __Pyx_INCREF(__pyx_kp_u__7);
    __pyx_t_7 += 2;
    __Pyx_GIVEREF(__pyx_kp_u__7);
    PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_kp_u__7);
    __pyx_t_13 = PyNumber_Multiply(__pyx_v_Tr, __pyx_v_Fsr); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __pyx_t_10 = __Pyx_PyObject_Format(__pyx_t_13, __pyx_kp_u_f_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_10);
    PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_t_10);
    __pyx_t_10 = 0;
    __Pyx_INCREF(__pyx_kp_u_or_Fsr_should_be_NrTr);
    __pyx_t_11 = (255 > __pyx_t_11) ? 255 : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_kp_u_or_Fsr_should_be_NrTr);
    __Pyx_GIVEREF(__pyx_kp_u_or_Fsr_should_be_NrTr);
    PyTuple_SET_ITEM(__pyx_t_1, 6, __pyx_kp_u_or_Fsr_should_be_NrTr);
    __pyx_t_10 = __Pyx_PyObject_Format(__pyx_v_Nr, __pyx_kp_u_f_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_10);
    PyTuple_SET_ITEM(__pyx_t_1, 7, __pyx_t_10);
    __pyx_t_10 = 0;
    __Pyx_INCREF(__pyx_kp_u__8);
    __pyx_t_7 += 1;
    __Pyx_GIVEREF(__pyx_kp_u__8);
    PyTuple_SET_ITEM(__pyx_t_1, 8, __pyx_kp_u__8);
    __pyx_t_10 = __Pyx_PyObject_Format(__pyx_v_Tr, __pyx_kp_u_f_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_10);
    PyTuple_SET_ITEM(__pyx_t_1, 9, __pyx_t_10);
    __pyx_t_10 = 0;
    __Pyx_INCREF(__pyx_kp_u__7);
    __pyx_t_7 += 2;
    __Pyx_GIVEREF(__pyx_kp_u__7);
    PyTuple_SET_ITEM(__pyx_t_1, 10, __pyx_kp_u__7);
    __pyx_t_10 = __Pyx_PyNumber_Divide(__pyx_v_Nr, __pyx_v_Tr); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_13 = __Pyx_PyObject_Format(__pyx_t_10, __pyx_kp_u_f_2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_13) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_13) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_13);
    __Pyx_GIVEREF(__pyx_t_13);
    PyTuple_SET_ITEM(__pyx_t_1, 11, __pyx_t_13);
    __pyx_t_13 = 0;
    __Pyx_INCREF(__pyx_kp_u_or_Abr_should_be_Abr);
    __pyx_t_7 += 50;
    __Pyx_GIVEREF(__pyx_kp_u_or_Abr_should_be_Abr);
    PyTuple_SET_ITEM(__pyx_t_1, 12, __pyx_kp_u_or_Abr_should_be_Abr);
    __pyx_t_13 = PyNumber_Multiply(__pyx_v_Abr, __pyx_float_180_); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_PI); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_2 = __Pyx_PyNumber_Divide(__pyx_t_13, __pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = __Pyx_PyObject_Format(__pyx_t_2, __pyx_kp_u_f_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_10);
    PyTuple_SET_ITEM(__pyx_t_1, 13, __pyx_t_10);
    __pyx_t_10 = 0;
    __Pyx_INCREF(__pyx_kp_u_degree);
    __pyx_t_7 += 8;
    __Pyx_GIVEREF(__pyx_kp_u_degree);
    PyTuple_SET_ITEM(__pyx_t_1, 14, __pyx_kp_u_degree);

    /* "torchsar/misc/sarplat.py":360
 *         if abs(Nr - Tr * Fsr) > 1:
 *             Abr = compute_range_beamwidth2(Nr, Fsr, H, Aon)
 *             logging.error("~~~Nr should be TrFsr= % f% f= % f\r\n\             # <<<<<<<<<<<<<<
 *                           or Fsr should be NrTr= % f\\% f= % f\r\n\
 *                           or Abr should be Abr= % f degree!" %
 */
    __pyx_t_10 = __Pyx_PyUnicode_Join(__pyx_t_1, 15, __pyx_t_7, __pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_14 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_1, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_10);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

    /* "torchsar/misc/sarplat.py":364
 *                           or Abr should be Abr= % f degree!" %
 *                           (Tr, Fsr, Tr * Fsr, Nr, Tr, Nr / Tr, Abr * 180. / PI))
 *             raise ValueError("Nr  Tr  Fsr! %f  %f  %f = %f" %             # <<<<<<<<<<<<<<
 *                              (Nr, Tr, Fsr, Tr * Fsr))
 * 
 */
    __pyx_t_14 = PyTuple_New(8); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 364, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __pyx_t_7 = 0;
    __pyx_t_11 = 127;
    __Pyx_INCREF(__pyx_kp_u_Nr_Tr_Fsr);
    __pyx_t_11 = (65535 > __pyx_t_11) ? 65535 : __pyx_t_11;
    __pyx_t_7 += 15;
    __Pyx_GIVEREF(__pyx_kp_u_Nr_Tr_Fsr);
    PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_kp_u_Nr_Tr_Fsr);

    /* "torchsar/misc/sarplat.py":365
 *                           (Tr, Fsr, Tr * Fsr, Nr, Tr, Nr / Tr, Abr * 180. / PI))
 *             raise ValueError("Nr  Tr  Fsr! %f  %f  %f = %f" %
 *                              (Nr, Tr, Fsr, Tr * Fsr))             # <<<<<<<<<<<<<<
 * 
 *         tr = np.linspace(tstart, tend, Nr)
 */
    __pyx_t_3 = __Pyx_PyObject_Format(__pyx_v_Nr, __pyx_n_u_f); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 365, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_t_3);
    __pyx_t_3 = 0;
    __Pyx_INCREF(__pyx_kp_u__9);
    __pyx_t_11 = (65535 > __pyx_t_11) ? 65535 : __pyx_t_11;
    __pyx_t_7 += 3;
    __Pyx_GIVEREF(__pyx_kp_u__9);
    PyTuple_SET_ITEM(__pyx_t_14, 2, __pyx_kp_u__9);
    __pyx_t_3 = __Pyx_PyObject_Format(__pyx_v_Tr, __pyx_n_u_f); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 365, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_14, 3, __pyx_t_3);
    __pyx_t_3 = 0;
    __Pyx_INCREF(__pyx_kp_u__10);
    __pyx_t_11 = (255 > __pyx_t_11) ? 255 : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_kp_u__10);
    __Pyx_GIVEREF(__pyx_kp_u__10);
    PyTuple_SET_ITEM(__pyx_t_14, 4, __pyx_kp_u__10);
    __pyx_t_3 = __Pyx_PyObject_Format(__pyx_v_Fsr, __pyx_n_u_f); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 365, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_14, 5, __pyx_t_3);
    __pyx_t_3 = 0;
    __Pyx_INCREF(__pyx_kp_u__11);
    __pyx_t_7 += 3;
    __Pyx_GIVEREF(__pyx_kp_u__11);
    PyTuple_SET_ITEM(__pyx_t_14, 6, __pyx_kp_u__11);
    __pyx_t_3 = PyNumber_Multiply(__pyx_v_Tr, __pyx_v_Fsr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 365, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_10 = __Pyx_PyObject_Format(__pyx_t_3, __pyx_n_u_f); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 365, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_10);
    PyTuple_SET_ITEM(__pyx_t_14, 7, __pyx_t_10);
    __pyx_t_10 = 0;

    /* "torchsar/misc/sarplat.py":364
 *                           or Abr should be Abr= % f degree!" %
 *                           (Tr, Fsr, Tr * Fsr, Nr, Tr, Nr / Tr, Abr * 180. / PI))
 *             raise ValueError("Nr  Tr  Fsr! %f  %f  %f = %f" %             # <<<<<<<<<<<<<<
 *                              (Nr, Tr, Fsr, Tr * Fsr))
 * 
 */
    __pyx_t_10 = __Pyx_PyUnicode_Join(__pyx_t_14, 8, __pyx_t_7, __pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 364, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    __pyx_t_14 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_10); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 364, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_Raise(__pyx_t_14, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    __PYX_ERR(0, 364, __pyx_L1_error)

    /* "torchsar/misc/sarplat.py":358
 * 
 *         Tr = tend - tstart
 *         if abs(Nr - Tr * Fsr) > 1:             # <<<<<<<<<<<<<<
 *             Abr = compute_range_beamwidth2(Nr, Fsr, H, Aon)
 *             logging.error("~~~Nr should be TrFsr= % f% f= % f\r\n\
 */
  }

  /* "torchsar/misc/sarplat.py":367
 *                              (Nr, Tr, Fsr, Tr * Fsr))
 * 
 *         tr = np.linspace(tstart, tend, Nr)             # <<<<<<<<<<<<<<
 *         # tr = np.linspace(-Tr / 2.0, Tr / 2.0, Nr) + t0
 *         fr = np.linspace(-Fsr / 2.0, Fsr / 2.0, Nr)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 367, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_linspace); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 367, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[4] = {__pyx_t_10, __pyx_v_tstart, __pyx_v_tend, __pyx_v_Nr};
    __pyx_t_14 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 367, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GOTREF(__pyx_t_14);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[4] = {__pyx_t_10, __pyx_v_tstart, __pyx_v_tend, __pyx_v_Nr};
    __pyx_t_14 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 367, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GOTREF(__pyx_t_14);
  } else
  #endif
  {
    __pyx_t_1 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 367, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__pyx_t_10) {
      __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_10); __pyx_t_10 = NULL;
    }
    __Pyx_INCREF(__pyx_v_tstart);
    __Pyx_GIVEREF(__pyx_v_tstart);
    PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_8, __pyx_v_tstart);
    __Pyx_INCREF(__pyx_v_tend);
    __Pyx_GIVEREF(__pyx_v_tend);
    PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_8, __pyx_v_tend);
    __Pyx_INCREF(__pyx_v_Nr);
    __Pyx_GIVEREF(__pyx_v_Nr);
    PyTuple_SET_ITEM(__pyx_t_1, 2+__pyx_t_8, __pyx_v_Nr);
    __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_1, NULL); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 367, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_tr = __pyx_t_14;
  __pyx_t_14 = 0;

  /* "torchsar/misc/sarplat.py":369
 *         tr = np.linspace(tstart, tend, Nr)
 *         # tr = np.linspace(-Tr / 2.0, Tr / 2.0, Nr) + t0
 *         fr = np.linspace(-Fsr / 2.0, Fsr / 2.0, Nr)             # <<<<<<<<<<<<<<
 * 
 *         # tac = -Yc / Vg
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_linspace); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Negative(__pyx_v_Fsr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_10 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_3, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyFloat_TrueDivideObjC(__pyx_v_Fsr, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 369, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_t_10, __pyx_t_3, __pyx_v_Nr};
    __pyx_t_14 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 369, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_t_10, __pyx_t_3, __pyx_v_Nr};
    __pyx_t_14 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 369, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_13 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 369, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    if (__pyx_t_2) {
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_13, 0, __pyx_t_2); __pyx_t_2 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_10);
    PyTuple_SET_ITEM(__pyx_t_13, 0+__pyx_t_8, __pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_13, 1+__pyx_t_8, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_Nr);
    __Pyx_GIVEREF(__pyx_v_Nr);
    PyTuple_SET_ITEM(__pyx_t_13, 2+__pyx_t_8, __pyx_v_Nr);
    __pyx_t_10 = 0;
    __pyx_t_3 = 0;
    __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_13, NULL); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 369, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_fr = __pyx_t_14;
  __pyx_t_14 = 0;

  /* "torchsar/misc/sarplat.py":373
 *         # tac = -Yc / Vg
 *         # tac = -R0 * np.tan(As) / Vg
 *         tac = -R0 * np.tan(Ar) / Vr             # <<<<<<<<<<<<<<
 * 
 *         # both are ok, equal
 */
  if (unlikely(!__pyx_v_R0)) { __Pyx_RaiseUnboundLocalError("R0"); __PYX_ERR(0, 373, __pyx_L1_error) }
  __pyx_t_14 = PyNumber_Negative(__pyx_v_R0); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_n_s_np); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_13, __pyx_n_s_tan); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_13)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_13);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_13, __pyx_v_Ar) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_Ar);
  __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Multiply(__pyx_t_14, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_t_3, __pyx_v_Vr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_tac = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":376
 * 
 *         # both are ok, equal
 *         fadc = 2.0 * Vr * np.sin(Ar) / Wl             # <<<<<<<<<<<<<<
 *         # fadc = 2.0 * Vs * np.sin(As) / Wl
 * 
 */
  __pyx_t_1 = PyNumber_Multiply(__pyx_float_2_0, __pyx_v_Vr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 376, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_n_s_np); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 376, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_14, __pyx_n_s_sin); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 376, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_14 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_13))) {
    __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_13);
    if (likely(__pyx_t_14)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_13);
      __Pyx_INCREF(__pyx_t_14);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_13, function);
    }
  }
  __pyx_t_3 = (__pyx_t_14) ? __Pyx_PyObject_Call2Args(__pyx_t_13, __pyx_t_14, __pyx_v_Ar) : __Pyx_PyObject_CallOneArg(__pyx_t_13, __pyx_v_Ar);
  __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 376, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = PyNumber_Multiply(__pyx_t_1, __pyx_t_3); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 376, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyNumber_Divide(__pyx_t_13, __pyx_v_Wl); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 376, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_v_fadc = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":379
 *         # fadc = 2.0 * Vs * np.sin(As) / Wl
 * 
 *         Bdop = 0.886 * (2 * Vs * np.cos(As) / La)             # <<<<<<<<<<<<<<
 * 
 *         xmin, xmax, ymin, ymax = SA + [Xc, Xc, Yc, Yc]
 */
  __pyx_t_3 = PyNumber_Multiply(__pyx_int_2, __pyx_v_Vs); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_cos); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_14))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_14);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_14, function);
    }
  }
  __pyx_t_13 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_14, __pyx_t_1, __pyx_v_As) : __Pyx_PyObject_CallOneArg(__pyx_t_14, __pyx_v_As);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_14 = PyNumber_Multiply(__pyx_t_3, __pyx_t_13); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = __Pyx_PyNumber_Divide(__pyx_t_14, __pyx_v_La); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_14 = PyNumber_Multiply(__pyx_float_0_886, __pyx_t_13); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 379, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_v_Bdop = __pyx_t_14;
  __pyx_t_14 = 0;

  /* "torchsar/misc/sarplat.py":381
 *         Bdop = 0.886 * (2 * Vs * np.cos(As) / La)
 * 
 *         xmin, xmax, ymin, ymax = SA + [Xc, Xc, Yc, Yc]             # <<<<<<<<<<<<<<
 * 
 *         if 1.0 / Fsa - Tp < Tr:
 */
  if (unlikely(!__pyx_v_Xc)) { __Pyx_RaiseUnboundLocalError("Xc"); __PYX_ERR(0, 381, __pyx_L1_error) }
  if (unlikely(!__pyx_v_Xc)) { __Pyx_RaiseUnboundLocalError("Xc"); __PYX_ERR(0, 381, __pyx_L1_error) }
  if (unlikely(!__pyx_v_Yc)) { __Pyx_RaiseUnboundLocalError("Yc"); __PYX_ERR(0, 381, __pyx_L1_error) }
  if (unlikely(!__pyx_v_Yc)) { __Pyx_RaiseUnboundLocalError("Yc"); __PYX_ERR(0, 381, __pyx_L1_error) }
  __pyx_t_14 = PyList_New(4); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 381, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_INCREF(__pyx_v_Xc);
  __Pyx_GIVEREF(__pyx_v_Xc);
  PyList_SET_ITEM(__pyx_t_14, 0, __pyx_v_Xc);
  __Pyx_INCREF(__pyx_v_Xc);
  __Pyx_GIVEREF(__pyx_v_Xc);
  PyList_SET_ITEM(__pyx_t_14, 1, __pyx_v_Xc);
  __Pyx_INCREF(__pyx_v_Yc);
  __Pyx_GIVEREF(__pyx_v_Yc);
  PyList_SET_ITEM(__pyx_t_14, 2, __pyx_v_Yc);
  __Pyx_INCREF(__pyx_v_Yc);
  __Pyx_GIVEREF(__pyx_v_Yc);
  PyList_SET_ITEM(__pyx_t_14, 3, __pyx_v_Yc);
  __pyx_t_13 = PyNumber_Add(__pyx_v_SA, __pyx_t_14); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 381, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  if ((likely(PyTuple_CheckExact(__pyx_t_13))) || (PyList_CheckExact(__pyx_t_13))) {
    PyObject* sequence = __pyx_t_13;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 4)) {
      if (size > 4) __Pyx_RaiseTooManyValuesError(4);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 381, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_14 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 2); 
      __pyx_t_10 = PyTuple_GET_ITEM(sequence, 3); 
    } else {
      __pyx_t_14 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
      __pyx_t_1 = PyList_GET_ITEM(sequence, 2); 
      __pyx_t_10 = PyList_GET_ITEM(sequence, 3); 
    }
    __Pyx_INCREF(__pyx_t_14);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_10);
    #else
    {
      Py_ssize_t i;
      PyObject** temps[4] = {&__pyx_t_14,&__pyx_t_3,&__pyx_t_1,&__pyx_t_10};
      for (i=0; i < 4; i++) {
        PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 381, __pyx_L1_error)
        __Pyx_GOTREF(item);
        *(temps[i]) = item;
      }
    }
    #endif
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  } else {
    Py_ssize_t index = -1;
    PyObject** temps[4] = {&__pyx_t_14,&__pyx_t_3,&__pyx_t_1,&__pyx_t_10};
    __pyx_t_2 = PyObject_GetIter(__pyx_t_13); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 381, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __pyx_t_12 = Py_TYPE(__pyx_t_2)->tp_iternext;
    for (index=0; index < 4; index++) {
      PyObject* item = __pyx_t_12(__pyx_t_2); if (unlikely(!item)) goto __pyx_L20_unpacking_failed;
      __Pyx_GOTREF(item);
      *(temps[index]) = item;
    }
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_12(__pyx_t_2), 4) < 0) __PYX_ERR(0, 381, __pyx_L1_error)
    __pyx_t_12 = NULL;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L21_unpacking_done;
    __pyx_L20_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_12 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 381, __pyx_L1_error)
    __pyx_L21_unpacking_done:;
  }
  __pyx_v_xmin = __pyx_t_14;
  __pyx_t_14 = 0;
  __pyx_v_xmax = __pyx_t_3;
  __pyx_t_3 = 0;
  __pyx_v_ymin = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_v_ymax = __pyx_t_10;
  __pyx_t_10 = 0;

  /* "torchsar/misc/sarplat.py":383
 *         xmin, xmax, ymin, ymax = SA + [Xc, Xc, Yc, Yc]
 * 
 *         if 1.0 / Fsa - Tp < Tr:             # <<<<<<<<<<<<<<
 *             logging.info(
 *                 "r~~~Sampling rate should be lower to avoid range ambiguity \r\n   \
 */
  __pyx_t_13 = __Pyx_PyFloat_TrueDivideCObj(__pyx_float_1_0, __pyx_v_Fsa, 1.0, 0, 1); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 383, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_10 = PyNumber_Subtract(__pyx_t_13, __pyx_v_Tp); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 383, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = PyObject_RichCompare(__pyx_t_10, __pyx_v_Tr, Py_LT); __Pyx_XGOTREF(__pyx_t_13); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 383, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_13); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 383, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  if (__pyx_t_5) {

    /* "torchsar/misc/sarplat.py":384
 * 
 *         if 1.0 / Fsa - Tp < Tr:
 *             logging.info(             # <<<<<<<<<<<<<<
 *                 "r~~~Sampling rate should be lower to avoid range ambiguity \r\n   \
 *                 (1.0  Fsa - Tp = 1.0  %f - %f > Tr = %f" % (Fsa, Tp, Tr))
 */
    __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_logging); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 384, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_info); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 384, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

    /* "torchsar/misc/sarplat.py":385
 *         if 1.0 / Fsa - Tp < Tr:
 *             logging.info(
 *                 "r~~~Sampling rate should be lower to avoid range ambiguity \r\n   \             # <<<<<<<<<<<<<<
 *                 (1.0  Fsa - Tp = 1.0  %f - %f > Tr = %f" % (Fsa, Tp, Tr))
 * 
 */
    __pyx_t_10 = PyTuple_New(6); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 385, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_7 = 0;
    __pyx_t_11 = 127;
    __Pyx_INCREF(__pyx_kp_u_r_Sampling_rate_should_be_lower);
    __pyx_t_11 = (255 > __pyx_t_11) ? 255 : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_kp_u_r_Sampling_rate_should_be_lower);
    __Pyx_GIVEREF(__pyx_kp_u_r_Sampling_rate_should_be_lower);
    PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_kp_u_r_Sampling_rate_should_be_lower);

    /* "torchsar/misc/sarplat.py":386
 *             logging.info(
 *                 "r~~~Sampling rate should be lower to avoid range ambiguity \r\n   \
 *                 (1.0  Fsa - Tp = 1.0  %f - %f > Tr = %f" % (Fsa, Tp, Tr))             # <<<<<<<<<<<<<<
 * 
 *         self._sensor['PRF'] = PRF
 */
    __pyx_t_3 = __Pyx_PyObject_Format(__pyx_v_Fsa, __pyx_n_u_f); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 386, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_3);
    __pyx_t_3 = 0;
    __Pyx_INCREF(__pyx_kp_u__12);
    __pyx_t_7 += 3;
    __Pyx_GIVEREF(__pyx_kp_u__12);
    PyTuple_SET_ITEM(__pyx_t_10, 2, __pyx_kp_u__12);
    __pyx_t_3 = __Pyx_PyObject_Format(__pyx_v_Tp, __pyx_n_u_f); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 386, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_10, 3, __pyx_t_3);
    __pyx_t_3 = 0;
    __Pyx_INCREF(__pyx_kp_u_Tr);
    __pyx_t_7 += 8;
    __Pyx_GIVEREF(__pyx_kp_u_Tr);
    PyTuple_SET_ITEM(__pyx_t_10, 4, __pyx_kp_u_Tr);
    __pyx_t_3 = __Pyx_PyObject_Format(__pyx_v_Tr, __pyx_n_u_f); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 386, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_10, 5, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "torchsar/misc/sarplat.py":385
 *         if 1.0 / Fsa - Tp < Tr:
 *             logging.info(
 *                 "r~~~Sampling rate should be lower to avoid range ambiguity \r\n   \             # <<<<<<<<<<<<<<
 *                 (1.0  Fsa - Tp = 1.0  %f - %f > Tr = %f" % (Fsa, Tp, Tr))
 * 
 */
    __pyx_t_3 = __Pyx_PyUnicode_Join(__pyx_t_10, 6, __pyx_t_7, __pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 385, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_13 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_10, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 384, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

    /* "torchsar/misc/sarplat.py":383
 *         xmin, xmax, ymin, ymax = SA + [Xc, Xc, Yc, Yc]
 * 
 *         if 1.0 / Fsa - Tp < Tr:             # <<<<<<<<<<<<<<
 *             logging.info(
 *                 "r~~~Sampling rate should be lower to avoid range ambiguity \r\n   \
 */
  }

  /* "torchsar/misc/sarplat.py":388
 *                 (1.0  Fsa - Tp = 1.0  %f - %f > Tr = %f" % (Fsa, Tp, Tr))
 * 
 *         self._sensor['PRF'] = PRF             # <<<<<<<<<<<<<<
 *         self._sensor['Fs'] = Fs
 * 
 */
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 388, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  if (unlikely(PyObject_SetItem(__pyx_t_13, __pyx_n_u_PRF, __pyx_v_PRF) < 0)) __PYX_ERR(0, 388, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "torchsar/misc/sarplat.py":389
 * 
 *         self._sensor['PRF'] = PRF
 *         self._sensor['Fs'] = Fs             # <<<<<<<<<<<<<<
 * 
 *         FPa = cr_footprint(Wl, H, La, Ad)
 */
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 389, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  if (unlikely(PyObject_SetItem(__pyx_t_13, __pyx_n_u_Fs, __pyx_v_Fs) < 0)) __PYX_ERR(0, 389, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "torchsar/misc/sarplat.py":391
 *         self._sensor['Fs'] = Fs
 * 
 *         FPa = cr_footprint(Wl, H, La, Ad)             # <<<<<<<<<<<<<<
 *         # FPr = ar_footprint(Wl, H, Lr, Ad)
 *         FPr = SA[1] - SA[0]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_cr_footprint); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 391, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[5] = {__pyx_t_3, __pyx_v_Wl, __pyx_v_H, __pyx_v_La, __pyx_v_Ad};
    __pyx_t_13 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_8, 4+__pyx_t_8); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 391, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_13);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[5] = {__pyx_t_3, __pyx_v_Wl, __pyx_v_H, __pyx_v_La, __pyx_v_Ad};
    __pyx_t_13 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_8, 4+__pyx_t_8); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 391, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_13);
  } else
  #endif
  {
    __pyx_t_10 = PyTuple_New(4+__pyx_t_8); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 391, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(__pyx_v_Wl);
    __Pyx_GIVEREF(__pyx_v_Wl);
    PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_8, __pyx_v_Wl);
    __Pyx_INCREF(__pyx_v_H);
    __Pyx_GIVEREF(__pyx_v_H);
    PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_8, __pyx_v_H);
    __Pyx_INCREF(__pyx_v_La);
    __Pyx_GIVEREF(__pyx_v_La);
    PyTuple_SET_ITEM(__pyx_t_10, 2+__pyx_t_8, __pyx_v_La);
    __Pyx_INCREF(__pyx_v_Ad);
    __Pyx_GIVEREF(__pyx_v_Ad);
    PyTuple_SET_ITEM(__pyx_t_10, 3+__pyx_t_8, __pyx_v_Ad);
    __pyx_t_13 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_10, NULL); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 391, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_FPa = __pyx_t_13;
  __pyx_t_13 = 0;

  /* "torchsar/misc/sarplat.py":393
 *         FPa = cr_footprint(Wl, H, La, Ad)
 *         # FPr = ar_footprint(Wl, H, Lr, Ad)
 *         FPr = SA[1] - SA[0]             # <<<<<<<<<<<<<<
 *         BWa = azimuth_beamwidth(Wl, La)
 * 
 */
  __pyx_t_13 = __Pyx_GetItemInt(__pyx_v_SA, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 393, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_SA, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 393, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_10 = PyNumber_Subtract(__pyx_t_13, __pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 393, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_FPr = __pyx_t_10;
  __pyx_t_10 = 0;

  /* "torchsar/misc/sarplat.py":394
 *         # FPr = ar_footprint(Wl, H, Lr, Ad)
 *         FPr = SA[1] - SA[0]
 *         BWa = azimuth_beamwidth(Wl, La)             # <<<<<<<<<<<<<<
 * 
 *         PRFmin = 2.0 * V / H
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_azimuth_beamwidth); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 394, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_13 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_13)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_13);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[3] = {__pyx_t_13, __pyx_v_Wl, __pyx_v_La};
    __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 394, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_GOTREF(__pyx_t_10);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[3] = {__pyx_t_13, __pyx_v_Wl, __pyx_v_La};
    __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 394, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_GOTREF(__pyx_t_10);
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 394, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_13) {
      __Pyx_GIVEREF(__pyx_t_13); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_13); __pyx_t_13 = NULL;
    }
    __Pyx_INCREF(__pyx_v_Wl);
    __Pyx_GIVEREF(__pyx_v_Wl);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_8, __pyx_v_Wl);
    __Pyx_INCREF(__pyx_v_La);
    __Pyx_GIVEREF(__pyx_v_La);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_8, __pyx_v_La);
    __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 394, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_BWa = __pyx_t_10;
  __pyx_t_10 = 0;

  /* "torchsar/misc/sarplat.py":396
 *         BWa = azimuth_beamwidth(Wl, La)
 * 
 *         PRFmin = 2.0 * V / H             # <<<<<<<<<<<<<<
 *         PRFmax = C / (2.0 * FPa * np.sin(Ad))
 * 
 */
  __pyx_t_10 = PyNumber_Multiply(__pyx_float_2_0, __pyx_v_V); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 396, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_t_10, __pyx_v_H); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 396, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_v_PRFmin = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":397
 * 
 *         PRFmin = 2.0 * V / H
 *         PRFmax = C / (2.0 * FPa * np.sin(Ad))             # <<<<<<<<<<<<<<
 * 
 *         if Lsar > FPa:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_C); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 397, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_10 = PyNumber_Multiply(__pyx_float_2_0, __pyx_v_FPa); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 397, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_n_s_np); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 397, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_13, __pyx_n_s_sin); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 397, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __pyx_t_13 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_14))) {
    __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_14);
    if (likely(__pyx_t_13)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
      __Pyx_INCREF(__pyx_t_13);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_14, function);
    }
  }
  __pyx_t_3 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_14, __pyx_t_13, __pyx_v_Ad) : __Pyx_PyObject_CallOneArg(__pyx_t_14, __pyx_v_Ad);
  __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 397, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_14 = PyNumber_Multiply(__pyx_t_10, __pyx_t_3); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 397, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyNumber_Divide(__pyx_t_1, __pyx_t_14); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 397, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_v_PRFmax = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":399
 *         PRFmax = C / (2.0 * FPa * np.sin(Ad))
 * 
 *         if Lsar > FPa:             # <<<<<<<<<<<<<<
 *             logging.warning(
 *                 r"~~~The synthetic aperture length Lsar=%s is large than \r\n   \
 */
  __pyx_t_3 = PyObject_RichCompare(__pyx_v_Lsar, __pyx_v_FPa, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 399, __pyx_L1_error)
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 399, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "torchsar/misc/sarplat.py":400
 * 
 *         if Lsar > FPa:
 *             logging.warning(             # <<<<<<<<<<<<<<
 *                 r"~~~The synthetic aperture length Lsar=%s is large than \r\n   \
 *                 the cross range footprint size Rcr=%s which maks a point outside the beam footprint." % (Lsar, FPa))
 */
    __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_n_s_logging); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 400, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_14, __pyx_n_s_warning); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 400, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

    /* "torchsar/misc/sarplat.py":401
 *         if Lsar > FPa:
 *             logging.warning(
 *                 r"~~~The synthetic aperture length Lsar=%s is large than \r\n   \             # <<<<<<<<<<<<<<
 *                 the cross range footprint size Rcr=%s which maks a point outside the beam footprint." % (Lsar, FPa))
 *         if PRF < PRFmin or PRF > PRFmax:
 */
    __pyx_t_14 = PyTuple_New(5); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 401, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __pyx_t_7 = 0;
    __pyx_t_11 = 127;
    __Pyx_INCREF(__pyx_kp_u_The_synthetic_aperture_length_L);
    __pyx_t_7 += 38;
    __Pyx_GIVEREF(__pyx_kp_u_The_synthetic_aperture_length_L);
    PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_kp_u_The_synthetic_aperture_length_L);

    /* "torchsar/misc/sarplat.py":402
 *             logging.warning(
 *                 r"~~~The synthetic aperture length Lsar=%s is large than \r\n   \
 *                 the cross range footprint size Rcr=%s which maks a point outside the beam footprint." % (Lsar, FPa))             # <<<<<<<<<<<<<<
 *         if PRF < PRFmin or PRF > PRFmax:
 *             logging.warning(r"~~~To avoid grating lobe issues, PRF=%s should in [%s, %s]."
 */
    __pyx_t_10 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Unicode(__pyx_v_Lsar), __pyx_empty_unicode); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 402, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_10);
    PyTuple_SET_ITEM(__pyx_t_14, 1, __pyx_t_10);
    __pyx_t_10 = 0;
    __Pyx_INCREF(__pyx_kp_u_is_large_than_r_n_the_cross_ran);
    __pyx_t_7 += 75;
    __Pyx_GIVEREF(__pyx_kp_u_is_large_than_r_n_the_cross_ran);
    PyTuple_SET_ITEM(__pyx_t_14, 2, __pyx_kp_u_is_large_than_r_n_the_cross_ran);
    __pyx_t_10 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Unicode(__pyx_v_FPa), __pyx_empty_unicode); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 402, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_10);
    __Pyx_GIVEREF(__pyx_t_10);
    PyTuple_SET_ITEM(__pyx_t_14, 3, __pyx_t_10);
    __pyx_t_10 = 0;
    __Pyx_INCREF(__pyx_kp_u_which_maks_a_point_outside_the);
    __pyx_t_7 += 47;
    __Pyx_GIVEREF(__pyx_kp_u_which_maks_a_point_outside_the);
    PyTuple_SET_ITEM(__pyx_t_14, 4, __pyx_kp_u_which_maks_a_point_outside_the);

    /* "torchsar/misc/sarplat.py":401
 *         if Lsar > FPa:
 *             logging.warning(
 *                 r"~~~The synthetic aperture length Lsar=%s is large than \r\n   \             # <<<<<<<<<<<<<<
 *                 the cross range footprint size Rcr=%s which maks a point outside the beam footprint." % (Lsar, FPa))
 *         if PRF < PRFmin or PRF > PRFmax:
 */
    __pyx_t_10 = __Pyx_PyUnicode_Join(__pyx_t_14, 5, __pyx_t_7, __pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 401, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    __pyx_t_14 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_14)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_14);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_3 = (__pyx_t_14) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_14, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_10);
    __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 400, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "torchsar/misc/sarplat.py":399
 *         PRFmax = C / (2.0 * FPa * np.sin(Ad))
 * 
 *         if Lsar > FPa:             # <<<<<<<<<<<<<<
 *             logging.warning(
 *                 r"~~~The synthetic aperture length Lsar=%s is large than \r\n   \
 */
  }

  /* "torchsar/misc/sarplat.py":403
 *                 r"~~~The synthetic aperture length Lsar=%s is large than \r\n   \
 *                 the cross range footprint size Rcr=%s which maks a point outside the beam footprint." % (Lsar, FPa))
 *         if PRF < PRFmin or PRF > PRFmax:             # <<<<<<<<<<<<<<
 *             logging.warning(r"~~~To avoid grating lobe issues, PRF=%s should in [%s, %s]."
 *                             % (PRF, PRFmin, PRFmax))
 */
  __pyx_t_3 = PyObject_RichCompare(__pyx_v_PRF, __pyx_v_PRFmin, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 403, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 403, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (!__pyx_t_4) {
  } else {
    __pyx_t_5 = __pyx_t_4;
    goto __pyx_L25_bool_binop_done;
  }
  __pyx_t_3 = PyObject_RichCompare(__pyx_v_PRF, __pyx_v_PRFmax, Py_GT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 403, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 403, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = __pyx_t_4;
  __pyx_L25_bool_binop_done:;
  if (__pyx_t_5) {

    /* "torchsar/misc/sarplat.py":404
 *                 the cross range footprint size Rcr=%s which maks a point outside the beam footprint." % (Lsar, FPa))
 *         if PRF < PRFmin or PRF > PRFmax:
 *             logging.warning(r"~~~To avoid grating lobe issues, PRF=%s should in [%s, %s]."             # <<<<<<<<<<<<<<
 *                             % (PRF, PRFmin, PRFmax))
 *         param = dict()
 */
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_logging); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 404, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_warning); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 404, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyTuple_New(7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 404, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_7 = 0;
    __pyx_t_11 = 127;
    __Pyx_INCREF(__pyx_kp_u_To_avoid_grating_lobe_issues_PR);
    __pyx_t_7 += 37;
    __Pyx_GIVEREF(__pyx_kp_u_To_avoid_grating_lobe_issues_PR);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_kp_u_To_avoid_grating_lobe_issues_PR);

    /* "torchsar/misc/sarplat.py":405
 *         if PRF < PRFmin or PRF > PRFmax:
 *             logging.warning(r"~~~To avoid grating lobe issues, PRF=%s should in [%s, %s]."
 *                             % (PRF, PRFmin, PRFmax))             # <<<<<<<<<<<<<<
 *         param = dict()
 *         param['GeometryMode'] = GM  # geometry mode: beam/scene
 */
    __pyx_t_14 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Unicode(__pyx_v_PRF), __pyx_empty_unicode); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 405, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_14) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_14) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_14);
    __Pyx_GIVEREF(__pyx_t_14);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_14);
    __pyx_t_14 = 0;
    __Pyx_INCREF(__pyx_kp_u_should_in);
    __pyx_t_7 += 12;
    __Pyx_GIVEREF(__pyx_kp_u_should_in);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_kp_u_should_in);
    __pyx_t_14 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Unicode(__pyx_v_PRFmin), __pyx_empty_unicode); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 405, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_14) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_14) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_14);
    __Pyx_GIVEREF(__pyx_t_14);
    PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_t_14);
    __pyx_t_14 = 0;
    __Pyx_INCREF(__pyx_kp_u__13);
    __pyx_t_7 += 2;
    __Pyx_GIVEREF(__pyx_kp_u__13);
    PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_kp_u__13);
    __pyx_t_14 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Unicode(__pyx_v_PRFmax), __pyx_empty_unicode); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 405, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __pyx_t_11 = (__Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_14) > __pyx_t_11) ? __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_14) : __pyx_t_11;
    __pyx_t_7 += __Pyx_PyUnicode_GET_LENGTH(__pyx_t_14);
    __Pyx_GIVEREF(__pyx_t_14);
    PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_t_14);
    __pyx_t_14 = 0;
    __Pyx_INCREF(__pyx_kp_u__14);
    __pyx_t_7 += 2;
    __Pyx_GIVEREF(__pyx_kp_u__14);
    PyTuple_SET_ITEM(__pyx_t_1, 6, __pyx_kp_u__14);

    /* "torchsar/misc/sarplat.py":404
 *                 the cross range footprint size Rcr=%s which maks a point outside the beam footprint." % (Lsar, FPa))
 *         if PRF < PRFmin or PRF > PRFmax:
 *             logging.warning(r"~~~To avoid grating lobe issues, PRF=%s should in [%s, %s]."             # <<<<<<<<<<<<<<
 *                             % (PRF, PRFmin, PRFmax))
 *         param = dict()
 */
    __pyx_t_14 = __Pyx_PyUnicode_Join(__pyx_t_1, 7, __pyx_t_7, __pyx_t_11); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 404, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_10);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_10, function);
      }
    }
    __pyx_t_3 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_1, __pyx_t_14) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_14);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 404, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "torchsar/misc/sarplat.py":403
 *                 r"~~~The synthetic aperture length Lsar=%s is large than \r\n   \
 *                 the cross range footprint size Rcr=%s which maks a point outside the beam footprint." % (Lsar, FPa))
 *         if PRF < PRFmin or PRF > PRFmax:             # <<<<<<<<<<<<<<
 *             logging.warning(r"~~~To avoid grating lobe issues, PRF=%s should in [%s, %s]."
 *                             % (PRF, PRFmin, PRFmax))
 */
  }

  /* "torchsar/misc/sarplat.py":406
 *             logging.warning(r"~~~To avoid grating lobe issues, PRF=%s should in [%s, %s]."
 *                             % (PRF, PRFmin, PRFmax))
 *         param = dict()             # <<<<<<<<<<<<<<
 *         param['GeometryMode'] = GM  # geometry mode: beam/scene
 *         param['Rb0'] = Rb0  # beam center distance
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 406, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_param = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":407
 *                             % (PRF, PRFmin, PRFmax))
 *         param = dict()
 *         param['GeometryMode'] = GM  # geometry mode: beam/scene             # <<<<<<<<<<<<<<
 *         param['Rb0'] = Rb0  # beam center distance
 *         param['Rbc'] = Rbc  # beam center distance
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_GeometryMode, __pyx_v_GM) < 0)) __PYX_ERR(0, 407, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":408
 *         param = dict()
 *         param['GeometryMode'] = GM  # geometry mode: beam/scene
 *         param['Rb0'] = Rb0  # beam center distance             # <<<<<<<<<<<<<<
 *         param['Rbc'] = Rbc  # beam center distance
 *         param['Tp'] = Tp  # Pulse repetition period
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Rb0, __pyx_v_Rb0) < 0)) __PYX_ERR(0, 408, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":409
 *         param['GeometryMode'] = GM  # geometry mode: beam/scene
 *         param['Rb0'] = Rb0  # beam center distance
 *         param['Rbc'] = Rbc  # beam center distance             # <<<<<<<<<<<<<<
 *         param['Tp'] = Tp  # Pulse repetition period
 *         param['PRT'] = PRT  # azimuth
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Rbc, __pyx_v_Rbc) < 0)) __PYX_ERR(0, 409, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":410
 *         param['Rb0'] = Rb0  # beam center distance
 *         param['Rbc'] = Rbc  # beam center distance
 *         param['Tp'] = Tp  # Pulse repetition period             # <<<<<<<<<<<<<<
 *         param['PRT'] = PRT  # azimuth
 *         param['PRF'] = PRF  # azimuth
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Tp, __pyx_v_Tp) < 0)) __PYX_ERR(0, 410, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":411
 *         param['Rbc'] = Rbc  # beam center distance
 *         param['Tp'] = Tp  # Pulse repetition period
 *         param['PRT'] = PRT  # azimuth             # <<<<<<<<<<<<<<
 *         param['PRF'] = PRF  # azimuth
 *         param['Fsa'] = Fsa  # azimuth, Fsa = PRF
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_PRT, __pyx_v_PRT) < 0)) __PYX_ERR(0, 411, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":412
 *         param['Tp'] = Tp  # Pulse repetition period
 *         param['PRT'] = PRT  # azimuth
 *         param['PRF'] = PRF  # azimuth             # <<<<<<<<<<<<<<
 *         param['Fsa'] = Fsa  # azimuth, Fsa = PRF
 *         param['Fs'] = Fs  # range
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_PRF, __pyx_v_PRF) < 0)) __PYX_ERR(0, 412, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":413
 *         param['PRT'] = PRT  # azimuth
 *         param['PRF'] = PRF  # azimuth
 *         param['Fsa'] = Fsa  # azimuth, Fsa = PRF             # <<<<<<<<<<<<<<
 *         param['Fs'] = Fs  # range
 *         param['Fsr'] = Fsr  # range, Fsr = Fs
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Fsa, __pyx_v_Fsa) < 0)) __PYX_ERR(0, 413, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":414
 *         param['PRF'] = PRF  # azimuth
 *         param['Fsa'] = Fsa  # azimuth, Fsa = PRF
 *         param['Fs'] = Fs  # range             # <<<<<<<<<<<<<<
 *         param['Fsr'] = Fsr  # range, Fsr = Fs
 *         param['Na'] = Na  # azimuth
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Fs, __pyx_v_Fs) < 0)) __PYX_ERR(0, 414, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":415
 *         param['Fsa'] = Fsa  # azimuth, Fsa = PRF
 *         param['Fs'] = Fs  # range
 *         param['Fsr'] = Fsr  # range, Fsr = Fs             # <<<<<<<<<<<<<<
 *         param['Na'] = Na  # azimuth
 *         param['Nr'] = Nr  # range
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Fsr, __pyx_v_Fsr) < 0)) __PYX_ERR(0, 415, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":416
 *         param['Fs'] = Fs  # range
 *         param['Fsr'] = Fsr  # range, Fsr = Fs
 *         param['Na'] = Na  # azimuth             # <<<<<<<<<<<<<<
 *         param['Nr'] = Nr  # range
 *         param['Ka'] = Ka  # azimuth
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Na, __pyx_v_Na) < 0)) __PYX_ERR(0, 416, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":417
 *         param['Fsr'] = Fsr  # range, Fsr = Fs
 *         param['Na'] = Na  # azimuth
 *         param['Nr'] = Nr  # range             # <<<<<<<<<<<<<<
 *         param['Ka'] = Ka  # azimuth
 *         param['Kr'] = Kr  # range
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Nr, __pyx_v_Nr) < 0)) __PYX_ERR(0, 417, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":418
 *         param['Na'] = Na  # azimuth
 *         param['Nr'] = Nr  # range
 *         param['Ka'] = Ka  # azimuth             # <<<<<<<<<<<<<<
 *         param['Kr'] = Kr  # range
 *         param['xmin'] = xmin
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Ka, __pyx_v_Ka) < 0)) __PYX_ERR(0, 418, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":419
 *         param['Nr'] = Nr  # range
 *         param['Ka'] = Ka  # azimuth
 *         param['Kr'] = Kr  # range             # <<<<<<<<<<<<<<
 *         param['xmin'] = xmin
 *         param['xmax'] = xmax
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Kr, __pyx_v_Kr) < 0)) __PYX_ERR(0, 419, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":420
 *         param['Ka'] = Ka  # azimuth
 *         param['Kr'] = Kr  # range
 *         param['xmin'] = xmin             # <<<<<<<<<<<<<<
 *         param['xmax'] = xmax
 *         param['ymin'] = ymin
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_xmin, __pyx_v_xmin) < 0)) __PYX_ERR(0, 420, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":421
 *         param['Kr'] = Kr  # range
 *         param['xmin'] = xmin
 *         param['xmax'] = xmax             # <<<<<<<<<<<<<<
 *         param['ymin'] = ymin
 *         param['ymax'] = ymax
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_xmax, __pyx_v_xmax) < 0)) __PYX_ERR(0, 421, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":422
 *         param['xmin'] = xmin
 *         param['xmax'] = xmax
 *         param['ymin'] = ymin             # <<<<<<<<<<<<<<
 *         param['ymax'] = ymax
 *         param['Rnear'] = Rnear
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_ymin, __pyx_v_ymin) < 0)) __PYX_ERR(0, 422, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":423
 *         param['xmax'] = xmax
 *         param['ymin'] = ymin
 *         param['ymax'] = ymax             # <<<<<<<<<<<<<<
 *         param['Rnear'] = Rnear
 *         param['Rfar'] = Rfar
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_ymax, __pyx_v_ymax) < 0)) __PYX_ERR(0, 423, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":424
 *         param['ymin'] = ymin
 *         param['ymax'] = ymax
 *         param['Rnear'] = Rnear             # <<<<<<<<<<<<<<
 *         param['Rfar'] = Rfar
 *         param['tnear'] = tnear
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Rnear, __pyx_v_Rnear) < 0)) __PYX_ERR(0, 424, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":425
 *         param['ymax'] = ymax
 *         param['Rnear'] = Rnear
 *         param['Rfar'] = Rfar             # <<<<<<<<<<<<<<
 *         param['tnear'] = tnear
 *         param['tfar'] = tfar
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Rfar, __pyx_v_Rfar) < 0)) __PYX_ERR(0, 425, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":426
 *         param['Rnear'] = Rnear
 *         param['Rfar'] = Rfar
 *         param['tnear'] = tnear             # <<<<<<<<<<<<<<
 *         param['tfar'] = tfar
 *         param['ta'] = ta  # time array in azimuth
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_tnear, __pyx_v_tnear) < 0)) __PYX_ERR(0, 426, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":427
 *         param['Rfar'] = Rfar
 *         param['tnear'] = tnear
 *         param['tfar'] = tfar             # <<<<<<<<<<<<<<
 *         param['ta'] = ta  # time array in azimuth
 *         param['fa'] = fa  # freq array in azimuth
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_tfar, __pyx_v_tfar) < 0)) __PYX_ERR(0, 427, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":428
 *         param['tnear'] = tnear
 *         param['tfar'] = tfar
 *         param['ta'] = ta  # time array in azimuth             # <<<<<<<<<<<<<<
 *         param['fa'] = fa  # freq array in azimuth
 *         param['tr'] = tr  # time array in range
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_ta, __pyx_v_ta) < 0)) __PYX_ERR(0, 428, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":429
 *         param['tfar'] = tfar
 *         param['ta'] = ta  # time array in azimuth
 *         param['fa'] = fa  # freq array in azimuth             # <<<<<<<<<<<<<<
 *         param['tr'] = tr  # time array in range
 *         param['fr'] = fr  # freq array in range
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_fa, __pyx_v_fa) < 0)) __PYX_ERR(0, 429, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":430
 *         param['ta'] = ta  # time array in azimuth
 *         param['fa'] = fa  # freq array in azimuth
 *         param['tr'] = tr  # time array in range             # <<<<<<<<<<<<<<
 *         param['fr'] = fr  # freq array in range
 *         param['Tsa'] = Tsa  # azimuth sampling period
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_tr, __pyx_v_tr) < 0)) __PYX_ERR(0, 430, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":431
 *         param['fa'] = fa  # freq array in azimuth
 *         param['tr'] = tr  # time array in range
 *         param['fr'] = fr  # freq array in range             # <<<<<<<<<<<<<<
 *         param['Tsa'] = Tsa  # azimuth sampling period
 *         param['Tsr'] = Tsr  # range sampling period
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_fr, __pyx_v_fr) < 0)) __PYX_ERR(0, 431, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":432
 *         param['tr'] = tr  # time array in range
 *         param['fr'] = fr  # freq array in range
 *         param['Tsa'] = Tsa  # azimuth sampling period             # <<<<<<<<<<<<<<
 *         param['Tsr'] = Tsr  # range sampling period
 *         param['Xc'] = Xc  # X scene center
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Tsa, __pyx_v_Tsa) < 0)) __PYX_ERR(0, 432, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":433
 *         param['fr'] = fr  # freq array in range
 *         param['Tsa'] = Tsa  # azimuth sampling period
 *         param['Tsr'] = Tsr  # range sampling period             # <<<<<<<<<<<<<<
 *         param['Xc'] = Xc  # X scene center
 *         param['Yc'] = Yc  # Y scene center
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Tsr, __pyx_v_Tsr) < 0)) __PYX_ERR(0, 433, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":434
 *         param['Tsa'] = Tsa  # azimuth sampling period
 *         param['Tsr'] = Tsr  # range sampling period
 *         param['Xc'] = Xc  # X scene center             # <<<<<<<<<<<<<<
 *         param['Yc'] = Yc  # Y scene center
 *         param['Rsc'] = Rsc  # R scene center
 */
  if (unlikely(!__pyx_v_Xc)) { __Pyx_RaiseUnboundLocalError("Xc"); __PYX_ERR(0, 434, __pyx_L1_error) }
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Xc, __pyx_v_Xc) < 0)) __PYX_ERR(0, 434, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":435
 *         param['Tsr'] = Tsr  # range sampling period
 *         param['Xc'] = Xc  # X scene center
 *         param['Yc'] = Yc  # Y scene center             # <<<<<<<<<<<<<<
 *         param['Rsc'] = Rsc  # R scene center
 *         param['Rs0'] = Rs0  # R min scene center
 */
  if (unlikely(!__pyx_v_Yc)) { __Pyx_RaiseUnboundLocalError("Yc"); __PYX_ERR(0, 435, __pyx_L1_error) }
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Yc, __pyx_v_Yc) < 0)) __PYX_ERR(0, 435, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":436
 *         param['Xc'] = Xc  # X scene center
 *         param['Yc'] = Yc  # Y scene center
 *         param['Rsc'] = Rsc  # R scene center             # <<<<<<<<<<<<<<
 *         param['Rs0'] = Rs0  # R min scene center
 *         param['Ad'] = Ad  # depression angle
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Rsc, __pyx_v_Rsc) < 0)) __PYX_ERR(0, 436, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":437
 *         param['Yc'] = Yc  # Y scene center
 *         param['Rsc'] = Rsc  # R scene center
 *         param['Rs0'] = Rs0  # R min scene center             # <<<<<<<<<<<<<<
 *         param['Ad'] = Ad  # depression angle
 *         param['Aon'] = Aon  # off-nadir angle
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Rs0, __pyx_v_Rs0) < 0)) __PYX_ERR(0, 437, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":438
 *         param['Rsc'] = Rsc  # R scene center
 *         param['Rs0'] = Rs0  # R min scene center
 *         param['Ad'] = Ad  # depression angle             # <<<<<<<<<<<<<<
 *         param['Aon'] = Aon  # off-nadir angle
 *         param['As'] = As  # squint angle
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Ad, __pyx_v_Ad) < 0)) __PYX_ERR(0, 438, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":439
 *         param['Rs0'] = Rs0  # R min scene center
 *         param['Ad'] = Ad  # depression angle
 *         param['Aon'] = Aon  # off-nadir angle             # <<<<<<<<<<<<<<
 *         param['As'] = As  # squint angle
 *         param['Ar'] = Ar  # squint angle
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Aon, __pyx_v_Aon) < 0)) __PYX_ERR(0, 439, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":440
 *         param['Ad'] = Ad  # depression angle
 *         param['Aon'] = Aon  # off-nadir angle
 *         param['As'] = As  # squint angle             # <<<<<<<<<<<<<<
 *         param['Ar'] = Ar  # squint angle
 *         param['Aba'] = Aba  # antenna elevation beamwidth
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_As, __pyx_v_As) < 0)) __PYX_ERR(0, 440, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":441
 *         param['Aon'] = Aon  # off-nadir angle
 *         param['As'] = As  # squint angle
 *         param['Ar'] = Ar  # squint angle             # <<<<<<<<<<<<<<
 *         param['Aba'] = Aba  # antenna elevation beamwidth
 *         param['Abr'] = Abr  # antenna elevation beamwidth
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Ar, __pyx_v_Ar) < 0)) __PYX_ERR(0, 441, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":442
 *         param['As'] = As  # squint angle
 *         param['Ar'] = Ar  # squint angle
 *         param['Aba'] = Aba  # antenna elevation beamwidth             # <<<<<<<<<<<<<<
 *         param['Abr'] = Abr  # antenna elevation beamwidth
 *         param['Ai'] = Ai  # incidence angle
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Aba, __pyx_v_Aba) < 0)) __PYX_ERR(0, 442, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":443
 *         param['Ar'] = Ar  # squint angle
 *         param['Aba'] = Aba  # antenna elevation beamwidth
 *         param['Abr'] = Abr  # antenna elevation beamwidth             # <<<<<<<<<<<<<<
 *         param['Ai'] = Ai  # incidence angle
 *         param['DA'] = La / 2.0  # low squint SAR
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Abr, __pyx_v_Abr) < 0)) __PYX_ERR(0, 443, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":444
 *         param['Aba'] = Aba  # antenna elevation beamwidth
 *         param['Abr'] = Abr  # antenna elevation beamwidth
 *         param['Ai'] = Ai  # incidence angle             # <<<<<<<<<<<<<<
 *         param['DA'] = La / 2.0  # low squint SAR
 *         # slant range resolution
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Ai, __pyx_v_Ai) < 0)) __PYX_ERR(0, 444, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":445
 *         param['Abr'] = Abr  # antenna elevation beamwidth
 *         param['Ai'] = Ai  # incidence angle
 *         param['DA'] = La / 2.0  # low squint SAR             # <<<<<<<<<<<<<<
 *         # slant range resolution
 *         param['DR'] = C / (2.0 * self._sensor['Br'])
 */
  __pyx_t_3 = __Pyx_PyFloat_TrueDivideObjC(__pyx_v_La, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_DA, __pyx_t_3) < 0)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":447
 *         param['DA'] = La / 2.0  # low squint SAR
 *         # slant range resolution
 *         param['DR'] = C / (2.0 * self._sensor['Br'])             # <<<<<<<<<<<<<<
 *         # ground range resolution
 *         param['DY'] = param['DA']  # low squint SAR
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_C); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_14 = __Pyx_PyObject_Dict_GetItem(__pyx_t_10, __pyx_n_u_Br); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyNumber_Multiply(__pyx_float_2_0, __pyx_t_14); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_14 = __Pyx_PyNumber_Divide(__pyx_t_3, __pyx_t_10); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_DR, __pyx_t_14) < 0)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

  /* "torchsar/misc/sarplat.py":449
 *         param['DR'] = C / (2.0 * self._sensor['Br'])
 *         # ground range resolution
 *         param['DY'] = param['DA']  # low squint SAR             # <<<<<<<<<<<<<<
 *         param['DX'] = param['DR'] / (np.sin(param['Ai']) + EPS)
 *         param['FPa'] = FPa
 */
  __pyx_t_14 = __Pyx_PyDict_GetItem(__pyx_v_param, __pyx_n_u_DA); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_DY, __pyx_t_14) < 0)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;

  /* "torchsar/misc/sarplat.py":450
 *         # ground range resolution
 *         param['DY'] = param['DA']  # low squint SAR
 *         param['DX'] = param['DR'] / (np.sin(param['Ai']) + EPS)             # <<<<<<<<<<<<<<
 *         param['FPa'] = FPa
 *         param['FPr'] = FPr
 */
  __pyx_t_14 = __Pyx_PyDict_GetItem(__pyx_v_param, __pyx_n_u_DR); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_sin); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_v_param, __pyx_n_u_Ai); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_13 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_13)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_13);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_10 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_13, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_EPS); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyNumber_Add(__pyx_t_10, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_t_14, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_DX, __pyx_t_1) < 0)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":451
 *         param['DY'] = param['DA']  # low squint SAR
 *         param['DX'] = param['DR'] / (np.sin(param['Ai']) + EPS)
 *         param['FPa'] = FPa             # <<<<<<<<<<<<<<
 *         param['FPr'] = FPr
 *         param['BWa'] = BWa
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_FPa, __pyx_v_FPa) < 0)) __PYX_ERR(0, 451, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":452
 *         param['DX'] = param['DR'] / (np.sin(param['Ai']) + EPS)
 *         param['FPa'] = FPa
 *         param['FPr'] = FPr             # <<<<<<<<<<<<<<
 *         param['BWa'] = BWa
 *         param['Tr'] = Tr  # range sampling times
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_FPr, __pyx_v_FPr) < 0)) __PYX_ERR(0, 452, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":453
 *         param['FPa'] = FPa
 *         param['FPr'] = FPr
 *         param['BWa'] = BWa             # <<<<<<<<<<<<<<
 *         param['Tr'] = Tr  # range sampling times
 *         param['Ta'] = Ta  # azimuth sampling times
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_BWa, __pyx_v_BWa) < 0)) __PYX_ERR(0, 453, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":454
 *         param['FPr'] = FPr
 *         param['BWa'] = BWa
 *         param['Tr'] = Tr  # range sampling times             # <<<<<<<<<<<<<<
 *         param['Ta'] = Ta  # azimuth sampling times
 *         param['t0'] = t0
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Tr_2, __pyx_v_Tr) < 0)) __PYX_ERR(0, 454, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":455
 *         param['BWa'] = BWa
 *         param['Tr'] = Tr  # range sampling times
 *         param['Ta'] = Ta  # azimuth sampling times             # <<<<<<<<<<<<<<
 *         param['t0'] = t0
 *         param['tac'] = tac
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Ta, __pyx_v_Ta) < 0)) __PYX_ERR(0, 455, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":456
 *         param['Tr'] = Tr  # range sampling times
 *         param['Ta'] = Ta  # azimuth sampling times
 *         param['t0'] = t0             # <<<<<<<<<<<<<<
 *         param['tac'] = tac
 *         param['fadc'] = fadc
 */
  if (unlikely(!__pyx_v_t0)) { __Pyx_RaiseUnboundLocalError("t0"); __PYX_ERR(0, 456, __pyx_L1_error) }
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_t0, __pyx_v_t0) < 0)) __PYX_ERR(0, 456, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":457
 *         param['Ta'] = Ta  # azimuth sampling times
 *         param['t0'] = t0
 *         param['tac'] = tac             # <<<<<<<<<<<<<<
 *         param['fadc'] = fadc
 *         param['Bdop'] = Bdop
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_tac, __pyx_v_tac) < 0)) __PYX_ERR(0, 457, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":458
 *         param['t0'] = t0
 *         param['tac'] = tac
 *         param['fadc'] = fadc             # <<<<<<<<<<<<<<
 *         param['Bdop'] = Bdop
 *         param['Lsar'] = Lsar
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_fadc, __pyx_v_fadc) < 0)) __PYX_ERR(0, 458, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":459
 *         param['tac'] = tac
 *         param['fadc'] = fadc
 *         param['Bdop'] = Bdop             # <<<<<<<<<<<<<<
 *         param['Lsar'] = Lsar
 *         param['Tsar'] = Tsar
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Bdop, __pyx_v_Bdop) < 0)) __PYX_ERR(0, 459, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":460
 *         param['fadc'] = fadc
 *         param['Bdop'] = Bdop
 *         param['Lsar'] = Lsar             # <<<<<<<<<<<<<<
 *         param['Tsar'] = Tsar
 *         param['Nsar'] = int(Tsar * Fsa)
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Lsar, __pyx_v_Lsar) < 0)) __PYX_ERR(0, 460, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":461
 *         param['Bdop'] = Bdop
 *         param['Lsar'] = Lsar
 *         param['Tsar'] = Tsar             # <<<<<<<<<<<<<<
 *         param['Nsar'] = int(Tsar * Fsa)
 *         param['tstart'] = tstart
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Tsar, __pyx_v_Tsar) < 0)) __PYX_ERR(0, 461, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":462
 *         param['Lsar'] = Lsar
 *         param['Tsar'] = Tsar
 *         param['Nsar'] = int(Tsar * Fsa)             # <<<<<<<<<<<<<<
 *         param['tstart'] = tstart
 *         param['tend'] = tend
 */
  __pyx_t_1 = PyNumber_Multiply(__pyx_v_Tsar, __pyx_v_Fsa); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyNumber_Int(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_Nsar, __pyx_t_3) < 0)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":463
 *         param['Tsar'] = Tsar
 *         param['Nsar'] = int(Tsar * Fsa)
 *         param['tstart'] = tstart             # <<<<<<<<<<<<<<
 *         param['tend'] = tend
 *         self._params = param
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_tstart, __pyx_v_tstart) < 0)) __PYX_ERR(0, 463, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":464
 *         param['Nsar'] = int(Tsar * Fsa)
 *         param['tstart'] = tstart
 *         param['tend'] = tend             # <<<<<<<<<<<<<<
 *         self._params = param
 *         logging.info('---Done!')
 */
  if (unlikely(PyDict_SetItem(__pyx_v_param, __pyx_n_u_tend, __pyx_v_tend) < 0)) __PYX_ERR(0, 464, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":465
 *         param['tstart'] = tstart
 *         param['tend'] = tend
 *         self._params = param             # <<<<<<<<<<<<<<
 *         logging.info('---Done!')
 * 
 */
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_params, __pyx_v_param) < 0) __PYX_ERR(0, 465, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":466
 *         param['tend'] = tend
 *         self._params = param
 *         logging.info('---Done!')             # <<<<<<<<<<<<<<
 * 
 *         return self._params
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_logging); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_info); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_14))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_14);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_14);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_14, function);
    }
  }
  __pyx_t_3 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_14, __pyx_t_1, __pyx_kp_u_Done) : __Pyx_PyObject_CallOneArg(__pyx_t_14, __pyx_kp_u_Done);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":468
 *         logging.info('---Done!')
 * 
 *         return self._params             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "torchsar/misc/sarplat.py":248
 * 
 *     @params.setter
 *     def params(self, value):             # <<<<<<<<<<<<<<
 * 
 *         logging.info("---Check and set algorithm parameters...")
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_AddTraceback("torchsar.misc.sarplat.SarPlat.params", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_k);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_XDECREF(__pyx_v_GM);
  __Pyx_XDECREF(__pyx_v_H);
  __Pyx_XDECREF(__pyx_v_V);
  __Pyx_XDECREF(__pyx_v_Vr);
  __Pyx_XDECREF(__pyx_v_Vg);
  __Pyx_XDECREF(__pyx_v_Vs);
  __Pyx_XDECREF(__pyx_v_La);
  __Pyx_XDECREF(__pyx_v_Fsa);
  __Pyx_XDECREF(__pyx_v_Fsr);
  __Pyx_XDECREF(__pyx_v_Tp);
  __Pyx_XDECREF(__pyx_v_Fc);
  __Pyx_XDECREF(__pyx_v_Wl);
  __Pyx_XDECREF(__pyx_v_Kr);
  __Pyx_XDECREF(__pyx_v_Rbc);
  __Pyx_XDECREF(__pyx_v_Rb0);
  __Pyx_XDECREF(__pyx_v_Rsc);
  __Pyx_XDECREF(__pyx_v_Rs0);
  __Pyx_XDECREF(__pyx_v_Rnear);
  __Pyx_XDECREF(__pyx_v_Rfar);
  __Pyx_XDECREF(__pyx_v_PC);
  __Pyx_XDECREF(__pyx_v_SC);
  __Pyx_XDECREF(__pyx_v_BC);
  __Pyx_XDECREF(__pyx_v_SA);
  __Pyx_XDECREF(__pyx_v_BA);
  __Pyx_XDECREF(__pyx_v_ES);
  __Pyx_XDECREF(__pyx_v_As);
  __Pyx_XDECREF(__pyx_v_Ar);
  __Pyx_XDECREF(__pyx_v_Ad);
  __Pyx_XDECREF(__pyx_v_Aon);
  __Pyx_XDECREF(__pyx_v_Aba);
  __Pyx_XDECREF(__pyx_v_Abr);
  __Pyx_XDECREF(__pyx_v_Ai);
  __Pyx_XDECREF(__pyx_v_Xc);
  __Pyx_XDECREF(__pyx_v_Yc);
  __Pyx_XDECREF(__pyx_v_t0);
  __Pyx_XDECREF(__pyx_v_R0);
  __Pyx_XDECREF(__pyx_v_Lsar);
  __Pyx_XDECREF(__pyx_v_Ka);
  __Pyx_XDECREF(__pyx_v_Tsar);
  __Pyx_XDECREF(__pyx_v_Ba);
  __Pyx_XDECREF(__pyx_v_PRF);
  __Pyx_XDECREF(__pyx_v_PRT);
  __Pyx_XDECREF(__pyx_v_Tsa);
  __Pyx_XDECREF(__pyx_v_Fs);
  __Pyx_XDECREF(__pyx_v_Tsr);
  __Pyx_XDECREF(__pyx_v_Na);
  __Pyx_XDECREF(__pyx_v_Nr);
  __Pyx_XDECREF(__pyx_v_Ta);
  __Pyx_XDECREF(__pyx_v_ta);
  __Pyx_XDECREF(__pyx_v_fa);
  __Pyx_XDECREF(__pyx_v_tnear);
  __Pyx_XDECREF(__pyx_v_tfar);
  __Pyx_XDECREF(__pyx_v_tstart);
  __Pyx_XDECREF(__pyx_v_tend);
  __Pyx_XDECREF(__pyx_v_Tr);
  __Pyx_XDECREF(__pyx_v_tr);
  __Pyx_XDECREF(__pyx_v_fr);
  __Pyx_XDECREF(__pyx_v_tac);
  __Pyx_XDECREF(__pyx_v_fadc);
  __Pyx_XDECREF(__pyx_v_Bdop);
  __Pyx_XDECREF(__pyx_v_xmin);
  __Pyx_XDECREF(__pyx_v_xmax);
  __Pyx_XDECREF(__pyx_v_ymin);
  __Pyx_XDECREF(__pyx_v_ymax);
  __Pyx_XDECREF(__pyx_v_FPa);
  __Pyx_XDECREF(__pyx_v_FPr);
  __Pyx_XDECREF(__pyx_v_BWa);
  __Pyx_XDECREF(__pyx_v_PRFmin);
  __Pyx_XDECREF(__pyx_v_PRFmax);
  __Pyx_XDECREF(__pyx_v_param);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "torchsar/misc/sarplat.py":471
 * 
 *     @property
 *     def selection(self):             # <<<<<<<<<<<<<<
 *         return self._selection
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_17selection(PyObject *__pyx_self, PyObject *__pyx_v_self); /*proto*/
static PyMethodDef __pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_17selection = {"selection", (PyCFunction)__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_17selection, METH_O, 0};
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_17selection(PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selection (wrapper)", 0);
  __pyx_r = __pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_16selection(__pyx_self, ((PyObject *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_16selection(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("selection", 0);

  /* "torchsar/misc/sarplat.py":472
 *     @property
 *     def selection(self):
 *         return self._selection             # <<<<<<<<<<<<<<
 * 
 *     @selection.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 472, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "torchsar/misc/sarplat.py":471
 * 
 *     @property
 *     def selection(self):             # <<<<<<<<<<<<<<
 *         return self._selection
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("torchsar.misc.sarplat.SarPlat.selection", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "torchsar/misc/sarplat.py":475
 * 
 *     @selection.setter
 *     def selection(self, value):             # <<<<<<<<<<<<<<
 *         logging.info("---Check and set sub area selection...")
 *         SA = self._acquisition['SceneArea']
 */

/* Python wrapper */
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_19selection(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_19selection = {"selection", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_19selection, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_19selection(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_value = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("selection (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_value,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("selection", 1, 2, 2, 1); __PYX_ERR(0, 475, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "selection") < 0)) __PYX_ERR(0, 475, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_value = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("selection", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 475, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("torchsar.misc.sarplat.SarPlat.selection", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_18selection(__pyx_self, __pyx_v_self, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_18selection(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  PyObject *__pyx_v_SA = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_SC = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_BC = NULL;
  PyObject *__pyx_v_ES = NULL;
  PyObject *__pyx_v_Ad = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_Aon = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_As = NULL;
  PyObject *__pyx_v_Ar = NULL;
  PyObject *__pyx_v_Fsr = NULL;
  PyObject *__pyx_v_Fsa = NULL;
  PyObject *__pyx_v_Tsa = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_Tsr = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_Na = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_Nr = NULL;
  PyObject *__pyx_v_Rnear = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_Rfar = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_FPr = NULL;
  PyObject *__pyx_v_H = NULL;
  PyObject *__pyx_v_Vr = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_Tp = NULL;
  PyObject *__pyx_v_Wl = NULL;
  PyObject *__pyx_v_La = NULL;
  PyObject *__pyx_v_GM = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_Lsar = NULL;
  PyObject *__pyx_v_SubSA = NULL;
  PyObject *__pyx_v_SubES = NULL;
  PyObject *__pyx_v_SubEA = NULL;
  PyObject *__pyx_v_SubNa = NULL;
  PyObject *__pyx_v_SubNr = NULL;
  PyObject *__pyx_v_SubRnear = NULL;
  PyObject *__pyx_v_SubRfar = NULL;
  PyObject *__pyx_v_SubAon = NULL;
  PyObject *__pyx_v_SubAbr = NULL;
  PyObject *__pyx_v_SubAd = NULL;
  PyObject *__pyx_v_SubFPr = NULL;
  PyObject *__pyx_v_SubTa = NULL;
  PyObject *__pyx_v_taSub = NULL;
  PyObject *__pyx_v_faSub = NULL;
  PyObject *__pyx_v_SubRsc = NULL;
  PyObject *__pyx_v_SubRs0 = NULL;
  PyObject *__pyx_v_SubYsc = NULL;
  PyObject *__pyx_v_SubXsc = NULL;
  PyObject *__pyx_v_SubSC = NULL;
  PyObject *__pyx_v_SubRbc = NULL;
  PyObject *__pyx_v_SubRb0 = NULL;
  PyObject *__pyx_v_SubYbc = NULL;
  PyObject *__pyx_v_SubXbc = NULL;
  PyObject *__pyx_v_SubBC = NULL;
  PyObject *__pyx_v_SubYnear = NULL;
  PyObject *__pyx_v_SubYfar = NULL;
  PyObject *__pyx_v_SubXnear = NULL;
  PyObject *__pyx_v_SubXfar = NULL;
  PyObject *__pyx_v_SubFPr0 = NULL;
  PyObject *__pyx_v_SubFPr1 = NULL;
  PyObject *__pyx_v_SubBA = NULL;
  PyObject *__pyx_v_tnearSub = NULL;
  PyObject *__pyx_v_tfarSub = NULL;
  PyObject *__pyx_v_tstartSub = NULL;
  PyObject *__pyx_v_tendSub = NULL;
  PyObject *__pyx_v_SubTr = NULL;
  PyObject *__pyx_v_Xc = NULL;
  PyObject *__pyx_v_Yc = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_SubR0 = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_t0Sub = NULL;
  PyObject *__pyx_v_trSub = NULL;
  PyObject *__pyx_v_xminSub = NULL;
  PyObject *__pyx_v_xmaxSub = NULL;
  PyObject *__pyx_v_yminSub = NULL;
  PyObject *__pyx_v_ymaxSub = NULL;
  PyObject *__pyx_v_frSub = NULL;
  PyObject *__pyx_v_SubKa = NULL;
  PyObject *__pyx_v_SubLsar = NULL;
  PyObject *__pyx_v_SubTsar = NULL;
  PyObject *__pyx_v_SubBa = NULL;
  PyObject *__pyx_v_SubFPa = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *(*__pyx_t_6)(PyObject *);
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("selection", 0);

  /* "torchsar/misc/sarplat.py":476
 *     @selection.setter
 *     def selection(self, value):
 *         logging.info("---Check and set sub area selection...")             # <<<<<<<<<<<<<<
 *         SA = self._acquisition['SceneArea']
 *         SC = self._acquisition['SceneCenter']
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_logging); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_info); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_kp_u_Check_and_set_sub_area_selectio) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_kp_u_Check_and_set_sub_area_selectio);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":477
 *     def selection(self, value):
 *         logging.info("---Check and set sub area selection...")
 *         SA = self._acquisition['SceneArea']             # <<<<<<<<<<<<<<
 *         SC = self._acquisition['SceneCenter']
 *         BC = self._acquisition['BeamCenter']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_SceneArea); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 477, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_SA = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":478
 *         logging.info("---Check and set sub area selection...")
 *         SA = self._acquisition['SceneArea']
 *         SC = self._acquisition['SceneCenter']             # <<<<<<<<<<<<<<
 *         BC = self._acquisition['BeamCenter']
 *         ES = self._acquisition['EchoSize']
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 478, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_SceneCenter); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 478, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_SC = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":479
 *         SA = self._acquisition['SceneArea']
 *         SC = self._acquisition['SceneCenter']
 *         BC = self._acquisition['BeamCenter']             # <<<<<<<<<<<<<<
 *         ES = self._acquisition['EchoSize']
 *         Ad = self._acquisition['Ad']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 479, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_BeamCenter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 479, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_BC = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":480
 *         SC = self._acquisition['SceneCenter']
 *         BC = self._acquisition['BeamCenter']
 *         ES = self._acquisition['EchoSize']             # <<<<<<<<<<<<<<
 *         Ad = self._acquisition['Ad']
 *         Aon = self._acquisition['Aon']
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 480, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_EchoSize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 480, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_ES = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":481
 *         BC = self._acquisition['BeamCenter']
 *         ES = self._acquisition['EchoSize']
 *         Ad = self._acquisition['Ad']             # <<<<<<<<<<<<<<
 *         Aon = self._acquisition['Aon']
 *         As = self._acquisition['As']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 481, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Ad); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 481, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Ad = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":482
 *         ES = self._acquisition['EchoSize']
 *         Ad = self._acquisition['Ad']
 *         Aon = self._acquisition['Aon']             # <<<<<<<<<<<<<<
 *         As = self._acquisition['As']
 *         Ar = self._acquisition['Ar']
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 482, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_Aon); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 482, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_Aon = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":483
 *         Ad = self._acquisition['Ad']
 *         Aon = self._acquisition['Aon']
 *         As = self._acquisition['As']             # <<<<<<<<<<<<<<
 *         Ar = self._acquisition['Ar']
 *         Fsr = self._params['Fsr']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_As); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_As = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":484
 *         Aon = self._acquisition['Aon']
 *         As = self._acquisition['As']
 *         Ar = self._acquisition['Ar']             # <<<<<<<<<<<<<<
 *         Fsr = self._params['Fsr']
 *         Fsa = self._params['Fsa']
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 484, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_Ar); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 484, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_Ar = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":485
 *         As = self._acquisition['As']
 *         Ar = self._acquisition['Ar']
 *         Fsr = self._params['Fsr']             # <<<<<<<<<<<<<<
 *         Fsa = self._params['Fsa']
 *         Tsa = self._params['Tsa']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Fsr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Fsr = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":486
 *         Ar = self._acquisition['Ar']
 *         Fsr = self._params['Fsr']
 *         Fsa = self._params['Fsa']             # <<<<<<<<<<<<<<
 *         Tsa = self._params['Tsa']
 *         Tsr = self._params['Tsr']
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_Fsa); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_Fsa = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":487
 *         Fsr = self._params['Fsr']
 *         Fsa = self._params['Fsa']
 *         Tsa = self._params['Tsa']             # <<<<<<<<<<<<<<
 *         Tsr = self._params['Tsr']
 *         Na = self._params['Na']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Tsa); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 487, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Tsa = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":488
 *         Fsa = self._params['Fsa']
 *         Tsa = self._params['Tsa']
 *         Tsr = self._params['Tsr']             # <<<<<<<<<<<<<<
 *         Na = self._params['Na']
 *         Nr = self._params['Nr']
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 488, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_Tsr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 488, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_Tsr = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":489
 *         Tsa = self._params['Tsa']
 *         Tsr = self._params['Tsr']
 *         Na = self._params['Na']             # <<<<<<<<<<<<<<
 *         Nr = self._params['Nr']
 *         Rnear = self._params['Rnear']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 489, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Na); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 489, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Na = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":490
 *         Tsr = self._params['Tsr']
 *         Na = self._params['Na']
 *         Nr = self._params['Nr']             # <<<<<<<<<<<<<<
 *         Rnear = self._params['Rnear']
 *         Rfar = self._params['Rfar']
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 490, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_Nr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 490, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_Nr = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":491
 *         Na = self._params['Na']
 *         Nr = self._params['Nr']
 *         Rnear = self._params['Rnear']             # <<<<<<<<<<<<<<
 *         Rfar = self._params['Rfar']
 *         FPr = self._params['FPr']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Rnear); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Rnear = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":492
 *         Nr = self._params['Nr']
 *         Rnear = self._params['Rnear']
 *         Rfar = self._params['Rfar']             # <<<<<<<<<<<<<<
 *         FPr = self._params['FPr']
 *         H = self._sensor['H']
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 492, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_Rfar); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 492, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_Rfar = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":493
 *         Rnear = self._params['Rnear']
 *         Rfar = self._params['Rfar']
 *         FPr = self._params['FPr']             # <<<<<<<<<<<<<<
 *         H = self._sensor['H']
 *         Vr = self._sensor['Vr']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 493, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_FPr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 493, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_FPr = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":494
 *         Rfar = self._params['Rfar']
 *         FPr = self._params['FPr']
 *         H = self._sensor['H']             # <<<<<<<<<<<<<<
 *         Vr = self._sensor['Vr']
 *         Tp = self._sensor['Tp']
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 494, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_H); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 494, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_H = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":495
 *         FPr = self._params['FPr']
 *         H = self._sensor['H']
 *         Vr = self._sensor['Vr']             # <<<<<<<<<<<<<<
 *         Tp = self._sensor['Tp']
 *         Wl = self._sensor['Wl']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 495, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Vr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 495, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Vr = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":496
 *         H = self._sensor['H']
 *         Vr = self._sensor['Vr']
 *         Tp = self._sensor['Tp']             # <<<<<<<<<<<<<<
 *         Wl = self._sensor['Wl']
 *         La = self._sensor['La']
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 496, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_Tp); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 496, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_Tp = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":497
 *         Vr = self._sensor['Vr']
 *         Tp = self._sensor['Tp']
 *         Wl = self._sensor['Wl']             # <<<<<<<<<<<<<<
 *         La = self._sensor['La']
 *         GM = self._params['GeometryMode']
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_Wl); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 497, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_Wl = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":498
 *         Tp = self._sensor['Tp']
 *         Wl = self._sensor['Wl']
 *         La = self._sensor['La']             # <<<<<<<<<<<<<<
 *         GM = self._params['GeometryMode']
 *         Lsar = self._params['Lsar']
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 498, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_La); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 498, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_La = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":499
 *         Wl = self._sensor['Wl']
 *         La = self._sensor['La']
 *         GM = self._params['GeometryMode']             # <<<<<<<<<<<<<<
 *         Lsar = self._params['Lsar']
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 499, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_GeometryMode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 499, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_GM = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":500
 *         La = self._sensor['La']
 *         GM = self._params['GeometryMode']
 *         Lsar = self._params['Lsar']             # <<<<<<<<<<<<<<
 * 
 *         if value is None:
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 500, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_t_3, __pyx_n_u_Lsar); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 500, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_Lsar = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":502
 *         Lsar = self._params['Lsar']
 * 
 *         if value is None:             # <<<<<<<<<<<<<<
 *             SubSA = SA
 *             SubES = ES
 */
  __pyx_t_4 = (__pyx_v_value == Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "torchsar/misc/sarplat.py":503
 * 
 *         if value is None:
 *             SubSA = SA             # <<<<<<<<<<<<<<
 *             SubES = ES
 *             SubEA = [0, 0]
 */
    __Pyx_INCREF(__pyx_v_SA);
    __pyx_v_SubSA = __pyx_v_SA;

    /* "torchsar/misc/sarplat.py":504
 *         if value is None:
 *             SubSA = SA
 *             SubES = ES             # <<<<<<<<<<<<<<
 *             SubEA = [0, 0]
 *         else:
 */
    __Pyx_INCREF(__pyx_v_ES);
    __pyx_v_SubES = __pyx_v_ES;

    /* "torchsar/misc/sarplat.py":505
 *             SubSA = SA
 *             SubES = ES
 *             SubEA = [0, 0]             # <<<<<<<<<<<<<<
 *         else:
 *             if value['SubSceneArea'] is not None:
 */
    __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 505, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_int_0);
    __Pyx_GIVEREF(__pyx_int_0);
    PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_0);
    __Pyx_INCREF(__pyx_int_0);
    __Pyx_GIVEREF(__pyx_int_0);
    PyList_SET_ITEM(__pyx_t_1, 1, __pyx_int_0);
    __pyx_v_SubEA = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "torchsar/misc/sarplat.py":502
 *         Lsar = self._params['Lsar']
 * 
 *         if value is None:             # <<<<<<<<<<<<<<
 *             SubSA = SA
 *             SubES = ES
 */
    goto __pyx_L3;
  }

  /* "torchsar/misc/sarplat.py":507
 *             SubEA = [0, 0]
 *         else:
 *             if value['SubSceneArea'] is not None:             # <<<<<<<<<<<<<<
 *                 SubSA = value['SubSceneArea']
 *             else:
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_value, __pyx_n_u_SubSceneArea); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 507, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = (__pyx_t_1 != Py_None);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_4 = (__pyx_t_5 != 0);
    if (__pyx_t_4) {

      /* "torchsar/misc/sarplat.py":508
 *         else:
 *             if value['SubSceneArea'] is not None:
 *                 SubSA = value['SubSceneArea']             # <<<<<<<<<<<<<<
 *             else:
 *                 SubSA = SA
 */
      __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_value, __pyx_n_u_SubSceneArea); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 508, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_SubSA = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "torchsar/misc/sarplat.py":507
 *             SubEA = [0, 0]
 *         else:
 *             if value['SubSceneArea'] is not None:             # <<<<<<<<<<<<<<
 *                 SubSA = value['SubSceneArea']
 *             else:
 */
      goto __pyx_L4;
    }

    /* "torchsar/misc/sarplat.py":510
 *                 SubSA = value['SubSceneArea']
 *             else:
 *                 SubSA = SA             # <<<<<<<<<<<<<<
 *             if value['SubEchoSize'] is not None:
 *                 SubES = value['SubEchoSize']
 */
    /*else*/ {
      __Pyx_INCREF(__pyx_v_SA);
      __pyx_v_SubSA = __pyx_v_SA;
    }
    __pyx_L4:;

    /* "torchsar/misc/sarplat.py":511
 *             else:
 *                 SubSA = SA
 *             if value['SubEchoSize'] is not None:             # <<<<<<<<<<<<<<
 *                 SubES = value['SubEchoSize']
 *             else:
 */
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_value, __pyx_n_u_SubEchoSize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 511, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = (__pyx_t_1 != Py_None);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = (__pyx_t_4 != 0);
    if (__pyx_t_5) {

      /* "torchsar/misc/sarplat.py":512
 *                 SubSA = SA
 *             if value['SubEchoSize'] is not None:
 *                 SubES = value['SubEchoSize']             # <<<<<<<<<<<<<<
 *             else:
 *                 SubES = ES
 */
      __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_value, __pyx_n_u_SubEchoSize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 512, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_SubES = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "torchsar/misc/sarplat.py":511
 *             else:
 *                 SubSA = SA
 *             if value['SubEchoSize'] is not None:             # <<<<<<<<<<<<<<
 *                 SubES = value['SubEchoSize']
 *             else:
 */
      goto __pyx_L5;
    }

    /* "torchsar/misc/sarplat.py":514
 *                 SubES = value['SubEchoSize']
 *             else:
 *                 SubES = ES             # <<<<<<<<<<<<<<
 *             if value['SubEchoAnchor'] is not None:
 *                 SubEA = value['SubEchoAnchor']
 */
    /*else*/ {
      __Pyx_INCREF(__pyx_v_ES);
      __pyx_v_SubES = __pyx_v_ES;
    }
    __pyx_L5:;

    /* "torchsar/misc/sarplat.py":515
 *             else:
 *                 SubES = ES
 *             if value['SubEchoAnchor'] is not None:             # <<<<<<<<<<<<<<
 *                 SubEA = value['SubEchoAnchor']
 *             else:
 */
    __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_value, __pyx_n_u_SubEchoAnchor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 515, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = (__pyx_t_1 != Py_None);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_4 = (__pyx_t_5 != 0);
    if (__pyx_t_4) {

      /* "torchsar/misc/sarplat.py":516
 *                 SubES = ES
 *             if value['SubEchoAnchor'] is not None:
 *                 SubEA = value['SubEchoAnchor']             # <<<<<<<<<<<<<<
 *             else:
 *                 SubEA = [0, 0]
 */
      __pyx_t_1 = __Pyx_PyObject_Dict_GetItem(__pyx_v_value, __pyx_n_u_SubEchoAnchor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 516, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_SubEA = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "torchsar/misc/sarplat.py":515
 *             else:
 *                 SubES = ES
 *             if value['SubEchoAnchor'] is not None:             # <<<<<<<<<<<<<<
 *                 SubEA = value['SubEchoAnchor']
 *             else:
 */
      goto __pyx_L6;
    }

    /* "torchsar/misc/sarplat.py":518
 *                 SubEA = value['SubEchoAnchor']
 *             else:
 *                 SubEA = [0, 0]             # <<<<<<<<<<<<<<
 * 
 *         SubNa, SubNr = SubES
 */
    /*else*/ {
      __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 518, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_INCREF(__pyx_int_0);
      __Pyx_GIVEREF(__pyx_int_0);
      PyList_SET_ITEM(__pyx_t_1, 0, __pyx_int_0);
      __Pyx_INCREF(__pyx_int_0);
      __Pyx_GIVEREF(__pyx_int_0);
      PyList_SET_ITEM(__pyx_t_1, 1, __pyx_int_0);
      __pyx_v_SubEA = __pyx_t_1;
      __pyx_t_1 = 0;
    }
    __pyx_L6:;
  }
  __pyx_L3:;

  /* "torchsar/misc/sarplat.py":520
 *                 SubEA = [0, 0]
 * 
 *         SubNa, SubNr = SubES             # <<<<<<<<<<<<<<
 *         # SubRnear = SubEA[1] * ((Rfar - Rnear) / float(Nr)) + Rnear
 *         # SubRfar = (SubEA[1] + SubNr) * ((Rfar - Rnear) / float(Nr)) + Rnear
 */
  if ((likely(PyTuple_CheckExact(__pyx_v_SubES))) || (PyList_CheckExact(__pyx_v_SubES))) {
    PyObject* sequence = __pyx_v_SubES;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 520, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_3);
    #else
    __pyx_t_1 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 520, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 520, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
  } else {
    Py_ssize_t index = -1;
    __pyx_t_2 = PyObject_GetIter(__pyx_v_SubES); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 520, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_6 = Py_TYPE(__pyx_t_2)->tp_iternext;
    index = 0; __pyx_t_1 = __pyx_t_6(__pyx_t_2); if (unlikely(!__pyx_t_1)) goto __pyx_L7_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    index = 1; __pyx_t_3 = __pyx_t_6(__pyx_t_2); if (unlikely(!__pyx_t_3)) goto __pyx_L7_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_2), 2) < 0) __PYX_ERR(0, 520, __pyx_L1_error)
    __pyx_t_6 = NULL;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    goto __pyx_L8_unpacking_done;
    __pyx_L7_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_6 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 520, __pyx_L1_error)
    __pyx_L8_unpacking_done:;
  }
  __pyx_v_SubNa = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_v_SubNr = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":523
 *         # SubRnear = SubEA[1] * ((Rfar - Rnear) / float(Nr)) + Rnear
 *         # SubRfar = (SubEA[1] + SubNr) * ((Rfar - Rnear) / float(Nr)) + Rnear
 *         SubRnear = min_slant_range(Rnear, Fsr, SubEA[1])             # <<<<<<<<<<<<<<
 *         SubRfar = min_slant_range(Rnear, Fsr, SubEA[1] + SubES[1])
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_min_slant_range); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 523, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_SubEA, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 523, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_Rnear, __pyx_v_Fsr, __pyx_t_2};
    __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 523, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_Rnear, __pyx_v_Fsr, __pyx_t_2};
    __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 523, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  } else
  #endif
  {
    __pyx_t_9 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 523, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    if (__pyx_t_7) {
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_7); __pyx_t_7 = NULL;
    }
    __Pyx_INCREF(__pyx_v_Rnear);
    __Pyx_GIVEREF(__pyx_v_Rnear);
    PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_v_Rnear);
    __Pyx_INCREF(__pyx_v_Fsr);
    __Pyx_GIVEREF(__pyx_v_Fsr);
    PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_v_Fsr);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_8, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_9, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 523, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_SubRnear = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":524
 *         # SubRfar = (SubEA[1] + SubNr) * ((Rfar - Rnear) / float(Nr)) + Rnear
 *         SubRnear = min_slant_range(Rnear, Fsr, SubEA[1])
 *         SubRfar = min_slant_range(Rnear, Fsr, SubEA[1] + SubES[1])             # <<<<<<<<<<<<<<
 * 
 *         SubAon = np.arccos(H / SubRnear)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_min_slant_range); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_9 = __Pyx_GetItemInt(__pyx_v_SubEA, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_SubES, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = PyNumber_Add(__pyx_t_9, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_Rnear, __pyx_v_Fsr, __pyx_t_7};
    __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 524, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_v_Rnear, __pyx_v_Fsr, __pyx_t_7};
    __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 524, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  {
    __pyx_t_9 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 524, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    if (__pyx_t_2) {
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_2); __pyx_t_2 = NULL;
    }
    __Pyx_INCREF(__pyx_v_Rnear);
    __Pyx_GIVEREF(__pyx_v_Rnear);
    PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_8, __pyx_v_Rnear);
    __Pyx_INCREF(__pyx_v_Fsr);
    __Pyx_GIVEREF(__pyx_v_Fsr);
    PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_8, __pyx_v_Fsr);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_9, 2+__pyx_t_8, __pyx_t_7);
    __pyx_t_7 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_9, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 524, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_SubRfar = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":526
 *         SubRfar = min_slant_range(Rnear, Fsr, SubEA[1] + SubES[1])
 * 
 *         SubAon = np.arccos(H / SubRnear)             # <<<<<<<<<<<<<<
 *         SubAbr = np.arccos(H / SubRfar) - SubAon
 *         SubAd = np.sign(SubAon) * (abs(SubAon) + SubAbr / 2.)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_arccos); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_v_H, __pyx_v_SubRnear); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
    }
  }
  __pyx_t_3 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_7, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 526, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_SubAon = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":527
 * 
 *         SubAon = np.arccos(H / SubRnear)
 *         SubAbr = np.arccos(H / SubRfar) - SubAon             # <<<<<<<<<<<<<<
 *         SubAd = np.sign(SubAon) * (abs(SubAon) + SubAbr / 2.)
 *         SubFPr = np.sqrt(SubRnear**2 + SubRfar**2 - 2. * SubRnear * SubRfar * np.cos(SubAbr))
 */
  __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_np); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 527, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_arccos); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 527, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyNumber_Divide(__pyx_v_H, __pyx_v_SubRfar); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 527, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_3 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_7, __pyx_t_9) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_9);
  __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 527, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Subtract(__pyx_t_3, __pyx_v_SubAon); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 527, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_SubAbr = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":528
 *         SubAon = np.arccos(H / SubRnear)
 *         SubAbr = np.arccos(H / SubRfar) - SubAon
 *         SubAd = np.sign(SubAon) * (abs(SubAon) + SubAbr / 2.)             # <<<<<<<<<<<<<<
 *         SubFPr = np.sqrt(SubRnear**2 + SubRfar**2 - 2. * SubRnear * SubRfar * np.cos(SubAbr))
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 528, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_sign); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 528, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_3, __pyx_v_SubAon) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_v_SubAon);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 528, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyNumber_Absolute(__pyx_v_SubAon); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 528, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_3 = __Pyx_PyFloat_TrueDivideObjC(__pyx_v_SubAbr, __pyx_float_2_, 2., 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 528, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = PyNumber_Add(__pyx_t_9, __pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 528, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Multiply(__pyx_t_1, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 528, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_SubAd = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":529
 *         SubAbr = np.arccos(H / SubRfar) - SubAon
 *         SubAd = np.sign(SubAon) * (abs(SubAon) + SubAbr / 2.)
 *         SubFPr = np.sqrt(SubRnear**2 + SubRfar**2 - 2. * SubRnear * SubRfar * np.cos(SubAbr))             # <<<<<<<<<<<<<<
 * 
 *         SubTa = SubNa * Tsa
 */
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_np); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyNumber_Power(__pyx_v_SubRnear, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = PyNumber_Power(__pyx_v_SubRfar, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_2 = PyNumber_Add(__pyx_t_7, __pyx_t_9); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = PyNumber_Multiply(__pyx_float_2_, __pyx_v_SubRnear); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_7 = PyNumber_Multiply(__pyx_t_9, __pyx_v_SubRfar); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_np); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_10, __pyx_n_s_cos); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_11))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_11);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_11, function);
    }
  }
  __pyx_t_9 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_11, __pyx_t_10, __pyx_v_SubAbr) : __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_v_SubAbr);
  __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = PyNumber_Multiply(__pyx_t_7, __pyx_t_9); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = PyNumber_Subtract(__pyx_t_2, __pyx_t_11); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_3 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_11, __pyx_t_9) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_SubFPr = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":531
 *         SubFPr = np.sqrt(SubRnear**2 + SubRfar**2 - 2. * SubRnear * SubRfar * np.cos(SubAbr))
 * 
 *         SubTa = SubNa * Tsa             # <<<<<<<<<<<<<<
 *         taSub = np.linspace(-SubTa / 2.0, SubTa / 2.0, SubNa)
 *         faSub = np.linspace(-Fsa / 2.0, Fsa / 2.0, SubNa)
 */
  __pyx_t_3 = PyNumber_Multiply(__pyx_v_SubNa, __pyx_v_Tsa); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 531, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_SubTa = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":532
 * 
 *         SubTa = SubNa * Tsa
 *         taSub = np.linspace(-SubTa / 2.0, SubTa / 2.0, SubNa)             # <<<<<<<<<<<<<<
 *         faSub = np.linspace(-Fsa / 2.0, Fsa / 2.0, SubNa)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 532, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_linspace); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 532, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Negative(__pyx_v_SubTa); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 532, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_11 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_1, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 532, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyFloat_TrueDivideObjC(__pyx_v_SubTa, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 532, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_9)) {
    PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_t_11, __pyx_t_1, __pyx_v_SubNa};
    __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 532, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
    PyObject *__pyx_temp[4] = {__pyx_t_2, __pyx_t_11, __pyx_t_1, __pyx_v_SubNa};
    __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 532, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else
  #endif
  {
    __pyx_t_7 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 532, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    if (__pyx_t_2) {
      __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_2); __pyx_t_2 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_11);
    PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_8, __pyx_t_11);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_8, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_SubNa);
    __Pyx_GIVEREF(__pyx_v_SubNa);
    PyTuple_SET_ITEM(__pyx_t_7, 2+__pyx_t_8, __pyx_v_SubNa);
    __pyx_t_11 = 0;
    __pyx_t_1 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_7, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 532, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  }
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_v_taSub = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":533
 *         SubTa = SubNa * Tsa
 *         taSub = np.linspace(-SubTa / 2.0, SubTa / 2.0, SubNa)
 *         faSub = np.linspace(-Fsa / 2.0, Fsa / 2.0, SubNa)             # <<<<<<<<<<<<<<
 * 
 *         SubRsc = 0.5 * np.sqrt(2. * SubRfar**2 + 2. * SubRnear**2 - SubFPr**2 + EPS)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_np); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_linspace); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = PyNumber_Negative(__pyx_v_Fsa); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_1 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_9, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = __Pyx_PyFloat_TrueDivideObjC(__pyx_v_Fsa, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_11 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_7)) {
    PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_t_1, __pyx_t_9, __pyx_v_SubNa};
    __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 533, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
    PyObject *__pyx_temp[4] = {__pyx_t_11, __pyx_t_1, __pyx_t_9, __pyx_v_SubNa};
    __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 533, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  } else
  #endif
  {
    __pyx_t_2 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 533, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (__pyx_t_11) {
      __Pyx_GIVEREF(__pyx_t_11); PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_11); __pyx_t_11 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0+__pyx_t_8, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_9);
    PyTuple_SET_ITEM(__pyx_t_2, 1+__pyx_t_8, __pyx_t_9);
    __Pyx_INCREF(__pyx_v_SubNa);
    __Pyx_GIVEREF(__pyx_v_SubNa);
    PyTuple_SET_ITEM(__pyx_t_2, 2+__pyx_t_8, __pyx_v_SubNa);
    __pyx_t_1 = 0;
    __pyx_t_9 = 0;
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 533, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_faSub = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":535
 *         faSub = np.linspace(-Fsa / 2.0, Fsa / 2.0, SubNa)
 * 
 *         SubRsc = 0.5 * np.sqrt(2. * SubRfar**2 + 2. * SubRnear**2 - SubFPr**2 + EPS)             # <<<<<<<<<<<<<<
 *         SubRs0 = SubRsc * np.cos(Ar)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_np); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyNumber_Power(__pyx_v_SubRfar, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = PyNumber_Multiply(__pyx_float_2_, __pyx_t_7); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyNumber_Power(__pyx_v_SubRnear, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = PyNumber_Multiply(__pyx_float_2_, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyNumber_Add(__pyx_t_9, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Power(__pyx_v_SubFPr, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_9 = PyNumber_Subtract(__pyx_t_7, __pyx_t_1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_EPS); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = PyNumber_Add(__pyx_t_9, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_7);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyNumber_Multiply(__pyx_float_0_5, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_SubRsc = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":536
 * 
 *         SubRsc = 0.5 * np.sqrt(2. * SubRfar**2 + 2. * SubRnear**2 - SubFPr**2 + EPS)
 *         SubRs0 = SubRsc * np.cos(Ar)             # <<<<<<<<<<<<<<
 * 
 *         SubYsc = SubRsc * np.sin(Ar)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 536, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_cos); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 536, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
    }
  }
  __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_3, __pyx_v_Ar) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_v_Ar);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 536, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyNumber_Multiply(__pyx_v_SubRsc, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 536, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_SubRs0 = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "torchsar/misc/sarplat.py":538
 *         SubRs0 = SubRsc * np.cos(Ar)
 * 
 *         SubYsc = SubRsc * np.sin(Ar)             # <<<<<<<<<<<<<<
 *         SubXsc = np.sqrt(np.abs(SubRs0**2 - H**2))
 *         SubSC = [SubXsc, SubYsc, 0.0]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 538, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_sin); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 538, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_7 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_v_Ar) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_Ar);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 538, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Multiply(__pyx_v_SubRsc, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 538, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_SubYsc = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":539
 * 
 *         SubYsc = SubRsc * np.sin(Ar)
 *         SubXsc = np.sqrt(np.abs(SubRs0**2 - H**2))             # <<<<<<<<<<<<<<
 *         SubSC = [SubXsc, SubYsc, 0.0]
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_np); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_abs); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Power(__pyx_v_SubRs0, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_11 = PyNumber_Power(__pyx_v_H, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_10 = PyNumber_Subtract(__pyx_t_1, __pyx_t_11); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_9))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_9);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_9, function);
    }
  }
  __pyx_t_7 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_11, __pyx_t_10) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_9)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 539, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_SubXsc = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":540
 *         SubYsc = SubRsc * np.sin(Ar)
 *         SubXsc = np.sqrt(np.abs(SubRs0**2 - H**2))
 *         SubSC = [SubXsc, SubYsc, 0.0]             # <<<<<<<<<<<<<<
 * 
 *         SubRbc = H / (np.sin(Ad) + EPS)
 */
  __pyx_t_3 = PyList_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 540, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_SubXsc);
  __Pyx_GIVEREF(__pyx_v_SubXsc);
  PyList_SET_ITEM(__pyx_t_3, 0, __pyx_v_SubXsc);
  __Pyx_INCREF(__pyx_v_SubYsc);
  __Pyx_GIVEREF(__pyx_v_SubYsc);
  PyList_SET_ITEM(__pyx_t_3, 1, __pyx_v_SubYsc);
  __Pyx_INCREF(__pyx_float_0_0);
  __Pyx_GIVEREF(__pyx_float_0_0);
  PyList_SET_ITEM(__pyx_t_3, 2, __pyx_float_0_0);
  __pyx_v_SubSC = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":542
 *         SubSC = [SubXsc, SubYsc, 0.0]
 * 
 *         SubRbc = H / (np.sin(Ad) + EPS)             # <<<<<<<<<<<<<<
 *         SubRb0 = SubRbc * np.cos(Ar)
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_sin); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
    }
  }
  __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_2, __pyx_v_Ad) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_v_Ad);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_EPS); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_2 = PyNumber_Add(__pyx_t_3, __pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyNumber_Divide(__pyx_v_H, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_SubRbc = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "torchsar/misc/sarplat.py":543
 * 
 *         SubRbc = H / (np.sin(Ad) + EPS)
 *         SubRb0 = SubRbc * np.cos(Ar)             # <<<<<<<<<<<<<<
 * 
 *         SubYbc = SubRbc * np.sin(Ar)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 543, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_cos); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 543, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_7 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_v_Ar) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_Ar);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 543, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Multiply(__pyx_v_SubRbc, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 543, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_SubRb0 = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":545
 *         SubRb0 = SubRbc * np.cos(Ar)
 * 
 *         SubYbc = SubRbc * np.sin(Ar)             # <<<<<<<<<<<<<<
 *         SubXbc = np.sqrt(np.abs(SubRb0**2 - H**2))
 *         SubBC = [SubXbc, SubYbc, 0.0]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_np); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 545, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_sin); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 545, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_7, __pyx_v_Ar) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_Ar);
  __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 545, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyNumber_Multiply(__pyx_v_SubRbc, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 545, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_SubYbc = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":546
 * 
 *         SubYbc = SubRbc * np.sin(Ar)
 *         SubXbc = np.sqrt(np.abs(SubRb0**2 - H**2))             # <<<<<<<<<<<<<<
 *         SubBC = [SubXbc, SubYbc, 0.0]
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 546, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 546, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_n_s_np); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 546, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_9, __pyx_n_s_abs); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 546, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_9 = PyNumber_Power(__pyx_v_SubRb0, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 546, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_11 = PyNumber_Power(__pyx_v_H, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 546, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_1 = PyNumber_Subtract(__pyx_t_9, __pyx_t_11); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 546, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_10, function);
    }
  }
  __pyx_t_3 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_11, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 546, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
    }
  }
  __pyx_t_2 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_10, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 546, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_SubXbc = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":547
 *         SubYbc = SubRbc * np.sin(Ar)
 *         SubXbc = np.sqrt(np.abs(SubRb0**2 - H**2))
 *         SubBC = [SubXbc, SubYbc, 0.0]             # <<<<<<<<<<<<<<
 * 
 *         SubYnear = SubRnear * np.sin(Ar)
 */
  __pyx_t_2 = PyList_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 547, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_SubXbc);
  __Pyx_GIVEREF(__pyx_v_SubXbc);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_v_SubXbc);
  __Pyx_INCREF(__pyx_v_SubYbc);
  __Pyx_GIVEREF(__pyx_v_SubYbc);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_v_SubYbc);
  __Pyx_INCREF(__pyx_float_0_0);
  __Pyx_GIVEREF(__pyx_float_0_0);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_float_0_0);
  __pyx_v_SubBC = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":549
 *         SubBC = [SubXbc, SubYbc, 0.0]
 * 
 *         SubYnear = SubRnear * np.sin(Ar)             # <<<<<<<<<<<<<<
 *         SubYfar = SubRfar * np.sin(Ar)
 *         SubXnear = np.sqrt((SubRnear ** 2 - H**2) - SubYnear ** 2 + EPS)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_np); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 549, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_sin); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 549, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_7, __pyx_v_Ar) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_Ar);
  __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 549, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Multiply(__pyx_v_SubRnear, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 549, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_SubYnear = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":550
 * 
 *         SubYnear = SubRnear * np.sin(Ar)
 *         SubYfar = SubRfar * np.sin(Ar)             # <<<<<<<<<<<<<<
 *         SubXnear = np.sqrt((SubRnear ** 2 - H**2) - SubYnear ** 2 + EPS)
 *         SubXfar = np.sqrt((SubRfar ** 2 - H**2) - SubYfar ** 2 + EPS)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 550, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_sin); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 550, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
    }
  }
  __pyx_t_3 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_2, __pyx_v_Ar) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_v_Ar);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 550, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyNumber_Multiply(__pyx_v_SubRfar, __pyx_t_3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 550, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_SubYfar = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "torchsar/misc/sarplat.py":551
 *         SubYnear = SubRnear * np.sin(Ar)
 *         SubYfar = SubRfar * np.sin(Ar)
 *         SubXnear = np.sqrt((SubRnear ** 2 - H**2) - SubYnear ** 2 + EPS)             # <<<<<<<<<<<<<<
 *         SubXfar = np.sqrt((SubRfar ** 2 - H**2) - SubYfar ** 2 + EPS)
 *         SubSA = [SubXnear - SubXsc, SubXfar - SubXsc, -SubTa * Vr / 2.0, SubTa * Vr / 2.0]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 551, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 551, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Power(__pyx_v_SubRnear, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 551, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_10 = PyNumber_Power(__pyx_v_H, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 551, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_1 = PyNumber_Subtract(__pyx_t_3, __pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 551, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyNumber_Power(__pyx_v_SubYnear, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 551, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_3 = PyNumber_Subtract(__pyx_t_1, __pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 551, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_EPS); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 551, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_1 = PyNumber_Add(__pyx_t_3, __pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 551, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_7 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_10, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 551, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_SubXnear = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "torchsar/misc/sarplat.py":552
 *         SubYfar = SubRfar * np.sin(Ar)
 *         SubXnear = np.sqrt((SubRnear ** 2 - H**2) - SubYnear ** 2 + EPS)
 *         SubXfar = np.sqrt((SubRfar ** 2 - H**2) - SubYfar ** 2 + EPS)             # <<<<<<<<<<<<<<
 *         SubSA = [SubXnear - SubXsc, SubXfar - SubXsc, -SubTa * Vr / 2.0, SubTa * Vr / 2.0]
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 552, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_sqrt); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 552, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyNumber_Power(__pyx_v_SubRfar, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 552, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_10 = PyNumber_Power(__pyx_v_H, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 552, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_3 = PyNumber_Subtract(__pyx_t_2, __pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 552, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyNumber_Power(__pyx_v_SubYfar, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 552, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_2 = PyNumber_Subtract(__pyx_t_3, __pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 552, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_10, __pyx_n_s_EPS); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 552, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_3 = PyNumber_Add(__pyx_t_2, __pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 552, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_7 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_10, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 552, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_SubXfar = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "torchsar/misc/sarplat.py":553
 *         SubXnear = np.sqrt((SubRnear ** 2 - H**2) - SubYnear ** 2 + EPS)
 *         SubXfar = np.sqrt((SubRfar ** 2 - H**2) - SubYfar ** 2 + EPS)
 *         SubSA = [SubXnear - SubXsc, SubXfar - SubXsc, -SubTa * Vr / 2.0, SubTa * Vr / 2.0]             # <<<<<<<<<<<<<<
 * 
 *         SubFPr0 = H / np.tan(Ad + SubAbr / 2.) - H / np.tan(Ad)
 */
  __pyx_t_7 = PyNumber_Subtract(__pyx_v_SubXnear, __pyx_v_SubXsc); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = PyNumber_Subtract(__pyx_v_SubXfar, __pyx_v_SubXsc); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyNumber_Negative(__pyx_v_SubTa); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_10 = PyNumber_Multiply(__pyx_t_3, __pyx_v_Vr); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_10, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyNumber_Multiply(__pyx_v_SubTa, __pyx_v_Vr); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_2 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_10, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyList_New(4); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_7);
  PyList_SET_ITEM(__pyx_t_10, 0, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_1);
  PyList_SET_ITEM(__pyx_t_10, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyList_SET_ITEM(__pyx_t_10, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyList_SET_ITEM(__pyx_t_10, 3, __pyx_t_2);
  __pyx_t_7 = 0;
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_t_2 = 0;
  __Pyx_DECREF_SET(__pyx_v_SubSA, __pyx_t_10);
  __pyx_t_10 = 0;

  /* "torchsar/misc/sarplat.py":555
 *         SubSA = [SubXnear - SubXsc, SubXfar - SubXsc, -SubTa * Vr / 2.0, SubTa * Vr / 2.0]
 * 
 *         SubFPr0 = H / np.tan(Ad + SubAbr / 2.) - H / np.tan(Ad)             # <<<<<<<<<<<<<<
 *         SubFPr1 = H / np.tan(Ad - SubAbr / 2.) - H / np.tan(Ad)
 *         SubBA = [SubFPr0, SubFPr1, -SubTa * Vr / 2.0, SubTa * Vr / 2.0]
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_tan); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyFloat_TrueDivideObjC(__pyx_v_SubAbr, __pyx_float_2_, 2., 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyNumber_Add(__pyx_v_Ad, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_10 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyNumber_Divide(__pyx_v_H, __pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_tan); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_10 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_v_Ad) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_Ad);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyNumber_Divide(__pyx_v_H, __pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyNumber_Subtract(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_SubFPr0 = __pyx_t_10;
  __pyx_t_10 = 0;

  /* "torchsar/misc/sarplat.py":556
 * 
 *         SubFPr0 = H / np.tan(Ad + SubAbr / 2.) - H / np.tan(Ad)
 *         SubFPr1 = H / np.tan(Ad - SubAbr / 2.) - H / np.tan(Ad)             # <<<<<<<<<<<<<<
 *         SubBA = [SubFPr0, SubFPr1, -SubTa * Vr / 2.0, SubTa * Vr / 2.0]
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 556, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_tan); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 556, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyFloat_TrueDivideObjC(__pyx_v_SubAbr, __pyx_float_2_, 2., 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 556, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyNumber_Subtract(__pyx_v_Ad, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 556, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_2)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_10 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_2, __pyx_t_1) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 556, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyNumber_Divide(__pyx_v_H, __pyx_t_10); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 556, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 556, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_tan); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 556, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_10 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_v_Ad) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_Ad);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 556, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyNumber_Divide(__pyx_v_H, __pyx_t_10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 556, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = PyNumber_Subtract(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 556, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_SubFPr1 = __pyx_t_10;
  __pyx_t_10 = 0;

  /* "torchsar/misc/sarplat.py":557
 *         SubFPr0 = H / np.tan(Ad + SubAbr / 2.) - H / np.tan(Ad)
 *         SubFPr1 = H / np.tan(Ad - SubAbr / 2.) - H / np.tan(Ad)
 *         SubBA = [SubFPr0, SubFPr1, -SubTa * Vr / 2.0, SubTa * Vr / 2.0]             # <<<<<<<<<<<<<<
 * 
 *         tnearSub = SubRnear / C
 */
  __pyx_t_10 = PyNumber_Negative(__pyx_v_SubTa); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 557, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_2 = PyNumber_Multiply(__pyx_t_10, __pyx_v_Vr); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 557, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  __pyx_t_10 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_2, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 557, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyNumber_Multiply(__pyx_v_SubTa, __pyx_v_Vr); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 557, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_2, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 557, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyList_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 557, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_SubFPr0);
  __Pyx_GIVEREF(__pyx_v_SubFPr0);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_v_SubFPr0);
  __Pyx_INCREF(__pyx_v_SubFPr1);
  __Pyx_GIVEREF(__pyx_v_SubFPr1);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_v_SubFPr1);
  __Pyx_GIVEREF(__pyx_t_10);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_t_10);
  __Pyx_GIVEREF(__pyx_t_3);
  PyList_SET_ITEM(__pyx_t_2, 3, __pyx_t_3);
  __pyx_t_10 = 0;
  __pyx_t_3 = 0;
  __pyx_v_SubBA = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":559
 *         SubBA = [SubFPr0, SubFPr1, -SubTa * Vr / 2.0, SubTa * Vr / 2.0]
 * 
 *         tnearSub = SubRnear / C             # <<<<<<<<<<<<<<
 *         tfarSub = SubRfar / C
 *         tstartSub = 2.0 * tnearSub
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_C); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 559, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyNumber_Divide(__pyx_v_SubRnear, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 559, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_tnearSub = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":560
 * 
 *         tnearSub = SubRnear / C
 *         tfarSub = SubRfar / C             # <<<<<<<<<<<<<<
 *         tstartSub = 2.0 * tnearSub
 *         tendSub = 2.0 * tfarSub
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_C); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 560, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyNumber_Divide(__pyx_v_SubRfar, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 560, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_tfarSub = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":561
 *         tnearSub = SubRnear / C
 *         tfarSub = SubRfar / C
 *         tstartSub = 2.0 * tnearSub             # <<<<<<<<<<<<<<
 *         tendSub = 2.0 * tfarSub
 *         # tendSub = 2.0 * tfarSub + Tp
 */
  __pyx_t_2 = PyNumber_Multiply(__pyx_float_2_0, __pyx_v_tnearSub); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 561, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_tstartSub = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":562
 *         tfarSub = SubRfar / C
 *         tstartSub = 2.0 * tnearSub
 *         tendSub = 2.0 * tfarSub             # <<<<<<<<<<<<<<
 *         # tendSub = 2.0 * tfarSub + Tp
 * 
 */
  __pyx_t_2 = PyNumber_Multiply(__pyx_float_2_0, __pyx_v_tfarSub); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 562, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_tendSub = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":565
 *         # tendSub = 2.0 * tfarSub + Tp
 * 
 *         SubTr = tendSub - tstartSub             # <<<<<<<<<<<<<<
 * 
 *         if GM == 'BG':
 */
  __pyx_t_2 = PyNumber_Subtract(__pyx_v_tendSub, __pyx_v_tstartSub); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 565, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_SubTr = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":567
 *         SubTr = tendSub - tstartSub
 * 
 *         if GM == 'BG':             # <<<<<<<<<<<<<<
 *             Xc = SubBC[0]
 *             Yc = SubBC[1]
 */
  __pyx_t_4 = (__Pyx_PyUnicode_Equals(__pyx_v_GM, __pyx_n_u_BG, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 567, __pyx_L1_error)
  if (__pyx_t_4) {

    /* "torchsar/misc/sarplat.py":568
 * 
 *         if GM == 'BG':
 *             Xc = SubBC[0]             # <<<<<<<<<<<<<<
 *             Yc = SubBC[1]
 *             SubR0 = SubRb0
 */
    __pyx_t_2 = __Pyx_GetItemInt_List(__pyx_v_SubBC, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 568, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_Xc = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "torchsar/misc/sarplat.py":569
 *         if GM == 'BG':
 *             Xc = SubBC[0]
 *             Yc = SubBC[1]             # <<<<<<<<<<<<<<
 *             SubR0 = SubRb0
 *             t0Sub = 2 * SubRbc / C
 */
    __pyx_t_2 = __Pyx_GetItemInt_List(__pyx_v_SubBC, 1, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 569, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_Yc = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "torchsar/misc/sarplat.py":570
 *             Xc = SubBC[0]
 *             Yc = SubBC[1]
 *             SubR0 = SubRb0             # <<<<<<<<<<<<<<
 *             t0Sub = 2 * SubRbc / C
 *             trSub = np.linspace(tstartSub, tendSub, SubNr)
 */
    __Pyx_INCREF(__pyx_v_SubRb0);
    __pyx_v_SubR0 = __pyx_v_SubRb0;

    /* "torchsar/misc/sarplat.py":571
 *             Yc = SubBC[1]
 *             SubR0 = SubRb0
 *             t0Sub = 2 * SubRbc / C             # <<<<<<<<<<<<<<
 *             trSub = np.linspace(tstartSub, tendSub, SubNr)
 *             # trSub = np.linspace(-SubTr / 2.0, SubTr / 2.0, SubNr) + t0Sub
 */
    __pyx_t_2 = PyNumber_Multiply(__pyx_int_2, __pyx_v_SubRbc); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 571, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_C); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 571, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_10 = __Pyx_PyNumber_Divide(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 571, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_t0Sub = __pyx_t_10;
    __pyx_t_10 = 0;

    /* "torchsar/misc/sarplat.py":572
 *             SubR0 = SubRb0
 *             t0Sub = 2 * SubRbc / C
 *             trSub = np.linspace(tstartSub, tendSub, SubNr)             # <<<<<<<<<<<<<<
 *             # trSub = np.linspace(-SubTr / 2.0, SubTr / 2.0, SubNr) + t0Sub
 *             xminSub, xmaxSub, yminSub, ymaxSub = np.array(SubBA) + np.array([Xc, Xc, Yc, Yc])
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 572, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_linspace); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 572, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_8 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_tstartSub, __pyx_v_tendSub, __pyx_v_SubNr};
      __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 572, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_10);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[4] = {__pyx_t_3, __pyx_v_tstartSub, __pyx_v_tendSub, __pyx_v_SubNr};
      __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 572, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_10);
    } else
    #endif
    {
      __pyx_t_1 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 572, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (__pyx_t_3) {
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3); __pyx_t_3 = NULL;
      }
      __Pyx_INCREF(__pyx_v_tstartSub);
      __Pyx_GIVEREF(__pyx_v_tstartSub);
      PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_8, __pyx_v_tstartSub);
      __Pyx_INCREF(__pyx_v_tendSub);
      __Pyx_GIVEREF(__pyx_v_tendSub);
      PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_8, __pyx_v_tendSub);
      __Pyx_INCREF(__pyx_v_SubNr);
      __Pyx_GIVEREF(__pyx_v_SubNr);
      PyTuple_SET_ITEM(__pyx_t_1, 2+__pyx_t_8, __pyx_v_SubNr);
      __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_1, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 572, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_trSub = __pyx_t_10;
    __pyx_t_10 = 0;

    /* "torchsar/misc/sarplat.py":574
 *             trSub = np.linspace(tstartSub, tendSub, SubNr)
 *             # trSub = np.linspace(-SubTr / 2.0, SubTr / 2.0, SubNr) + t0Sub
 *             xminSub, xmaxSub, yminSub, ymaxSub = np.array(SubBA) + np.array([Xc, Xc, Yc, Yc])             # <<<<<<<<<<<<<<
 *         if GM == 'SG':
 *             Xc = SubSC[0]
 */
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 574, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_array); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 574, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_10 = (__pyx_t_2) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_2, __pyx_v_SubBA) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_SubBA);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 574, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_np); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 574, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_array); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 574, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = PyList_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 574, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_Xc);
    __Pyx_GIVEREF(__pyx_v_Xc);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_v_Xc);
    __Pyx_INCREF(__pyx_v_Xc);
    __Pyx_GIVEREF(__pyx_v_Xc);
    PyList_SET_ITEM(__pyx_t_2, 1, __pyx_v_Xc);
    __Pyx_INCREF(__pyx_v_Yc);
    __Pyx_GIVEREF(__pyx_v_Yc);
    PyList_SET_ITEM(__pyx_t_2, 2, __pyx_v_Yc);
    __Pyx_INCREF(__pyx_v_Yc);
    __Pyx_GIVEREF(__pyx_v_Yc);
    PyList_SET_ITEM(__pyx_t_2, 3, __pyx_v_Yc);
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_7) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_7, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2);
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 574, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyNumber_Add(__pyx_t_10, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 574, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {
      PyObject* sequence = __pyx_t_3;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 4)) {
        if (size > 4) __Pyx_RaiseTooManyValuesError(4);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 574, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_10 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_2 = PyTuple_GET_ITEM(sequence, 2); 
        __pyx_t_7 = PyTuple_GET_ITEM(sequence, 3); 
      } else {
        __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_10 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_2 = PyList_GET_ITEM(sequence, 2); 
        __pyx_t_7 = PyList_GET_ITEM(sequence, 3); 
      }
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_7);
      #else
      {
        Py_ssize_t i;
        PyObject** temps[4] = {&__pyx_t_1,&__pyx_t_10,&__pyx_t_2,&__pyx_t_7};
        for (i=0; i < 4; i++) {
          PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 574, __pyx_L1_error)
          __Pyx_GOTREF(item);
          *(temps[i]) = item;
        }
      }
      #endif
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else {
      Py_ssize_t index = -1;
      PyObject** temps[4] = {&__pyx_t_1,&__pyx_t_10,&__pyx_t_2,&__pyx_t_7};
      __pyx_t_11 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 574, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = Py_TYPE(__pyx_t_11)->tp_iternext;
      for (index=0; index < 4; index++) {
        PyObject* item = __pyx_t_6(__pyx_t_11); if (unlikely(!item)) goto __pyx_L10_unpacking_failed;
        __Pyx_GOTREF(item);
        *(temps[index]) = item;
      }
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_11), 4) < 0) __PYX_ERR(0, 574, __pyx_L1_error)
      __pyx_t_6 = NULL;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      goto __pyx_L11_unpacking_done;
      __pyx_L10_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_6 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 574, __pyx_L1_error)
      __pyx_L11_unpacking_done:;
    }
    __pyx_v_xminSub = __pyx_t_1;
    __pyx_t_1 = 0;
    __pyx_v_xmaxSub = __pyx_t_10;
    __pyx_t_10 = 0;
    __pyx_v_yminSub = __pyx_t_2;
    __pyx_t_2 = 0;
    __pyx_v_ymaxSub = __pyx_t_7;
    __pyx_t_7 = 0;

    /* "torchsar/misc/sarplat.py":567
 *         SubTr = tendSub - tstartSub
 * 
 *         if GM == 'BG':             # <<<<<<<<<<<<<<
 *             Xc = SubBC[0]
 *             Yc = SubBC[1]
 */
  }

  /* "torchsar/misc/sarplat.py":575
 *             # trSub = np.linspace(-SubTr / 2.0, SubTr / 2.0, SubNr) + t0Sub
 *             xminSub, xmaxSub, yminSub, ymaxSub = np.array(SubBA) + np.array([Xc, Xc, Yc, Yc])
 *         if GM == 'SG':             # <<<<<<<<<<<<<<
 *             Xc = SubSC[0]
 *             Yc = SubSC[1]
 */
  __pyx_t_4 = (__Pyx_PyUnicode_Equals(__pyx_v_GM, __pyx_n_u_SG, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 575, __pyx_L1_error)
  if (__pyx_t_4) {

    /* "torchsar/misc/sarplat.py":576
 *             xminSub, xmaxSub, yminSub, ymaxSub = np.array(SubBA) + np.array([Xc, Xc, Yc, Yc])
 *         if GM == 'SG':
 *             Xc = SubSC[0]             # <<<<<<<<<<<<<<
 *             Yc = SubSC[1]
 *             SubR0 = SubRs0
 */
    __pyx_t_3 = __Pyx_GetItemInt_List(__pyx_v_SubSC, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 576, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_Xc, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "torchsar/misc/sarplat.py":577
 *         if GM == 'SG':
 *             Xc = SubSC[0]
 *             Yc = SubSC[1]             # <<<<<<<<<<<<<<
 *             SubR0 = SubRs0
 *             t0Sub = 2 * SubRsc / C
 */
    __pyx_t_3 = __Pyx_GetItemInt_List(__pyx_v_SubSC, 1, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 577, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_Yc, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "torchsar/misc/sarplat.py":578
 *             Xc = SubSC[0]
 *             Yc = SubSC[1]
 *             SubR0 = SubRs0             # <<<<<<<<<<<<<<
 *             t0Sub = 2 * SubRsc / C
 *             trSub = np.linspace(tstartSub, tendSub, SubNr)
 */
    __Pyx_INCREF(__pyx_v_SubRs0);
    __Pyx_XDECREF_SET(__pyx_v_SubR0, __pyx_v_SubRs0);

    /* "torchsar/misc/sarplat.py":579
 *             Yc = SubSC[1]
 *             SubR0 = SubRs0
 *             t0Sub = 2 * SubRsc / C             # <<<<<<<<<<<<<<
 *             trSub = np.linspace(tstartSub, tendSub, SubNr)
 *             # trSub = np.linspace(-SubTr / 2.0, SubTr / 2.0, SubNr) + t0Sub
 */
    __pyx_t_3 = PyNumber_Multiply(__pyx_int_2, __pyx_v_SubRsc); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 579, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_C); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 579, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_2 = __Pyx_PyNumber_Divide(__pyx_t_3, __pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 579, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF_SET(__pyx_v_t0Sub, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "torchsar/misc/sarplat.py":580
 *             SubR0 = SubRs0
 *             t0Sub = 2 * SubRsc / C
 *             trSub = np.linspace(tstartSub, tendSub, SubNr)             # <<<<<<<<<<<<<<
 *             # trSub = np.linspace(-SubTr / 2.0, SubTr / 2.0, SubNr) + t0Sub
 *             xminSub, xmaxSub, yminSub, ymaxSub = np.array(SubSA) + np.array([Xc, Xc, Yc, Yc])
 */
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_n_s_np); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 580, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_linspace); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 580, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = NULL;
    __pyx_t_8 = 0;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
        __pyx_t_8 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_tstartSub, __pyx_v_tendSub, __pyx_v_SubNr};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 580, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[4] = {__pyx_t_7, __pyx_v_tstartSub, __pyx_v_tendSub, __pyx_v_SubNr};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 580, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    {
      __pyx_t_10 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 580, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      if (__pyx_t_7) {
        __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_7); __pyx_t_7 = NULL;
      }
      __Pyx_INCREF(__pyx_v_tstartSub);
      __Pyx_GIVEREF(__pyx_v_tstartSub);
      PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_8, __pyx_v_tstartSub);
      __Pyx_INCREF(__pyx_v_tendSub);
      __Pyx_GIVEREF(__pyx_v_tendSub);
      PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_8, __pyx_v_tendSub);
      __Pyx_INCREF(__pyx_v_SubNr);
      __Pyx_GIVEREF(__pyx_v_SubNr);
      PyTuple_SET_ITEM(__pyx_t_10, 2+__pyx_t_8, __pyx_v_SubNr);
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_10, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 580, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_trSub, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "torchsar/misc/sarplat.py":582
 *             trSub = np.linspace(tstartSub, tendSub, SubNr)
 *             # trSub = np.linspace(-SubTr / 2.0, SubTr / 2.0, SubNr) + t0Sub
 *             xminSub, xmaxSub, yminSub, ymaxSub = np.array(SubSA) + np.array([Xc, Xc, Yc, Yc])             # <<<<<<<<<<<<<<
 * 
 *         frSub = np.linspace(-Fsr / 2.0, Fsr / 2.0, SubNr)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 582, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_array); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 582, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_10);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_10, function);
      }
    }
    __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_10, __pyx_t_3, __pyx_v_SubSA) : __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_v_SubSA);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 582, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 582, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_array); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 582, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyList_New(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 582, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_Xc);
    __Pyx_GIVEREF(__pyx_v_Xc);
    PyList_SET_ITEM(__pyx_t_3, 0, __pyx_v_Xc);
    __Pyx_INCREF(__pyx_v_Xc);
    __Pyx_GIVEREF(__pyx_v_Xc);
    PyList_SET_ITEM(__pyx_t_3, 1, __pyx_v_Xc);
    __Pyx_INCREF(__pyx_v_Yc);
    __Pyx_GIVEREF(__pyx_v_Yc);
    PyList_SET_ITEM(__pyx_t_3, 2, __pyx_v_Yc);
    __Pyx_INCREF(__pyx_v_Yc);
    __Pyx_GIVEREF(__pyx_v_Yc);
    PyList_SET_ITEM(__pyx_t_3, 3, __pyx_v_Yc);
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_10 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_1, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_3);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 582, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __pyx_t_7 = PyNumber_Add(__pyx_t_2, __pyx_t_10); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 582, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    if ((likely(PyTuple_CheckExact(__pyx_t_7))) || (PyList_CheckExact(__pyx_t_7))) {
      PyObject* sequence = __pyx_t_7;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 4)) {
        if (size > 4) __Pyx_RaiseTooManyValuesError(4);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 582, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_10 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 2); 
        __pyx_t_1 = PyTuple_GET_ITEM(sequence, 3); 
      } else {
        __pyx_t_10 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_3 = PyList_GET_ITEM(sequence, 2); 
        __pyx_t_1 = PyList_GET_ITEM(sequence, 3); 
      }
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_1);
      #else
      {
        Py_ssize_t i;
        PyObject** temps[4] = {&__pyx_t_10,&__pyx_t_2,&__pyx_t_3,&__pyx_t_1};
        for (i=0; i < 4; i++) {
          PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 582, __pyx_L1_error)
          __Pyx_GOTREF(item);
          *(temps[i]) = item;
        }
      }
      #endif
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else {
      Py_ssize_t index = -1;
      PyObject** temps[4] = {&__pyx_t_10,&__pyx_t_2,&__pyx_t_3,&__pyx_t_1};
      __pyx_t_11 = PyObject_GetIter(__pyx_t_7); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 582, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_6 = Py_TYPE(__pyx_t_11)->tp_iternext;
      for (index=0; index < 4; index++) {
        PyObject* item = __pyx_t_6(__pyx_t_11); if (unlikely(!item)) goto __pyx_L13_unpacking_failed;
        __Pyx_GOTREF(item);
        *(temps[index]) = item;
      }
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_6(__pyx_t_11), 4) < 0) __PYX_ERR(0, 582, __pyx_L1_error)
      __pyx_t_6 = NULL;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      goto __pyx_L14_unpacking_done;
      __pyx_L13_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_t_6 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 582, __pyx_L1_error)
      __pyx_L14_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_xminSub, __pyx_t_10);
    __pyx_t_10 = 0;
    __Pyx_XDECREF_SET(__pyx_v_xmaxSub, __pyx_t_2);
    __pyx_t_2 = 0;
    __Pyx_XDECREF_SET(__pyx_v_yminSub, __pyx_t_3);
    __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_ymaxSub, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "torchsar/misc/sarplat.py":575
 *             # trSub = np.linspace(-SubTr / 2.0, SubTr / 2.0, SubNr) + t0Sub
 *             xminSub, xmaxSub, yminSub, ymaxSub = np.array(SubBA) + np.array([Xc, Xc, Yc, Yc])
 *         if GM == 'SG':             # <<<<<<<<<<<<<<
 *             Xc = SubSC[0]
 *             Yc = SubSC[1]
 */
  }

  /* "torchsar/misc/sarplat.py":584
 *             xminSub, xmaxSub, yminSub, ymaxSub = np.array(SubSA) + np.array([Xc, Xc, Yc, Yc])
 * 
 *         frSub = np.linspace(-Fsr / 2.0, Fsr / 2.0, SubNr)             # <<<<<<<<<<<<<<
 * 
 *         SubKa = (-2.0 * Vr**2 * np.cos(Ar)**3) / (Wl * SubRnear)
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_np); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_linspace); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Negative(__pyx_v_Fsr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyFloat_TrueDivideObjC(__pyx_t_1, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyFloat_TrueDivideObjC(__pyx_v_Fsr, __pyx_float_2_0, 2.0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 584, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_10 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_10)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_10);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[4] = {__pyx_t_10, __pyx_t_2, __pyx_t_1, __pyx_v_SubNr};
    __pyx_t_7 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 584, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[4] = {__pyx_t_10, __pyx_t_2, __pyx_t_1, __pyx_v_SubNr};
    __pyx_t_7 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_8, 3+__pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 584, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else
  #endif
  {
    __pyx_t_11 = PyTuple_New(3+__pyx_t_8); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 584, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    if (__pyx_t_10) {
      __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_11, 0+__pyx_t_8, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_11, 1+__pyx_t_8, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_SubNr);
    __Pyx_GIVEREF(__pyx_v_SubNr);
    PyTuple_SET_ITEM(__pyx_t_11, 2+__pyx_t_8, __pyx_v_SubNr);
    __pyx_t_2 = 0;
    __pyx_t_1 = 0;
    __pyx_t_7 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_11, NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 584, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_frSub = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "torchsar/misc/sarplat.py":586
 *         frSub = np.linspace(-Fsr / 2.0, Fsr / 2.0, SubNr)
 * 
 *         SubKa = (-2.0 * Vr**2 * np.cos(Ar)**3) / (Wl * SubRnear)             # <<<<<<<<<<<<<<
 *         SubLsar = Wl * SubRnear / La
 * 
 */
  __pyx_t_7 = PyNumber_Power(__pyx_v_Vr, __pyx_int_2, Py_None); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 586, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_3 = PyNumber_Multiply(__pyx_float_neg_2_0, __pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 586, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_11, __pyx_n_s_np); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 586, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_n_s_cos); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 586, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_7 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_11, __pyx_v_Ar) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_Ar);
  __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
  if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 586, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Power(__pyx_t_7, __pyx_int_3, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 586, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = PyNumber_Multiply(__pyx_t_3, __pyx_t_1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 586, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyNumber_Multiply(__pyx_v_Wl, __pyx_v_SubRnear); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 586, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyNumber_Divide(__pyx_t_7, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 586, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_SubKa = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":587
 * 
 *         SubKa = (-2.0 * Vr**2 * np.cos(Ar)**3) / (Wl * SubRnear)
 *         SubLsar = Wl * SubRnear / La             # <<<<<<<<<<<<<<
 * 
 *         SubTsar = SubLsar / Vr
 */
  __pyx_t_3 = PyNumber_Multiply(__pyx_v_Wl, __pyx_v_SubRnear); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_t_3, __pyx_v_La); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 587, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_SubLsar = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":589
 *         SubLsar = Wl * SubRnear / La
 * 
 *         SubTsar = SubLsar / Vr             # <<<<<<<<<<<<<<
 *         SubBa = np.abs(SubKa * SubTsar)
 * 
 */
  __pyx_t_1 = __Pyx_PyNumber_Divide(__pyx_v_SubLsar, __pyx_v_Vr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 589, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_SubTsar = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":590
 * 
 *         SubTsar = SubLsar / Vr
 *         SubBa = np.abs(SubKa * SubTsar)             # <<<<<<<<<<<<<<
 * 
 *         SubFPa = self._params['FPa']
 */
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_np); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 590, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_abs); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 590, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_Multiply(__pyx_v_SubKa, __pyx_v_SubTsar); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 590, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_11 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
    }
  }
  __pyx_t_1 = (__pyx_t_11) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_11, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 590, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_v_SubBa = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":592
 *         SubBa = np.abs(SubKa * SubTsar)
 * 
 *         SubFPa = self._params['FPa']             # <<<<<<<<<<<<<<
 *         SubFPr = SubSA[1] - SubSA[0]
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 592, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_Dict_GetItem(__pyx_t_1, __pyx_n_u_FPa); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 592, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_SubFPa = __pyx_t_7;
  __pyx_t_7 = 0;

  /* "torchsar/misc/sarplat.py":593
 * 
 *         SubFPa = self._params['FPa']
 *         SubFPr = SubSA[1] - SubSA[0]             # <<<<<<<<<<<<<<
 * 
 *         self._selection = dict()
 */
  __pyx_t_7 = __Pyx_GetItemInt_List(__pyx_v_SubSA, 1, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 593, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_SubSA, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 593, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyNumber_Subtract(__pyx_t_7, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 593, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF_SET(__pyx_v_SubFPr, __pyx_t_3);
  __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":595
 *         SubFPr = SubSA[1] - SubSA[0]
 * 
 *         self._selection = dict()             # <<<<<<<<<<<<<<
 *         self._selection['taSub'] = taSub
 *         self._selection['faSub'] = faSub
 */
  __pyx_t_3 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 595, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_self, __pyx_n_s_selection, __pyx_t_3) < 0) __PYX_ERR(0, 595, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":596
 * 
 *         self._selection = dict()
 *         self._selection['taSub'] = taSub             # <<<<<<<<<<<<<<
 *         self._selection['faSub'] = faSub
 *         self._selection['trSub'] = trSub
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 596, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_taSub, __pyx_v_taSub) < 0)) __PYX_ERR(0, 596, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":597
 *         self._selection = dict()
 *         self._selection['taSub'] = taSub
 *         self._selection['faSub'] = faSub             # <<<<<<<<<<<<<<
 *         self._selection['trSub'] = trSub
 *         self._selection['frSub'] = frSub
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 597, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_faSub, __pyx_v_faSub) < 0)) __PYX_ERR(0, 597, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":598
 *         self._selection['taSub'] = taSub
 *         self._selection['faSub'] = faSub
 *         self._selection['trSub'] = trSub             # <<<<<<<<<<<<<<
 *         self._selection['frSub'] = frSub
 *         self._selection['SubNa'] = SubNa
 */
  if (unlikely(!__pyx_v_trSub)) { __Pyx_RaiseUnboundLocalError("trSub"); __PYX_ERR(0, 598, __pyx_L1_error) }
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 598, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_trSub, __pyx_v_trSub) < 0)) __PYX_ERR(0, 598, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":599
 *         self._selection['faSub'] = faSub
 *         self._selection['trSub'] = trSub
 *         self._selection['frSub'] = frSub             # <<<<<<<<<<<<<<
 *         self._selection['SubNa'] = SubNa
 *         self._selection['SubNr'] = SubNr
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_frSub, __pyx_v_frSub) < 0)) __PYX_ERR(0, 599, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":600
 *         self._selection['trSub'] = trSub
 *         self._selection['frSub'] = frSub
 *         self._selection['SubNa'] = SubNa             # <<<<<<<<<<<<<<
 *         self._selection['SubNr'] = SubNr
 *         self._selection['SubTa'] = SubTa
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 600, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubNa, __pyx_v_SubNa) < 0)) __PYX_ERR(0, 600, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":601
 *         self._selection['frSub'] = frSub
 *         self._selection['SubNa'] = SubNa
 *         self._selection['SubNr'] = SubNr             # <<<<<<<<<<<<<<
 *         self._selection['SubTa'] = SubTa
 *         self._selection['SubTr'] = SubTr
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 601, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubNr, __pyx_v_SubNr) < 0)) __PYX_ERR(0, 601, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":602
 *         self._selection['SubNa'] = SubNa
 *         self._selection['SubNr'] = SubNr
 *         self._selection['SubTa'] = SubTa             # <<<<<<<<<<<<<<
 *         self._selection['SubTr'] = SubTr
 *         self._selection['tnearSub'] = tnearSub
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 602, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubTa, __pyx_v_SubTa) < 0)) __PYX_ERR(0, 602, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":603
 *         self._selection['SubNr'] = SubNr
 *         self._selection['SubTa'] = SubTa
 *         self._selection['SubTr'] = SubTr             # <<<<<<<<<<<<<<
 *         self._selection['tnearSub'] = tnearSub
 *         self._selection['tfarSub'] = tfarSub
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 603, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubTr, __pyx_v_SubTr) < 0)) __PYX_ERR(0, 603, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":604
 *         self._selection['SubTa'] = SubTa
 *         self._selection['SubTr'] = SubTr
 *         self._selection['tnearSub'] = tnearSub             # <<<<<<<<<<<<<<
 *         self._selection['tfarSub'] = tfarSub
 *         self._selection['tstartSub'] = tstartSub
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 604, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_tnearSub, __pyx_v_tnearSub) < 0)) __PYX_ERR(0, 604, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":605
 *         self._selection['SubTr'] = SubTr
 *         self._selection['tnearSub'] = tnearSub
 *         self._selection['tfarSub'] = tfarSub             # <<<<<<<<<<<<<<
 *         self._selection['tstartSub'] = tstartSub
 *         self._selection['tendSub'] = tendSub
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 605, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_tfarSub, __pyx_v_tfarSub) < 0)) __PYX_ERR(0, 605, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":606
 *         self._selection['tnearSub'] = tnearSub
 *         self._selection['tfarSub'] = tfarSub
 *         self._selection['tstartSub'] = tstartSub             # <<<<<<<<<<<<<<
 *         self._selection['tendSub'] = tendSub
 *         self._selection['SubRnear'] = SubRnear
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 606, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_tstartSub, __pyx_v_tstartSub) < 0)) __PYX_ERR(0, 606, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":607
 *         self._selection['tfarSub'] = tfarSub
 *         self._selection['tstartSub'] = tstartSub
 *         self._selection['tendSub'] = tendSub             # <<<<<<<<<<<<<<
 *         self._selection['SubRnear'] = SubRnear
 *         self._selection['SubRfar'] = SubRfar
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 607, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_tendSub, __pyx_v_tendSub) < 0)) __PYX_ERR(0, 607, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":608
 *         self._selection['tstartSub'] = tstartSub
 *         self._selection['tendSub'] = tendSub
 *         self._selection['SubRnear'] = SubRnear             # <<<<<<<<<<<<<<
 *         self._selection['SubRfar'] = SubRfar
 *         self._selection['SubSceneArea'] = SubSA
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 608, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubRnear, __pyx_v_SubRnear) < 0)) __PYX_ERR(0, 608, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":609
 *         self._selection['tendSub'] = tendSub
 *         self._selection['SubRnear'] = SubRnear
 *         self._selection['SubRfar'] = SubRfar             # <<<<<<<<<<<<<<
 *         self._selection['SubSceneArea'] = SubSA
 *         self._selection['SubBeamArea'] = SubBA
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 609, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubRfar, __pyx_v_SubRfar) < 0)) __PYX_ERR(0, 609, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":610
 *         self._selection['SubRnear'] = SubRnear
 *         self._selection['SubRfar'] = SubRfar
 *         self._selection['SubSceneArea'] = SubSA             # <<<<<<<<<<<<<<
 *         self._selection['SubBeamArea'] = SubBA
 *         self._selection['SubSceneCenter'] = SubSC
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 610, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubSceneArea, __pyx_v_SubSA) < 0)) __PYX_ERR(0, 610, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":611
 *         self._selection['SubRfar'] = SubRfar
 *         self._selection['SubSceneArea'] = SubSA
 *         self._selection['SubBeamArea'] = SubBA             # <<<<<<<<<<<<<<
 *         self._selection['SubSceneCenter'] = SubSC
 *         self._selection['SubBeamCenter'] = SubBC
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 611, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubBeamArea, __pyx_v_SubBA) < 0)) __PYX_ERR(0, 611, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":612
 *         self._selection['SubSceneArea'] = SubSA
 *         self._selection['SubBeamArea'] = SubBA
 *         self._selection['SubSceneCenter'] = SubSC             # <<<<<<<<<<<<<<
 *         self._selection['SubBeamCenter'] = SubBC
 *         self._selection['SubEchoAnchor'] = SubEA
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 612, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubSceneCenter, __pyx_v_SubSC) < 0)) __PYX_ERR(0, 612, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":613
 *         self._selection['SubBeamArea'] = SubBA
 *         self._selection['SubSceneCenter'] = SubSC
 *         self._selection['SubBeamCenter'] = SubBC             # <<<<<<<<<<<<<<
 *         self._selection['SubEchoAnchor'] = SubEA
 *         self._selection['SubEchoSize'] = SubES
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 613, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubBeamCenter, __pyx_v_SubBC) < 0)) __PYX_ERR(0, 613, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":614
 *         self._selection['SubSceneCenter'] = SubSC
 *         self._selection['SubBeamCenter'] = SubBC
 *         self._selection['SubEchoAnchor'] = SubEA             # <<<<<<<<<<<<<<
 *         self._selection['SubEchoSize'] = SubES
 *         self._selection['SubFPa'] = SubFPa
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 614, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubEchoAnchor, __pyx_v_SubEA) < 0)) __PYX_ERR(0, 614, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":615
 *         self._selection['SubBeamCenter'] = SubBC
 *         self._selection['SubEchoAnchor'] = SubEA
 *         self._selection['SubEchoSize'] = SubES             # <<<<<<<<<<<<<<
 *         self._selection['SubFPa'] = SubFPa
 *         self._selection['SubFPr'] = SubFPr
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 615, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubEchoSize, __pyx_v_SubES) < 0)) __PYX_ERR(0, 615, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":616
 *         self._selection['SubEchoAnchor'] = SubEA
 *         self._selection['SubEchoSize'] = SubES
 *         self._selection['SubFPa'] = SubFPa             # <<<<<<<<<<<<<<
 *         self._selection['SubFPr'] = SubFPr
 *         self._selection['SubRsc'] = SubRsc
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 616, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubFPa, __pyx_v_SubFPa) < 0)) __PYX_ERR(0, 616, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":617
 *         self._selection['SubEchoSize'] = SubES
 *         self._selection['SubFPa'] = SubFPa
 *         self._selection['SubFPr'] = SubFPr             # <<<<<<<<<<<<<<
 *         self._selection['SubRsc'] = SubRsc
 *         self._selection['SubRs0'] = SubRs0
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 617, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubFPr, __pyx_v_SubFPr) < 0)) __PYX_ERR(0, 617, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":618
 *         self._selection['SubFPa'] = SubFPa
 *         self._selection['SubFPr'] = SubFPr
 *         self._selection['SubRsc'] = SubRsc             # <<<<<<<<<<<<<<
 *         self._selection['SubRs0'] = SubRs0
 *         self._selection['SubRbc'] = SubRbc
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubRsc, __pyx_v_SubRsc) < 0)) __PYX_ERR(0, 618, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":619
 *         self._selection['SubFPr'] = SubFPr
 *         self._selection['SubRsc'] = SubRsc
 *         self._selection['SubRs0'] = SubRs0             # <<<<<<<<<<<<<<
 *         self._selection['SubRbc'] = SubRbc
 *         self._selection['SubRb0'] = SubRb0
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 619, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubRs0, __pyx_v_SubRs0) < 0)) __PYX_ERR(0, 619, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":620
 *         self._selection['SubRsc'] = SubRsc
 *         self._selection['SubRs0'] = SubRs0
 *         self._selection['SubRbc'] = SubRbc             # <<<<<<<<<<<<<<
 *         self._selection['SubRb0'] = SubRb0
 *         self._selection['xminSub'] = xminSub
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 620, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubRbc, __pyx_v_SubRbc) < 0)) __PYX_ERR(0, 620, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":621
 *         self._selection['SubRs0'] = SubRs0
 *         self._selection['SubRbc'] = SubRbc
 *         self._selection['SubRb0'] = SubRb0             # <<<<<<<<<<<<<<
 *         self._selection['xminSub'] = xminSub
 *         self._selection['xmaxSub'] = xmaxSub
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 621, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubRb0, __pyx_v_SubRb0) < 0)) __PYX_ERR(0, 621, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":622
 *         self._selection['SubRbc'] = SubRbc
 *         self._selection['SubRb0'] = SubRb0
 *         self._selection['xminSub'] = xminSub             # <<<<<<<<<<<<<<
 *         self._selection['xmaxSub'] = xmaxSub
 *         self._selection['yminSub'] = yminSub
 */
  if (unlikely(!__pyx_v_xminSub)) { __Pyx_RaiseUnboundLocalError("xminSub"); __PYX_ERR(0, 622, __pyx_L1_error) }
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 622, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_xminSub, __pyx_v_xminSub) < 0)) __PYX_ERR(0, 622, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":623
 *         self._selection['SubRb0'] = SubRb0
 *         self._selection['xminSub'] = xminSub
 *         self._selection['xmaxSub'] = xmaxSub             # <<<<<<<<<<<<<<
 *         self._selection['yminSub'] = yminSub
 *         self._selection['ymaxSub'] = ymaxSub
 */
  if (unlikely(!__pyx_v_xmaxSub)) { __Pyx_RaiseUnboundLocalError("xmaxSub"); __PYX_ERR(0, 623, __pyx_L1_error) }
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 623, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_xmaxSub, __pyx_v_xmaxSub) < 0)) __PYX_ERR(0, 623, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":624
 *         self._selection['xminSub'] = xminSub
 *         self._selection['xmaxSub'] = xmaxSub
 *         self._selection['yminSub'] = yminSub             # <<<<<<<<<<<<<<
 *         self._selection['ymaxSub'] = ymaxSub
 *         self._selection['SubAbr'] = SubAbr
 */
  if (unlikely(!__pyx_v_yminSub)) { __Pyx_RaiseUnboundLocalError("yminSub"); __PYX_ERR(0, 624, __pyx_L1_error) }
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 624, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_yminSub, __pyx_v_yminSub) < 0)) __PYX_ERR(0, 624, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":625
 *         self._selection['xmaxSub'] = xmaxSub
 *         self._selection['yminSub'] = yminSub
 *         self._selection['ymaxSub'] = ymaxSub             # <<<<<<<<<<<<<<
 *         self._selection['SubAbr'] = SubAbr
 *         self._selection['SubAd'] = SubAd
 */
  if (unlikely(!__pyx_v_ymaxSub)) { __Pyx_RaiseUnboundLocalError("ymaxSub"); __PYX_ERR(0, 625, __pyx_L1_error) }
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 625, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_ymaxSub, __pyx_v_ymaxSub) < 0)) __PYX_ERR(0, 625, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":626
 *         self._selection['yminSub'] = yminSub
 *         self._selection['ymaxSub'] = ymaxSub
 *         self._selection['SubAbr'] = SubAbr             # <<<<<<<<<<<<<<
 *         self._selection['SubAd'] = SubAd
 *         self._selection['SubAon'] = SubAon
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 626, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubAbr, __pyx_v_SubAbr) < 0)) __PYX_ERR(0, 626, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":627
 *         self._selection['ymaxSub'] = ymaxSub
 *         self._selection['SubAbr'] = SubAbr
 *         self._selection['SubAd'] = SubAd             # <<<<<<<<<<<<<<
 *         self._selection['SubAon'] = SubAon
 *         self._selection['SubKa'] = SubKa
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 627, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubAd, __pyx_v_SubAd) < 0)) __PYX_ERR(0, 627, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":628
 *         self._selection['SubAbr'] = SubAbr
 *         self._selection['SubAd'] = SubAd
 *         self._selection['SubAon'] = SubAon             # <<<<<<<<<<<<<<
 *         self._selection['SubKa'] = SubKa
 *         self._selection['SubBa'] = SubBa
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 628, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubAon, __pyx_v_SubAon) < 0)) __PYX_ERR(0, 628, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":629
 *         self._selection['SubAd'] = SubAd
 *         self._selection['SubAon'] = SubAon
 *         self._selection['SubKa'] = SubKa             # <<<<<<<<<<<<<<
 *         self._selection['SubBa'] = SubBa
 *         self._selection['SubTsar'] = SubTsar
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 629, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubKa, __pyx_v_SubKa) < 0)) __PYX_ERR(0, 629, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":630
 *         self._selection['SubAon'] = SubAon
 *         self._selection['SubKa'] = SubKa
 *         self._selection['SubBa'] = SubBa             # <<<<<<<<<<<<<<
 *         self._selection['SubTsar'] = SubTsar
 *         self._selection['SubLsar'] = SubLsar
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 630, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubBa, __pyx_v_SubBa) < 0)) __PYX_ERR(0, 630, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":631
 *         self._selection['SubKa'] = SubKa
 *         self._selection['SubBa'] = SubBa
 *         self._selection['SubTsar'] = SubTsar             # <<<<<<<<<<<<<<
 *         self._selection['SubLsar'] = SubLsar
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 631, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubTsar, __pyx_v_SubTsar) < 0)) __PYX_ERR(0, 631, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":632
 *         self._selection['SubBa'] = SubBa
 *         self._selection['SubTsar'] = SubTsar
 *         self._selection['SubLsar'] = SubLsar             # <<<<<<<<<<<<<<
 * 
 *         self._params['taSub'] = taSub
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 632, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubLsar, __pyx_v_SubLsar) < 0)) __PYX_ERR(0, 632, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":634
 *         self._selection['SubLsar'] = SubLsar
 * 
 *         self._params['taSub'] = taSub             # <<<<<<<<<<<<<<
 *         self._params['faSub'] = faSub
 *         self._params['trSub'] = trSub
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_taSub, __pyx_v_taSub) < 0)) __PYX_ERR(0, 634, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":635
 * 
 *         self._params['taSub'] = taSub
 *         self._params['faSub'] = faSub             # <<<<<<<<<<<<<<
 *         self._params['trSub'] = trSub
 *         self._params['frSub'] = frSub
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 635, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_faSub, __pyx_v_faSub) < 0)) __PYX_ERR(0, 635, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":636
 *         self._params['taSub'] = taSub
 *         self._params['faSub'] = faSub
 *         self._params['trSub'] = trSub             # <<<<<<<<<<<<<<
 *         self._params['frSub'] = frSub
 *         self._params['SubNa'] = SubNa
 */
  if (unlikely(!__pyx_v_trSub)) { __Pyx_RaiseUnboundLocalError("trSub"); __PYX_ERR(0, 636, __pyx_L1_error) }
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 636, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_trSub, __pyx_v_trSub) < 0)) __PYX_ERR(0, 636, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":637
 *         self._params['faSub'] = faSub
 *         self._params['trSub'] = trSub
 *         self._params['frSub'] = frSub             # <<<<<<<<<<<<<<
 *         self._params['SubNa'] = SubNa
 *         self._params['SubNr'] = SubNr
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 637, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_frSub, __pyx_v_frSub) < 0)) __PYX_ERR(0, 637, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":638
 *         self._params['trSub'] = trSub
 *         self._params['frSub'] = frSub
 *         self._params['SubNa'] = SubNa             # <<<<<<<<<<<<<<
 *         self._params['SubNr'] = SubNr
 *         self._params['SubTa'] = SubTa
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 638, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubNa, __pyx_v_SubNa) < 0)) __PYX_ERR(0, 638, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":639
 *         self._params['frSub'] = frSub
 *         self._params['SubNa'] = SubNa
 *         self._params['SubNr'] = SubNr             # <<<<<<<<<<<<<<
 *         self._params['SubTa'] = SubTa
 *         self._params['SubTr'] = SubTr
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 639, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubNr, __pyx_v_SubNr) < 0)) __PYX_ERR(0, 639, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":640
 *         self._params['SubNa'] = SubNa
 *         self._params['SubNr'] = SubNr
 *         self._params['SubTa'] = SubTa             # <<<<<<<<<<<<<<
 *         self._params['SubTr'] = SubTr
 *         self._params['tnearSub'] = tnearSub
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 640, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubTa, __pyx_v_SubTa) < 0)) __PYX_ERR(0, 640, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":641
 *         self._params['SubNr'] = SubNr
 *         self._params['SubTa'] = SubTa
 *         self._params['SubTr'] = SubTr             # <<<<<<<<<<<<<<
 *         self._params['tnearSub'] = tnearSub
 *         self._params['tfarSub'] = tfarSub
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 641, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubTr, __pyx_v_SubTr) < 0)) __PYX_ERR(0, 641, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":642
 *         self._params['SubTa'] = SubTa
 *         self._params['SubTr'] = SubTr
 *         self._params['tnearSub'] = tnearSub             # <<<<<<<<<<<<<<
 *         self._params['tfarSub'] = tfarSub
 *         self._params['tstartSub'] = tstartSub
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 642, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_tnearSub, __pyx_v_tnearSub) < 0)) __PYX_ERR(0, 642, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":643
 *         self._params['SubTr'] = SubTr
 *         self._params['tnearSub'] = tnearSub
 *         self._params['tfarSub'] = tfarSub             # <<<<<<<<<<<<<<
 *         self._params['tstartSub'] = tstartSub
 *         self._params['tendSub'] = tendSub
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 643, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_tfarSub, __pyx_v_tfarSub) < 0)) __PYX_ERR(0, 643, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":644
 *         self._params['tnearSub'] = tnearSub
 *         self._params['tfarSub'] = tfarSub
 *         self._params['tstartSub'] = tstartSub             # <<<<<<<<<<<<<<
 *         self._params['tendSub'] = tendSub
 *         self._params['SubRnear'] = SubRnear
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 644, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_tstartSub, __pyx_v_tstartSub) < 0)) __PYX_ERR(0, 644, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":645
 *         self._params['tfarSub'] = tfarSub
 *         self._params['tstartSub'] = tstartSub
 *         self._params['tendSub'] = tendSub             # <<<<<<<<<<<<<<
 *         self._params['SubRnear'] = SubRnear
 *         self._params['SubRfar'] = SubRfar
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 645, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_tendSub, __pyx_v_tendSub) < 0)) __PYX_ERR(0, 645, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":646
 *         self._params['tstartSub'] = tstartSub
 *         self._params['tendSub'] = tendSub
 *         self._params['SubRnear'] = SubRnear             # <<<<<<<<<<<<<<
 *         self._params['SubRfar'] = SubRfar
 *         self._params['SubSceneArea'] = SubSA
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 646, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubRnear, __pyx_v_SubRnear) < 0)) __PYX_ERR(0, 646, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":647
 *         self._params['tendSub'] = tendSub
 *         self._params['SubRnear'] = SubRnear
 *         self._params['SubRfar'] = SubRfar             # <<<<<<<<<<<<<<
 *         self._params['SubSceneArea'] = SubSA
 *         self._params['SubBeamArea'] = SubBA
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 647, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubRfar, __pyx_v_SubRfar) < 0)) __PYX_ERR(0, 647, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":648
 *         self._params['SubRnear'] = SubRnear
 *         self._params['SubRfar'] = SubRfar
 *         self._params['SubSceneArea'] = SubSA             # <<<<<<<<<<<<<<
 *         self._params['SubBeamArea'] = SubBA
 *         self._params['SubSceneCenter'] = SubSC
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 648, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubSceneArea, __pyx_v_SubSA) < 0)) __PYX_ERR(0, 648, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":649
 *         self._params['SubRfar'] = SubRfar
 *         self._params['SubSceneArea'] = SubSA
 *         self._params['SubBeamArea'] = SubBA             # <<<<<<<<<<<<<<
 *         self._params['SubSceneCenter'] = SubSC
 *         self._params['SubBeamCenter'] = SubBC
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 649, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubBeamArea, __pyx_v_SubBA) < 0)) __PYX_ERR(0, 649, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":650
 *         self._params['SubSceneArea'] = SubSA
 *         self._params['SubBeamArea'] = SubBA
 *         self._params['SubSceneCenter'] = SubSC             # <<<<<<<<<<<<<<
 *         self._params['SubBeamCenter'] = SubBC
 *         self._params['SubEchoAnchor'] = SubEA
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 650, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubSceneCenter, __pyx_v_SubSC) < 0)) __PYX_ERR(0, 650, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":651
 *         self._params['SubBeamArea'] = SubBA
 *         self._params['SubSceneCenter'] = SubSC
 *         self._params['SubBeamCenter'] = SubBC             # <<<<<<<<<<<<<<
 *         self._params['SubEchoAnchor'] = SubEA
 *         self._params['SubEchoSize'] = SubES
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 651, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubBeamCenter, __pyx_v_SubBC) < 0)) __PYX_ERR(0, 651, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":652
 *         self._params['SubSceneCenter'] = SubSC
 *         self._params['SubBeamCenter'] = SubBC
 *         self._params['SubEchoAnchor'] = SubEA             # <<<<<<<<<<<<<<
 *         self._params['SubEchoSize'] = SubES
 *         self._params['SubFPa'] = SubFPa
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 652, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubEchoAnchor, __pyx_v_SubEA) < 0)) __PYX_ERR(0, 652, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":653
 *         self._params['SubBeamCenter'] = SubBC
 *         self._params['SubEchoAnchor'] = SubEA
 *         self._params['SubEchoSize'] = SubES             # <<<<<<<<<<<<<<
 *         self._params['SubFPa'] = SubFPa
 *         self._params['SubFPr'] = SubFPr
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 653, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubEchoSize, __pyx_v_SubES) < 0)) __PYX_ERR(0, 653, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":654
 *         self._params['SubEchoAnchor'] = SubEA
 *         self._params['SubEchoSize'] = SubES
 *         self._params['SubFPa'] = SubFPa             # <<<<<<<<<<<<<<
 *         self._params['SubFPr'] = SubFPr
 *         self._params['SubRsc'] = SubRsc
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 654, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubFPa, __pyx_v_SubFPa) < 0)) __PYX_ERR(0, 654, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":655
 *         self._params['SubEchoSize'] = SubES
 *         self._params['SubFPa'] = SubFPa
 *         self._params['SubFPr'] = SubFPr             # <<<<<<<<<<<<<<
 *         self._params['SubRsc'] = SubRsc
 *         self._params['SubRs0'] = SubRs0
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 655, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubFPr, __pyx_v_SubFPr) < 0)) __PYX_ERR(0, 655, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":656
 *         self._params['SubFPa'] = SubFPa
 *         self._params['SubFPr'] = SubFPr
 *         self._params['SubRsc'] = SubRsc             # <<<<<<<<<<<<<<
 *         self._params['SubRs0'] = SubRs0
 *         self._params['SubRbc'] = SubRbc
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubRsc, __pyx_v_SubRsc) < 0)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":657
 *         self._params['SubFPr'] = SubFPr
 *         self._params['SubRsc'] = SubRsc
 *         self._params['SubRs0'] = SubRs0             # <<<<<<<<<<<<<<
 *         self._params['SubRbc'] = SubRbc
 *         self._params['SubRb0'] = SubRb0
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 657, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubRs0, __pyx_v_SubRs0) < 0)) __PYX_ERR(0, 657, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":658
 *         self._params['SubRsc'] = SubRsc
 *         self._params['SubRs0'] = SubRs0
 *         self._params['SubRbc'] = SubRbc             # <<<<<<<<<<<<<<
 *         self._params['SubRb0'] = SubRb0
 *         self._params['xminSub'] = xminSub
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 658, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubRbc, __pyx_v_SubRbc) < 0)) __PYX_ERR(0, 658, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":659
 *         self._params['SubRs0'] = SubRs0
 *         self._params['SubRbc'] = SubRbc
 *         self._params['SubRb0'] = SubRb0             # <<<<<<<<<<<<<<
 *         self._params['xminSub'] = xminSub
 *         self._params['xmaxSub'] = xmaxSub
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 659, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubRb0, __pyx_v_SubRb0) < 0)) __PYX_ERR(0, 659, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":660
 *         self._params['SubRbc'] = SubRbc
 *         self._params['SubRb0'] = SubRb0
 *         self._params['xminSub'] = xminSub             # <<<<<<<<<<<<<<
 *         self._params['xmaxSub'] = xmaxSub
 *         self._params['yminSub'] = yminSub
 */
  if (unlikely(!__pyx_v_xminSub)) { __Pyx_RaiseUnboundLocalError("xminSub"); __PYX_ERR(0, 660, __pyx_L1_error) }
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 660, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_xminSub, __pyx_v_xminSub) < 0)) __PYX_ERR(0, 660, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":661
 *         self._params['SubRb0'] = SubRb0
 *         self._params['xminSub'] = xminSub
 *         self._params['xmaxSub'] = xmaxSub             # <<<<<<<<<<<<<<
 *         self._params['yminSub'] = yminSub
 *         self._params['ymaxSub'] = ymaxSub
 */
  if (unlikely(!__pyx_v_xmaxSub)) { __Pyx_RaiseUnboundLocalError("xmaxSub"); __PYX_ERR(0, 661, __pyx_L1_error) }
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 661, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_xmaxSub, __pyx_v_xmaxSub) < 0)) __PYX_ERR(0, 661, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":662
 *         self._params['xminSub'] = xminSub
 *         self._params['xmaxSub'] = xmaxSub
 *         self._params['yminSub'] = yminSub             # <<<<<<<<<<<<<<
 *         self._params['ymaxSub'] = ymaxSub
 *         self._params['SubAbr'] = SubAbr
 */
  if (unlikely(!__pyx_v_yminSub)) { __Pyx_RaiseUnboundLocalError("yminSub"); __PYX_ERR(0, 662, __pyx_L1_error) }
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 662, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_yminSub, __pyx_v_yminSub) < 0)) __PYX_ERR(0, 662, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":663
 *         self._params['xmaxSub'] = xmaxSub
 *         self._params['yminSub'] = yminSub
 *         self._params['ymaxSub'] = ymaxSub             # <<<<<<<<<<<<<<
 *         self._params['SubAbr'] = SubAbr
 *         self._params['SubAd'] = SubAd
 */
  if (unlikely(!__pyx_v_ymaxSub)) { __Pyx_RaiseUnboundLocalError("ymaxSub"); __PYX_ERR(0, 663, __pyx_L1_error) }
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 663, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_ymaxSub, __pyx_v_ymaxSub) < 0)) __PYX_ERR(0, 663, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":664
 *         self._params['yminSub'] = yminSub
 *         self._params['ymaxSub'] = ymaxSub
 *         self._params['SubAbr'] = SubAbr             # <<<<<<<<<<<<<<
 *         self._params['SubAd'] = SubAd
 *         self._params['SubAon'] = SubAon
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 664, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubAbr, __pyx_v_SubAbr) < 0)) __PYX_ERR(0, 664, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":665
 *         self._params['ymaxSub'] = ymaxSub
 *         self._params['SubAbr'] = SubAbr
 *         self._params['SubAd'] = SubAd             # <<<<<<<<<<<<<<
 *         self._params['SubAon'] = SubAon
 *         self._params['SubKa'] = SubKa
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 665, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubAd, __pyx_v_SubAd) < 0)) __PYX_ERR(0, 665, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":666
 *         self._params['SubAbr'] = SubAbr
 *         self._params['SubAd'] = SubAd
 *         self._params['SubAon'] = SubAon             # <<<<<<<<<<<<<<
 *         self._params['SubKa'] = SubKa
 *         self._params['SubBa'] = SubBa
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 666, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubAon, __pyx_v_SubAon) < 0)) __PYX_ERR(0, 666, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":667
 *         self._params['SubAd'] = SubAd
 *         self._params['SubAon'] = SubAon
 *         self._params['SubKa'] = SubKa             # <<<<<<<<<<<<<<
 *         self._params['SubBa'] = SubBa
 *         self._params['SubTsar'] = SubTsar
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 667, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubKa, __pyx_v_SubKa) < 0)) __PYX_ERR(0, 667, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":668
 *         self._params['SubAon'] = SubAon
 *         self._params['SubKa'] = SubKa
 *         self._params['SubBa'] = SubBa             # <<<<<<<<<<<<<<
 *         self._params['SubTsar'] = SubTsar
 *         self._params['SubLsar'] = SubLsar
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 668, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubBa, __pyx_v_SubBa) < 0)) __PYX_ERR(0, 668, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":669
 *         self._params['SubKa'] = SubKa
 *         self._params['SubBa'] = SubBa
 *         self._params['SubTsar'] = SubTsar             # <<<<<<<<<<<<<<
 *         self._params['SubLsar'] = SubLsar
 *         logging.info('---Done!')
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 669, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubTsar, __pyx_v_SubTsar) < 0)) __PYX_ERR(0, 669, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":670
 *         self._params['SubBa'] = SubBa
 *         self._params['SubTsar'] = SubTsar
 *         self._params['SubLsar'] = SubLsar             # <<<<<<<<<<<<<<
 *         logging.info('---Done!')
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 670, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (unlikely(PyObject_SetItem(__pyx_t_3, __pyx_n_u_SubLsar, __pyx_v_SubLsar) < 0)) __PYX_ERR(0, 670, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":671
 *         self._params['SubTsar'] = SubTsar
 *         self._params['SubLsar'] = SubLsar
 *         logging.info('---Done!')             # <<<<<<<<<<<<<<
 * 
 *     def printsp(self, verbose=False):
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_logging); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 671, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_info); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 671, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_7);
    if (likely(__pyx_t_1)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_7, function);
    }
  }
  __pyx_t_3 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_1, __pyx_kp_u_Done) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_kp_u_Done);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 671, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":475
 * 
 *     @selection.setter
 *     def selection(self, value):             # <<<<<<<<<<<<<<
 *         logging.info("---Check and set sub area selection...")
 *         SA = self._acquisition['SceneArea']
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("torchsar.misc.sarplat.SarPlat.selection", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_SA);
  __Pyx_XDECREF(__pyx_v_SC);
  __Pyx_XDECREF(__pyx_v_BC);
  __Pyx_XDECREF(__pyx_v_ES);
  __Pyx_XDECREF(__pyx_v_Ad);
  __Pyx_XDECREF(__pyx_v_Aon);
  __Pyx_XDECREF(__pyx_v_As);
  __Pyx_XDECREF(__pyx_v_Ar);
  __Pyx_XDECREF(__pyx_v_Fsr);
  __Pyx_XDECREF(__pyx_v_Fsa);
  __Pyx_XDECREF(__pyx_v_Tsa);
  __Pyx_XDECREF(__pyx_v_Tsr);
  __Pyx_XDECREF(__pyx_v_Na);
  __Pyx_XDECREF(__pyx_v_Nr);
  __Pyx_XDECREF(__pyx_v_Rnear);
  __Pyx_XDECREF(__pyx_v_Rfar);
  __Pyx_XDECREF(__pyx_v_FPr);
  __Pyx_XDECREF(__pyx_v_H);
  __Pyx_XDECREF(__pyx_v_Vr);
  __Pyx_XDECREF(__pyx_v_Tp);
  __Pyx_XDECREF(__pyx_v_Wl);
  __Pyx_XDECREF(__pyx_v_La);
  __Pyx_XDECREF(__pyx_v_GM);
  __Pyx_XDECREF(__pyx_v_Lsar);
  __Pyx_XDECREF(__pyx_v_SubSA);
  __Pyx_XDECREF(__pyx_v_SubES);
  __Pyx_XDECREF(__pyx_v_SubEA);
  __Pyx_XDECREF(__pyx_v_SubNa);
  __Pyx_XDECREF(__pyx_v_SubNr);
  __Pyx_XDECREF(__pyx_v_SubRnear);
  __Pyx_XDECREF(__pyx_v_SubRfar);
  __Pyx_XDECREF(__pyx_v_SubAon);
  __Pyx_XDECREF(__pyx_v_SubAbr);
  __Pyx_XDECREF(__pyx_v_SubAd);
  __Pyx_XDECREF(__pyx_v_SubFPr);
  __Pyx_XDECREF(__pyx_v_SubTa);
  __Pyx_XDECREF(__pyx_v_taSub);
  __Pyx_XDECREF(__pyx_v_faSub);
  __Pyx_XDECREF(__pyx_v_SubRsc);
  __Pyx_XDECREF(__pyx_v_SubRs0);
  __Pyx_XDECREF(__pyx_v_SubYsc);
  __Pyx_XDECREF(__pyx_v_SubXsc);
  __Pyx_XDECREF(__pyx_v_SubSC);
  __Pyx_XDECREF(__pyx_v_SubRbc);
  __Pyx_XDECREF(__pyx_v_SubRb0);
  __Pyx_XDECREF(__pyx_v_SubYbc);
  __Pyx_XDECREF(__pyx_v_SubXbc);
  __Pyx_XDECREF(__pyx_v_SubBC);
  __Pyx_XDECREF(__pyx_v_SubYnear);
  __Pyx_XDECREF(__pyx_v_SubYfar);
  __Pyx_XDECREF(__pyx_v_SubXnear);
  __Pyx_XDECREF(__pyx_v_SubXfar);
  __Pyx_XDECREF(__pyx_v_SubFPr0);
  __Pyx_XDECREF(__pyx_v_SubFPr1);
  __Pyx_XDECREF(__pyx_v_SubBA);
  __Pyx_XDECREF(__pyx_v_tnearSub);
  __Pyx_XDECREF(__pyx_v_tfarSub);
  __Pyx_XDECREF(__pyx_v_tstartSub);
  __Pyx_XDECREF(__pyx_v_tendSub);
  __Pyx_XDECREF(__pyx_v_SubTr);
  __Pyx_XDECREF(__pyx_v_Xc);
  __Pyx_XDECREF(__pyx_v_Yc);
  __Pyx_XDECREF(__pyx_v_SubR0);
  __Pyx_XDECREF(__pyx_v_t0Sub);
  __Pyx_XDECREF(__pyx_v_trSub);
  __Pyx_XDECREF(__pyx_v_xminSub);
  __Pyx_XDECREF(__pyx_v_xmaxSub);
  __Pyx_XDECREF(__pyx_v_yminSub);
  __Pyx_XDECREF(__pyx_v_ymaxSub);
  __Pyx_XDECREF(__pyx_v_frSub);
  __Pyx_XDECREF(__pyx_v_SubKa);
  __Pyx_XDECREF(__pyx_v_SubLsar);
  __Pyx_XDECREF(__pyx_v_SubTsar);
  __Pyx_XDECREF(__pyx_v_SubBa);
  __Pyx_XDECREF(__pyx_v_SubFPa);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "torchsar/misc/sarplat.py":673
 *         logging.info('---Done!')
 * 
 *     def printsp(self, verbose=False):             # <<<<<<<<<<<<<<
 *         print("===============SAR platform: ", self._name, "================")
 *         print("--------------------------sensor-----------------------------")
 */

/* Python wrapper */
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_21printsp(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_21printsp = {"printsp", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_21printsp, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_8torchsar_4misc_7sarplat_7SarPlat_21printsp(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  PyObject *__pyx_v_verbose = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("printsp (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_verbose,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)((PyObject *)Py_False));
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_verbose);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "printsp") < 0)) __PYX_ERR(0, 673, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_self = values[0];
    __pyx_v_verbose = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("printsp", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 673, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("torchsar.misc.sarplat.SarPlat.printsp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_20printsp(__pyx_self, __pyx_v_self, __pyx_v_verbose);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_8torchsar_4misc_7sarplat_7SarPlat_20printsp(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, PyObject *__pyx_v_verbose) {
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  Py_ssize_t __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("printsp", 0);

  /* "torchsar/misc/sarplat.py":674
 * 
 *     def printsp(self, verbose=False):
 *         print("===============SAR platform: ", self._name, "================")             # <<<<<<<<<<<<<<
 *         print("--------------------------sensor-----------------------------")
 *         for key in sorted(self._sensor):
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 674, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 674, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_kp_u_SAR_platform);
  __Pyx_GIVEREF(__pyx_kp_u_SAR_platform);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_kp_u_SAR_platform);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
  __Pyx_INCREF(__pyx_kp_u__15);
  __Pyx_GIVEREF(__pyx_kp_u__15);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_kp_u__15);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 674, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":675
 *     def printsp(self, verbose=False):
 *         print("===============SAR platform: ", self._name, "================")
 *         print("--------------------------sensor-----------------------------")             # <<<<<<<<<<<<<<
 *         for key in sorted(self._sensor):
 *             print(key, self._sensor[key])
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 675, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":676
 *         print("===============SAR platform: ", self._name, "================")
 *         print("--------------------------sensor-----------------------------")
 *         for key in sorted(self._sensor):             # <<<<<<<<<<<<<<
 *             print(key, self._sensor[key])
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 676, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PySequence_List(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 676, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_1 = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_t_4 = PyList_Sort(__pyx_t_1); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 676, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 676, __pyx_L1_error)
  }
  __pyx_t_3 = __pyx_t_1; __Pyx_INCREF(__pyx_t_3); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_3)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_1 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_5); __Pyx_INCREF(__pyx_t_1); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 676, __pyx_L1_error)
    #else
    __pyx_t_1 = PySequence_ITEM(__pyx_t_3, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 676, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "torchsar/misc/sarplat.py":677
 *         print("--------------------------sensor-----------------------------")
 *         for key in sorted(self._sensor):
 *             print(key, self._sensor[key])             # <<<<<<<<<<<<<<
 * 
 *         print("-----------------------acquisition---------------------------")
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_sensor); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 677, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_v_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 677, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 677, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_key);
    __Pyx_GIVEREF(__pyx_v_key);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_key);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 677, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "torchsar/misc/sarplat.py":676
 *         print("===============SAR platform: ", self._name, "================")
 *         print("--------------------------sensor-----------------------------")
 *         for key in sorted(self._sensor):             # <<<<<<<<<<<<<<
 *             print(key, self._sensor[key])
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":679
 *             print(key, self._sensor[key])
 * 
 *         print("-----------------------acquisition---------------------------")             # <<<<<<<<<<<<<<
 *         for key in sorted(self._acquisition):
 *             print(key, self._acquisition[key])
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 679, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":680
 * 
 *         print("-----------------------acquisition---------------------------")
 *         for key in sorted(self._acquisition):             # <<<<<<<<<<<<<<
 *             print(key, self._acquisition[key])
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 680, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PySequence_List(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 680, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_4 = PyList_Sort(__pyx_t_3); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 680, __pyx_L1_error)
  if (unlikely(__pyx_t_3 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 680, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_t_3; __Pyx_INCREF(__pyx_t_1); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  for (;;) {
    if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_3); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 680, __pyx_L1_error)
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 680, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "torchsar/misc/sarplat.py":681
 *         print("-----------------------acquisition---------------------------")
 *         for key in sorted(self._acquisition):
 *             print(key, self._acquisition[key])             # <<<<<<<<<<<<<<
 * 
 *         print("--------------------------params-----------------------------")
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_acquisition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 681, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_t_3, __pyx_v_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 681, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 681, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v_key);
    __Pyx_GIVEREF(__pyx_v_key);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_key);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 681, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "torchsar/misc/sarplat.py":680
 * 
 *         print("-----------------------acquisition---------------------------")
 *         for key in sorted(self._acquisition):             # <<<<<<<<<<<<<<
 *             print(key, self._acquisition[key])
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":683
 *             print(key, self._acquisition[key])
 * 
 *         print("--------------------------params-----------------------------")             # <<<<<<<<<<<<<<
 *         if verbose:
 *             print(self._params)
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 683, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":684
 * 
 *         print("--------------------------params-----------------------------")
 *         if verbose:             # <<<<<<<<<<<<<<
 *             print(self._params)
 *         else:
 */
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_verbose); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 684, __pyx_L1_error)
  if (__pyx_t_6) {

    /* "torchsar/misc/sarplat.py":685
 *         print("--------------------------params-----------------------------")
 *         if verbose:
 *             print(self._params)             # <<<<<<<<<<<<<<
 *         else:
 *             for key in sorted(self._params):
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 685, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_print, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 685, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "torchsar/misc/sarplat.py":684
 * 
 *         print("--------------------------params-----------------------------")
 *         if verbose:             # <<<<<<<<<<<<<<
 *             print(self._params)
 *         else:
 */
    goto __pyx_L7;
  }

  /* "torchsar/misc/sarplat.py":687
 *             print(self._params)
 *         else:
 *             for key in sorted(self._params):             # <<<<<<<<<<<<<<
 *                 value = self._params[key]
 *                 if type(value) in [list, tuple, th.Tensor] and len(value) > 10:
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 687, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PySequence_List(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 687, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_2 = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_4 = PyList_Sort(__pyx_t_2); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 687, __pyx_L1_error)
    if (unlikely(__pyx_t_2 == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 687, __pyx_L1_error)
    }
    __pyx_t_3 = __pyx_t_2; __Pyx_INCREF(__pyx_t_3); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    for (;;) {
      if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_3)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_2 = PyList_GET_ITEM(__pyx_t_3, __pyx_t_5); __Pyx_INCREF(__pyx_t_2); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 687, __pyx_L1_error)
      #else
      __pyx_t_2 = PySequence_ITEM(__pyx_t_3, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 687, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      #endif
      __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "torchsar/misc/sarplat.py":688
 *         else:
 *             for key in sorted(self._params):
 *                 value = self._params[key]             # <<<<<<<<<<<<<<
 *                 if type(value) in [list, tuple, th.Tensor] and len(value) > 10:
 *                     print("shape of " + key + ": ", value.shape)
 */
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_t_2, __pyx_v_key); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 688, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "torchsar/misc/sarplat.py":689
 *             for key in sorted(self._params):
 *                 value = self._params[key]
 *                 if type(value) in [list, tuple, th.Tensor] and len(value) > 10:             # <<<<<<<<<<<<<<
 *                     print("shape of " + key + ": ", value.shape)
 *                 else:
 */
      __Pyx_INCREF(((PyObject *)Py_TYPE(__pyx_v_value)));
      __pyx_t_1 = ((PyObject *)Py_TYPE(__pyx_v_value));
      __pyx_t_2 = PyObject_RichCompare(((PyObject *)__pyx_t_1), ((PyObject *)(&PyList_Type)), Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 689, __pyx_L1_error)
      __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 689, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (!__pyx_t_8) {
      } else {
        __pyx_t_7 = __pyx_t_8;
        goto __pyx_L13_bool_binop_done;
      }
      __pyx_t_2 = PyObject_RichCompare(((PyObject *)__pyx_t_1), ((PyObject *)(&PyTuple_Type)), Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 689, __pyx_L1_error)
      __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 689, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (!__pyx_t_8) {
      } else {
        __pyx_t_7 = __pyx_t_8;
        goto __pyx_L13_bool_binop_done;
      }
      __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_th); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 689, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_Tensor); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 689, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = PyObject_RichCompare(((PyObject *)__pyx_t_1), __pyx_t_9, Py_EQ); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 689, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 689, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_7 = __pyx_t_8;
      __pyx_L13_bool_binop_done:;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_8 = (__pyx_t_7 != 0);
      if (__pyx_t_8) {
      } else {
        __pyx_t_6 = __pyx_t_8;
        goto __pyx_L11_bool_binop_done;
      }
      __pyx_t_10 = PyObject_Length(__pyx_v_value); if (unlikely(__pyx_t_10 == ((Py_ssize_t)-1))) __PYX_ERR(0, 689, __pyx_L1_error)
      __pyx_t_8 = ((__pyx_t_10 > 10) != 0);
      __pyx_t_6 = __pyx_t_8;
      __pyx_L11_bool_binop_done:;
      if (__pyx_t_6) {

        /* "torchsar/misc/sarplat.py":690
 *                 value = self._params[key]
 *                 if type(value) in [list, tuple, th.Tensor] and len(value) > 10:
 *                     print("shape of " + key + ": ", value.shape)             # <<<<<<<<<<<<<<
 *                 else:
 *                     print(key, self._params[key])
 */
        __pyx_t_1 = PyNumber_Add(__pyx_kp_u_shape_of, __pyx_v_key); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 690, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_2 = PyNumber_Add(__pyx_t_1, __pyx_kp_u__19); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 690, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_shape); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 690, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 690, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_GIVEREF(__pyx_t_2);
        PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_2);
        __Pyx_GIVEREF(__pyx_t_1);
        PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_1);
        __pyx_t_2 = 0;
        __pyx_t_1 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 690, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "torchsar/misc/sarplat.py":689
 *             for key in sorted(self._params):
 *                 value = self._params[key]
 *                 if type(value) in [list, tuple, th.Tensor] and len(value) > 10:             # <<<<<<<<<<<<<<
 *                     print("shape of " + key + ": ", value.shape)
 *                 else:
 */
        goto __pyx_L10;
      }

      /* "torchsar/misc/sarplat.py":692
 *                     print("shape of " + key + ": ", value.shape)
 *                 else:
 *                     print(key, self._params[key])             # <<<<<<<<<<<<<<
 * 
 *         print("-------------------------selection---------------------------")
 */
      /*else*/ {
        __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_params); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 692, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_9 = __Pyx_PyObject_GetItem(__pyx_t_1, __pyx_v_key); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 692, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 692, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_v_key);
        __Pyx_GIVEREF(__pyx_v_key);
        PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_key);
        __Pyx_GIVEREF(__pyx_t_9);
        PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_9);
        __pyx_t_9 = 0;
        __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_1, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 692, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
      __pyx_L10:;

      /* "torchsar/misc/sarplat.py":687
 *             print(self._params)
 *         else:
 *             for key in sorted(self._params):             # <<<<<<<<<<<<<<
 *                 value = self._params[key]
 *                 if type(value) in [list, tuple, th.Tensor] and len(value) > 10:
 */
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __pyx_L7:;

  /* "torchsar/misc/sarplat.py":694
 *                     print(key, self._params[key])
 * 
 *         print("-------------------------selection---------------------------")             # <<<<<<<<<<<<<<
 *         for key in sorted(self._selection):
 *             value = self._selection[key]
 */
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_tuple__20, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 694, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":695
 * 
 *         print("-------------------------selection---------------------------")
 *         for key in sorted(self._selection):             # <<<<<<<<<<<<<<
 *             value = self._selection[key]
 *             if type(value) in [list, tuple, th.Tensor] and len(value) > 10:
 */
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 695, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_1 = PySequence_List(__pyx_t_9); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 695, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
  __pyx_t_3 = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_4 = PyList_Sort(__pyx_t_3); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 695, __pyx_L1_error)
  if (unlikely(__pyx_t_3 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 695, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_t_3; __Pyx_INCREF(__pyx_t_1); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  for (;;) {
    if (__pyx_t_5 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_5); __Pyx_INCREF(__pyx_t_3); __pyx_t_5++; if (unlikely(0 < 0)) __PYX_ERR(0, 695, __pyx_L1_error)
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_5); __pyx_t_5++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 695, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "torchsar/misc/sarplat.py":696
 *         print("-------------------------selection---------------------------")
 *         for key in sorted(self._selection):
 *             value = self._selection[key]             # <<<<<<<<<<<<<<
 *             if type(value) in [list, tuple, th.Tensor] and len(value) > 10:
 *                 print("shape of " + key + ": ", value.shape)
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 696, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_9 = __Pyx_PyObject_GetItem(__pyx_t_3, __pyx_v_key); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 696, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_9);
    __pyx_t_9 = 0;

    /* "torchsar/misc/sarplat.py":697
 *         for key in sorted(self._selection):
 *             value = self._selection[key]
 *             if type(value) in [list, tuple, th.Tensor] and len(value) > 10:             # <<<<<<<<<<<<<<
 *                 print("shape of " + key + ": ", value.shape)
 *             else:
 */
    __Pyx_INCREF(((PyObject *)Py_TYPE(__pyx_v_value)));
    __pyx_t_9 = ((PyObject *)Py_TYPE(__pyx_v_value));
    __pyx_t_3 = PyObject_RichCompare(((PyObject *)__pyx_t_9), ((PyObject *)(&PyList_Type)), Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 697, __pyx_L1_error)
    __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 697, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L21_bool_binop_done;
    }
    __pyx_t_3 = PyObject_RichCompare(((PyObject *)__pyx_t_9), ((PyObject *)(&PyTuple_Type)), Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 697, __pyx_L1_error)
    __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 697, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L21_bool_binop_done;
    }
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_th); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 697, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_Tensor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 697, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyObject_RichCompare(((PyObject *)__pyx_t_9), __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 697, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 697, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_8 = __pyx_t_7;
    __pyx_L21_bool_binop_done:;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __pyx_t_7 = (__pyx_t_8 != 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L19_bool_binop_done;
    }
    __pyx_t_10 = PyObject_Length(__pyx_v_value); if (unlikely(__pyx_t_10 == ((Py_ssize_t)-1))) __PYX_ERR(0, 697, __pyx_L1_error)
    __pyx_t_7 = ((__pyx_t_10 > 10) != 0);
    __pyx_t_6 = __pyx_t_7;
    __pyx_L19_bool_binop_done:;
    if (__pyx_t_6) {

      /* "torchsar/misc/sarplat.py":698
 *             value = self._selection[key]
 *             if type(value) in [list, tuple, th.Tensor] and len(value) > 10:
 *                 print("shape of " + key + ": ", value.shape)             # <<<<<<<<<<<<<<
 *             else:
 *                 print(key, self._selection[key])
 */
      __pyx_t_9 = PyNumber_Add(__pyx_kp_u_shape_of, __pyx_v_key); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 698, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_3 = PyNumber_Add(__pyx_t_9, __pyx_kp_u__19); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 698, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_value, __pyx_n_s_shape); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 698, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 698, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_9);
      PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_9);
      __pyx_t_3 = 0;
      __pyx_t_9 = 0;
      __pyx_t_9 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_2, NULL); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 698, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "torchsar/misc/sarplat.py":697
 *         for key in sorted(self._selection):
 *             value = self._selection[key]
 *             if type(value) in [list, tuple, th.Tensor] and len(value) > 10:             # <<<<<<<<<<<<<<
 *                 print("shape of " + key + ": ", value.shape)
 *             else:
 */
      goto __pyx_L18;
    }

    /* "torchsar/misc/sarplat.py":700
 *                 print("shape of " + key + ": ", value.shape)
 *             else:
 *                 print(key, self._selection[key])             # <<<<<<<<<<<<<<
 *         print("=============================================================")
 */
    /*else*/ {
      __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_self, __pyx_n_s_selection); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 700, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_t_9, __pyx_v_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 700, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_9 = PyTuple_New(2); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 700, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_INCREF(__pyx_v_key);
      __Pyx_GIVEREF(__pyx_v_key);
      PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_v_key);
      __Pyx_GIVEREF(__pyx_t_2);
      PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_t_2);
      __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_t_9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 700, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __pyx_L18:;

    /* "torchsar/misc/sarplat.py":695
 * 
 *         print("-------------------------selection---------------------------")
 *         for key in sorted(self._selection):             # <<<<<<<<<<<<<<
 *             value = self._selection[key]
 *             if type(value) in [list, tuple, th.Tensor] and len(value) > 10:
 */
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":701
 *             else:
 *                 print(key, self._selection[key])
 *         print("=============================================================")             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_print, __pyx_tuple__22, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 701, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":673
 *         logging.info('---Done!')
 * 
 *     def printsp(self, verbose=False):             # <<<<<<<<<<<<<<
 *         print("===============SAR platform: ", self._name, "================")
 *         print("--------------------------sensor-----------------------------")
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("torchsar.misc.sarplat.SarPlat.printsp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

static int __pyx_import_star_set(PyObject *o, PyObject* py_name, char *name) {
  static const char* internal_type_names[] = {
    0
  };
  const char** type_name = internal_type_names;
  while (*type_name) {
    if (__Pyx_StrEq(name, *type_name)) {
      PyErr_Format(PyExc_TypeError, "Cannot overwrite C type %s", name);
      goto bad;
    }
    type_name++;
  }
  if (0);
  else {
    if (PyObject_SetAttr(__pyx_m, py_name, o) < 0) goto bad;
  }
  return 0;
  bad:
  return -1;
}

static int
__Pyx_import_all_from(PyObject *locals, PyObject *v)
{
    PyObject *all = PyObject_GetAttrString(v, "__all__");
    PyObject *dict, *name, *value;
    int skip_leading_underscores = 0;
    int pos, err;
    if (all == NULL) {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError))
            return -1;
        PyErr_Clear();
        dict = PyObject_GetAttrString(v, "__dict__");
        if (dict == NULL) {
            if (!PyErr_ExceptionMatches(PyExc_AttributeError))
                return -1;
            PyErr_SetString(PyExc_ImportError,
            "from-import-* object has no __dict__ and no __all__");
            return -1;
        }
#if PY_MAJOR_VERSION < 3
        all = PyObject_CallMethod(dict, (char *)"keys", NULL);
#else
        all = PyMapping_Keys(dict);
#endif
        Py_DECREF(dict);
        if (all == NULL)
            return -1;
        skip_leading_underscores = 1;
    }
    for (pos = 0, err = 0; ; pos++) {
        name = PySequence_GetItem(all, pos);
        if (name == NULL) {
            if (!PyErr_ExceptionMatches(PyExc_IndexError))
                err = -1;
            else
                PyErr_Clear();
            break;
        }
        if (skip_leading_underscores &&
#if PY_MAJOR_VERSION < 3
            likely(PyString_Check(name)) &&
            PyString_AS_STRING(name)[0] == '_')
#else
            likely(PyUnicode_Check(name)) &&
            likely(__Pyx_PyUnicode_GET_LENGTH(name)) &&
            __Pyx_PyUnicode_READ_CHAR(name, 0) == '_')
#endif
        {
            Py_DECREF(name);
            continue;
        }
        value = PyObject_GetAttr(v, name);
        if (value == NULL)
            err = -1;
        else if (PyDict_CheckExact(locals))
            err = PyDict_SetItem(locals, name, value);
        else
            err = PyObject_SetItem(locals, name, value);
        Py_DECREF(name);
        Py_XDECREF(value);
        if (err != 0)
            break;
    }
    Py_DECREF(all);
    return err;
}
static int __pyx_import_star(PyObject* m) {
    int i;
    int ret = -1;
    char* s;
    PyObject *locals = 0;
    PyObject *list = 0;
#if PY_MAJOR_VERSION >= 3
    PyObject *utf8_name = 0;
#endif
    PyObject *name;
    PyObject *item;
    locals = PyDict_New();              if (!locals) goto bad;
    if (__Pyx_import_all_from(locals, m) < 0) goto bad;
    list = PyDict_Items(locals);        if (!list) goto bad;
    for(i=0; i<PyList_GET_SIZE(list); i++) {
        name = PyTuple_GET_ITEM(PyList_GET_ITEM(list, i), 0);
        item = PyTuple_GET_ITEM(PyList_GET_ITEM(list, i), 1);
#if PY_MAJOR_VERSION >= 3
        utf8_name = PyUnicode_AsUTF8String(name);
        if (!utf8_name) goto bad;
        s = PyBytes_AS_STRING(utf8_name);
        if (__pyx_import_star_set(item, name, s) < 0) goto bad;
        Py_DECREF(utf8_name); utf8_name = 0;
#else
        s = PyString_AsString(name);
        if (!s) goto bad;
        if (__pyx_import_star_set(item, name, s) < 0) goto bad;
#endif
    }
    ret = 0;
bad:
    Py_XDECREF(locals);
    Py_XDECREF(list);
#if PY_MAJOR_VERSION >= 3
    Py_XDECREF(utf8_name);
#endif
    return ret;
}



#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_sarplat(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_sarplat},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "sarplat",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_Aba, __pyx_k_Aba, sizeof(__pyx_k_Aba), 0, 0, 1, 1},
  {&__pyx_n_u_Aba, __pyx_k_Aba, sizeof(__pyx_k_Aba), 0, 1, 0, 1},
  {&__pyx_n_s_Abr, __pyx_k_Abr, sizeof(__pyx_k_Abr), 0, 0, 1, 1},
  {&__pyx_n_u_Abr, __pyx_k_Abr, sizeof(__pyx_k_Abr), 0, 1, 0, 1},
  {&__pyx_n_s_Ad, __pyx_k_Ad, sizeof(__pyx_k_Ad), 0, 0, 1, 1},
  {&__pyx_n_u_Ad, __pyx_k_Ad, sizeof(__pyx_k_Ad), 0, 1, 0, 1},
  {&__pyx_n_s_Ai, __pyx_k_Ai, sizeof(__pyx_k_Ai), 0, 0, 1, 1},
  {&__pyx_n_u_Ai, __pyx_k_Ai, sizeof(__pyx_k_Ai), 0, 1, 0, 1},
  {&__pyx_kp_u_Airbone, __pyx_k_Airbone, sizeof(__pyx_k_Airbone), 0, 1, 0, 0},
  {&__pyx_n_s_AirboneSatelliteHeightBoundary, __pyx_k_AirboneSatelliteHeightBoundary, sizeof(__pyx_k_AirboneSatelliteHeightBoundary), 0, 0, 1, 1},
  {&__pyx_n_s_Ao, __pyx_k_Ao, sizeof(__pyx_k_Ao), 0, 0, 1, 1},
  {&__pyx_n_s_Aon, __pyx_k_Aon, sizeof(__pyx_k_Aon), 0, 0, 1, 1},
  {&__pyx_n_u_Aon, __pyx_k_Aon, sizeof(__pyx_k_Aon), 0, 1, 0, 1},
  {&__pyx_n_s_Ar, __pyx_k_Ar, sizeof(__pyx_k_Ar), 0, 0, 1, 1},
  {&__pyx_n_u_Ar, __pyx_k_Ar, sizeof(__pyx_k_Ar), 0, 1, 0, 1},
  {&__pyx_n_s_As, __pyx_k_As, sizeof(__pyx_k_As), 0, 0, 1, 1},
  {&__pyx_n_u_As, __pyx_k_As, sizeof(__pyx_k_As), 0, 1, 0, 1},
  {&__pyx_n_s_BA, __pyx_k_BA, sizeof(__pyx_k_BA), 0, 0, 1, 1},
  {&__pyx_n_s_BAV, __pyx_k_BAV, sizeof(__pyx_k_BAV), 0, 0, 1, 1},
  {&__pyx_n_s_BC, __pyx_k_BC, sizeof(__pyx_k_BC), 0, 0, 1, 1},
  {&__pyx_n_u_BG, __pyx_k_BG, sizeof(__pyx_k_BG), 0, 1, 0, 1},
  {&__pyx_n_s_BWa, __pyx_k_BWa, sizeof(__pyx_k_BWa), 0, 0, 1, 1},
  {&__pyx_n_u_BWa, __pyx_k_BWa, sizeof(__pyx_k_BWa), 0, 1, 0, 1},
  {&__pyx_kp_u_Ba, __pyx_k_Ba, sizeof(__pyx_k_Ba), 0, 1, 0, 0},
  {&__pyx_n_s_Ba_2, __pyx_k_Ba_2, sizeof(__pyx_k_Ba_2), 0, 0, 1, 1},
  {&__pyx_n_s_Bdop, __pyx_k_Bdop, sizeof(__pyx_k_Bdop), 0, 0, 1, 1},
  {&__pyx_n_u_Bdop, __pyx_k_Bdop, sizeof(__pyx_k_Bdop), 0, 1, 0, 1},
  {&__pyx_n_u_BeamArea, __pyx_k_BeamArea, sizeof(__pyx_k_BeamArea), 0, 1, 0, 1},
  {&__pyx_n_u_BeamCenter, __pyx_k_BeamCenter, sizeof(__pyx_k_BeamCenter), 0, 1, 0, 1},
  {&__pyx_n_u_Br, __pyx_k_Br, sizeof(__pyx_k_Br), 0, 1, 0, 1},
  {&__pyx_n_s_C, __pyx_k_C, sizeof(__pyx_k_C), 0, 0, 1, 1},
  {&__pyx_kp_u_Check_and_set_SAR_platform_name, __pyx_k_Check_and_set_SAR_platform_name, sizeof(__pyx_k_Check_and_set_SAR_platform_name), 0, 1, 0, 0},
  {&__pyx_kp_u_Check_and_set_SAR_sensor_parame, __pyx_k_Check_and_set_SAR_sensor_parame, sizeof(__pyx_k_Check_and_set_SAR_sensor_parame), 0, 1, 0, 0},
  {&__pyx_kp_u_Check_and_set_acquisition_param, __pyx_k_Check_and_set_acquisition_param, sizeof(__pyx_k_Check_and_set_acquisition_param), 0, 1, 0, 0},
  {&__pyx_kp_u_Check_and_set_algorithm_paramet, __pyx_k_Check_and_set_algorithm_paramet, sizeof(__pyx_k_Check_and_set_algorithm_paramet), 0, 1, 0, 0},
  {&__pyx_kp_u_Check_and_set_sub_area_selectio, __pyx_k_Check_and_set_sub_area_selectio, sizeof(__pyx_k_Check_and_set_sub_area_selectio), 0, 1, 0, 0},
  {&__pyx_kp_u_Computed_equivalent_velocity_is, __pyx_k_Computed_equivalent_velocity_is, sizeof(__pyx_k_Computed_equivalent_velocity_is), 0, 1, 0, 0},
  {&__pyx_n_u_DA, __pyx_k_DA, sizeof(__pyx_k_DA), 0, 1, 0, 1},
  {&__pyx_n_u_DR, __pyx_k_DR, sizeof(__pyx_k_DR), 0, 1, 0, 1},
  {&__pyx_n_u_DX, __pyx_k_DX, sizeof(__pyx_k_DX), 0, 1, 0, 1},
  {&__pyx_n_u_DY, __pyx_k_DY, sizeof(__pyx_k_DY), 0, 1, 0, 1},
  {&__pyx_kp_u_Done, __pyx_k_Done, sizeof(__pyx_k_Done), 0, 1, 0, 0},
  {&__pyx_n_s_EPS, __pyx_k_EPS, sizeof(__pyx_k_EPS), 0, 0, 1, 1},
  {&__pyx_n_s_ES, __pyx_k_ES, sizeof(__pyx_k_ES), 0, 0, 1, 1},
  {&__pyx_n_u_EchoSize, __pyx_k_EchoSize, sizeof(__pyx_k_EchoSize), 0, 1, 0, 1},
  {&__pyx_kp_u_EchoSize_should_be_specified, __pyx_k_EchoSize_should_be_specified, sizeof(__pyx_k_EchoSize_should_be_specified), 0, 1, 0, 0},
  {&__pyx_n_s_FPa, __pyx_k_FPa, sizeof(__pyx_k_FPa), 0, 0, 1, 1},
  {&__pyx_n_u_FPa, __pyx_k_FPa, sizeof(__pyx_k_FPa), 0, 1, 0, 1},
  {&__pyx_n_s_FPr, __pyx_k_FPr, sizeof(__pyx_k_FPr), 0, 0, 1, 1},
  {&__pyx_n_u_FPr, __pyx_k_FPr, sizeof(__pyx_k_FPr), 0, 1, 0, 1},
  {&__pyx_n_s_FPr0, __pyx_k_FPr0, sizeof(__pyx_k_FPr0), 0, 0, 1, 1},
  {&__pyx_n_s_FPr1, __pyx_k_FPr1, sizeof(__pyx_k_FPr1), 0, 0, 1, 1},
  {&__pyx_n_s_Fc, __pyx_k_Fc, sizeof(__pyx_k_Fc), 0, 0, 1, 1},
  {&__pyx_n_u_Fc, __pyx_k_Fc, sizeof(__pyx_k_Fc), 0, 1, 0, 1},
  {&__pyx_n_s_Fs, __pyx_k_Fs, sizeof(__pyx_k_Fs), 0, 0, 1, 1},
  {&__pyx_n_u_Fs, __pyx_k_Fs, sizeof(__pyx_k_Fs), 0, 1, 0, 1},
  {&__pyx_n_s_Fsa, __pyx_k_Fsa, sizeof(__pyx_k_Fsa), 0, 0, 1, 1},
  {&__pyx_n_u_Fsa, __pyx_k_Fsa, sizeof(__pyx_k_Fsa), 0, 1, 0, 1},
  {&__pyx_n_s_Fsr, __pyx_k_Fsr, sizeof(__pyx_k_Fsr), 0, 0, 1, 1},
  {&__pyx_n_u_Fsr, __pyx_k_Fsr, sizeof(__pyx_k_Fsr), 0, 1, 0, 1},
  {&__pyx_n_s_GM, __pyx_k_GM, sizeof(__pyx_k_GM), 0, 0, 1, 1},
  {&__pyx_n_s_Ge, __pyx_k_Ge, sizeof(__pyx_k_Ge), 0, 0, 1, 1},
  {&__pyx_n_u_GeometryMode, __pyx_k_GeometryMode, sizeof(__pyx_k_GeometryMode), 0, 1, 0, 1},
  {&__pyx_n_s_H, __pyx_k_H, sizeof(__pyx_k_H), 0, 0, 1, 1},
  {&__pyx_n_u_H, __pyx_k_H, sizeof(__pyx_k_H), 0, 1, 0, 1},
  {&__pyx_n_s_Ka, __pyx_k_Ka, sizeof(__pyx_k_Ka), 0, 0, 1, 1},
  {&__pyx_n_u_Ka, __pyx_k_Ka, sizeof(__pyx_k_Ka), 0, 1, 0, 1},
  {&__pyx_n_s_Kr, __pyx_k_Kr, sizeof(__pyx_k_Kr), 0, 0, 1, 1},
  {&__pyx_n_u_Kr, __pyx_k_Kr, sizeof(__pyx_k_Kr), 0, 1, 0, 1},
  {&__pyx_n_s_La, __pyx_k_La, sizeof(__pyx_k_La), 0, 0, 1, 1},
  {&__pyx_n_u_La, __pyx_k_La, sizeof(__pyx_k_La), 0, 1, 0, 1},
  {&__pyx_n_s_Lsar, __pyx_k_Lsar, sizeof(__pyx_k_Lsar), 0, 0, 1, 1},
  {&__pyx_n_u_Lsar, __pyx_k_Lsar, sizeof(__pyx_k_Lsar), 0, 1, 0, 1},
  {&__pyx_n_s_Na, __pyx_k_Na, sizeof(__pyx_k_Na), 0, 0, 1, 1},
  {&__pyx_n_u_Na, __pyx_k_Na, sizeof(__pyx_k_Na), 0, 1, 0, 1},
  {&__pyx_n_s_Nr, __pyx_k_Nr, sizeof(__pyx_k_Nr), 0, 0, 1, 1},
  {&__pyx_n_u_Nr, __pyx_k_Nr, sizeof(__pyx_k_Nr), 0, 1, 0, 1},
  {&__pyx_kp_u_Nr_Tr_Fsr, __pyx_k_Nr_Tr_Fsr, sizeof(__pyx_k_Nr_Tr_Fsr), 0, 1, 0, 0},
  {&__pyx_kp_u_Nr_should_be_TrFsr, __pyx_k_Nr_should_be_TrFsr, sizeof(__pyx_k_Nr_should_be_TrFsr), 0, 1, 0, 0},
  {&__pyx_n_u_Nsar, __pyx_k_Nsar, sizeof(__pyx_k_Nsar), 0, 1, 0, 1},
  {&__pyx_n_s_PC, __pyx_k_PC, sizeof(__pyx_k_PC), 0, 0, 1, 1},
  {&__pyx_n_s_PI, __pyx_k_PI, sizeof(__pyx_k_PI), 0, 0, 1, 1},
  {&__pyx_n_s_PRF, __pyx_k_PRF, sizeof(__pyx_k_PRF), 0, 0, 1, 1},
  {&__pyx_n_u_PRF, __pyx_k_PRF, sizeof(__pyx_k_PRF), 0, 1, 0, 1},
  {&__pyx_n_s_PRFmax, __pyx_k_PRFmax, sizeof(__pyx_k_PRFmax), 0, 0, 1, 1},
  {&__pyx_n_s_PRFmin, __pyx_k_PRFmin, sizeof(__pyx_k_PRFmin), 0, 0, 1, 1},
  {&__pyx_n_s_PRT, __pyx_k_PRT, sizeof(__pyx_k_PRT), 0, 0, 1, 1},
  {&__pyx_n_u_PRT, __pyx_k_PRT, sizeof(__pyx_k_PRT), 0, 1, 0, 1},
  {&__pyx_n_u_PlatCenter, __pyx_k_PlatCenter, sizeof(__pyx_k_PlatCenter), 0, 1, 0, 1},
  {&__pyx_n_s_R0, __pyx_k_R0, sizeof(__pyx_k_R0), 0, 0, 1, 1},
  {&__pyx_n_s_Rb0, __pyx_k_Rb0, sizeof(__pyx_k_Rb0), 0, 0, 1, 1},
  {&__pyx_n_u_Rb0, __pyx_k_Rb0, sizeof(__pyx_k_Rb0), 0, 1, 0, 1},
  {&__pyx_n_s_Rbc, __pyx_k_Rbc, sizeof(__pyx_k_Rbc), 0, 0, 1, 1},
  {&__pyx_n_u_Rbc, __pyx_k_Rbc, sizeof(__pyx_k_Rbc), 0, 1, 0, 1},
  {&__pyx_n_s_Re, __pyx_k_Re, sizeof(__pyx_k_Re), 0, 0, 1, 1},
  {&__pyx_n_s_Rea, __pyx_k_Rea, sizeof(__pyx_k_Rea), 0, 0, 1, 1},
  {&__pyx_n_s_Ree, __pyx_k_Ree, sizeof(__pyx_k_Ree), 0, 0, 1, 1},
  {&__pyx_n_s_Rfar, __pyx_k_Rfar, sizeof(__pyx_k_Rfar), 0, 0, 1, 1},
  {&__pyx_n_u_Rfar, __pyx_k_Rfar, sizeof(__pyx_k_Rfar), 0, 1, 0, 1},
  {&__pyx_n_s_Rnear, __pyx_k_Rnear, sizeof(__pyx_k_Rnear), 0, 0, 1, 1},
  {&__pyx_n_u_Rnear, __pyx_k_Rnear, sizeof(__pyx_k_Rnear), 0, 1, 0, 1},
  {&__pyx_n_s_Rs, __pyx_k_Rs, sizeof(__pyx_k_Rs), 0, 0, 1, 1},
  {&__pyx_n_u_Rs, __pyx_k_Rs, sizeof(__pyx_k_Rs), 0, 1, 0, 1},
  {&__pyx_n_s_Rs0, __pyx_k_Rs0, sizeof(__pyx_k_Rs0), 0, 0, 1, 1},
  {&__pyx_n_u_Rs0, __pyx_k_Rs0, sizeof(__pyx_k_Rs0), 0, 1, 0, 1},
  {&__pyx_n_s_Rsc, __pyx_k_Rsc, sizeof(__pyx_k_Rsc), 0, 0, 1, 1},
  {&__pyx_n_u_Rsc, __pyx_k_Rsc, sizeof(__pyx_k_Rsc), 0, 1, 0, 1},
  {&__pyx_n_s_SA, __pyx_k_SA, sizeof(__pyx_k_SA), 0, 0, 1, 1},
  {&__pyx_kp_u_SAR_platform, __pyx_k_SAR_platform, sizeof(__pyx_k_SAR_platform), 0, 1, 0, 0},
  {&__pyx_kp_s_SAR_platform_class_sensor_acquis, __pyx_k_SAR_platform_class_sensor_acquis, sizeof(__pyx_k_SAR_platform_class_sensor_acquis), 0, 0, 1, 0},
  {&__pyx_n_s_SAV, __pyx_k_SAV, sizeof(__pyx_k_SAV), 0, 0, 1, 1},
  {&__pyx_n_s_SC, __pyx_k_SC, sizeof(__pyx_k_SC), 0, 0, 1, 1},
  {&__pyx_n_u_SG, __pyx_k_SG, sizeof(__pyx_k_SG), 0, 1, 0, 1},
  {&__pyx_n_s_SV, __pyx_k_SV, sizeof(__pyx_k_SV), 0, 0, 1, 1},
  {&__pyx_kp_u_Sampling_rate_should_be_in_rang, __pyx_k_Sampling_rate_should_be_in_rang, sizeof(__pyx_k_Sampling_rate_should_be_in_rang), 0, 1, 0, 0},
  {&__pyx_n_s_SarPlat, __pyx_k_SarPlat, sizeof(__pyx_k_SarPlat), 0, 0, 1, 1},
  {&__pyx_n_s_SarPlat_acquisition, __pyx_k_SarPlat_acquisition, sizeof(__pyx_k_SarPlat_acquisition), 0, 0, 1, 1},
  {&__pyx_n_s_SarPlat_name, __pyx_k_SarPlat_name, sizeof(__pyx_k_SarPlat_name), 0, 0, 1, 1},
  {&__pyx_n_s_SarPlat_params, __pyx_k_SarPlat_params, sizeof(__pyx_k_SarPlat_params), 0, 0, 1, 1},
  {&__pyx_n_s_SarPlat_printsp, __pyx_k_SarPlat_printsp, sizeof(__pyx_k_SarPlat_printsp), 0, 0, 1, 1},
  {&__pyx_n_s_SarPlat_selection, __pyx_k_SarPlat_selection, sizeof(__pyx_k_SarPlat_selection), 0, 0, 1, 1},
  {&__pyx_n_s_SarPlat_sensor, __pyx_k_SarPlat_sensor, sizeof(__pyx_k_SarPlat_sensor), 0, 0, 1, 1},
  {&__pyx_kp_u_Satellite, __pyx_k_Satellite, sizeof(__pyx_k_Satellite), 0, 1, 0, 0},
  {&__pyx_n_u_SceneArea, __pyx_k_SceneArea, sizeof(__pyx_k_SceneArea), 0, 1, 0, 1},
  {&__pyx_n_u_SceneCenter, __pyx_k_SceneCenter, sizeof(__pyx_k_SceneCenter), 0, 1, 0, 1},
  {&__pyx_n_u_StateVector, __pyx_k_StateVector, sizeof(__pyx_k_StateVector), 0, 1, 0, 1},
  {&__pyx_n_s_SubAbr, __pyx_k_SubAbr, sizeof(__pyx_k_SubAbr), 0, 0, 1, 1},
  {&__pyx_n_u_SubAbr, __pyx_k_SubAbr, sizeof(__pyx_k_SubAbr), 0, 1, 0, 1},
  {&__pyx_n_s_SubAd, __pyx_k_SubAd, sizeof(__pyx_k_SubAd), 0, 0, 1, 1},
  {&__pyx_n_u_SubAd, __pyx_k_SubAd, sizeof(__pyx_k_SubAd), 0, 1, 0, 1},
  {&__pyx_n_s_SubAon, __pyx_k_SubAon, sizeof(__pyx_k_SubAon), 0, 0, 1, 1},
  {&__pyx_n_u_SubAon, __pyx_k_SubAon, sizeof(__pyx_k_SubAon), 0, 1, 0, 1},
  {&__pyx_n_s_SubBA, __pyx_k_SubBA, sizeof(__pyx_k_SubBA), 0, 0, 1, 1},
  {&__pyx_n_s_SubBC, __pyx_k_SubBC, sizeof(__pyx_k_SubBC), 0, 0, 1, 1},
  {&__pyx_n_s_SubBa, __pyx_k_SubBa, sizeof(__pyx_k_SubBa), 0, 0, 1, 1},
  {&__pyx_n_u_SubBa, __pyx_k_SubBa, sizeof(__pyx_k_SubBa), 0, 1, 0, 1},
  {&__pyx_n_u_SubBeamArea, __pyx_k_SubBeamArea, sizeof(__pyx_k_SubBeamArea), 0, 1, 0, 1},
  {&__pyx_n_u_SubBeamCenter, __pyx_k_SubBeamCenter, sizeof(__pyx_k_SubBeamCenter), 0, 1, 0, 1},
  {&__pyx_n_s_SubEA, __pyx_k_SubEA, sizeof(__pyx_k_SubEA), 0, 0, 1, 1},
  {&__pyx_n_s_SubES, __pyx_k_SubES, sizeof(__pyx_k_SubES), 0, 0, 1, 1},
  {&__pyx_n_u_SubEchoAnchor, __pyx_k_SubEchoAnchor, sizeof(__pyx_k_SubEchoAnchor), 0, 1, 0, 1},
  {&__pyx_n_u_SubEchoSize, __pyx_k_SubEchoSize, sizeof(__pyx_k_SubEchoSize), 0, 1, 0, 1},
  {&__pyx_n_s_SubFPa, __pyx_k_SubFPa, sizeof(__pyx_k_SubFPa), 0, 0, 1, 1},
  {&__pyx_n_u_SubFPa, __pyx_k_SubFPa, sizeof(__pyx_k_SubFPa), 0, 1, 0, 1},
  {&__pyx_n_s_SubFPr, __pyx_k_SubFPr, sizeof(__pyx_k_SubFPr), 0, 0, 1, 1},
  {&__pyx_n_u_SubFPr, __pyx_k_SubFPr, sizeof(__pyx_k_SubFPr), 0, 1, 0, 1},
  {&__pyx_n_s_SubFPr0, __pyx_k_SubFPr0, sizeof(__pyx_k_SubFPr0), 0, 0, 1, 1},
  {&__pyx_n_s_SubFPr1, __pyx_k_SubFPr1, sizeof(__pyx_k_SubFPr1), 0, 0, 1, 1},
  {&__pyx_n_s_SubKa, __pyx_k_SubKa, sizeof(__pyx_k_SubKa), 0, 0, 1, 1},
  {&__pyx_n_u_SubKa, __pyx_k_SubKa, sizeof(__pyx_k_SubKa), 0, 1, 0, 1},
  {&__pyx_n_s_SubLsar, __pyx_k_SubLsar, sizeof(__pyx_k_SubLsar), 0, 0, 1, 1},
  {&__pyx_n_u_SubLsar, __pyx_k_SubLsar, sizeof(__pyx_k_SubLsar), 0, 1, 0, 1},
  {&__pyx_n_s_SubNa, __pyx_k_SubNa, sizeof(__pyx_k_SubNa), 0, 0, 1, 1},
  {&__pyx_n_u_SubNa, __pyx_k_SubNa, sizeof(__pyx_k_SubNa), 0, 1, 0, 1},
  {&__pyx_n_s_SubNr, __pyx_k_SubNr, sizeof(__pyx_k_SubNr), 0, 0, 1, 1},
  {&__pyx_n_u_SubNr, __pyx_k_SubNr, sizeof(__pyx_k_SubNr), 0, 1, 0, 1},
  {&__pyx_n_s_SubR0, __pyx_k_SubR0, sizeof(__pyx_k_SubR0), 0, 0, 1, 1},
  {&__pyx_n_s_SubRb0, __pyx_k_SubRb0, sizeof(__pyx_k_SubRb0), 0, 0, 1, 1},
  {&__pyx_n_u_SubRb0, __pyx_k_SubRb0, sizeof(__pyx_k_SubRb0), 0, 1, 0, 1},
  {&__pyx_n_s_SubRbc, __pyx_k_SubRbc, sizeof(__pyx_k_SubRbc), 0, 0, 1, 1},
  {&__pyx_n_u_SubRbc, __pyx_k_SubRbc, sizeof(__pyx_k_SubRbc), 0, 1, 0, 1},
  {&__pyx_n_s_SubRfar, __pyx_k_SubRfar, sizeof(__pyx_k_SubRfar), 0, 0, 1, 1},
  {&__pyx_n_u_SubRfar, __pyx_k_SubRfar, sizeof(__pyx_k_SubRfar), 0, 1, 0, 1},
  {&__pyx_n_s_SubRnear, __pyx_k_SubRnear, sizeof(__pyx_k_SubRnear), 0, 0, 1, 1},
  {&__pyx_n_u_SubRnear, __pyx_k_SubRnear, sizeof(__pyx_k_SubRnear), 0, 1, 0, 1},
  {&__pyx_n_s_SubRs0, __pyx_k_SubRs0, sizeof(__pyx_k_SubRs0), 0, 0, 1, 1},
  {&__pyx_n_u_SubRs0, __pyx_k_SubRs0, sizeof(__pyx_k_SubRs0), 0, 1, 0, 1},
  {&__pyx_n_s_SubRsc, __pyx_k_SubRsc, sizeof(__pyx_k_SubRsc), 0, 0, 1, 1},
  {&__pyx_n_u_SubRsc, __pyx_k_SubRsc, sizeof(__pyx_k_SubRsc), 0, 1, 0, 1},
  {&__pyx_n_s_SubSA, __pyx_k_SubSA, sizeof(__pyx_k_SubSA), 0, 0, 1, 1},
  {&__pyx_n_s_SubSC, __pyx_k_SubSC, sizeof(__pyx_k_SubSC), 0, 0, 1, 1},
  {&__pyx_n_u_SubSceneArea, __pyx_k_SubSceneArea, sizeof(__pyx_k_SubSceneArea), 0, 1, 0, 1},
  {&__pyx_n_u_SubSceneCenter, __pyx_k_SubSceneCenter, sizeof(__pyx_k_SubSceneCenter), 0, 1, 0, 1},
  {&__pyx_n_s_SubTa, __pyx_k_SubTa, sizeof(__pyx_k_SubTa), 0, 0, 1, 1},
  {&__pyx_n_u_SubTa, __pyx_k_SubTa, sizeof(__pyx_k_SubTa), 0, 1, 0, 1},
  {&__pyx_n_s_SubTr, __pyx_k_SubTr, sizeof(__pyx_k_SubTr), 0, 0, 1, 1},
  {&__pyx_n_u_SubTr, __pyx_k_SubTr, sizeof(__pyx_k_SubTr), 0, 1, 0, 1},
  {&__pyx_n_s_SubTsar, __pyx_k_SubTsar, sizeof(__pyx_k_SubTsar), 0, 0, 1, 1},
  {&__pyx_n_u_SubTsar, __pyx_k_SubTsar, sizeof(__pyx_k_SubTsar), 0, 1, 0, 1},
  {&__pyx_n_s_SubXbc, __pyx_k_SubXbc, sizeof(__pyx_k_SubXbc), 0, 0, 1, 1},
  {&__pyx_n_s_SubXfar, __pyx_k_SubXfar, sizeof(__pyx_k_SubXfar), 0, 0, 1, 1},
  {&__pyx_n_s_SubXnear, __pyx_k_SubXnear, sizeof(__pyx_k_SubXnear), 0, 0, 1, 1},
  {&__pyx_n_s_SubXsc, __pyx_k_SubXsc, sizeof(__pyx_k_SubXsc), 0, 0, 1, 1},
  {&__pyx_n_s_SubYbc, __pyx_k_SubYbc, sizeof(__pyx_k_SubYbc), 0, 0, 1, 1},
  {&__pyx_n_s_SubYfar, __pyx_k_SubYfar, sizeof(__pyx_k_SubYfar), 0, 0, 1, 1},
  {&__pyx_n_s_SubYnear, __pyx_k_SubYnear, sizeof(__pyx_k_SubYnear), 0, 0, 1, 1},
  {&__pyx_n_s_SubYsc, __pyx_k_SubYsc, sizeof(__pyx_k_SubYsc), 0, 0, 1, 1},
  {&__pyx_n_s_Ta, __pyx_k_Ta, sizeof(__pyx_k_Ta), 0, 0, 1, 1},
  {&__pyx_n_u_Ta, __pyx_k_Ta, sizeof(__pyx_k_Ta), 0, 1, 0, 1},
  {&__pyx_n_s_Tensor, __pyx_k_Tensor, sizeof(__pyx_k_Tensor), 0, 0, 1, 1},
  {&__pyx_kp_u_The_antenna_elevation_beamwidth, __pyx_k_The_antenna_elevation_beamwidth, sizeof(__pyx_k_The_antenna_elevation_beamwidth), 0, 1, 0, 0},
  {&__pyx_kp_u_The_synthetic_aperture_length_L, __pyx_k_The_synthetic_aperture_length_L, sizeof(__pyx_k_The_synthetic_aperture_length_L), 0, 1, 0, 0},
  {&__pyx_n_u_To, __pyx_k_To, sizeof(__pyx_k_To), 0, 1, 0, 1},
  {&__pyx_kp_u_To_avoid_grating_lobe_issues_PR, __pyx_k_To_avoid_grating_lobe_issues_PR, sizeof(__pyx_k_To_avoid_grating_lobe_issues_PR), 0, 1, 0, 0},
  {&__pyx_n_s_Tp, __pyx_k_Tp, sizeof(__pyx_k_Tp), 0, 0, 1, 1},
  {&__pyx_n_u_Tp, __pyx_k_Tp, sizeof(__pyx_k_Tp), 0, 1, 0, 1},
  {&__pyx_kp_u_Tr, __pyx_k_Tr, sizeof(__pyx_k_Tr), 0, 1, 0, 0},
  {&__pyx_n_s_Tr_2, __pyx_k_Tr_2, sizeof(__pyx_k_Tr_2), 0, 0, 1, 1},
  {&__pyx_n_u_Tr_2, __pyx_k_Tr_2, sizeof(__pyx_k_Tr_2), 0, 1, 0, 1},
  {&__pyx_n_s_Tsa, __pyx_k_Tsa, sizeof(__pyx_k_Tsa), 0, 0, 1, 1},
  {&__pyx_n_u_Tsa, __pyx_k_Tsa, sizeof(__pyx_k_Tsa), 0, 1, 0, 1},
  {&__pyx_n_s_Tsar, __pyx_k_Tsar, sizeof(__pyx_k_Tsar), 0, 0, 1, 1},
  {&__pyx_n_u_Tsar, __pyx_k_Tsar, sizeof(__pyx_k_Tsar), 0, 1, 0, 1},
  {&__pyx_n_s_Tsr, __pyx_k_Tsr, sizeof(__pyx_k_Tsr), 0, 0, 1, 1},
  {&__pyx_n_u_Tsr, __pyx_k_Tsr, sizeof(__pyx_k_Tsr), 0, 1, 0, 1},
  {&__pyx_n_s_V, __pyx_k_V, sizeof(__pyx_k_V), 0, 0, 1, 1},
  {&__pyx_n_u_V, __pyx_k_V, sizeof(__pyx_k_V), 0, 1, 0, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_Vg, __pyx_k_Vg, sizeof(__pyx_k_Vg), 0, 0, 1, 1},
  {&__pyx_n_u_Vg, __pyx_k_Vg, sizeof(__pyx_k_Vg), 0, 1, 0, 1},
  {&__pyx_n_s_Vr, __pyx_k_Vr, sizeof(__pyx_k_Vr), 0, 0, 1, 1},
  {&__pyx_n_u_Vr, __pyx_k_Vr, sizeof(__pyx_k_Vr), 0, 1, 0, 1},
  {&__pyx_n_s_Vs, __pyx_k_Vs, sizeof(__pyx_k_Vs), 0, 0, 1, 1},
  {&__pyx_n_u_Vs, __pyx_k_Vs, sizeof(__pyx_k_Vs), 0, 1, 0, 1},
  {&__pyx_n_s_Wl, __pyx_k_Wl, sizeof(__pyx_k_Wl), 0, 0, 1, 1},
  {&__pyx_n_u_Wl, __pyx_k_Wl, sizeof(__pyx_k_Wl), 0, 1, 0, 1},
  {&__pyx_n_u_Ws, __pyx_k_Ws, sizeof(__pyx_k_Ws), 0, 1, 0, 1},
  {&__pyx_n_s_Xbc, __pyx_k_Xbc, sizeof(__pyx_k_Xbc), 0, 0, 1, 1},
  {&__pyx_n_s_Xc, __pyx_k_Xc, sizeof(__pyx_k_Xc), 0, 0, 1, 1},
  {&__pyx_n_u_Xc, __pyx_k_Xc, sizeof(__pyx_k_Xc), 0, 1, 0, 1},
  {&__pyx_n_s_Xfar, __pyx_k_Xfar, sizeof(__pyx_k_Xfar), 0, 0, 1, 1},
  {&__pyx_n_s_Xnear, __pyx_k_Xnear, sizeof(__pyx_k_Xnear), 0, 0, 1, 1},
  {&__pyx_n_s_Xsc, __pyx_k_Xsc, sizeof(__pyx_k_Xsc), 0, 0, 1, 1},
  {&__pyx_n_s_Ybc, __pyx_k_Ybc, sizeof(__pyx_k_Ybc), 0, 0, 1, 1},
  {&__pyx_n_s_Yc, __pyx_k_Yc, sizeof(__pyx_k_Yc), 0, 0, 1, 1},
  {&__pyx_n_u_Yc, __pyx_k_Yc, sizeof(__pyx_k_Yc), 0, 1, 0, 1},
  {&__pyx_n_s_Yfar, __pyx_k_Yfar, sizeof(__pyx_k_Yfar), 0, 0, 1, 1},
  {&__pyx_n_s_Ynear, __pyx_k_Ynear, sizeof(__pyx_k_Ynear), 0, 0, 1, 1},
  {&__pyx_n_s_Ysc, __pyx_k_Ysc, sizeof(__pyx_k_Ysc), 0, 0, 1, 1},
  {&__pyx_kp_u__10, __pyx_k__10, sizeof(__pyx_k__10), 0, 1, 0, 0},
  {&__pyx_kp_u__11, __pyx_k__11, sizeof(__pyx_k__11), 0, 1, 0, 0},
  {&__pyx_kp_u__12, __pyx_k__12, sizeof(__pyx_k__12), 0, 1, 0, 0},
  {&__pyx_kp_u__13, __pyx_k__13, sizeof(__pyx_k__13), 0, 1, 0, 0},
  {&__pyx_kp_u__14, __pyx_k__14, sizeof(__pyx_k__14), 0, 1, 0, 0},
  {&__pyx_kp_u__15, __pyx_k__15, sizeof(__pyx_k__15), 0, 1, 0, 0},
  {&__pyx_kp_u__19, __pyx_k__19, sizeof(__pyx_k__19), 0, 1, 0, 0},
  {&__pyx_kp_u__21, __pyx_k__21, sizeof(__pyx_k__21), 0, 1, 0, 0},
  {&__pyx_n_s__23, __pyx_k__23, sizeof(__pyx_k__23), 0, 0, 1, 1},
  {&__pyx_kp_u__5, __pyx_k__5, sizeof(__pyx_k__5), 0, 1, 0, 0},
  {&__pyx_kp_u__6, __pyx_k__6, sizeof(__pyx_k__6), 0, 1, 0, 0},
  {&__pyx_kp_u__7, __pyx_k__7, sizeof(__pyx_k__7), 0, 1, 0, 0},
  {&__pyx_kp_u__8, __pyx_k__8, sizeof(__pyx_k__8), 0, 1, 0, 0},
  {&__pyx_kp_u__9, __pyx_k__9, sizeof(__pyx_k__9), 0, 1, 0, 0},
  {&__pyx_n_s_abs, __pyx_k_abs, sizeof(__pyx_k_abs), 0, 0, 1, 1},
  {&__pyx_n_s_acquisition, __pyx_k_acquisition, sizeof(__pyx_k_acquisition), 0, 0, 1, 1},
  {&__pyx_n_u_acquisition, __pyx_k_acquisition, sizeof(__pyx_k_acquisition), 0, 1, 0, 1},
  {&__pyx_kp_u_acquisition_2, __pyx_k_acquisition_2, sizeof(__pyx_k_acquisition_2), 0, 1, 0, 0},
  {&__pyx_n_s_acquisition_3, __pyx_k_acquisition_3, sizeof(__pyx_k_acquisition_3), 0, 0, 1, 1},
  {&__pyx_kp_u_acquisition_must_be_a_dict, __pyx_k_acquisition_must_be_a_dict, sizeof(__pyx_k_acquisition_must_be_a_dict), 0, 1, 0, 0},
  {&__pyx_n_s_arccos, __pyx_k_arccos, sizeof(__pyx_k_arccos), 0, 0, 1, 1},
  {&__pyx_n_s_array, __pyx_k_array, sizeof(__pyx_k_array), 0, 0, 1, 1},
  {&__pyx_n_s_azimuth_beamwidth, __pyx_k_azimuth_beamwidth, sizeof(__pyx_k_azimuth_beamwidth), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_compute_range_beamwidth2, __pyx_k_compute_range_beamwidth2, sizeof(__pyx_k_compute_range_beamwidth2), 0, 0, 1, 1},
  {&__pyx_n_s_cos, __pyx_k_cos, sizeof(__pyx_k_cos), 0, 0, 1, 1},
  {&__pyx_n_s_cr_footprint, __pyx_k_cr_footprint, sizeof(__pyx_k_cr_footprint), 0, 0, 1, 1},
  {&__pyx_kp_u_degree, __pyx_k_degree, sizeof(__pyx_k_degree), 0, 1, 0, 0},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_error, __pyx_k_error, sizeof(__pyx_k_error), 0, 0, 1, 1},
  {&__pyx_n_u_f, __pyx_k_f, sizeof(__pyx_k_f), 0, 1, 0, 1},
  {&__pyx_kp_u_f_2, __pyx_k_f_2, sizeof(__pyx_k_f_2), 0, 1, 0, 0},
  {&__pyx_n_s_fa, __pyx_k_fa, sizeof(__pyx_k_fa), 0, 0, 1, 1},
  {&__pyx_n_u_fa, __pyx_k_fa, sizeof(__pyx_k_fa), 0, 1, 0, 1},
  {&__pyx_n_s_faSub, __pyx_k_faSub, sizeof(__pyx_k_faSub), 0, 0, 1, 1},
  {&__pyx_n_u_faSub, __pyx_k_faSub, sizeof(__pyx_k_faSub), 0, 1, 0, 1},
  {&__pyx_n_s_fadc, __pyx_k_fadc, sizeof(__pyx_k_fadc), 0, 0, 1, 1},
  {&__pyx_n_u_fadc, __pyx_k_fadc, sizeof(__pyx_k_fadc), 0, 1, 0, 1},
  {&__pyx_n_s_fr, __pyx_k_fr, sizeof(__pyx_k_fr), 0, 0, 1, 1},
  {&__pyx_n_u_fr, __pyx_k_fr, sizeof(__pyx_k_fr), 0, 1, 0, 1},
  {&__pyx_n_s_frSub, __pyx_k_frSub, sizeof(__pyx_k_frSub), 0, 0, 1, 1},
  {&__pyx_n_u_frSub, __pyx_k_frSub, sizeof(__pyx_k_frSub), 0, 1, 0, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_info, __pyx_k_info, sizeof(__pyx_k_info), 0, 0, 1, 1},
  {&__pyx_kp_u_is_large_than_r_n_the_cross_ran, __pyx_k_is_large_than_r_n_the_cross_ran, sizeof(__pyx_k_is_large_than_r_n_the_cross_ran), 0, 1, 0, 0},
  {&__pyx_n_s_items, __pyx_k_items, sizeof(__pyx_k_items), 0, 0, 1, 1},
  {&__pyx_n_s_k, __pyx_k_k, sizeof(__pyx_k_k), 0, 0, 1, 1},
  {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {&__pyx_n_s_linspace, __pyx_k_linspace, sizeof(__pyx_k_linspace), 0, 0, 1, 1},
  {&__pyx_n_s_logging, __pyx_k_logging, sizeof(__pyx_k_logging), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_min_slant_range, __pyx_k_min_slant_range, sizeof(__pyx_k_min_slant_range), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_n_s_name_3, __pyx_k_name_3, sizeof(__pyx_k_name_3), 0, 0, 1, 1},
  {&__pyx_kp_u_name_must_be_a_string, __pyx_k_name_must_be_a_string, sizeof(__pyx_k_name_must_be_a_string), 0, 1, 0, 0},
  {&__pyx_n_s_np, __pyx_k_np, sizeof(__pyx_k_np), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_n_s_object, __pyx_k_object, sizeof(__pyx_k_object), 0, 0, 1, 1},
  {&__pyx_kp_u_or_Abr_should_be_Abr, __pyx_k_or_Abr_should_be_Abr, sizeof(__pyx_k_or_Abr_should_be_Abr), 0, 1, 0, 0},
  {&__pyx_kp_u_or_Fsr_should_be_NrTr, __pyx_k_or_Fsr_should_be_NrTr, sizeof(__pyx_k_or_Fsr_should_be_NrTr), 0, 1, 0, 0},
  {&__pyx_n_s_param, __pyx_k_param, sizeof(__pyx_k_param), 0, 0, 1, 1},
  {&__pyx_n_s_params, __pyx_k_params, sizeof(__pyx_k_params), 0, 0, 1, 1},
  {&__pyx_n_u_params, __pyx_k_params, sizeof(__pyx_k_params), 0, 1, 0, 1},
  {&__pyx_kp_u_params_2, __pyx_k_params_2, sizeof(__pyx_k_params_2), 0, 1, 0, 0},
  {&__pyx_n_s_params_3, __pyx_k_params_3, sizeof(__pyx_k_params_3), 0, 0, 1, 1},
  {&__pyx_kp_u_params_must_be_a_dict, __pyx_k_params_must_be_a_dict, sizeof(__pyx_k_params_must_be_a_dict), 0, 1, 0, 0},
  {&__pyx_n_s_pi, __pyx_k_pi, sizeof(__pyx_k_pi), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_print, __pyx_k_print, sizeof(__pyx_k_print), 0, 0, 1, 1},
  {&__pyx_n_s_printsp, __pyx_k_printsp, sizeof(__pyx_k_printsp), 0, 0, 1, 1},
  {&__pyx_n_s_property, __pyx_k_property, sizeof(__pyx_k_property), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_kp_u_r_Sampling_rate_should_be_lower, __pyx_k_r_Sampling_rate_should_be_lower, sizeof(__pyx_k_r_Sampling_rate_should_be_lower), 0, 1, 0, 0},
  {&__pyx_n_s_selection, __pyx_k_selection, sizeof(__pyx_k_selection), 0, 0, 1, 1},
  {&__pyx_kp_u_selection_2, __pyx_k_selection_2, sizeof(__pyx_k_selection_2), 0, 1, 0, 0},
  {&__pyx_n_s_selection_3, __pyx_k_selection_3, sizeof(__pyx_k_selection_3), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_n_s_sensor, __pyx_k_sensor, sizeof(__pyx_k_sensor), 0, 0, 1, 1},
  {&__pyx_n_u_sensor, __pyx_k_sensor, sizeof(__pyx_k_sensor), 0, 1, 0, 1},
  {&__pyx_kp_u_sensor_2, __pyx_k_sensor_2, sizeof(__pyx_k_sensor_2), 0, 1, 0, 0},
  {&__pyx_n_s_sensor_3, __pyx_k_sensor_3, sizeof(__pyx_k_sensor_3), 0, 0, 1, 1},
  {&__pyx_kp_u_sensor_must_be_a_dict, __pyx_k_sensor_must_be_a_dict, sizeof(__pyx_k_sensor_must_be_a_dict), 0, 1, 0, 0},
  {&__pyx_kp_u_setted_equivalent_velocity_is, __pyx_k_setted_equivalent_velocity_is, sizeof(__pyx_k_setted_equivalent_velocity_is), 0, 1, 0, 0},
  {&__pyx_n_s_setter, __pyx_k_setter, sizeof(__pyx_k_setter), 0, 0, 1, 1},
  {&__pyx_n_s_shape, __pyx_k_shape, sizeof(__pyx_k_shape), 0, 0, 1, 1},
  {&__pyx_kp_u_shape_of, __pyx_k_shape_of, sizeof(__pyx_k_shape_of), 0, 1, 0, 0},
  {&__pyx_kp_u_should_in, __pyx_k_should_in, sizeof(__pyx_k_should_in), 0, 1, 0, 0},
  {&__pyx_n_s_sign, __pyx_k_sign, sizeof(__pyx_k_sign), 0, 0, 1, 1},
  {&__pyx_n_s_sin, __pyx_k_sin, sizeof(__pyx_k_sin), 0, 0, 1, 1},
  {&__pyx_n_s_sqrt, __pyx_k_sqrt, sizeof(__pyx_k_sqrt), 0, 0, 1, 1},
  {&__pyx_n_s_t0, __pyx_k_t0, sizeof(__pyx_k_t0), 0, 0, 1, 1},
  {&__pyx_n_u_t0, __pyx_k_t0, sizeof(__pyx_k_t0), 0, 1, 0, 1},
  {&__pyx_n_s_t0Sub, __pyx_k_t0Sub, sizeof(__pyx_k_t0Sub), 0, 0, 1, 1},
  {&__pyx_n_s_ta, __pyx_k_ta, sizeof(__pyx_k_ta), 0, 0, 1, 1},
  {&__pyx_n_u_ta, __pyx_k_ta, sizeof(__pyx_k_ta), 0, 1, 0, 1},
  {&__pyx_n_s_taSub, __pyx_k_taSub, sizeof(__pyx_k_taSub), 0, 0, 1, 1},
  {&__pyx_n_u_taSub, __pyx_k_taSub, sizeof(__pyx_k_taSub), 0, 1, 0, 1},
  {&__pyx_n_s_tac, __pyx_k_tac, sizeof(__pyx_k_tac), 0, 0, 1, 1},
  {&__pyx_n_u_tac, __pyx_k_tac, sizeof(__pyx_k_tac), 0, 1, 0, 1},
  {&__pyx_n_s_tan, __pyx_k_tan, sizeof(__pyx_k_tan), 0, 0, 1, 1},
  {&__pyx_n_s_tend, __pyx_k_tend, sizeof(__pyx_k_tend), 0, 0, 1, 1},
  {&__pyx_n_u_tend, __pyx_k_tend, sizeof(__pyx_k_tend), 0, 1, 0, 1},
  {&__pyx_n_s_tendSub, __pyx_k_tendSub, sizeof(__pyx_k_tendSub), 0, 0, 1, 1},
  {&__pyx_n_u_tendSub, __pyx_k_tendSub, sizeof(__pyx_k_tendSub), 0, 1, 0, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_tfar, __pyx_k_tfar, sizeof(__pyx_k_tfar), 0, 0, 1, 1},
  {&__pyx_n_u_tfar, __pyx_k_tfar, sizeof(__pyx_k_tfar), 0, 1, 0, 1},
  {&__pyx_n_s_tfarSub, __pyx_k_tfarSub, sizeof(__pyx_k_tfarSub), 0, 0, 1, 1},
  {&__pyx_n_u_tfarSub, __pyx_k_tfarSub, sizeof(__pyx_k_tfarSub), 0, 1, 0, 1},
  {&__pyx_n_s_th, __pyx_k_th, sizeof(__pyx_k_th), 0, 0, 1, 1},
  {&__pyx_n_s_tnear, __pyx_k_tnear, sizeof(__pyx_k_tnear), 0, 0, 1, 1},
  {&__pyx_n_u_tnear, __pyx_k_tnear, sizeof(__pyx_k_tnear), 0, 1, 0, 1},
  {&__pyx_n_s_tnearSub, __pyx_k_tnearSub, sizeof(__pyx_k_tnearSub), 0, 0, 1, 1},
  {&__pyx_n_u_tnearSub, __pyx_k_tnearSub, sizeof(__pyx_k_tnearSub), 0, 1, 0, 1},
  {&__pyx_n_s_torch, __pyx_k_torch, sizeof(__pyx_k_torch), 0, 0, 1, 1},
  {&__pyx_n_s_torchbox_utils_const, __pyx_k_torchbox_utils_const, sizeof(__pyx_k_torchbox_utils_const), 0, 0, 1, 1},
  {&__pyx_n_s_torchsar_misc_sarplat, __pyx_k_torchsar_misc_sarplat, sizeof(__pyx_k_torchsar_misc_sarplat), 0, 0, 1, 1},
  {&__pyx_kp_s_torchsar_misc_sarplat_py, __pyx_k_torchsar_misc_sarplat_py, sizeof(__pyx_k_torchsar_misc_sarplat_py), 0, 0, 1, 0},
  {&__pyx_n_s_torchsar_sharing_beamwidth_footp, __pyx_k_torchsar_sharing_beamwidth_footp, sizeof(__pyx_k_torchsar_sharing_beamwidth_footp), 0, 0, 1, 1},
  {&__pyx_n_s_torchsar_sharing_slant_ground_ra, __pyx_k_torchsar_sharing_slant_ground_ra, sizeof(__pyx_k_torchsar_sharing_slant_ground_ra), 0, 0, 1, 1},
  {&__pyx_n_s_tr, __pyx_k_tr, sizeof(__pyx_k_tr), 0, 0, 1, 1},
  {&__pyx_n_u_tr, __pyx_k_tr, sizeof(__pyx_k_tr), 0, 1, 0, 1},
  {&__pyx_n_s_trSub, __pyx_k_trSub, sizeof(__pyx_k_trSub), 0, 0, 1, 1},
  {&__pyx_n_u_trSub, __pyx_k_trSub, sizeof(__pyx_k_trSub), 0, 1, 0, 1},
  {&__pyx_n_s_tstart, __pyx_k_tstart, sizeof(__pyx_k_tstart), 0, 0, 1, 1},
  {&__pyx_n_u_tstart, __pyx_k_tstart, sizeof(__pyx_k_tstart), 0, 1, 0, 1},
  {&__pyx_n_s_tstartSub, __pyx_k_tstartSub, sizeof(__pyx_k_tstartSub), 0, 0, 1, 1},
  {&__pyx_n_u_tstartSub, __pyx_k_tstartSub, sizeof(__pyx_k_tstartSub), 0, 1, 0, 1},
  {&__pyx_kp_u_used, __pyx_k_used, sizeof(__pyx_k_used), 0, 1, 0, 0},
  {&__pyx_n_s_v, __pyx_k_v, sizeof(__pyx_k_v), 0, 0, 1, 1},
  {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {&__pyx_n_s_verbose, __pyx_k_verbose, sizeof(__pyx_k_verbose), 0, 0, 1, 1},
  {&__pyx_n_s_warning, __pyx_k_warning, sizeof(__pyx_k_warning), 0, 0, 1, 1},
  {&__pyx_kp_u_which_maks_a_point_outside_the, __pyx_k_which_maks_a_point_outside_the, sizeof(__pyx_k_which_maks_a_point_outside_the), 0, 1, 0, 0},
  {&__pyx_n_s_xmax, __pyx_k_xmax, sizeof(__pyx_k_xmax), 0, 0, 1, 1},
  {&__pyx_n_u_xmax, __pyx_k_xmax, sizeof(__pyx_k_xmax), 0, 1, 0, 1},
  {&__pyx_n_s_xmaxSub, __pyx_k_xmaxSub, sizeof(__pyx_k_xmaxSub), 0, 0, 1, 1},
  {&__pyx_n_u_xmaxSub, __pyx_k_xmaxSub, sizeof(__pyx_k_xmaxSub), 0, 1, 0, 1},
  {&__pyx_n_s_xmin, __pyx_k_xmin, sizeof(__pyx_k_xmin), 0, 0, 1, 1},
  {&__pyx_n_u_xmin, __pyx_k_xmin, sizeof(__pyx_k_xmin), 0, 1, 0, 1},
  {&__pyx_n_s_xminSub, __pyx_k_xminSub, sizeof(__pyx_k_xminSub), 0, 0, 1, 1},
  {&__pyx_n_u_xminSub, __pyx_k_xminSub, sizeof(__pyx_k_xminSub), 0, 1, 0, 1},
  {&__pyx_n_s_ymax, __pyx_k_ymax, sizeof(__pyx_k_ymax), 0, 0, 1, 1},
  {&__pyx_n_u_ymax, __pyx_k_ymax, sizeof(__pyx_k_ymax), 0, 1, 0, 1},
  {&__pyx_n_s_ymaxSub, __pyx_k_ymaxSub, sizeof(__pyx_k_ymaxSub), 0, 0, 1, 1},
  {&__pyx_n_u_ymaxSub, __pyx_k_ymaxSub, sizeof(__pyx_k_ymaxSub), 0, 1, 0, 1},
  {&__pyx_n_s_ymin, __pyx_k_ymin, sizeof(__pyx_k_ymin), 0, 0, 1, 1},
  {&__pyx_n_u_ymin, __pyx_k_ymin, sizeof(__pyx_k_ymin), 0, 1, 0, 1},
  {&__pyx_n_s_yminSub, __pyx_k_yminSub, sizeof(__pyx_k_yminSub), 0, 0, 1, 1},
  {&__pyx_n_u_yminSub, __pyx_k_yminSub, sizeof(__pyx_k_yminSub), 0, 1, 0, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_object = __Pyx_GetBuiltinName(__pyx_n_s_object); if (!__pyx_builtin_object) __PYX_ERR(0, 38, __pyx_L1_error)
  __pyx_builtin_property = __Pyx_GetBuiltinName(__pyx_n_s_property); if (!__pyx_builtin_property) __PYX_ERR(0, 45, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 54, __pyx_L1_error)
  __pyx_builtin_print = __Pyx_GetBuiltinName(__pyx_n_s_print); if (!__pyx_builtin_print) __PYX_ERR(0, 330, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "torchsar/misc/sarplat.py":54
 * 
 *         if not isinstance(value, str):
 *             raise ValueError('name must be a string!')             # <<<<<<<<<<<<<<
 *         self._name = value
 *         logging.info("---Done!")
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_u_name_must_be_a_string); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "torchsar/misc/sarplat.py":67
 * 
 *         if not isinstance(value, dict):
 *             raise ValueError('sensor must be a dict!')             # <<<<<<<<<<<<<<
 *         if hasattr(self, '_sensor'):
 *             for k, v in value.items():
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_u_sensor_must_be_a_dict); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "torchsar/misc/sarplat.py":113
 *         if value is not None:
 *             if not isinstance(value, dict):
 *                 raise ValueError('acquisition must be a dict!')             # <<<<<<<<<<<<<<
 *             if hasattr(self, '_acquisition'):
 *                 for k, v in value.items():
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_u_acquisition_must_be_a_dict); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "torchsar/misc/sarplat.py":254
 *         if value is not None:
 *             if not isinstance(value, dict):
 *                 raise ValueError('params must be a dict!')             # <<<<<<<<<<<<<<
 *             if hasattr(self, '_params'):
 *                 for k, v in value.items():
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_u_params_must_be_a_dict); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "torchsar/misc/sarplat.py":675
 *     def printsp(self, verbose=False):
 *         print("===============SAR platform: ", self._name, "================")
 *         print("--------------------------sensor-----------------------------")             # <<<<<<<<<<<<<<
 *         for key in sorted(self._sensor):
 *             print(key, self._sensor[key])
 */
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_kp_u_sensor_2); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(0, 675, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "torchsar/misc/sarplat.py":679
 *             print(key, self._sensor[key])
 * 
 *         print("-----------------------acquisition---------------------------")             # <<<<<<<<<<<<<<
 *         for key in sorted(self._acquisition):
 *             print(key, self._acquisition[key])
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_kp_u_acquisition_2); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 679, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "torchsar/misc/sarplat.py":683
 *             print(key, self._acquisition[key])
 * 
 *         print("--------------------------params-----------------------------")             # <<<<<<<<<<<<<<
 *         if verbose:
 *             print(self._params)
 */
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_kp_u_params_2); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(0, 683, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "torchsar/misc/sarplat.py":694
 *                     print(key, self._params[key])
 * 
 *         print("-------------------------selection---------------------------")             # <<<<<<<<<<<<<<
 *         for key in sorted(self._selection):
 *             value = self._selection[key]
 */
  __pyx_tuple__20 = PyTuple_Pack(1, __pyx_kp_u_selection_2); if (unlikely(!__pyx_tuple__20)) __PYX_ERR(0, 694, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__20);
  __Pyx_GIVEREF(__pyx_tuple__20);

  /* "torchsar/misc/sarplat.py":701
 *             else:
 *                 print(key, self._selection[key])
 *         print("=============================================================")             # <<<<<<<<<<<<<<
 */
  __pyx_tuple__22 = PyTuple_Pack(1, __pyx_kp_u__21); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(0, 701, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);

  /* "torchsar/misc/sarplat.py":38
 * 
 * 
 * class SarPlat(object):             # <<<<<<<<<<<<<<
 *     r"""SAR platform class.
 *         sensor
 */
  __pyx_tuple__24 = PyTuple_Pack(1, __pyx_builtin_object); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);

  /* "torchsar/misc/sarplat.py":46
 * 
 *     @property
 *     def name(self):             # <<<<<<<<<<<<<<
 *         return self._name
 * 
 */
  __pyx_tuple__25 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(0, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);
  __pyx_codeobj__26 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__25, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_torchsar_misc_sarplat_py, __pyx_n_s_name_3, 46, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__26)) __PYX_ERR(0, 46, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":50
 * 
 *     @name.setter
 *     def name(self, value):             # <<<<<<<<<<<<<<
 *         logging.info("---Check and set SAR platform name...")
 * 
 */
  __pyx_tuple__27 = PyTuple_Pack(2, __pyx_n_s_self, __pyx_n_s_value); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);
  __pyx_codeobj__28 = (PyObject*)__Pyx_PyCode_New(2, 0, 2, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__27, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_torchsar_misc_sarplat_py, __pyx_n_s_name_3, 50, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__28)) __PYX_ERR(0, 50, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":59
 * 
 *     @property
 *     def sensor(self):             # <<<<<<<<<<<<<<
 *         return self._sensor
 * 
 */
  __pyx_tuple__29 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);
  __pyx_codeobj__30 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__29, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_torchsar_misc_sarplat_py, __pyx_n_s_sensor_3, 59, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__30)) __PYX_ERR(0, 59, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":63
 * 
 *     @sensor.setter
 *     def sensor(self, value):             # <<<<<<<<<<<<<<
 *         logging.info("---Check and set SAR sensor parameters...")
 * 
 */
  __pyx_tuple__31 = PyTuple_Pack(7, __pyx_n_s_self, __pyx_n_s_value, __pyx_n_s_k, __pyx_n_s_v, __pyx_n_s_Re, __pyx_n_s_Rs, __pyx_n_s_Vr); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);
  __pyx_codeobj__32 = (PyObject*)__Pyx_PyCode_New(2, 0, 7, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__31, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_torchsar_misc_sarplat_py, __pyx_n_s_sensor_3, 63, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__32)) __PYX_ERR(0, 63, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":104
 * 
 *     @property
 *     def acquisition(self):             # <<<<<<<<<<<<<<
 *         return self._acquisition
 * 
 */
  __pyx_tuple__33 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__33)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);
  __pyx_codeobj__34 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__33, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_torchsar_misc_sarplat_py, __pyx_n_s_acquisition_3, 104, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__34)) __PYX_ERR(0, 104, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":108
 * 
 *     @acquisition.setter
 *     def acquisition(self, value):             # <<<<<<<<<<<<<<
 *         logging.info("---Check and set acquisition parameters...")
 * 
 */
  __pyx_tuple__35 = PyTuple_Pack(48, __pyx_n_s_self, __pyx_n_s_value, __pyx_n_s_k, __pyx_n_s_v, __pyx_n_s_H, __pyx_n_s_V, __pyx_n_s_Vr, __pyx_n_s_Vs, __pyx_n_s_PRF, __pyx_n_s_Fsr, __pyx_n_s_Tp, __pyx_n_s_Rs, __pyx_n_s_As, __pyx_n_s_Ad, __pyx_n_s_Aon, __pyx_n_s_Aba, __pyx_n_s_Abr, __pyx_n_s_SA, __pyx_n_s_PC, __pyx_n_s_ES, __pyx_n_s_SV, __pyx_n_s_Ar, __pyx_n_s_SAV, __pyx_n_s_BAV, __pyx_n_s_Nr, __pyx_n_s_Rnear, __pyx_n_s_Rfar, __pyx_n_s_FPr, __pyx_n_s_Rsc, __pyx_n_s_Rs0, __pyx_n_s_Ysc, __pyx_n_s_Xsc, __pyx_n_s_SC, __pyx_n_s_Rbc, __pyx_n_s_Rb0, __pyx_n_s_Ybc, __pyx_n_s_Xbc, __pyx_n_s_BC, __pyx_n_s_Ynear, __pyx_n_s_Yfar, __pyx_n_s_Xnear, __pyx_n_s_Xfar, __pyx_n_s_Na, __pyx_n_s_Ta, __pyx_n_s_FPr0, __pyx_n_s_FPr1, __pyx_n_s_Ai, __pyx_n_s_Ao); if (unlikely(!__pyx_tuple__35)) __PYX_ERR(0, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__35);
  __Pyx_GIVEREF(__pyx_tuple__35);
  __pyx_codeobj__36 = (PyObject*)__Pyx_PyCode_New(2, 0, 48, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__35, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_torchsar_misc_sarplat_py, __pyx_n_s_acquisition_3, 108, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__36)) __PYX_ERR(0, 108, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":244
 * 
 *     @property
 *     def params(self):             # <<<<<<<<<<<<<<
 *         return self._params
 * 
 */
  __pyx_tuple__37 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__37)) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__37);
  __Pyx_GIVEREF(__pyx_tuple__37);
  __pyx_codeobj__38 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__37, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_torchsar_misc_sarplat_py, __pyx_n_s_params_3, 244, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__38)) __PYX_ERR(0, 244, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":248
 * 
 *     @params.setter
 *     def params(self, value):             # <<<<<<<<<<<<<<
 * 
 *         logging.info("---Check and set algorithm parameters...")
 */
  __pyx_tuple__39 = PyTuple_Pack(74, __pyx_n_s_self, __pyx_n_s_value, __pyx_n_s_k, __pyx_n_s_v, __pyx_n_s_GM, __pyx_n_s_H, __pyx_n_s_V, __pyx_n_s_Vr, __pyx_n_s_Vg, __pyx_n_s_Vs, __pyx_n_s_La, __pyx_n_s_Fsa, __pyx_n_s_Fsr, __pyx_n_s_Tp, __pyx_n_s_Fc, __pyx_n_s_Wl, __pyx_n_s_Kr, __pyx_n_s_Rbc, __pyx_n_s_Rb0, __pyx_n_s_Rsc, __pyx_n_s_Rs0, __pyx_n_s_Rnear, __pyx_n_s_Rfar, __pyx_n_s_PC, __pyx_n_s_SC, __pyx_n_s_BC, __pyx_n_s_SA, __pyx_n_s_BA, __pyx_n_s_ES, __pyx_n_s_As, __pyx_n_s_Ar, __pyx_n_s_Ad, __pyx_n_s_Aon, __pyx_n_s_Aba, __pyx_n_s_Abr, __pyx_n_s_Ai, __pyx_n_s_Xc, __pyx_n_s_Yc, __pyx_n_s_t0, __pyx_n_s_R0, __pyx_n_s_Lsar, __pyx_n_s_Ka, __pyx_n_s_Tsar, __pyx_n_s_Ba_2, __pyx_n_s_PRF, __pyx_n_s_PRT, __pyx_n_s_Tsa, __pyx_n_s_Fs, __pyx_n_s_Tsr, __pyx_n_s_Na, __pyx_n_s_Nr, __pyx_n_s_Ta, __pyx_n_s_ta, __pyx_n_s_fa, __pyx_n_s_tnear, __pyx_n_s_tfar, __pyx_n_s_tstart, __pyx_n_s_tend, __pyx_n_s_Tr_2, __pyx_n_s_tr, __pyx_n_s_fr, __pyx_n_s_tac, __pyx_n_s_fadc, __pyx_n_s_Bdop, __pyx_n_s_xmin, __pyx_n_s_xmax, __pyx_n_s_ymin, __pyx_n_s_ymax, __pyx_n_s_FPa, __pyx_n_s_FPr, __pyx_n_s_BWa, __pyx_n_s_PRFmin, __pyx_n_s_PRFmax, __pyx_n_s_param); if (unlikely(!__pyx_tuple__39)) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__39);
  __Pyx_GIVEREF(__pyx_tuple__39);
  __pyx_codeobj__40 = (PyObject*)__Pyx_PyCode_New(2, 0, 74, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__39, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_torchsar_misc_sarplat_py, __pyx_n_s_params_3, 248, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__40)) __PYX_ERR(0, 248, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":471
 * 
 *     @property
 *     def selection(self):             # <<<<<<<<<<<<<<
 *         return self._selection
 * 
 */
  __pyx_tuple__41 = PyTuple_Pack(1, __pyx_n_s_self); if (unlikely(!__pyx_tuple__41)) __PYX_ERR(0, 471, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__41);
  __Pyx_GIVEREF(__pyx_tuple__41);
  __pyx_codeobj__42 = (PyObject*)__Pyx_PyCode_New(1, 0, 1, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__41, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_torchsar_misc_sarplat_py, __pyx_n_s_selection_3, 471, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__42)) __PYX_ERR(0, 471, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":475
 * 
 *     @selection.setter
 *     def selection(self, value):             # <<<<<<<<<<<<<<
 *         logging.info("---Check and set sub area selection...")
 *         SA = self._acquisition['SceneArea']
 */
  __pyx_tuple__43 = PyTuple_Pack(77, __pyx_n_s_self, __pyx_n_s_value, __pyx_n_s_SA, __pyx_n_s_SC, __pyx_n_s_BC, __pyx_n_s_ES, __pyx_n_s_Ad, __pyx_n_s_Aon, __pyx_n_s_As, __pyx_n_s_Ar, __pyx_n_s_Fsr, __pyx_n_s_Fsa, __pyx_n_s_Tsa, __pyx_n_s_Tsr, __pyx_n_s_Na, __pyx_n_s_Nr, __pyx_n_s_Rnear, __pyx_n_s_Rfar, __pyx_n_s_FPr, __pyx_n_s_H, __pyx_n_s_Vr, __pyx_n_s_Tp, __pyx_n_s_Wl, __pyx_n_s_La, __pyx_n_s_GM, __pyx_n_s_Lsar, __pyx_n_s_SubSA, __pyx_n_s_SubES, __pyx_n_s_SubEA, __pyx_n_s_SubNa, __pyx_n_s_SubNr, __pyx_n_s_SubRnear, __pyx_n_s_SubRfar, __pyx_n_s_SubAon, __pyx_n_s_SubAbr, __pyx_n_s_SubAd, __pyx_n_s_SubFPr, __pyx_n_s_SubTa, __pyx_n_s_taSub, __pyx_n_s_faSub, __pyx_n_s_SubRsc, __pyx_n_s_SubRs0, __pyx_n_s_SubYsc, __pyx_n_s_SubXsc, __pyx_n_s_SubSC, __pyx_n_s_SubRbc, __pyx_n_s_SubRb0, __pyx_n_s_SubYbc, __pyx_n_s_SubXbc, __pyx_n_s_SubBC, __pyx_n_s_SubYnear, __pyx_n_s_SubYfar, __pyx_n_s_SubXnear, __pyx_n_s_SubXfar, __pyx_n_s_SubFPr0, __pyx_n_s_SubFPr1, __pyx_n_s_SubBA, __pyx_n_s_tnearSub, __pyx_n_s_tfarSub, __pyx_n_s_tstartSub, __pyx_n_s_tendSub, __pyx_n_s_SubTr, __pyx_n_s_Xc, __pyx_n_s_Yc, __pyx_n_s_SubR0, __pyx_n_s_t0Sub, __pyx_n_s_trSub, __pyx_n_s_xminSub, __pyx_n_s_xmaxSub, __pyx_n_s_yminSub, __pyx_n_s_ymaxSub, __pyx_n_s_frSub, __pyx_n_s_SubKa, __pyx_n_s_SubLsar, __pyx_n_s_SubTsar, __pyx_n_s_SubBa, __pyx_n_s_SubFPa); if (unlikely(!__pyx_tuple__43)) __PYX_ERR(0, 475, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__43);
  __Pyx_GIVEREF(__pyx_tuple__43);
  __pyx_codeobj__44 = (PyObject*)__Pyx_PyCode_New(2, 0, 77, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__43, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_torchsar_misc_sarplat_py, __pyx_n_s_selection_3, 475, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__44)) __PYX_ERR(0, 475, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":673
 *         logging.info('---Done!')
 * 
 *     def printsp(self, verbose=False):             # <<<<<<<<<<<<<<
 *         print("===============SAR platform: ", self._name, "================")
 *         print("--------------------------sensor-----------------------------")
 */
  __pyx_tuple__45 = PyTuple_Pack(4, __pyx_n_s_self, __pyx_n_s_verbose, __pyx_n_s_key, __pyx_n_s_value); if (unlikely(!__pyx_tuple__45)) __PYX_ERR(0, 673, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__45);
  __Pyx_GIVEREF(__pyx_tuple__45);
  __pyx_codeobj__46 = (PyObject*)__Pyx_PyCode_New(2, 0, 4, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__45, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_torchsar_misc_sarplat_py, __pyx_n_s_printsp, 673, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__46)) __PYX_ERR(0, 673, __pyx_L1_error)
  __pyx_tuple__47 = PyTuple_Pack(1, ((PyObject *)Py_False)); if (unlikely(!__pyx_tuple__47)) __PYX_ERR(0, 673, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__47);
  __Pyx_GIVEREF(__pyx_tuple__47);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float_2_ = PyFloat_FromDouble(2.); if (unlikely(!__pyx_float_2_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_0_0 = PyFloat_FromDouble(0.0); if (unlikely(!__pyx_float_0_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_0_5 = PyFloat_FromDouble(0.5); if (unlikely(!__pyx_float_0_5)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_1_0 = PyFloat_FromDouble(1.0); if (unlikely(!__pyx_float_1_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_1_1 = PyFloat_FromDouble(1.1); if (unlikely(!__pyx_float_1_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_2_0 = PyFloat_FromDouble(2.0); if (unlikely(!__pyx_float_2_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_1_25 = PyFloat_FromDouble(1.25); if (unlikely(!__pyx_float_1_25)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_180_ = PyFloat_FromDouble(180.); if (unlikely(!__pyx_float_180_)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_60e3 = PyFloat_FromDouble(60e3); if (unlikely(!__pyx_float_60e3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_0_886 = PyFloat_FromDouble(0.886); if (unlikely(!__pyx_float_0_886)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_1_905 = PyFloat_FromDouble(1.905); if (unlikely(!__pyx_float_1_905)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_float_neg_2_0 = PyFloat_FromDouble(-2.0); if (unlikely(!__pyx_float_neg_2_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initsarplat(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initsarplat(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_sarplat(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_sarplat(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_sarplat(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'sarplat' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_sarplat(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("sarplat", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_torchsar__misc__sarplat) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name_2, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "torchsar.misc.sarplat")) {
      if (unlikely(PyDict_SetItemString(modules, "torchsar.misc.sarplat", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  (void)__Pyx_modinit_type_init_code();
  (void)__Pyx_modinit_type_import_code();
  (void)__Pyx_modinit_variable_import_code();
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "torchsar/misc/sarplat.py":28
 * #
 * 
 * import logging             # <<<<<<<<<<<<<<
 * import torch as th
 * import numpy as np
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_logging, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_logging, __pyx_t_1) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":29
 * 
 * import logging
 * import torch as th             # <<<<<<<<<<<<<<
 * import numpy as np
 * from torchbox.utils.const import *
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_torch, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_th, __pyx_t_1) < 0) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":30
 * import logging
 * import torch as th
 * import numpy as np             # <<<<<<<<<<<<<<
 * from torchbox.utils.const import *
 * from torchsar.sharing.beamwidth_footprint import azimuth_beamwidth, compute_range_beamwidth2, cr_footprint
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_np, __pyx_t_1) < 0) __PYX_ERR(0, 30, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":31
 * import torch as th
 * import numpy as np
 * from torchbox.utils.const import *             # <<<<<<<<<<<<<<
 * from torchsar.sharing.beamwidth_footprint import azimuth_beamwidth, compute_range_beamwidth2, cr_footprint
 * from torchsar.sharing.slant_ground_range import min_slant_range
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s__23);
  __Pyx_GIVEREF(__pyx_n_s__23);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s__23);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_torchbox_utils_const, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_import_star(__pyx_t_2) < 0) __PYX_ERR(0, 31, __pyx_L1_error);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":32
 * import numpy as np
 * from torchbox.utils.const import *
 * from torchsar.sharing.beamwidth_footprint import azimuth_beamwidth, compute_range_beamwidth2, cr_footprint             # <<<<<<<<<<<<<<
 * from torchsar.sharing.slant_ground_range import min_slant_range
 * 
 */
  __pyx_t_2 = PyList_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_azimuth_beamwidth);
  __Pyx_GIVEREF(__pyx_n_s_azimuth_beamwidth);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_azimuth_beamwidth);
  __Pyx_INCREF(__pyx_n_s_compute_range_beamwidth2);
  __Pyx_GIVEREF(__pyx_n_s_compute_range_beamwidth2);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_s_compute_range_beamwidth2);
  __Pyx_INCREF(__pyx_n_s_cr_footprint);
  __Pyx_GIVEREF(__pyx_n_s_cr_footprint);
  PyList_SET_ITEM(__pyx_t_2, 2, __pyx_n_s_cr_footprint);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_torchsar_sharing_beamwidth_footp, __pyx_t_2, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_azimuth_beamwidth); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_azimuth_beamwidth, __pyx_t_2) < 0) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_compute_range_beamwidth2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_compute_range_beamwidth2, __pyx_t_2) < 0) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_cr_footprint); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_cr_footprint, __pyx_t_2) < 0) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "torchsar/misc/sarplat.py":33
 * from torchbox.utils.const import *
 * from torchsar.sharing.beamwidth_footprint import azimuth_beamwidth, compute_range_beamwidth2, cr_footprint
 * from torchsar.sharing.slant_ground_range import min_slant_range             # <<<<<<<<<<<<<<
 * 
 * AirboneSatelliteHeightBoundary = 60e3  # 60 km
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_min_slant_range);
  __Pyx_GIVEREF(__pyx_n_s_min_slant_range);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_min_slant_range);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_torchsar_sharing_slant_ground_ra, __pyx_t_1, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_min_slant_range); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_min_slant_range, __pyx_t_1) < 0) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":35
 * from torchsar.sharing.slant_ground_range import min_slant_range
 * 
 * AirboneSatelliteHeightBoundary = 60e3  # 60 km             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_AirboneSatelliteHeightBoundary, __pyx_float_60e3) < 0) __PYX_ERR(0, 35, __pyx_L1_error)

  /* "torchsar/misc/sarplat.py":38
 * 
 * 
 * class SarPlat(object):             # <<<<<<<<<<<<<<
 *     r"""SAR platform class.
 *         sensor
 */
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_tuple__24); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_tuple__24, __pyx_n_s_SarPlat, __pyx_n_s_SarPlat, (PyObject *) NULL, __pyx_n_s_torchsar_misc_sarplat, __pyx_kp_s_SAR_platform_class_sensor_acquis); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "torchsar/misc/sarplat.py":46
 * 
 *     @property
 *     def name(self):             # <<<<<<<<<<<<<<
 *         return self._name
 * 
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_1name, 0, __pyx_n_s_SarPlat_name, NULL, __pyx_n_s_torchsar_misc_sarplat, __pyx_d, ((PyObject *)__pyx_codeobj__26)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "torchsar/misc/sarplat.py":45
 *     """
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def name(self):
 *         return self._name
 */
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_property, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_1, __pyx_n_s_name_3, __pyx_t_4) < 0) __PYX_ERR(0, 46, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "torchsar/misc/sarplat.py":49
 *         return self._name
 * 
 *     @name.setter             # <<<<<<<<<<<<<<
 *     def name(self, value):
 *         logging.info("---Check and set SAR platform name...")
 */
  __pyx_t_3 = PyObject_GetItem(__pyx_t_1, __pyx_n_s_name_3);
  if (unlikely(!__pyx_t_3)) {
    PyErr_Clear();
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_name_3);
  }
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_setter); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":50
 * 
 *     @name.setter
 *     def name(self, value):             # <<<<<<<<<<<<<<
 *         logging.info("---Check and set SAR platform name...")
 * 
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_3name, 0, __pyx_n_s_SarPlat_name, NULL, __pyx_n_s_torchsar_misc_sarplat, __pyx_d, ((PyObject *)__pyx_codeobj__28)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_1, __pyx_n_s_name_3, __pyx_t_4) < 0) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "torchsar/misc/sarplat.py":59
 * 
 *     @property
 *     def sensor(self):             # <<<<<<<<<<<<<<
 *         return self._sensor
 * 
 */
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_5sensor, 0, __pyx_n_s_SarPlat_sensor, NULL, __pyx_n_s_torchsar_misc_sarplat, __pyx_d, ((PyObject *)__pyx_codeobj__30)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "torchsar/misc/sarplat.py":58
 *         logging.info("---Done!")
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def sensor(self):
 *         return self._sensor
 */
  __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_builtin_property, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_1, __pyx_n_s_sensor_3, __pyx_t_5) < 0) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "torchsar/misc/sarplat.py":62
 *         return self._sensor
 * 
 *     @sensor.setter             # <<<<<<<<<<<<<<
 *     def sensor(self, value):
 *         logging.info("---Check and set SAR sensor parameters...")
 */
  __pyx_t_4 = PyObject_GetItem(__pyx_t_1, __pyx_n_s_sensor_3);
  if (unlikely(!__pyx_t_4)) {
    PyErr_Clear();
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_sensor_3);
  }
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 62, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_setter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 62, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "torchsar/misc/sarplat.py":63
 * 
 *     @sensor.setter
 *     def sensor(self, value):             # <<<<<<<<<<<<<<
 *         logging.info("---Check and set SAR sensor parameters...")
 * 
 */
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_7sensor, 0, __pyx_n_s_SarPlat_sensor, NULL, __pyx_n_s_torchsar_misc_sarplat, __pyx_d, ((PyObject *)__pyx_codeobj__32)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_5 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_6, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 62, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_1, __pyx_n_s_sensor_3, __pyx_t_5) < 0) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "torchsar/misc/sarplat.py":104
 * 
 *     @property
 *     def acquisition(self):             # <<<<<<<<<<<<<<
 *         return self._acquisition
 * 
 */
  __pyx_t_5 = __Pyx_CyFunction_New(&__pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_9acquisition, 0, __pyx_n_s_SarPlat_acquisition, NULL, __pyx_n_s_torchsar_misc_sarplat, __pyx_d, ((PyObject *)__pyx_codeobj__34)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);

  /* "torchsar/misc/sarplat.py":103
 *         logging.info("---Done!")
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def acquisition(self):
 *         return self._acquisition
 */
  __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_property, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_1, __pyx_n_s_acquisition_3, __pyx_t_3) < 0) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":107
 *         return self._acquisition
 * 
 *     @acquisition.setter             # <<<<<<<<<<<<<<
 *     def acquisition(self, value):
 *         logging.info("---Check and set acquisition parameters...")
 */
  __pyx_t_5 = PyObject_GetItem(__pyx_t_1, __pyx_n_s_acquisition_3);
  if (unlikely(!__pyx_t_5)) {
    PyErr_Clear();
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_acquisition_3);
  }
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_setter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "torchsar/misc/sarplat.py":108
 * 
 *     @acquisition.setter
 *     def acquisition(self, value):             # <<<<<<<<<<<<<<
 *         logging.info("---Check and set acquisition parameters...")
 * 
 */
  __pyx_t_5 = __Pyx_CyFunction_New(&__pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_11acquisition, 0, __pyx_n_s_SarPlat_acquisition, NULL, __pyx_n_s_torchsar_misc_sarplat, __pyx_d, ((PyObject *)__pyx_codeobj__36)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 108, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_3 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_1, __pyx_n_s_acquisition_3, __pyx_t_3) < 0) __PYX_ERR(0, 108, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":244
 * 
 *     @property
 *     def params(self):             # <<<<<<<<<<<<<<
 *         return self._params
 * 
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_13params, 0, __pyx_n_s_SarPlat_params, NULL, __pyx_n_s_torchsar_misc_sarplat, __pyx_d, ((PyObject *)__pyx_codeobj__38)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "torchsar/misc/sarplat.py":243
 *         self._acquisition = value
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def params(self):
 *         return self._params
 */
  __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_property, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 243, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_1, __pyx_n_s_params_3, __pyx_t_4) < 0) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "torchsar/misc/sarplat.py":247
 *         return self._params
 * 
 *     @params.setter             # <<<<<<<<<<<<<<
 *     def params(self, value):
 * 
 */
  __pyx_t_3 = PyObject_GetItem(__pyx_t_1, __pyx_n_s_params_3);
  if (unlikely(!__pyx_t_3)) {
    PyErr_Clear();
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_params_3);
  }
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_setter); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "torchsar/misc/sarplat.py":248
 * 
 *     @params.setter
 *     def params(self, value):             # <<<<<<<<<<<<<<
 * 
 *         logging.info("---Check and set algorithm parameters...")
 */
  __pyx_t_3 = __Pyx_CyFunction_New(&__pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_15params, 0, __pyx_n_s_SarPlat_params, NULL, __pyx_n_s_torchsar_misc_sarplat, __pyx_d, ((PyObject *)__pyx_codeobj__40)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  __pyx_t_4 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_6, __pyx_t_3) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_1, __pyx_n_s_params_3, __pyx_t_4) < 0) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "torchsar/misc/sarplat.py":471
 * 
 *     @property
 *     def selection(self):             # <<<<<<<<<<<<<<
 *         return self._selection
 * 
 */
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_17selection, 0, __pyx_n_s_SarPlat_selection, NULL, __pyx_n_s_torchsar_misc_sarplat, __pyx_d, ((PyObject *)__pyx_codeobj__42)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 471, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "torchsar/misc/sarplat.py":470
 *         return self._params
 * 
 *     @property             # <<<<<<<<<<<<<<
 *     def selection(self):
 *         return self._selection
 */
  __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_builtin_property, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_1, __pyx_n_s_selection_3, __pyx_t_5) < 0) __PYX_ERR(0, 471, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "torchsar/misc/sarplat.py":474
 *         return self._selection
 * 
 *     @selection.setter             # <<<<<<<<<<<<<<
 *     def selection(self, value):
 *         logging.info("---Check and set sub area selection...")
 */
  __pyx_t_4 = PyObject_GetItem(__pyx_t_1, __pyx_n_s_selection_3);
  if (unlikely(!__pyx_t_4)) {
    PyErr_Clear();
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_selection_3);
  }
  if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_setter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "torchsar/misc/sarplat.py":475
 * 
 *     @selection.setter
 *     def selection(self, value):             # <<<<<<<<<<<<<<
 *         logging.info("---Check and set sub area selection...")
 *         SA = self._acquisition['SceneArea']
 */
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_19selection, 0, __pyx_n_s_SarPlat_selection, NULL, __pyx_n_s_torchsar_misc_sarplat, __pyx_d, ((PyObject *)__pyx_codeobj__44)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 475, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_5 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_6, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 474, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__Pyx_SetNameInClass(__pyx_t_1, __pyx_n_s_selection_3, __pyx_t_5) < 0) __PYX_ERR(0, 475, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "torchsar/misc/sarplat.py":673
 *         logging.info('---Done!')
 * 
 *     def printsp(self, verbose=False):             # <<<<<<<<<<<<<<
 *         print("===============SAR platform: ", self._name, "================")
 *         print("--------------------------sensor-----------------------------")
 */
  __pyx_t_5 = __Pyx_CyFunction_New(&__pyx_mdef_8torchsar_4misc_7sarplat_7SarPlat_21printsp, 0, __pyx_n_s_SarPlat_printsp, NULL, __pyx_n_s_torchsar_misc_sarplat, __pyx_d, ((PyObject *)__pyx_codeobj__46)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 673, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_5, __pyx_tuple__47);
  if (__Pyx_SetNameInClass(__pyx_t_1, __pyx_n_s_printsp, __pyx_t_5) < 0) __PYX_ERR(0, 673, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "torchsar/misc/sarplat.py":38
 * 
 * 
 * class SarPlat(object):             # <<<<<<<<<<<<<<
 *     r"""SAR platform class.
 *         sensor
 */
  __pyx_t_5 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_n_s_SarPlat, __pyx_tuple__24, __pyx_t_1, NULL, 0, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SarPlat, __pyx_t_5) < 0) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "torchsar/misc/sarplat.py":1
 * #!/usr/bin/env python             # <<<<<<<<<<<<<<
 * #-*- coding: utf-8 -*-
 * # @file      : sarplat.py
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init torchsar.misc.sarplat", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init torchsar.misc.sarplat");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (descr != NULL) {
        *method = descr;
        return 0;
    }
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(name));
#endif
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod0 */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* RaiseNoneIterError */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* UnpackTupleError */
static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else if (PyTuple_GET_SIZE(t) < index) {
      __Pyx_RaiseNeedMoreValuesError(PyTuple_GET_SIZE(t));
    } else {
      __Pyx_RaiseTooManyValuesError(index);
    }
}

/* UnpackTuple2 */
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
        PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2, int decref_tuple) {
    PyObject *value1 = NULL, *value2 = NULL;
#if CYTHON_COMPILING_IN_PYPY
    value1 = PySequence_ITEM(tuple, 0);  if (unlikely(!value1)) goto bad;
    value2 = PySequence_ITEM(tuple, 1);  if (unlikely(!value2)) goto bad;
#else
    value1 = PyTuple_GET_ITEM(tuple, 0);  Py_INCREF(value1);
    value2 = PyTuple_GET_ITEM(tuple, 1);  Py_INCREF(value2);
#endif
    if (decref_tuple) {
        Py_DECREF(tuple);
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
#if CYTHON_COMPILING_IN_PYPY
bad:
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
#endif
}
static int __Pyx_unpack_tuple2_generic(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                       int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    iternextfunc iternext;
    iter = PyObject_GetIter(tuple);
    if (unlikely(!iter)) goto bad;
    if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
    iternext = Py_TYPE(iter)->tp_iternext;
    value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
    value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
    if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
    Py_DECREF(iter);
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

/* dict_iter */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
    if (is_dict) {
#if !CYTHON_COMPILING_IN_PYPY
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
#elif PY_MAJOR_VERSION >= 3
        static PyObject *py_items = NULL, *py_keys = NULL, *py_values = NULL;
        PyObject **pp = NULL;
        if (method_name) {
            const char *name = PyUnicode_AsUTF8(method_name);
            if (strcmp(name, "iteritems") == 0) pp = &py_items;
            else if (strcmp(name, "iterkeys") == 0) pp = &py_keys;
            else if (strcmp(name, "itervalues") == 0) pp = &py_values;
            if (pp) {
                if (!*pp) {
                    *pp = PyUnicode_FromString(name + 4);
                    if (!*pp)
                        return NULL;
                }
                method_name = *pp;
            }
        }
#endif
    }
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_COMPILING_IN_PYPY
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_COMPILING_IN_PYPY
    if (source_is_dict) {
        PyObject *key, *value;
        if (unlikely(orig_length != PyDict_Size(iter_obj))) {
            PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
            return -1;
        }
        if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
            return 0;
        }
        if (pitem) {
            PyObject* tuple = PyTuple_New(2);
            if (unlikely(!tuple)) {
                return -1;
            }
            Py_INCREF(key);
            Py_INCREF(value);
            PyTuple_SET_ITEM(tuple, 0, key);
            PyTuple_SET_ITEM(tuple, 1, value);
            *pitem = tuple;
        } else {
            if (pkey) {
                Py_INCREF(key);
                *pkey = key;
            }
            if (pvalue) {
                Py_INCREF(value);
                *pvalue = value;
            }
        }
        return 1;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyTuple_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        if (unlikely(pos >= PyList_GET_SIZE(iter_obj))) return 0;
        *ppos = pos + 1;
        next_item = PyList_GET_ITEM(iter_obj, pos);
        Py_INCREF(next_item);
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

/* DictGetItem */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            if (unlikely(PyTuple_Check(key))) {
                PyObject* args = PyTuple_Pack(1, key);
                if (likely(args)) {
                    PyErr_SetObject(PyExc_KeyError, args);
                    Py_DECREF(args);
                }
            } else {
                PyErr_SetObject(PyExc_KeyError, key);
            }
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* PyFloatBinop */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_TrueDivideObjC(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check) {
    const double b = floatval;
    double a, result;
    (void)inplace;
    (void)zerodivision_check;
    if (likely(PyFloat_CheckExact(op1))) {
        a = PyFloat_AS_DOUBLE(op1);
        
    } else
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        a = (double) PyInt_AS_LONG(op1);
        
    } else
    #endif
    if (likely(PyLong_CheckExact(op1))) {
        #if CYTHON_USE_PYLONG_INTERNALS
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        switch (size) {
            case  0: a = 0.0; break;
            case -1: a = -(double) digits[0]; break;
            case  1: a = (double) digits[0]; break;
            case -2:
            case 2:
                if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (1 * PyLong_SHIFT < 53))) {
                    a = (double) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -2)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -3:
            case 3:
                if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53))) {
                    a = (double) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -3)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -4:
            case 4:
                if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53))) {
                    a = (double) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (4 * PyLong_SHIFT < 53) || (a < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -4)
                            a = -a;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            default:
        #else
        {
        #endif
            a = PyLong_AsDouble(op1);
            if (unlikely(a == -1.0 && PyErr_Occurred())) return NULL;
            
        }
    } else {
        return (inplace ? PyNumber_InPlaceTrueDivide : PyNumber_TrueDivide)(op1, op2);
    }
        
        PyFPE_START_PROTECT("divide", return NULL)
        result = a / b;
        PyFPE_END_PROTECT(result)
        return PyFloat_FromDouble(result);
}
#endif

/* py_abs */
  #if CYTHON_USE_PYLONG_INTERNALS
static PyObject *__Pyx_PyLong_AbsNeg(PyObject *n) {
    if (likely(Py_SIZE(n) == -1)) {
        return PyLong_FromLong(((PyLongObject*)n)->ob_digit[0]);
    }
#if CYTHON_COMPILING_IN_CPYTHON
    {
        PyObject *copy = _PyLong_Copy((PyLongObject*)n);
        if (likely(copy)) {
            __Pyx_SET_SIZE(copy, -Py_SIZE(copy));
        }
        return copy;
    }
#else
    return PyNumber_Negative(n);
#endif
}
#endif

/* SetItemInt */
  static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (!j) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o)))) {
            PyObject* old = PyList_GET_ITEM(o, n);
            Py_INCREF(v);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 1;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return m->sq_ass_item(o, i, v);
        }
    }
#else
#if CYTHON_COMPILING_IN_PYPY
    if (is_list || (PySequence_Check(o) && !PyDict_Check(o)))
#else
    if (is_list || PySequence_Check(o))
#endif
    {
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyInt_FromSsize_t(i), v);
}

/* BytesEquals */
  static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
  static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* PyObjectFormat */
  #if CYTHON_USE_UNICODE_WRITER
static PyObject* __Pyx_PyObject_Format(PyObject* obj, PyObject* format_spec) {
    int ret;
    _PyUnicodeWriter writer;
    if (likely(PyFloat_CheckExact(obj))) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x03040000
        _PyUnicodeWriter_Init(&writer, 0);
#else
        _PyUnicodeWriter_Init(&writer);
#endif
        ret = _PyFloat_FormatAdvancedWriter(
            &writer,
            obj,
            format_spec, 0, PyUnicode_GET_LENGTH(format_spec));
    } else if (likely(PyLong_CheckExact(obj))) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x03040000
        _PyUnicodeWriter_Init(&writer, 0);
#else
        _PyUnicodeWriter_Init(&writer);
#endif
        ret = _PyLong_FormatAdvancedWriter(
            &writer,
            obj,
            format_spec, 0, PyUnicode_GET_LENGTH(format_spec));
    } else {
        return PyObject_Format(obj, format_spec);
    }
    if (unlikely(ret == -1)) {
        _PyUnicodeWriter_Dealloc(&writer);
        return NULL;
    }
    return _PyUnicodeWriter_Finish(&writer);
}
#endif

/* JoinPyUnicode */
  static PyObject* __Pyx_PyUnicode_Join(PyObject* value_tuple, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      CYTHON_UNUSED Py_UCS4 max_char) {
#if CYTHON_USE_UNICODE_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyObject *result_uval;
    int result_ukind;
    Py_ssize_t i, char_pos;
    void *result_udata;
#if CYTHON_PEP393_ENABLED
    result_uval = PyUnicode_New(result_ulength, max_char);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = (max_char <= 255) ? PyUnicode_1BYTE_KIND : (max_char <= 65535) ? PyUnicode_2BYTE_KIND : PyUnicode_4BYTE_KIND;
    result_udata = PyUnicode_DATA(result_uval);
#else
    result_uval = PyUnicode_FromUnicode(NULL, result_ulength);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = sizeof(Py_UNICODE);
    result_udata = PyUnicode_AS_UNICODE(result_uval);
#endif
    char_pos = 0;
    for (i=0; i < value_count; i++) {
        int ukind;
        Py_ssize_t ulength;
        void *udata;
        PyObject *uval = PyTuple_GET_ITEM(value_tuple, i);
        if (unlikely(__Pyx_PyUnicode_READY(uval)))
            goto bad;
        ulength = __Pyx_PyUnicode_GET_LENGTH(uval);
        if (unlikely(!ulength))
            continue;
        if (unlikely(char_pos + ulength < 0))
            goto overflow;
        ukind = __Pyx_PyUnicode_KIND(uval);
        udata = __Pyx_PyUnicode_DATA(uval);
        if (!CYTHON_PEP393_ENABLED || ukind == result_ukind) {
            memcpy((char *)result_udata + char_pos * result_ukind, udata, (size_t) (ulength * result_ukind));
        } else {
            #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030300F0 || defined(_PyUnicode_FastCopyCharacters)
            _PyUnicode_FastCopyCharacters(result_uval, char_pos, uval, 0, ulength);
            #else
            Py_ssize_t j;
            for (j=0; j < ulength; j++) {
                Py_UCS4 uchar = __Pyx_PyUnicode_READ(ukind, udata, j);
                __Pyx_PyUnicode_WRITE(result_ukind, result_udata, char_pos+j, uchar);
            }
            #endif
        }
        char_pos += ulength;
    }
    return result_uval;
overflow:
    PyErr_SetString(PyExc_OverflowError, "join() result is too long for a Python string");
bad:
    Py_DECREF(result_uval);
    return NULL;
#else
    result_ulength++;
    value_count++;
    return PyUnicode_Join(__pyx_empty_unicode, value_tuple);
#endif
}

/* PyFloatBinop */
  #if !CYTHON_COMPILING_IN_PYPY
#define __Pyx_PyFloat_TrueDivideCObj_ZeroDivisionError(operand) if (unlikely(zerodivision_check && ((operand) == 0))) {\
    PyErr_SetString(PyExc_ZeroDivisionError, "float division by zero");\
    return NULL;\
}
static PyObject* __Pyx_PyFloat_TrueDivideCObj(PyObject *op1, PyObject *op2, double floatval, int inplace, int zerodivision_check) {
    const double a = floatval;
    double b, result;
    (void)inplace;
    (void)zerodivision_check;
    if (likely(PyFloat_CheckExact(op2))) {
        b = PyFloat_AS_DOUBLE(op2);
        __Pyx_PyFloat_TrueDivideCObj_ZeroDivisionError(b)
    } else
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op2))) {
        b = (double) PyInt_AS_LONG(op2);
        __Pyx_PyFloat_TrueDivideCObj_ZeroDivisionError(b)
    } else
    #endif
    if (likely(PyLong_CheckExact(op2))) {
        #if CYTHON_USE_PYLONG_INTERNALS
        const digit* digits = ((PyLongObject*)op2)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op2);
        switch (size) {
            case  0: __Pyx_PyFloat_TrueDivideCObj_ZeroDivisionError(0) break;
            case -1: b = -(double) digits[0]; break;
            case  1: b = (double) digits[0]; break;
            case -2:
            case 2:
                if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (1 * PyLong_SHIFT < 53))) {
                    b = (double) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53) || (b < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -2)
                            b = -b;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -3:
            case 3:
                if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53))) {
                    b = (double) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53) || (b < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -3)
                            b = -b;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            case -4:
            case 4:
                if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53))) {
                    b = (double) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (4 * PyLong_SHIFT < 53) || (b < (double) ((PY_LONG_LONG)1 << 53))) {
                        if (size == -4)
                            b = -b;
                        break;
                    }
                }
                CYTHON_FALLTHROUGH;
            default:
        #else
        {
        #endif
            b = PyLong_AsDouble(op2);
            if (unlikely(b == -1.0 && PyErr_Occurred())) return NULL;
            __Pyx_PyFloat_TrueDivideCObj_ZeroDivisionError(b)
        }
    } else {
        return (inplace ? PyNumber_InPlaceTrueDivide : PyNumber_TrueDivide)(op1, op2);
    }
        __Pyx_PyFloat_TrueDivideCObj_ZeroDivisionError(b)
        PyFPE_START_PROTECT("divide", return NULL)
        result = a / b;
        PyFPE_END_PROTECT(result)
        return PyFloat_FromDouble(result);
}
#endif

/* None */
    static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

/* PyObjectFormatAndDecref */
    static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f) {
    if (unlikely(!s)) return NULL;
    if (likely(PyUnicode_CheckExact(s))) return s;
    #if PY_MAJOR_VERSION < 3
    if (likely(PyString_CheckExact(s))) {
        PyObject *result = PyUnicode_FromEncodedObject(s, NULL, "strict");
        Py_DECREF(s);
        return result;
    }
    #endif
    return __Pyx_PyObject_FormatAndDecref(s, f);
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f) {
    PyObject *result = PyObject_Format(s, f);
    Py_DECREF(s);
    return result;
}

/* ObjectGetItem */
    #if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* Import */
    static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
    static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* CalculateMetaclass */
    static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* FetchCommonType */
    static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunctionShared */
    #include <structmember.h>
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    Py_INCREF(m->func_qualname);
    return m->func_qualname;
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    if (unlikely(op == NULL))
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
#if PY_MAJOR_VERSION < 3
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
#endif
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1 && (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800)
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
    static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* Py3ClassCreate */
    static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

/* CLineInTraceback */
    #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        (void) PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
    static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
    #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_srcfile = NULL;
    py_srcfile = PyString_FromString(filename);
    if (!py_srcfile) goto bad;
    #endif
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        if (!py_funcname) goto bad;
        #endif
    }
    #if PY_MAJOR_VERSION < 3
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    #else
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    #endif
    Py_XDECREF(py_funcname);  // XDECREF since it's only set on Py3 if cline
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_srcfile);
    #endif
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
    static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPyVerify */
    #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntFromPy */
    static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
    static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* FastTypeChecks */
    #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CStringEquals */
    static CYTHON_INLINE int __Pyx_StrEq(const char *s1, const char *s2) {
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    return *s1 == *s2;
}

/* CheckBinaryVersion */
    static int __Pyx_check_binary_version(void) {
    char ctversion[5];
    int same=1, i, found_dot;
    const char* rt_from_call = Py_GetVersion();
    PyOS_snprintf(ctversion, 5, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    found_dot = 0;
    for (i = 0; i < 4; i++) {
        if (!ctversion[i]) {
            same = (rt_from_call[i] < '0' || rt_from_call[i] > '9');
            break;
        }
        if (rt_from_call[i] != ctversion[i]) {
            same = 0;
            break;
        }
    }
    if (!same) {
        char rtversion[5] = {'\0'};
        char message[200];
        for (i=0; i<4; ++i) {
            if (rt_from_call[i] == '.') {
                if (found_dot) break;
                found_dot = 1;
            } else if (rt_from_call[i] < '0' || rt_from_call[i] > '9') {
                break;
            }
            rtversion[i] = rt_from_call[i];
        }
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
    static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
#if PY_MAJOR_VERSION < 3
  } else if (likely(PyInt_CheckExact(o))) {
    return PyInt_AS_LONG(o);
#endif
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyInt_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
